// -----------------------------------------------------------------------
//  <copyright file="MakoAssembly.cpp" company="Global Graphics Software Ltd">
//      Copyright (c) 2017 Global Graphics Software Ltd. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

#include "MakoAssembly.h"
#include <jawsmako/jawsmako.h>
#include <jawsmako/pdfoutput.h>
#include <edl/idommetadata.h>
#include <algorithm>

const U8String DEFAULT_PDF_VERSION = "1.7";

MakoAssembly::MakoAssembly(IJawsMakoPtr mako, IDocumentAssemblyPtr assembly, std::string sourceFilePath) :
    m_mako(mako), m_assembly(assembly), m_pdfVersion(DEFAULT_PDF_VERSION), m_sourceFilePath(std::move(sourceFilePath))
{
    IDOMMetadataPtr metadata = m_assembly->getJobMetadata();
    if (metadata)
        m_pdfVersion = getPdfVersion(metadata);
}

MakoDocument* MakoAssembly::openDocument(int documentIndex = 0) const
{
    const IDocumentPtr document = m_assembly->getDocument(documentIndex);
    return new MakoDocument(m_mako, document, m_sourceFilePath);
}

U8String MakoAssembly::getPdfVersion(IDOMMetadataPtr& domMetadata)
{
    PValue value;
    if (domMetadata->getProperty(IDOMMetadata::ePDFInfo, "Version", value))
        return StringToU8String(value.getString());

    return U8String(DEFAULT_PDF_VERSION);
}

void MakoAssembly::setPdfVersion(IPDFOutputPtr pdfOutput, const U8String version)
{
    try
    {
        pdfOutput->setParameter("PDFVersion", version);
    }
    catch (IError& e)
    {
        // If it's not a configuration error, throw onwards.
        if (e.getErrorCode() != JM_ERR_BAD_CONFIGURATION)
            throw;

        // Failed to set the version, this may be because the source PDF version is
        // not a version that mako supports. In this case, lets continue and let mako 
        // work out the best version for the resulting document.
    }
}

// Determine the associated format for a given path from the file extension
static eFileFormat formatFromPath(const U8String &path)
{
    // Get the extension in lower case
    if (path.size() < 4)
    {
        // Cannot determine the extension if there isn't one!
        std::string message("Cannot determine file extension for path ");
        message += path.c_str();
        throw std::length_error(message);
    }

    size_t extensionPosition = path.find_last_of('.');
    U8String extension = path.substr(extensionPosition);
    std::transform(extension.begin(), extension.end(), extension.begin(), towlower);

    if (extension == ".xps")
        return eFFXPS;
    if (extension == ".pdf")
        return eFFPDF;
     if (extension == ".svg")
        return eFFSVG;
     if (extension == ".ps")
        return eFFPS;
     if (extension == ".pcl")
        return eFFPCL5;
     if (extension == ".pxl")
        return eFFPCLXL;

    std::string message("Unsupported file type for path ");
    message += path.c_str();
    throw std::invalid_argument(message);
}

void MakoAssembly::save(const U8String path, const CEDLSysStringVect output_params) const
{
    IOutputPtr fileOutput = IOutput::create(m_mako, formatFromPath(path));
    for (int i = 0; i < output_params.size(); i++)
    {
        size_t equalsPos = output_params[i].find('=');
        // Set the output parameter
        if (equalsPos != String::npos)
            fileOutput->setParameter(output_params[i].substr(0, equalsPos), output_params[i].substr(equalsPos + 1));
    }

    fileOutput->writeAssembly(m_assembly, path);
}

void MakoAssembly::saveAsPdf(const U8String path, bool incremental) const
{
    IPDFOutputPtr pdfOutput = IPDFOutput::create(m_mako);

    setPdfVersion(pdfOutput, m_pdfVersion);    
    pdfOutput->setEnableIncrementalOutput(incremental);

    pdfOutput->writeAssembly(m_assembly, path);
}

MakoAssembly::~MakoAssembly() = default;
