/*
 * Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file        edlerrors.h
 * @ingroup     IController
 * @brief       EDL errors
 *
 * Many of the EDL APIs return an (unsigned) integer result code
 * Where EDL_OK (0) indicates "success" and a non-zero value indicates
 * an error of some kind.
 *
 * There is a separate file "edlerrors.cpp" that provides a look-up function
 * to convert these integer result codes into (localizable) strings
 */

#ifndef __EDLERRORS_H__
#define __EDLERRORS_H__

#include <typeinfo>
#include <edl/edltypes.h>
#include <edl/edlstring.h>

_BEGIN_EDL_NAMESPACE

const  uint32 EDL_OK  = 0;

// Commmon Errors 1..499
const uint32 EDL_ERR_PANIC = 1;                 //!< Unstable state - abort the application
const uint32 EDL_ERR_UNDEFINED = 2;             //!< Undefined error
const uint32 EDL_ERR_LICENSE = 3;               //!< Invalid serial number or licence provided
const uint32 EDL_ERR_FILENOTFOUND = 4;          //!< File not found
const uint32 EDL_ERR_CORRUPT_PNG_IMAGE = 5;     //!< Corrupt PNG image encountered
const uint32 EDL_ERR_OUTOFMEMORY = 6;           //!< Out of memory
const uint32 EDL_ERR_READ = 7;                  //!< Read error
const uint32 EDL_ERR_LARGE_FILE = 8;            //!< File too large
const uint32 EDL_ERR_OPENFORWRITE = 9;          //!< Open file for write failed
const uint32 EDL_ERR_WRITE = 10;                //!< Write failed
const uint32 EDL_ERR_IOERROR = 11;              //!< General IO error
const uint32 EDL_ERR_COMPRESS = 12;             //!< Compression error
const uint32 EDL_ERR_CONFIG = 13;               //!< Configuration error
const uint32 EDL_ERR_RESTRICTED_FONT = 14;      //!< A restricted font has been encountered
const uint32 EDL_ERR_ABORTED = 15;              //!< Job aborted by user
const uint32 EDL_ERR_BAD_ARGUMENTS = 16;        //!< General error for bad arguments passed to an API function
const uint32 EDL_ERR_VECTOR_ERROR = 17;         //!< An out-of range value was accessed in a CEDLVector
const uint32 EDL_ERR_BAD_FONT_MAP = 18;         //!< An error was found in a FontMap file
const uint32 EDL_ERR_UNICODE = 19;              //!< An invalid Unicode string was encountered
const uint32 EDL_ERR_PAGE_TOO_LARGE = 20;       //!< A page was too large to be used

// JawsMako API-specific errors
const uint32 JM_ERR_GENERAL = 99;              //!< A general error occurred. Additional information will be provided with the exception
const uint32 JM_ERR_UNSUPPORTED = 100;         //!< An attempt was made to use an unsupported, unimplemented feature
const uint32 JM_ERR_REVERT_FAILED = 101;       //!< An attempt was made to revert a page that could not be performed
const uint32 JM_ERR_TARGET_NOT_FOUND = 102;    //!< A target with the given DOMid was not found
const uint32 JM_ERR_RANGE_ERROR = 103;         //!< An attempt was made to request or add an item from the API with an out-of-bounds index
const uint32 JM_ERR_PAGE_NOT_FOUND = 104;      //!< The given page was not found in a document
const uint32 JM_ERR_DOCUMENT_NOT_FOUND = 105;  //!< The given document was not found in an assembly
const uint32 JM_DUPLICATE_FIELD_PARTIAL_NAME = 106; //!< A field with a duplicate partial name was found
const uint32 JM_ERR_FORM_FIELD_NOT_FOUND = 107;//!< The given form field was not found
const uint32 JM_ERR_DUPLICATE_WIDGET = 108;    //!< A widget with a duplicate reference was not found
const uint32 JM_ERR_WIDGET_NOT_FOUND = 109;    //!< The given widget annotation or annotation reference could not be found
const uint32 JM_ERR_BAD_CONFIGURATION = 110;   //!< General configuration error
const uint32 JM_ERR_APPEARANCE_NOT_FOUND = 111;//!< The required annotation appearance was not found
const uint32 JM_ERR_ANNOTATION_NOT_FOUND = 112;//!< The required annotation was not found
const uint32 JM_ERR_ASSEMBLY_WRITE_FORBIDDEN = 113; //!< The given assembly cannot be written due to security permissions
const uint32 JM_ERR_FONT_NOT_FOUND = 114;      //!< The requested font was not found
const uint32 JM_ERR_TOO_MANY_PDFOUT_WRITERS = 115; //!< There are too many PDF Out writers currently in progress
const uint32 JM_ERR_INFORMATION_NOT_AVAILABLE = 116; //!< The requested information is not yet available
const uint32 JM_ERR_DIRECTORY_DOESNT_EXIST = 117; //!< A required directory does not exist on disk
const uint32 JM_ERR_RESOURCE_NOT_FOUND = 118; //!< The requested resource could not be found
const uint32 JM_ERR_BAD_UNICODE_CMAP = 119; //!< A bad ToUnicode CMap was encountered
const uint32 JM_ERR_OPTIONAL_CONTENT_GROUP_NOT_FOUND = 120; //!< The requested optional content group was not found
const uint32 JM_ERR_INVALID_OPTIONAL_CONTENT = 121; //!< Invalid optional content information encountered
const uint32 JM_ERR_PCLXL = 122;               //!< PCL/XL Processing error
const uint32 JM_ERR_PCL = 123;                 //!< PCL5 Processing error
const uint32 JM_ERR_PJL = 124;                 //!< PJL Processing error
const uint32 JM_ERR_SYMBOLSET_NOT_FOUND = 125; //!< The requested PCL or PCL/XL symbol set was not found
const uint32 JM_ERR_PDF_OBJECT_NOT_FOUND = 126;//!< An expected PDF object was not found

// DOM Errors 500..999
const uint32 EDL_ERR_BAD_DIMENSIONS = 501;      //!< Bad dimensions specified for an object
const uint32 EDL_ERR_NULL_NODE = 502;           //!< An unexpected null DOM node pointer was encountered
const uint32 EDL_ERR_BAD_BRUSH = 503;           //!< A bad brush was encountered
const uint32 EDL_ERR_DOM_CREATION_FAILED = 504; //!< Failure while attempting to create a DOM object
const uint32 EDL_ERR_INVALID_DOM_ID = 505;      //!< An invalid DOMid was encountered
const uint32 EDL_ERR_INVALID_TYPE3_GLYPH = 506; //!< An invalid Type 3 Glyphs was encountered
const uint32 EDL_ERR_INVALID_GLYPHS = 507;      //!< An invalid Glyphs node was encountered.

// XPS specific errors
const uint32 EDL_ERR_INVALID_ZIP = 1001;        //!< XPS input: Invalid ZIP file encountered
const uint32 EDL_ERR_NO_CONTENT_TYPE = 1002;    //!< XPS input: Unable to retrieve the content type for the part
const uint32 EDL_ERR_PROCESS_PART = 1003;       //!< XPS input: Error processing the part
const uint32 EDL_ERR_XPS_PROVIDER = 1004;       //!< XPS input: XPS provider error
const uint32 EDL_ERR_MISSING_RESOURCE = 1005;   //!< XPS input: Missing resource
const uint32 EDL_ERR_DTD_CONTENT = 1006;        //!< XPS input: DTD content detected
const uint32 EDL_ERR_MISSING_REQ_ATTRIBUTE = 1007;  //!< XPS input: An element is missing a required attribute
const uint32 EDL_ERR_INVALID_ATTRIBUTE = 1008;      //!< XPS input: An invalid attribute has been encountered in the element
const uint32 EDL_ERR_PROPERTY_ALREADY_SET = 1009;   //!< XPS input: Attempt to set the same property twice
const uint32 EDL_ERR_REMOTERESOURCE_REF = 1010;     //!< XPS input: Remote resource dictionary references another remote resource dictionary
const uint32 EDL_ERR_KEY_SET_NON_RESOURCE = 1011;   //!< XPS input: Key attribute is set for an element that is not in the resource dictionary
const uint32 EDL_ERR_UNSUPPORTED_GLYPHS = 1012;     //!< XPS input: Unsupported glyphs encountered
const uint32 EDL_ERR_INVALID_URI = 1013;            //!< XPS input: Invalid URI encountered
const uint32 EDL_ERR_DEFLATE = 1014;                //!< XPS input: Deflate error
const uint32 EDL_ERR_NO_KEY_RESDICT_OBJECT = 1015;  //!< XPS input: Resource dictionary is missing the key attribute
const uint32 EDL_ERR_ADD_OBJECT_RESDICT = 1016;     //!< XPS input: Error adding the element to the resource dictionary
const uint32 EDL_ERR_INVALID_ABBR_PATH = 1017;      //!< XPS input: Error processing abbreviated path
const uint32 EDL_ERR_NAMESPACE_NOT_UNDERSTOOD = 1018;   //!< XPS input: Namespace is not understood
const uint32 EDL_ERR_XML_PARSER = 1019;                 //!< XPS input: Parser error encountered
const uint32 EDL_ERR_NO_REQ_ELEMENT = 1020;             //!< XPS input: Element is missing a required subelement
const uint32 EDL_ERR_MORE_THAN_ONE_ELEMENT = 1021;      //!< XPS input: Element has more than one instance of a subelement
const uint32 EDL_ERR_INVALID_ELEMENT = 1022;            //!< XPS input: Element contains an invalid subelement
const uint32 EDL_ERR_FALLBACK_BEFORE_CHOICE = 1023;     //!< XPS input: Fallback element has been found before choice
const uint32 EDL_ERR_PREFIX_NOT_DEFINED = 1024;         //!< XPS input: Undefined prefix encountered
const uint32 EDL_ERR_UNPREFIXED_ATTRIBUTE = 1025;       //!< XPS input: Unprefixed attribute encountered
const uint32 EDL_ERR_BROKEN_ELEMENT_SEQUENCE = 1026;    //!< XPS input: Broken element sequence encountered
const uint32 EDL_ERR_PREFIXED_ATTRIBUTE = 1027;         //!< XPS input: Prefixed attribute encountered
const uint32 EDL_ERR_INVALID_IMAGE = 1028;              //!< XPS input: Invalid image encountered
const uint32 EDL_ERR_NUMBER_REQ_ELEMENT = 1029;         //!< XPS input: Insufficient elements of a particular type encountered
const uint32 EDL_ERR_MISSING_XPS_PART = 1030;           //!< XPS input: Missing XPS part
const uint32 EDL_ERR_PRINTTICKET_EXISTS = 1031;         //!< XPS input: PrintTicket already exists
const uint32 EDL_ERR_MULTIPLE_STARTPARTS_RELS = 1032;   //!< XPS input: Multiple StartPart relationships encountered
const uint32 EDL_ERR_DICT_ITEM_PRESENT = 1033;          //!< XPS input: Multiple items with the same key present
const uint32 EDL_ERR_MULTIPLE_COREPROPERTIES_RELS = 1034;   //!< XPS input: Multiple CoreProperty relationships encountered
const uint32 EDL_ERR_MULTIPLE_DOCSTRUCTURE_RELS = 1035;     //!< XPS input: Multiple DocumentStructure relationships encountered
const uint32 EDL_ERR_MULTIPLE_THUMBNAIL_RELS = 1036;        //!< XPS input: Multiple thumbnail relationships encountered
const uint32 EDL_ERR_INVALID_THUMBNAIL_TYPE = 1037;         //!< XPS input: Invalid thumbnail type encountered
const uint32 EDL_ERR_PRESERVED_ITEMS_NOT_DECLARED_IGNORABLE = 1038; //!< XPS input: Unable to preserve items! The namespace is not declared as ignorable.
const uint32 EDL_ERR_MORE_THAN_ONE_GROUP_ELEMENT = 1039;    //!< XPS input: Element contains more than one subelement
const uint32 EDL_ERR_NO_REQ_GROUP_ELEMENT = 1040;           //!< XPS input: Element requires one subelement
const uint32 EDL_ERR_INVALID_COLOR_SPECIFICATION = 1041;    //!< XPS input: Invalid color specification encountered
const uint32 EDL_ERR_PROCESS_CONTENT_NOT_DECLARED_IGNORABLE = 1042; //!< XPS input: Namespace in ProcessContent attribute is not declared as ignorable
const uint32 EDL_ERR_DUPLICATE_URI = 1043;                  //!< XPS input: Duplicate URI encountered

// XPS out specific errors
const uint32 EDL_ERR_UNEXPECTED_RESOURCE = 1300;            //!< XPS output: Unexpected resource encountered
const uint32 EDL_ERR_DEVICE_OUTOFMEMORY = 1301;             //!< XPS output: Insufficient device memory to process page
const uint32 EDL_ERR_XPS_FROM_ENCRYPTED_SOURCE = 1302;      //!< XPS output: Attempt to create XPS from an encrypted document

// Outline specific errors
const uint32 EDL_ERR_DELETE_OUTLINE_WITH_CLOSED_PARENT = 1600;  //!< Outline: Attempt to delete outline entry with closed parent
const uint32 EDL_ERR_APPEND_OUTLINE_TO_CLOSED_NODE = 1601;      //!< Outline: Attempt to append outline entry to a closed node
const uint32 EDL_ERR_FAILS_TO_ADD_OUTLINE_TO_METADATA_STORAGE = 1602;   //!< Outline: Failed to append an outline entry to the metadata storage node

// Annotation specific errors
const uint32 EDL_ERR_APPEND_EMPTY_ANNOTATION = 1700;
const uint32 EDL_ERR_APPEND_ANNOTATION_WITHOUT_RECT = 1701;

// internal RIP erors
const uint32 EDL_ERR_INTERNAL_RIP = 2000;   //!< Internal RIP error

// PDF in specific errors
const uint32 EDL_ERR_OPEN_PDF = 2300;   //!< PDF input: Error opening PDF
const uint32 EDL_ERR_INVALID_PDF_PASSWORD = 2301;   //!< PDF input: Error opening encrypted PDF; Invalid password

// PDF/PS-out specific errors
const uint32 EDL_ERR_PSOUT_GENERAL_FAILURE = 2400; //!<PS output: General error 
const uint32 EDL_ERR_PDFOUT_GENERAL_FAILURE = 2401; //!<PDF output: General error 
const uint32 EDL_ERR_INVALID_FONT = 2402; //!<PDF output: Error processing a font
const uint32 EDL_ERR_INCOMPATIBLE_PDFA = 2403; //!<PDF/A output: Incompatible content found
const uint32 EDL_ERR_INCOMPATIBLE_PDFX = 2404; //!<PDF/X output: Incompatible content found

// Color specific errors
const uint32 EDL_ERR_BAD_COLOR_SPACE              = 2500; //!< Color: A bad color space was encountered.
const uint32 EDL_ERR_COLOR_CONVERSION_FAILURE     = 2501; //!< Color: Color conversion failed

// Image specific errors
const uint32 EDL_ERR_INCOMPATIBLE_IMAGE_PARAMETER = 2600; //!< Image: An image encoder has been given an image or parameter that it cannot handle
const uint32 EDL_ERR_IMAGE_PARAMETER_OUT_OF_RANGE = 2601; //!< Image: An image parameter for encoding or decoding is outside the expected range
const uint32 EDL_ERR_IMAGE_DECODE_FAILURE         = 2602; //!< Image: An image could not be decoded
const uint32 EDL_ERR_IMAGE_ENCODE_FAILURE         = 2603; //!< Image: An image could not be encoded

// ZIP specific errors
const uint32 EDL_ERR_ZIP_WRITE_ONLY     = 2700; //!< Zip: An attempt was made to retrieve a stream from a write-only Zip file
const uint32 EDL_ERR_ZIP_PATH_NOT_FOUND = 2701; //!< Zip: An expected entry in a Zip package was not found


struct EDL_ERR_LIST {
        uint32    errCode;
        EDLString errStr;
};

/**
 * @brief An abstract class for EDL exceptions.
 *
 *        what() may return only a generic message. For more detailed error messages
 *        add edl/edlerrors.cpp to your project, and use IEDLError::getErrorDescription(),
 *        passing the the results of getEDLErrorString(). For example:
 *
 *        catch (const EDL::IEDLError& e)
 *        {
 *              EDLString formatString = EDL::getEDLErrorString(e.getErrorCode());
 *              EDLString description = e.getErrorDescription(formatString);
 *              // deal with the description as seen fit
 *        }
 */
class IEDLError : public std::exception
{
  public:
    virtual uint32 getErrorCode() const = 0;
    virtual EDLString getErrorDescription(const EDLString &formatErrorString) = 0;
};

/**
 * @brief	Implements a simple numeric-error-code-to-corresponding-language-specific-error-message-string look-up
 *          using the "errorList" array/table
 * @param	errorCode The error code number
 * @return	EDLString Error message text
 */
EDLString getEDLErrorString(uint32 errorCode);

/**
 * @brief  Utility - Throw an IEDLError exception with the given error code
 * @param	errorcode The error code number
 */
EDL_API void throwEDLError(uint32 errorcode);

_END_EDL_NAMESPACE

#endif // __EDLERRORS_H__
