/*
 * Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file edlgeom.h
 * @ingroup IController
 * @brief Geometry primitives including: point, rectangle and matrix types
 *           supporting both integer and floating point values within.
 */

#ifndef EDLGEOM_H
#define EDLGEOM_H

#include <edl/edltypes.h>
#include <edl/edlmath.h>
#include <edl/edlvector.h>

_BEGIN_EDL_NAMESPACE

template <typename PointType>
class PointTmpl
{
public:
    PointTmpl() : x(0), y(0)
    {
    }
    PointTmpl(PointType ax, PointType ay) : x(ax), y(ay)
    {
    }
    PointTmpl<PointType> &operator += (const PointTmpl<PointType> &pt)
    {
        x += pt.x;
        y += pt.y;
        return *this;
    }
	
	bool equal(const PointTmpl &point) const
	{
		return ((x == point.x) && (y == point.y));
	}

    /**
    * @brief  Find the mid-point between this and another point.
    */
    PointTmpl<PointType> getMidPoint(const PointTmpl<PointType> &pt)
    {
        return PointTmpl<PointType>(x + (pt.x - x) / 2.0, y + (pt.y - y) / 2.0);
    }

    /**
    * @brief  Find the distance to another point.
    */
    double getDistance(const PointTmpl<PointType> &pt) const
    {
        return hypot(pt.x - x, pt.y - y);
    }

    PointType x, y;
};

template <typename PointType>
const PointTmpl<PointType> operator +(const PointTmpl<PointType>& lhp, const PointTmpl<PointType>& rhp)
{
    return PointTmpl<PointType>(lhp.x + rhp.x, lhp.y + rhp.y);
}
template <typename PointType>
const PointTmpl<PointType> operator -(const PointTmpl<PointType>& lhp, const PointTmpl<PointType>& rhp)
{
    return PointTmpl<PointType>(lhp.x - rhp.x, lhp.y - rhp.y);
}

typedef PointTmpl<double>               FPoint;
typedef PointTmpl<int32>                IntPoint;
typedef PointTmpl<int64>                Int64Point;
typedef CEDLVector<FPoint>              CFPointVect;

// Point type may not be an unsigned type
template <typename PointType>
class RectTmpl
{
public:
    RectTmpl() :
      x(0), y(0), dX(-1), dY(-1) 
    {};
    RectTmpl(PointType ax, PointType ay, PointType adX, PointType adY) :
      x(ax), y(ay), dX(adX), dY(adY) 
    {};
    PointType x;
    PointType y;
    PointType dX;
    PointType dY;
    void setEmpty()
    {
            dX = (PointType) -1;
            dY = (PointType) -1;
    }
    bool isEmpty() const
    {
            return (dX < (PointType) 0 || dY < (PointType) 0);
    }
	bool equal(const RectTmpl &rect) const
	{
		return ((x == rect.x) && (y == rect.y) &&
                (dX == rect.dX) && (dY == rect.dY));
	}
    bool similar(const RectTmpl &rect, float epsilon) const
    {
        return (fabs(x - rect.x) < epsilon &&
                fabs(y - rect.y) < epsilon &&
                fabs(dX - rect.dX) < epsilon &&
                fabs(dY - rect.dY) < epsilon);
    }
    RectTmpl &offset(PointType offX, PointType offY)
    {
        if (!isEmpty())
        {
            x += offX;
            y += offY;
        }
        return *this;
    }
    /** @brief Expand this rect if necessary to include point
     *  @param point The point to consider
     *  @return returns true if the box needed to be expanded */
    bool expandToPoint(const PointTmpl<PointType> &point)
    {
        bool result = false;
        if (isEmpty())
        {
            x = point.x; y = point.y;
            dX = 0; dY = 0;
            result = true;
        }
        else
        {
            if (point.x < x)
            {
                dX += (x - point.x);
                x = point.x;
                result = true;
            }
            if (point.y < y)
            {
                dY += (y - point.y);
                y = point.y;
                result = true;
            }                       
            if (point.x > x + dX)
            {
                dX = point.x - x;
                result = true;
            }
            if (point.y > y + dY)
            {
                dY = point.y - y;
                result = true;
            }
        }
        return result;
    }
    /** @brief Intersect this rect with another rect.
     *         If the rects do not intersect, the result is an empty rect.
     */
    void intersectRect(const RectTmpl<PointType> &rect)
    {
        if (!isEmpty() && !rect.isEmpty())
        {
            PointType llX = (x > rect.x) ? x : rect.x;
            PointType urX = (x + dX < rect.x + rect.dX) ? x + dX : rect.x + rect.dX;
            PointType llY = (y > rect.y) ? y : rect.y;
            PointType urY = (y + dY < rect.y + rect.dY) ? y + dY : rect.y + rect.dY;
            x = llX; y = llY; dX = urX - x; dY = urY - y;
        }
        else
        {
            setEmpty(); // Empty result
        }
    }
    /** @brief Unite this rect with another rect */
    void unionRect(const RectTmpl<PointType> &rect)
    {
        if (!isEmpty())
        {
            if (!rect.isEmpty())
            {
                PointType llX = (x < rect.x) ? x : rect.x;
                PointType urX = (x + dX > rect.x + rect.dX) ? x + dX : rect.x + rect.dX;
                PointType llY = (y < rect.y) ? y : rect.y;
                PointType urY = (y + dY > rect.y + rect.dY) ? y + dY : rect.y + rect.dY;
                x = llX; y = llY; dX = urX - x; dY = urY - y;
            }
        }
        else
        {
            x = rect.x; y = rect.y; dX = rect.dX; dY = rect.dY;
        }
    }
    /** @brief Does this rect intersect with another rect? */
    bool intersectsWithRect(const RectTmpl<PointType> &rect) const
    {
        RectTmpl<PointType> copy = rect;
        copy.intersectRect(*this);
        return !copy.isEmpty();
    }
    /** @brief Does this rectangle completely contain the given rect? */
    bool containsRect(const RectTmpl<PointType> &rect) const
    {
        if (rect.isEmpty() || isEmpty())
        {
            return false;
        }
        RectTmpl<PointType> copy = rect;
        copy.unionRect(*this);
        return equal(copy);
    }
    bool containsPoint(PointType px, PointType py) const
    {
        return x <= px && px <= getRight() && y <= py && py <= getBottom();
    }
    double getRight() const
    {
        return x + dX;
    }
    double getBottom() const
    {
        return y + dY;
    }
};

typedef RectTmpl<double>                FRect;
typedef RectTmpl<int32>                 IntRect;
typedef RectTmpl<int64>                 Int64Rect;

/// Matrix class - special 3x2 matrix
template <typename TItem>
class CTransformMatrix
{
public:

	//! Classification of operation type flags of the transform
	typedef enum {
	
		eDoesTranslate			= 0x1, //!< Does translate	
		eDoesScale				= 0x2, //!< Does scale		
		eDoesRotate				= 0x4, //!< Does rotate		
		eIsComplex				= 0x8  //!< Is complex		 
		
	} eOperationTypes;
	
    /**
     *  \brief Creates zero matrix
     */
	CTransformMatrix() 
    {
        set();
    }

    /**
	 *  \brief Creates the matrix elementary
	 *  
	 *  \param _xx xx component
	 *  \param _xy xy component
	 *  \param _yx yx component
	 *  \param _yy yy component
	 *  \param _dx dx component
	 *  \param _dy dy component
	 *  \return \link CTransformMatrix \endlink The matrix
	 */
	CTransformMatrix(TItem _xx,TItem _xy,TItem _yx,TItem _yy,TItem _dx,TItem _dy) {
        set(_xx, _xy, _yx, _yy, _dx, _dy);/*type = BMAT_NONE*/;
    }
    /**
	 *  \brief Copy constructor
	 *  \param m Transform matrix
	 */
	CTransformMatrix(const CTransformMatrix<TItem> &m) 
    { 
        *this=m; 
    }
    
	/**
	 *  \brief Creates a matrix that transforms from one rectangle to another
	 *  
	 *  \param sourceRect Source rectangle
	 *  \param destRect Destination rectangle
	 *  \return \link CTransformMatrix \endlink The matrix
	 */
	CTransformMatrix(const RectTmpl<TItem> &sourceRect, const RectTmpl<TItem> &destRect)
    {
        set();
        if (sourceRect.dX > 0 && destRect.dX > 0 && sourceRect.dY > 0 && destRect.dY > 0) 
        {
            m_xx = destRect.dX / sourceRect.dX;
            m_yy = destRect.dY / sourceRect.dY;
            m_dx = destRect.x - sourceRect.x * m_xx;
            m_dy = destRect.y - sourceRect.y * m_yy;
        }
    }
    
    /**
     *  \brief Create blank matrix
     *  
     *  \param _xx xx component
     *  \param _xy xy component
     *  \param _yx yx component
     *  \param _yy yy component
     *  \param _dx dx component
     *  \param _dy dy component
     */	
    void set(TItem _xx = 1, TItem _xy = 0, TItem _yx = 0, TItem _yy = 1, TItem _dx = 0, TItem _dy = 0) 
    {
        m_xx = _xx; m_xy = _xy; m_yx = _yx; m_yy = _yy; m_dx = _dx; m_dy = _dy;
    }

    /**
     *  \brief Returns xx component
     *  \return @b TItem xx component
     */
    TItem xx()const{return m_xx;}

    /**
     *  \brief Returns xy component
     *  \return @b TItem xy component
     */
    TItem xy()const{return m_xy;}

    /**
     *  \brief Returns yx component
     *  \return @b TItem yx component
     */
	TItem yx()const{return m_yx;}

    /**
     *  \brief Returns yy component
     *  \return @b TItem yy component
     */
    TItem yy()const{return m_yy;}

    /**
     *  \brief Returns dx component
     *  \return @b TItem dx component
     */
    TItem dx()const{return m_dx;}

    /**
     *  \brief Returns dy component
     *  \return @b TItem dy component
     */
    TItem dy()const{return m_dy;}

    /**
     *  \brief Sets xx component
     *  \param x value to set
     */
    void setXX(TItem x) {m_xx = x;}

    /**
     *  \brief Sets xy component
     *  \param x value to set
     */
    void setXY(TItem x) {m_xy = x;}

    /**
     *  \brief Sets yx component
     *  \param x value to set
     */
    void setYX(TItem x) {m_yx = x;}

    /**
     *  \brief Sets yy component
     *  \param x value to set
     */
    void setYY(TItem x) {m_yy = x;}

    /**
     *  \brief Sets dx component
     *  \param x value to set
     */
    void setDX(TItem x) {m_dx = x;}

    /**
     *  \brief Sets dy component
     *  \param x value to set
     */
    void setDY(TItem x) {m_dy = x;}

    /**
     *  \brief Compare to another matrix
     *  \param matrix Matrix to compare current instance to
     *  \param ignoreDXDY Ignore DX & DY for this comparison
     *  \return @b bool True if equal, false if not
     */
    bool equal(const CTransformMatrix<TItem> &matrix, bool ignoreDXDY = false) const 
    {
        return  (m_xx == matrix.xx()) && 
        (m_xy == matrix.xy()) &&
        (m_yx == matrix.yx()) &&
        (m_yy == matrix.yy()) &&
        (ignoreDXDY || (
                        (m_dx == matrix.dx()) &&
                        (m_dy == matrix.dy())));
    }

    /**
     *  \brief Determine if identity matrix
     *  \param ignoreDXDY Ignore DX & DY for this operation
     *  \return @b bool True if identity matrix, false if not
     */
    bool identity(bool ignoreDXDY = false) const 
    {
        return  (m_xx == 1.0) && 
        (m_xy == 0.0) &&
        (m_yx == 0.0) &&
        (m_yy == 1.0) &&
        (ignoreDXDY || (
                        (m_dx == 0.0) &&
                        (m_dy == 0.0)));
    }

	/**
	 *  \brief Premultiply by given matrix
	 *  \param matrix Matrix premultiplier
	 *  \return @b CTransformMatrix<TItem> Resulting matrix
	 */
	CTransformMatrix<TItem> &preMul(const CTransformMatrix<TItem> &matrix) {
        TItem a=matrix.xx()*m_xx + matrix.xy()*m_yx;
        TItem b=matrix.xx()*m_xy + matrix.xy()*m_yy;
        TItem c=matrix.yx()*m_xx + matrix.yy()*m_yx;
        TItem d=matrix.yx()*m_xy + matrix.yy()*m_yy;
        TItem e=matrix.dx()*m_xx + matrix.dy()*m_yx + m_dx;
        TItem f=matrix.dx()*m_xy + matrix.dy()*m_yy + m_dy;
        m_xx=a; m_xy=b; m_yx=c; m_yy=d; m_dx=e; m_dy=f;
        return *this;
    }
	
    /**
     *  \brief Postmultiply by given matrix
     *  \param matrix Matrix postmultiplier
     *  \return @b CTransformMatrix<TItem> Resulting matrix
     */
    CTransformMatrix<TItem> &postMul(const CTransformMatrix<TItem> &matrix) {
        TItem a=m_xx*matrix.xx() + m_xy*matrix.yx();
        TItem b=m_xx*matrix.xy() + m_xy*matrix.yy();
        TItem c=m_yx*matrix.xx() + m_yy*matrix.yx();
        TItem d=m_yx*matrix.xy() + m_yy*matrix.yy();
        TItem e=m_dx*matrix.xx() + m_dy*matrix.yx() + matrix.dx();
        TItem f=m_dx*matrix.xy() + m_dy*matrix.yy() + matrix.dy();
        m_xx=a; m_xy=b; m_yx=c; m_yy=d; m_dx=e; m_dy=f;
        return *this;
    }

    /**
     *  \brief Invert the matrix
     *  \return @b bool True if matrix was inverted, false if inversion was not possible
     */
    bool invert() {
        TItem det;
        
        // Find the determinant
        det = m_xx * m_yy - m_yx * m_xy;
        if (det == 0.0)
        {
            // This matrix cannot be inverted
            return false;
        }
        
        TItem a = m_yy / det;
        TItem b = -m_xy / det;
        TItem c = -m_yx / det;
        TItem d = m_xx / det;
        TItem e = -(m_dx * m_yy - m_dy * m_yx) / det;
        TItem f = (m_dx * m_xy - m_dy * m_xx) / det;
        m_xx=a; m_xy=b; m_yx=c; m_yy=d; m_dx=e; m_dy=f;
        return true;
    }
    /// 
    /**
     *  \brief Transform a point
     *  \param result The point after transformation
     *  \param point The point before transformation
     *  \param ignoreDXDY Ignore DX & DY for this operation
     */
    void transform(PointTmpl<TItem> &result, const PointTmpl<TItem> &point, bool ignoreDXDY = false) const
    {
        TItem x = point.x * m_xx + point.y * m_yx;
        TItem y = point.x * m_xy + point.y * m_yy;
        if (!ignoreDXDY)
        {
            x += m_dx;
            y += m_dy;
        }
        result.x = x;
        result.y = y;
    }

    /**
     *  \brief Transform a point by the inverse of the matrix
     *  \param result PointTmpl<TItem> to accept result 
     *  \param point The point
     *  \param ignoreDXDY Ignore DX & DY for this operation
     *  \return @b bool True if point was transformed, false if transformation was not possible
     */
    bool iTransform(PointTmpl<TItem> &result, const PointTmpl<TItem> &point, bool ignoreDXDY = false) const
    {
        TItem det = m_xx * m_yy - m_yx * m_xy;
        if (det == 0.0)
        {
            // This matrix cannot be inverted
            return false;
        }
        TItem iDet = 1 / det;
        
        TItem x = point.x;
        TItem y = point.y;
        if (!ignoreDXDY)
        {
            x -= m_dx;
            y -= m_dy;
        }
        
        result.x = ((x * m_yy - y * m_yx) * iDet);
        result.y = ((-x * m_xy + y * m_xx) * iDet);
        
        return true;
    }

    /**
     *  \brief Add a rotation, clockwise, in radians
     *  \param radians Angle in radians
     */
    void rotate(double radians)
    {
        CTransformMatrix rotate(cos(radians), sin(radians), -sin(radians), cos(radians), 0.0, 0.0);
        preMul(rotate);
    }
    
    /**
     *  \brief Scale
     *  \param xscale X-axis scaling factor
     *  \param yscale Y-axis scaling factor
     */
    void scale (TItem xscale, TItem yscale)
    {
        CTransformMatrix mat (xscale, 0.0, 0.0, yscale, 0.0, 0.0);
        preMul (mat);
    }

    /**
     *  \brief Translate
     *  \param dx x-axis transformation value
     *  \param dy y-axis transformation value
     */
    void translate (TItem dx, TItem dy)
    {
        CTransformMatrix mat (1.0, 0.0, 0.0, 1.0, dx, dy);
        preMul (mat);
    }

    /// Transform a rectangle
    /**
     *  \brief Transform a rectangle
     *  \param rect Rectangle to transform
     *  \param ignoreDXDY Ignore DX & DY for this operation
     */
    void transformRect (RectTmpl<TItem> &rect, bool ignoreDXDY = false) const
    {
        TItem llX, llY, urX, urY;
        PointTmpl<TItem> point, transformedPoint;
        
        if (rect.isEmpty())
            return;
#define EDLTMIN(a, b) (a) < (b) ? (a) : (b)
#define EDLTMAX(a, b) (a) < (b) ? (b) : (a)
        // Transform each of the points that make up the corners,
        // and find the extremes.
        point.x = rect.x;
        point.y = rect.y;
        transform(transformedPoint, point, ignoreDXDY);
        llX = urX = transformedPoint.x;
        llY = urY = transformedPoint.y;
        point.x = rect.x + rect.dX;
        transform(transformedPoint, point, ignoreDXDY);
        llX = EDLTMIN(llX, transformedPoint.x);
        llY = EDLTMIN(llY, transformedPoint.y);
        urX = EDLTMAX(urX, transformedPoint.x);
        urY = EDLTMAX(urY, transformedPoint.y);
        point.y = rect.y + rect.dY;
        transform(transformedPoint, point, ignoreDXDY);
        llX = EDLTMIN(llX, transformedPoint.x);
        llY = EDLTMIN(llY, transformedPoint.y);
        urX = EDLTMAX(urX, transformedPoint.x);
        urY = EDLTMAX(urY, transformedPoint.y);
        point.x = rect.x;
        transform(transformedPoint, point, ignoreDXDY);
        llX = EDLTMIN(llX, transformedPoint.x);
        llY = EDLTMIN(llY, transformedPoint.y);
        urX = EDLTMAX(urX, transformedPoint.x);
        urY = EDLTMAX(urY, transformedPoint.y);
#undef EDLTMIN
#undef EDLTMAX
        rect.x = llX; rect.y = llY; rect.dX = urX - llX; rect.dY = urY - llY;
    }    
    
    /**
     *  \brief Classify the transform
     *  \return uint32 Operation flags
     */
    uint32 classify() const
    {
		uint32 operationFlags = 0;
		
		if ((m_dx != 0.0) || (m_dy != 0))
			operationFlags |= eDoesTranslate;
        
		if ((m_xy == 0) && (m_yx == 0))
			operationFlags |= eDoesScale;
        
		if ((m_xx == m_yy) && (m_xy == -m_yx)) {
			operationFlags |= eDoesRotate;
			
			double det	= m_xx * m_yy - m_yx * m_xy;
			double eps	=  1.0e-06;
		
			if (fabs(det - 1.0) > eps)
				operationFlags |= eDoesScale;
				
		} else {
		
			operationFlags |= eIsComplex;
			
		}
      
        return operationFlags;
    }
                 
    
    // Decompose the transform into an equivalent set of translate + scale + shear + rotate
    void decompose(PointTmpl<TItem> &translate, FPoint &scale, FPoint &shear, double &rotationAngle, double eps = 1.0e-06) const
    {
        // unit square decomposition of matrix ...
        // A={a,c}, B={b,d}, Q=angle between vectors (ie PI/2-shear angle)!
        // |A|=sqrt(aa+cc)
        // |B|=sqrt(bb+dd)
        // AxB=|A||B|sinQ=ad-bc=(new area of unit square)
        // A.B=|A||B|cosQ=ab+cd
        
        // ScaleX   = |A|
        // ScaleY   = |B|
        // Scale    = ScaleY*cos(PI/2-Q) = ScaleY*sinQ = AxB/|A|
        // AbsScale = abs(Scale) // effectively remove any mirror
        // Aspect   = ScaleX/abs(Scale)
        // Shear    = PI/2-Q = PI/2-acos((A.B)/(|A||B|))
        // Rotate   = atan2(c,a)
        
        double a=m_xx;
        double b=m_xy; 
        double c=m_yx;
        double d=m_yy;
        
        // get cross product (determinant), modulus (length) of A and scale
//      double AxB      = a*d-b*c;
        double AdotB    = a*b+c*d;
        double ModA     = ::sqrt(a*a+c*c);
        double ModB     = ::sqrt(b*b+d*d);
        double pi       = PI;
        double Shear    = (pi/2-::acos(AdotB/(ModB*ModA)));
        
        // Set the translate
        translate.x = m_dx;
        translate.y = m_dy;
        
        // Set the rotation angle
		if ((b == 0) || (c == 0))
			rotationAngle = 0;
		else {
			rotationAngle = ::atan2(c, a);
			if (fabs(rotationAngle) < eps)
				rotationAngle = 0.0;
		}
        
        //Set the scale
		scale.x = ((a != 0) && (a > 0)) || ((a == 0) && (c * b <= 0)) ? ModA : -ModA;
		scale.y = ((d != 0) && (d > 0)) || ((d == 0) && (c * b <= 0)) ? ModB : -ModB;
            
        // Set shear
        if (fabs(Shear) < eps) {
            
            // Case: no shear
            shear.x     = 0;
            shear.y     = 0;
            
        } else {
            
            // Case: shear present.
            shear.x     = -tan(Shear);
            shear.y     = 0;
            
            if (a == d) {
                
                // Shear will handle differential x and y scaling
                scale.x = a;
                scale.y = d;
                
            }
        }
        
    }
    
    double getScale() const
    {
        double a=m_xx;
        double b=m_xy; 
        double c=m_yx;
        double d=m_yy;
        
        double AxB      = a*d-b*c;
        double ModA     = ::sqrt(a*a+c*c);
        
        return ::fabs(AxB/ModA);
    }
    
    bool isSimpleScaled(TItem &scale) const
    {        
        bool isSimple = ((m_xx == m_yy) && (m_xy == 0) && (m_yx == 0));
        
        if (isSimple)    
            scale = m_xx;
        
        return isSimple;
    }
    
private:
        TItem m_xx,m_xy,m_yx,m_yy,m_dx,m_dy;
};

typedef CTransformMatrix<double> FMatrix;

_END_EDL_NAMESPACE

#endif /* __EDLGEOM_H__ */
