/*
 * Copyright (C) 2006 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file edlmath.h
 * @ingroup IController
 * @brief (very thin) portability layer around operating system provided math functionality
 * but also includes a definition of "pi" and functions to interconvert between (angular) degrees and radians
 */

#ifndef __EDLMATH_H__
#define __EDLMATH_H__

#include <edl/edlnamespaces.h>
#include <math.h>

/**
 * @brief Local definition of PI to 20 decimal places
 *
 * This should give sufficient precision for calculations
 * using double-precision floating-point math
 */
#define PI 3.14159265358979323846 

/**
 * @brief convertDegreesToRadians converts degrees (double) to radians (double)
 */
static inline double convertDegreesToRadians(double degrees)
{
    const double convConst = PI * 2.0 / 360.0; /* Should compile precalculated */
    
    return degrees * convConst;
}

/**
 * @brief convertRadiansToDegrees converts radians (double) to degrees (double)
 */
static inline double convertRadiansToDegrees(double radians)
{
    const double convConst = 360.0 / (PI * 2.0); /* Should compile precalculated */
    
    return radians * convConst;
}

_BEGIN_EDL_NAMESPACE

// Simple matrix class with flat array storage
template <class T, int W, int H>
class CMatrix
{
public:
    /// Parameter is an array of WxH values of Type T.
    /// Order is across first, then down
    CMatrix(T *values)
    {
        setValues(values);
    }
    CMatrix()
    {
        for (int i = 0; i < W * H; i++)
        {
            m_values[i] = 0;
        }
    }
    /// Get all the values in the Matrix as a flat array
    T * getValues()
    {
        return m_values;
    }
    /// Set all the values as a flat array
    void setValues(T *values)
    {
        for (int i = 0; i < W * H; i++)
        {
            m_values[i] = values[i];
        }
    }
    bool getValue(int x, int y, T &value)
    {
        if (x >= W || x < 0 || y >= H || y < 0)
        {
            return false;
        }
        value = m_values[y * W + x];
    }
    bool setValue(int x, int y, T &value)
    {
        if (x >= W || x < 0 || y >= H || y < 0)
        {
            return false;
        }
        m_values[y * W + x] = value;
    }
    // Matrix mult/inverse/etc methods would go here
private:
    T m_values[W * H];
};

typedef CMatrix<float,3,3> SFMatrix3x3;

_END_EDL_NAMESPACE
#endif /* __EDLMATH_H__ */
