/*
 * Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file        edlnamespaces.h
 * @ingroup     IController
 * @brief       EDL C++ namespace(s)
 *
 * This file defines a collection of macros
 * that collectively define the namespace(s)
 * that EDL provides.
 *
 * For each namespace (the primary one being "EDL")
 * there is a collection of macros:
 *
 * _BEGIN_\<x\>_NAMESPACE intended to mark the start of a series of
 * declarations/definitions that are in namespace \<x\>.
 * This is typically used near the start of the EDL header files
 * *after* all the included dependent header files
 * *before* any local declarations
 *
 * _END_\<x\>_NAMESPACE intended to mark the end of a series of declarations
 * Typically used at the end of an EDL header file
 *
 * _USING_\<x\>_NAMESPACE is used whenever EDL code wishes to use
 * types, data, functions from within namespace \<x\>
 * without having to explicitly specify namespace "\<x\>::" everywhere
 * This is typically found at the start of most C++ source files.
 *
 * _FROM_\<x\>_NAMESPACE is used for isolated individual locations
 * where a type, data item or function from namespace \<x\> is needed
 * and where a explicit namespace qualifier is needed for disambiguation
 * This macro takes a single argument which is the name of the
 * type, data item or function that is in the particular \<x\> namespace
 *
 * Note that for all EDL namespaces, the above-named macros
 * are *always* defined.
 *
 * However they are only defined to expand to some namespace-specific text
 * when both __cplusplus and the _\<x\>_NAMESPACE macro is defined
 */

#ifndef edl_edlnamespaces_h
#define edl_edlnamespaces_h

/*
 * The following macros define the names of the various EDL namespaces
 * The _BEGIN_<x>_NAMESPACE, _END_<x>_NAMESPACE, _USING_<x>_NAMESPACE
 * and _FROM_<x>_NAMESPACE only expand to any namespace-specific text
 * IFF their corresponding _<x>_NAMESPACE macro is defined
 */

#define _EDL_NAMESPACE EDL
#define _HQN_NAMESPACE HQN
#define _EDL_DOM_NAMESPACE EDL
#define _EDL_PCL5_NAMESPACE PCL5
#define _EDL_PCL6_NAMESPACE XL
#define _EDL_XPS_NAMESPACE EDL
#define _EDL_COMMON_NAMESPACE EDL_COMMON
#define _EDL_UNICODE_NAMESPACE EDL_UNICODE
#define _EDL_FONT_NAMESPACE Font
#define _EDL_J2_NAMESPACE EDLJ2


#if defined(__cplusplus) && defined(_EDL_NAMESPACE)

#define _BEGIN_EDL_NAMESPACE  namespace _EDL_NAMESPACE {
#define _END_EDL_NAMESPACE    };
#define _USING_EDL_NAMESPACE  using namespace _EDL_NAMESPACE;
#define _FROM_EDL_NAMESPACE(_EDL_NS_SYMBOL)  _EDL_NAMESPACE::_EDL_NS_SYMBOL

#else

#define _BEGIN_EDL_NAMESPACE
#define _END_EDL_NAMESPACE
#define _USING_EDL_NAMESPACE
#define _FROM_EDL_NAMESPACE(_EDL_NS_SYMBOL)   _EDL_NS_SYMBOL

#endif


#if defined(__cplusplus) && defined(_HQN_NAMESPACE)

#define _BEGIN_HQN_NAMESPACE  namespace _HQN_NAMESPACE {
#define _END_HQN_NAMESPACE    };
#define _USING_HQN_NAMESPACE  using namespace _HQN_NAMESPACE;
#define _FROM_HQN_NAMESPACE(_HQN_NS_SYMBOL)  _HQN_NAMESPACE::_HQN_NS_SYMBOL

#else

#define _BEGIN_HQN_NAMESPACE
#define _END_HQN_NAMESPACE
#define _USING_HQN_NAMESPACE
#define _FROM_HQN_NAMESPACE(_HQN_NS_SYMBOL)   _HQN_NS_SYMBOL

#endif


#if defined(__cplusplus) && defined(_EDL_DOM_NAMESPACE)

#define _BEGIN_EDL_DOM_NAMESPACE  namespace _EDL_DOM_NAMESPACE {
#define _END_EDL_DOM_NAMESPACE    };
#define _USING_EDL_DOM_NAMESPACE  using namespace _EDL_DOM_NAMESPACE;
#define _FROM_EDL_DOM_NAMESPACE(_EDL_DOM_NS_SYMBOL)  _EDL_DOM_NAMESPACE::_EDL_DOM_NS_SYMBOL

#else

#define _BEGIN_EDL_DOM_NAMESPACE
#define _END_EDL_DOM_NAMESPACE
#define _USING_EDL_DOM_NAMESPACE
#define _FROM_EDL_DOM_NAMESPACE(_EDL_DOM_NS_SYMBOL)   _EDL_DOM_NS_SYMBOL

#endif


#if defined(__cplusplus) && defined(_EDL_PCL5_NAMESPACE)

#define _BEGIN_EDL_PCL5_NAMESPACE  namespace _EDL_PCL5_NAMESPACE {
#define _END_EDL_PCL5_NAMESPACE    };
#define _USING_EDL_PCL5_NAMESPACE  using namespace _EDL_PCL5_NAMESPACE;
#define _FROM_EDL_PCL5_NAMESPACE(_EDL_PCL5_NS_SYMBOL)  _EDL_PCL5_NAMESPACE::_EDL_PCL5_NS_SYMBOL

#else

#define _BEGIN_EDL_XPS_NAMESPACE
#define _END_EDL_XPS_NAMESPACE
#define _USING_EDL_XPS_NAMESPACE
#define _FROM_EDL_XPS_NAMESPACE(_EDL_XPS_NS_SYMBOL)   _EDL_XPS_NS_SYMBOL

#endif



#if defined(__cplusplus) && defined(_EDL_PCL6_NAMESPACE)

#define _BEGIN_EDL_PCL6_NAMESPACE  namespace _EDL_PCL6_NAMESPACE {
#define _END_EDL_PCL6_NAMESPACE    };
#define _USING_EDL_PCL6_NAMESPACE  using namespace _EDL_PCL6_NAMESPACE;
#define _FROM_EDL_PCL6_NAMESPACE(_EDL_PCL6_NS_SYMBOL)  _EDL_PCL6_NAMESPACE::_EDL_PCL6_NS_SYMBOL

#else

#define _BEGIN_EDL_XPS_NAMESPACE
#define _END_EDL_XPS_NAMESPACE
#define _USING_EDL_XPS_NAMESPACE
#define _FROM_EDL_XPS_NAMESPACE(_EDL_XPS_NS_SYMBOL)   _EDL_XPS_NS_SYMBOL

#endif


#if defined(__cplusplus) && defined(_EDL_XPS_NAMESPACE)

#define _BEGIN_EDL_XPS_NAMESPACE  namespace _EDL_XPS_NAMESPACE {
#define _END_EDL_XPS_NAMESPACE    };
#define _USING_EDL_XPS_NAMESPACE  using namespace _EDL_XPS_NAMESPACE;
#define _FROM_EDL_XPS_NAMESPACE(_EDL_XPS_NS_SYMBOL)  _EDL_XPS_NAMESPACE::_EDL_XPS_NS_SYMBOL

#else

#define _BEGIN_EDL_XPS_NAMESPACE
#define _END_EDL_XPS_NAMESPACE
#define _USING_EDL_XPS_NAMESPACE
#define _FROM_EDL_XPS_NAMESPACE(_EDL_XPS_NS_SYMBOL)   _EDL_XPS_NS_SYMBOL

#endif


#if defined(__cplusplus) && defined(_EDL_UNICODE_NAMESPACE)

#define _BEGIN_EDL_UNICODE_NAMESPACE  namespace _EDL_UNICODE_NAMESPACE {
#define _END_EDL_UNICODE_NAMESPACE    };
#define _USING_EDL_UNICODE_NAMESPACE  using namespace _EDL_UNICODE_NAMESPACE;
#define _FROM_EDL_UNICODE_NAMESPACE(_EDL_UNICODE_NS_SYMBOL)  _EDL_UNICODE_NAMESPACE::_EDL_UNICODE_NS_SYMBOL

#else

#define _BEGIN_EDL_UNICODE_NAMESPACE
#define _END_EDL_UNICODE_NAMESPACE
#define _USING_EDL_UNICODE_NAMESPACE
#define _FROM_EDL_UNICODE_NAMESPACE(_EDL_UNICODE_NS_SYMBOL)   _EDL_UNICODE_NS_SYMBOL

#endif

#if defined(__cplusplus) && defined(_EDL_COMMON_NAMESPACE)

#define _BEGIN_EDL_COMMON_NAMESPACE  namespace _EDL_COMMON_NAMESPACE {
#define _END_EDL_COMMON_NAMESPACE    };
#define _USING_EDL_COMMON_NAMESPACE  using namespace _EDL_COMMON_NAMESPACE;
#define _FROM_EDL_COMMON_NAMESPACE(_EDL_COMMON_NS_SYMBOL)  _EDL_COMMON_NAMESPACE::_EDL_COMMON_NS_SYMBOL

#else

#define _BEGIN_EDL_COMMON_NAMESPACE
#define _END_EDL_COMMON_NAMESPACE
#define _USING_EDL_COMMON_NAMESPACE
#define _FROM_EDL_COMMON_NAMESPACE(_EDL_COMMON_NS_SYMBOL)   _EDL_COMMON_NS_SYMBOL

#endif


#if defined(__cplusplus) && defined(_EDL_COMMON_NAMESPACE)

#define _BEGIN_EDL_FONT_NAMESPACE  namespace _EDL_FONT_NAMESPACE {
#define _END_EDL_FONT_NAMESPACE    };
#define _USING_EDL_FONT_NAMESPACE  using namespace _EDL_FONT_NAMESPACE;
#define _FROM_EDL_FONT_NAMESPACE(_EDL_FONT_NS_SYMBOL)  _EDL_FONT_NAMESPACE::_EDL_FONT_NS_SYMBOL

#else

#define _BEGIN_EDL_FONT_NAMESPACE
#define _END_EDL_FONT_NAMESPACE
#define _USING_EDL_FONT_NAMESPACE
#define _FROM_EDL_FONT_NAMESPACE(_EDL_FONT_NS_SYMBOL)   _EDL_FONT_NS_SYMBOL

#endif

#if defined(__cplusplus) && defined(_EDL_J2_NAMESPACE)

#define _BEGIN_EDL_J2_NAMESPACE  namespace _EDL_J2_NAMESPACE {
#define _END_EDL_J2_NAMESPACE    };
#define _USING_EDL_J2_NAMESPACE  using namespace _EDL_J2_NAMESPACE;
#define _FROM_EDL_J2_NAMESPACE(_EDL_NS_SYMBOL)  _EDL_J2_NAMESPACE::_EDL_NS_SYMBOL

#else

#define _BEGIN_EDL_NAMESPACE
#define _END_EDL_NAMESPACE
#define _USING_EDL_NAMESPACE
#define _FROM_EDL_NAMESPACE(_EDL_NS_SYMBOL)   _EDL_NS_SYMBOL

#endif


#endif // edl_edlnamespaces_h
