/*
 * Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file    edlqname.h
 * @ingroup IController
 * @brief   IEDLNamespace and EDLQName classes
 */

#ifndef __EDLQNAME_H__
#define __EDLQNAME_H__

#include <edl/edltypes.h>
#include <edl/iedlobject.h>
#include <edl/iedlenum.h>
#include <edl/edlstring.h>

_BEGIN_EDL_NAMESPACE

/**
* @class    IEDLNamespace
* @ingroup  IDom
* @brief    Interface to EDL Namespace calss
*/
class IEDLNamespace : public IEDLObject
{
#define IEDLNamespaceID 0xb836b26d, 0x2e4e45d2, 0x82aa6c63, 0x6ca45484
public:
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        EDLSysString prefix;
        EDLSysString nameSpace;
    };
    /**
    * @brief    Retrieves class id of IEDLNamespace
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IEDLNamespaceID);
        return id;
    };
    /**
    * @brief    Retrieves the namespace prefix
    * @param    sPrefix Namespace prefix
    * @return   bool Returns true on success
    */
    virtual bool getPrefix(EDLSysString &sPrefix) const = 0;
    /**
    * @brief    Sets the name space prefix
    * @param    sPrefix Namespace prefix
    * @return   bool Returns true on success
    */
    virtual bool setPrefix(const EDLSysString &sPrefix) = 0;
    /**
    * @brief    Retrieves the namespace
    * @param    sNamespace Namespace
    * @return   bool Returns true on success
    */
    virtual bool getNamespace(EDLSysString &sNamespace) const = 0;
    /**
    * @brief    Sets the namespace
    * @param    sNamespace Namespace 
    * @return   bool Returns true on success
    */
    virtual bool setNamespace(const EDLSysString &sNamespace) = 0;
};
DECL_SMART_PTR(IEDLNamespace);
#define edlobj2IEDLNamespace(src) edl_cast((IEDLNamespace *)NULL, src)

typedef IInputEnumRC<IEDLNamespacePtr> IEDLNamespaceCollectionEnum;
DECL_SMART_PTR(IEDLNamespaceCollectionEnum);

#define NamespaceSeparatorL L":"
#define NamespaceSeparator ":"

/**
* @class    EDLQName
* @ingroup  IDom
* @brief    Implementation of qualified name class
*/
class EDLQName
{
public:
    /**
    * @brief    Empty constructor of EDLQName
    */
    EDLQName()
    {
        m_ptrNamespace = IEDLNamespacePtr(NULL);
        m_name = EDLString();
    }

    /**
    * @brief    constructor of EDLQName
    * @param    ptrNamespace Smart pointer to the namespace interface
    * @param    name The name value 
    */
    EDLQName(IEDLNamespacePtr ptrNamespace, const EDLString &name)
    {
        m_ptrNamespace = ptrNamespace;
        m_name = name;
    }

    /**
    * @brief    constructor of EDLQName
    * @param    ptrNamespace Smart pointer to the namespace interface
    * @param    name The name value 
    */
    EDLQName(IEDLNamespacePtr ptrNamespace, const EDLSysString &name)
    {
        m_ptrNamespace = ptrNamespace;
        m_name = EDLSysStringToEDLString(name);
    }

    /**
    * @brief    operator =
	* @param	another EDLQName
    */
    void operator =(const EDLQName &another)
    {
        if (this != &another) 
        {
            m_ptrNamespace = another.m_ptrNamespace; 
            m_name = another.m_name; 
        }
    }

    /**
    * @brief    operator ==
	* @param	another EDLQName
	* @return	<b>bool</b> True if equal, false if not
    */
    bool operator ==(const EDLQName &another) const
    {
        if (this != &another) 
        {

            EDLSysString ns1, ns2;
            if (m_ptrNamespace->getNamespace(ns1) && another.m_ptrNamespace->getNamespace(ns2))
                return (ns1 == ns2) && (m_name == another.m_name);
            else
                return false;

        } 
        else
            return true;
    }

    /**
    * @brief    Retrieves namespace
    * @return   IEDLNamespacePtr Returns smart pointer to the namespace interface
    */
    IEDLNamespacePtr getNamespace() const
    {
        return m_ptrNamespace;
    }

    /**
    * @brief    Sets namespace
    * @param    ns Smart pointer to the namespace interface
    * @return   bool Returns true
    */
    bool setNamespace(const IEDLNamespacePtr &ns)
    {
        m_ptrNamespace = ns;
        return true;
    }

    /**
    * @brief    Retrieves member name of EDLQName
    * @return   EDLString Returns name value as EDLString
    */
    EDLString getName() const
    {
        return m_name;
    }

    /**
    * @brief    Sets member name of EDLQName
    * @param    name The name value
    * @return   bool Returns true
    */
    bool setName(const EDLString &name)
    {
        m_name = name;
        return true;
    }

    /**
    * @brief    Retrieves member name of EDLQName
    * @return   EDLSysString Returns name value as EDLSysString
    */
    EDLSysString getSysName() const
    {
        EDLSysString sysname = EDLStringToEDLSysString(m_name);
        return sysname;
    }

    /**
    * @brief    Sets member name of EDLQName
    * @param    sysname The name value
    * @return   bool Returns true
    */
    bool setName(const EDLSysString &sysname)
    {
        m_name = EDLSysStringToEDLString(sysname);
        return true;
    }

    /**
    * @brief    Returns EDLString that is combination of prefix and name (prefix:name)
    * @return   EDLString Returns fullname
    */
    EDLString getNameWithPrefix() const
    {
        EDLString result(L"");
        EDLSysString sPrefix;

        if (m_ptrNamespace && m_ptrNamespace->getPrefix(sPrefix))
            result = EDLSysStringToEDLString(sPrefix) + NamespaceSeparatorL + m_name;
        else
            result = m_name;
        return result;
    }

private:
    IEDLNamespacePtr m_ptrNamespace;
    EDLString m_name;
};

_END_EDL_NAMESPACE

#endif /* __EDLQNAME_H__ */
