/*
 * Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file edlstring.h
 * @ingroup IController
 * @brief EDLString and EDLSysString classes
 * and associated EDL string manipulation functions
 */

#ifndef __EDLSTRING_H__
#define __EDLSTRING_H__

#include <string>
#include <sstream>
#include <limits>
#ifdef ANDROID
#include <ctype.h>
#endif
#include <edl/edltypes.h>
#include <edl/edlvector.h>

_BEGIN_EDL_NAMESPACE

#ifdef _WIN32
// An STL Allocator, used for strings to allow
// strings to be passed between DLLs that use
// statically-linked runtimes.

EDL_API void *edlStringAlloc(size_t);
EDL_API void  edlStringFree(void*);
template <class T>
class CEDLStringAllocator
{
  public:
    inline CEDLStringAllocator()
    {
        allocFunc = edlStringAlloc;
        freeFunc = edlStringFree;
    }
    CEDLStringAllocator(CEDLStringAllocator const &r)
    {
        allocFunc = r.allocFunc;
        freeFunc = r.freeFunc;
    }
    template <class U>
    inline explicit CEDLStringAllocator(CEDLStringAllocator<U> const &r)
    {
        allocFunc = r.allocFunc;
        freeFunc = r.freeFunc;
    }
    inline ~CEDLStringAllocator() {}

  public:
    void *(*allocFunc)(size_t);
    void (*freeFunc)(void*);

  public:
    typedef T              value_type;
    typedef T*             pointer;
    typedef const T*       const_pointer;
    typedef T&             reference;
    typedef const T&       const_reference;
    typedef std::size_t    size_type;
    typedef std::ptrdiff_t difference_type;

    template <class U>
    struct rebind
    {
        typedef CEDLStringAllocator<U> other;
    };

    inline pointer address (reference v) const { return &v; }
    const_pointer address (const_reference v) const { return &v; }
    size_type max_size () const
    {
#undef max
        return std::numeric_limits<std::size_t>::max() / sizeof(T);
    }

    pointer allocate (size_type n, const void * = 0)
    {
        pointer p = (pointer) allocFunc(n * sizeof(T));
        if (!p)
            throw std::bad_alloc();
        return p;
    }
    void deallocate (pointer p, size_type)
    {
        freeFunc(p);
    }

    void construct (pointer p, const T& v)
    {
        new ((void*)p) T(v);
    }

    void destroy (pointer p)
    {
        p->~T();
    }
};

template <typename T, typename U>
bool operator== (CEDLStringAllocator<T> const &l, CEDLStringAllocator<U> const &r)
{
	return (l.freeFunc == r.freeFunc);
}

template<typename T, typename U>
bool operator!= (CEDLStringAllocator<T> const &l, CEDLStringAllocator<U> const &r)
{
	return (l.freeFunc != r.freeFunc);
}

// Basic EDLString types
typedef std::basic_string<wchar_t, std::char_traits<wchar_t>, CEDLStringAllocator<wchar_t> > EDLString;
typedef std::basic_string<char,    std::char_traits<char>,    CEDLStringAllocator<char>    > EDLSysString;

typedef std::basic_istringstream<wchar_t, std::char_traits<wchar_t>, CEDLStringAllocator<wchar_t> > EDLStringIStream;
typedef std::basic_istringstream<char,    std::char_traits<char>,    CEDLStringAllocator<char>    > EDLSysStringIStream;

#else
// Basic EDLString types
typedef std::wstring EDLString;
typedef std::string  EDLSysString;

typedef std::basic_istringstream<wchar_t> EDLStringIStream;
typedef std::basic_istringstream<char> EDLSysStringIStream;

#endif


// Explicit UTF16 and UTF32 strings. Do not use BOMs.
#ifdef _WIN32
typedef wchar_t u16char;
typedef EDLString EDLU16String;
typedef uint32 u32char;
typedef std::basic_string<u32char,    std::char_traits<u32char>,    CEDLStringAllocator<u32char>    > EDLU32String;

#else
typedef uint16 u16char;
typedef std::basic_string<u16char> EDLU16String;
typedef wchar_t u32char;
typedef EDLString EDLU32String;
#endif





typedef CEDLVector<EDLString> CEDLStringVect;
typedef CEDLVector<EDLSysString> CEDLSysStringVect;


inline bool equal(const EDLSysString &str1,const EDLSysString &str2) { return (str1 == str2); }
uint32 hashValue(const EDLSysString &str);
uint32 hashValue(const char *pStr, uint32 lenStr);

/**
 * @brief sxtob converts a hexadecimal character into corresponding
 * (unsigned) integer value.
 * (Note that there is no guard against being supplied with
 * non-hexadecimal characters)
 * @param c The hexadecimal digit
 * @return <b>uint8</b> The unsigned integer value
 */
inline uint8 sxtob(char c) 
{ 
    return (c >= '0' && c <= '9') ? (c - '0') : ((c >= 'a' && c <= 'f') ? (c - 'a' + 10) : (c - 'A' + 10)); 
}

/**
 * @brief edlstringEqualsIgnoreCaseCnt performs a case-insensitive maximum-length-constrained string equality test
 * @param str1 First string
 * @param str2 Second string
 * @param len Number of characters to compare
 * @return <b>bool</b> True if the two strings are equal, false if not
 */
 inline bool edlstringEqualsIgnoreCaseCnt(const EDLString &str1,const EDLString &str2, size_t len) 
{
    wchar_t c1;
    wchar_t c2;
    size_t  minLen = (str1.size() < str2.size()) ? str1.size() : str2.size();

    if (minLen < len)
        return false;
    
    for (size_t i = 0; i < len; i++) {
        c1 = ::tolower(str1.at(i));
        c2 = ::tolower(str2.at(i));

        if (c1 != c2)
            return false;
    }

    return true;
}

/**
 * @brief edlstringFind searches for a substring within a larger string.
 * @param str String to be searched
 * @param searchStr Substring to search for
 * @return <b>size_t</b> The starting position of the (first instance of the) substring.
 */
inline size_t edlstringFind(const EDLString &str, EDLString &searchStr) 
{
    return str.find(searchStr);
}

/**
 * @brief edlstringSubstr returns a substring from specified start and end points from a larger string
 * @param str Source string
 * @param result String to accept result
 * @param start Start point
 * @param end End point
 */
 inline void edlstringSubstr(const EDLString &str, EDLString& result, size_t start, size_t end) 
{
    result = str.substr(start, end);
}


/**
 * @brief EDLSysStringToEDLString converts an EDLSysString (UTF8) to an EDLString (UTF16 or UTF32 depending on platform).
 * @param edlSysString Source string
 * @return <b>EDLString</b> Result of conversion
 */

EDL_API EDLString EDLSysStringToEDLString(const EDLSysString &edlSysString);

/**
 * @brief EDLStringToEDLSysString converts an (UTF16 or UTF32 depending on platform) EDLString to an EDLSysString (UTF8).
 * @param edlString Source string
 * @return <b>EDLSysString</b> Result of conversion
 */

EDL_API EDLSysString EDLStringToEDLSysString(const EDLString &edlString);

_END_EDL_NAMESPACE

#endif /* __EDLSTRING_H__ */
