/*
 * Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file    edltime.h
 * @ingroup IController
 * @brief   IEDLTime class that represents the EDL date-time
 */

#ifndef __EDLTIME_H__
#define __EDLTIME_H__

#include <edl/iedlobject.h>
#include <edl/iedlfactory.h>
#include "edltypes.h"
#include "edlstring.h"
#include <time.h>

_BEGIN_EDL_NAMESPACE

class IEDLTime;
DECL_SMART_PTR(IEDLTime);

/**
* @class    IEDLTime
* @ingroup  EDL Core
* @brief    Interface to EDL date-time class
*/
class IEDLTime: public IEDLObject
{
#define IEDLTimeID 0x88d388a7, 0xcd6146bf, 0xa29e9830, 0x722cf8b9
public:

    DECLARE_CLASS_ID(IEDLTimeID);
    
    /**
    * @brief    Simplified creator to create the time as of now.
    */
    static IEDLTimePtr createNow(IEDLClassFactory *pFactory);

    /**
    * @class    Data
    * @ingroup  IEDLTime
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : year(0), month(0), day(0), hour(0), minute(0), second(0), ms(0),
                 tzd_sign(1), tzd_hour(0), tzd_min(0)
        {}

        uint16 year;     //year
        uint16 month;    //month (1=January, etc.)
        uint16 day;      //day of month (01 through 31)
        uint16 hour;     //hour (0 through 23)
        uint16 minute;   //minute (00 through 59)
        uint16 second;   //second (00 through 59)
        uint16 ms;       //decimal fraction of a second
        int16  tzd_sign; //time zone designator - sign (+1 or -1)
        uint16 tzd_hour; //time zone designator - hour
        uint16 tzd_min;  //time zone designator - min
    };

    /**
    * @brief    Sets the year
    * @param    year The year value
    */
    virtual bool setYear(uint16 year) = 0;

    /**
    * @brief    Retrieves the year value
    * @return   bool Returns true on success, false if the call fails.
    */
    virtual uint16 getYear() const = 0;

    /**
    * @brief    Sets the month
    * @param    month The month value
    */
    virtual bool setMonth(uint16 month) = 0; 

    /**
    * @brief    Retrieves the month value
    * @return   bool Returns true on success, false if the call fails.
    */
    virtual uint16 getMonth() const = 0;

    /**
    * @brief    Sets the day
    * @param    day The day value
    */
    virtual bool setDay(uint16 day) = 0;

    /**
    * @brief    Retrieves the day value
    * @return   bool Returns true on success, false if the call fails.
    */
    virtual uint16 getDay() const = 0;

    /**
    * @brief    Sets the time
    * @param    hour The hour component
    * @param    min The minute component
    * @param    sec The second component
    * @param    ms The fractional second component
    * @param    tzd_sign The timezone sign component
    * @param    tzd_hour The timezone hour component
    * @param    tzd_min The timezone minute component
    * @return   bool Returns true on success, false if the call fails.
    */
    virtual bool setTime(uint16 hour = 0, uint16 min = 0, uint16 sec = 0, uint16 ms = 0, int16 tzd_sign = 1, uint16 tzd_hour = 0, uint16 tzd_min = 0) = 0;

    /**
    * @brief    Retrieves the time
    * @param    hour The hour component
    * @param    min The minute component
    * @param    sec The second component
    * @param    ms The fractional second component
    * @param    tzd_sign The timezone sign component
    * @param    tzd_hour The timezone hour component
    * @param    tzd_min The timezone minute component
    */
    virtual void getTime(uint16 &hour, uint16 &min, uint16 &sec, uint16 &ms, int16 &tzd_sign, uint16 &tzd_hour, uint16 &tzd_min) const = 0;

    /**
    * @brief    Convert IEDLTime to string with W3CDTF format
    * @return   EDLSysString Returns IEDLTime value as string in W3CDTF format
    */
    virtual EDLSysString toW3CDTF() const = 0;

    /**
    * @brief    Fill IEDLTime value from string with W3CDTF format
    * @param    EDLSysString The string
    */
    virtual bool fromW3CDTF(const EDLSysString &strTime) = 0;

    /**
    * @brief    Convert IEDLTime to string with PDF date format
    * @return   EDLSysString Returns IEDLTime value as string in W3CDTF format
    */
    virtual EDLSysString toPDFDate() const = 0;

    /**
    * @brief    Fill IEDLTime value from string with PDF date format
    * @param    EDLSysString The string
    */
    virtual bool fromPDFDate(const EDLSysString &strTime) = 0;

    /**
    * @brief    Check this IEDLTime for equality to another IEDLTime
    * @param    ptrTime The time to compare.
    * @return   bool. Returns true if the times are the same.
    */
    virtual bool isEqualTo(const IEDLTimePtr &ptrTime) const = 0;

    /**
     * @brief    Compare this time against ptrTime.
     * @param    ptrTime The time to compare.
     * @return   -1 if this time is earlier than ptrTime,
     *            0 if this time is equal to ptrTime or
     *            1 if this time is later than ptrTime
     */
    virtual int32 compare(const IEDLTimePtr &ptrTime) const = 0;

    /**
    * @brief    Sets date/time to the current system date and time
    */
    virtual void now() = 0;

    /**
    * @brief    Convert UTC date/time
    */
    virtual void toUTC() = 0;

    /**
    * @brief    Convert to local date/time
    */
    virtual void toLocalTime() = 0;
};

#define edlobj2IEDLTime(src) edl_cast((IEDLTime *)NULL, src)

_END_EDL_NAMESPACE

#endif /* __EDLTIME_H__ */
