/*
 * Copyright (C) 2008-2018 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file    edlvector.h
 * @ingroup EDL
 * @brief   Simple template vector class for general use.
 */

#ifndef edlvector_h
#define edlvector_h


#include <edl/edltypes.h>
#include <edl/edlnamespaces.h>
#include <edl/edlallocator.h>

_BEGIN_EDL_NAMESPACE

EDL_API void throwVectorError();

template <class T, uint32 STATICBUFFERLEN = 1>
class CEDLVector
{
    public:
    
        // Copy constructor
        CEDLVector<T, STATICBUFFERLEN>(const CEDLVector<T, STATICBUFFERLEN> &incoming)
        {
            init();
            assign(incoming);
        }
        // Allocate an empty vector
        CEDLVector<T, STATICBUFFERLEN>(uint32 size = 0)
        {
            init();
            if (size > 0)
            {
                resize(size);
            }
        }
        
        ~CEDLVector<T, STATICBUFFERLEN>()
        {
            clear();
        }

        void clear()
        {
            resizeVector(0);
            m_vectorSize = 0;
        }
        
        void reserve(uint32 reserveSize)
        {
            if (reserveSize > m_reservedSize)
            {
                resizeVector(reserveSize);
            }
        }
        
        // Get size
        uint32 size() const
        {
            return m_vectorSize;
        }
    
        bool empty() const
        {
            return m_vectorSize == 0;
        }
    

        // Dereference via index
        T& operator[](uint32 index)
        {
            if (index >= m_vectorSize)
                throwVectorError();

            return m_vector[index];
        }
        
        T& operator[](uint32 index) const
        {
            if (index >= m_vectorSize)
                throwVectorError(); 

            return m_vector[index];
        }
        
        T& last()
        {
            if (empty())
                throwVectorError();

            return m_vector[m_vectorSize - 1];
        }
        
        T& last() const
        {
            if (empty())
                throwVectorError();
            
            return m_vector[m_vectorSize - 1];
        }

        CEDLVector<T, STATICBUFFERLEN>& operator=(const CEDLVector<T, STATICBUFFERLEN> &incoming)
        {
            assign(incoming);
            return *this;
        }

        void reverse()
        {
            if (!m_vectorSize)
                return;

            for (uint32 i = 0, x = m_vectorSize - 1; i < x; ++i, --x)
            {
                T cache = m_vector[i];
                m_vector[i] = m_vector[x];
                m_vector[x] = cache;
            }
        }
            
        void resize(uint32 newSize)
        {
            if (newSize == m_vectorSize)
                return;
            
            if (newSize == 0 && m_vector)
            {
                clear();
            }
            else if (newSize > m_vectorSize && newSize <= m_reservedSize && m_vector)
            {
                for (uint32 i = m_vectorSize; i < newSize; i++)
                    new (m_vector + i) T();
            }
            else if (newSize < m_vectorSize && m_vector)
            {
                for (uint32 i = newSize; i < m_vectorSize; i++)
                    m_vector[i].~T();
            }
            else
            {
                uint32 reserveSize = newSize;
                if (newSize > m_vectorSize)
                {
                    reserveSize = m_vectorSize * 3 / 2;
                    if (reserveSize < newSize)
                        reserveSize = newSize;
                }
                if (reserveSize > 0 && reserveSize < STATICBUFFERLEN)
                {
                    reserveSize = STATICBUFFERLEN;
                }
                resizeVector(reserveSize);
                m_reservedSize = reserveSize;
                if (newSize > m_vectorSize)
                {
                    for (uint32 i = m_vectorSize; i < newSize; i++)
                        new (m_vector + i) T();
                }
            }
            m_vectorSize = newSize;
        }
    
        void append(const T& obj)
        {
            resize(m_vectorSize + 1);
            m_vector[m_vectorSize - 1] = obj;
        }
    
        void insert(uint32 index, const T& obj)
        {
            if (index > m_vectorSize)
                throwVectorError();
            
            if (index == m_vectorSize)
            {
                append(obj);
            }
            else
            {
                resize(m_vectorSize + 1);
                for (uint32 i = m_vectorSize - 1; i > index; i--)
                {
                    m_vector[i] = m_vector[i - 1];
                }
                m_vector[index] = obj;
            }
        }

        void insert(uint32 index, const CEDLVector<T>& items)
        {
            for (uint32 i = 0; i < items.size(); ++i)
            {
                insert(index++, items[i]);
            }
        }
    
        // Erase the entry at the given index, shortening the vector by one.
        void erase(uint32 index)
        {
            if (index >= m_vectorSize)
                throwVectorError();
            for (uint32 i = index + 1; i < m_vectorSize; i++)
                m_vector[i-1] = m_vector[i];
            resize(m_vectorSize - 1);
        }

        // Find the specified entry in the collection, throwing if none found.
        uint32 indexOf(const T &obj) const
        {
            for (uint32 i = 0; i < m_vectorSize; i++)
            {
                if (m_vector[i] == obj)
                {
                    return i;
                }
            }
            throwVectorError(); // Object not found.
            return (uint32)-1; // Should never be reached.
        }
        
    private:
        void init()
        {
            m_vector = NULL;
            m_vectorSize = 0;
            m_reservedSize = 0;
        }
        
        void assign(const CEDLVector<T, STATICBUFFERLEN> &incoming)
        {
            uint32 newSize = incoming.size();

            if (newSize != m_vectorSize)
            {
                resize(0);
                if (newSize > 0)
                    resize(newSize);
            }

            for (uint32 i = 0; i < newSize; i++)
            {
                m_vector[i] = incoming[i];
            }
        }
        
        // Change the size of the actual storage array
        void resizeVector(uint32 newSize)
        {
            // We will impose a minimum reserved size of SBLEN
            if (newSize > 0 && STATICBUFFERLEN > 0 && newSize < STATICBUFFERLEN)
            {
                newSize = STATICBUFFERLEN;
            }
            
            if (newSize == m_reservedSize)
            {
                return;
            }
            
            T *newVect = NULL;
            if (newSize > 0)
            {
                if (newSize == STATICBUFFERLEN)
                {
                    newVect = (T *) m_staticVector;
                }
                else
                {
                    newVect = (T *) m_allocator.allocate(sizeof(T) * newSize);
                    if (!newVect) throwVectorError();
                }
                uint32 cpSz = newSize > m_vectorSize ? m_vectorSize : newSize;
                for (uint32 i = 0; i < cpSz; i++)
                    new (newVect + i) T(m_vector[i]);
            }
            if (m_vector)
            {
                for (uint32 i = 0; i < m_vectorSize; i++)
                    m_vector[i].~T();
                if (m_vector != (T *) m_staticVector)
                {
                    m_allocator.deallocate(m_vector);
                }
            }
            m_vector = newVect;
            m_reservedSize = newSize;
        }

        T *m_vector;
        uint8  m_staticVector[sizeof(T) * STATICBUFFERLEN];
        uint32 m_vectorSize;
        uint32 m_reservedSize;
        EDLAllocator m_allocator;
};


_END_EDL_NAMESPACE

#endif

