/*
* Copyright (C) 2007,2012-2019 Global Graphics Software Ltd. All rights reserved.
*/

#ifndef __EDLIDOMBRUSH_H_DECLS__
#define __EDLIDOMBRUSH_H_DECLS__

/**
* @file     idombrush.h
* @ingroup  IDom
* @brief    Interfaces to DOM brush objects
*/

#include <edl/edlnamespaces.h>
#include <edl/idomnode.h>
#include <edl/idomhashable.h>
#include <jawsmako/optionalcontent.h>

// Forward declarations, to avoid problems with
// circular dependencies.
_BEGIN_EDL_DOM_NAMESPACE

class IDOMBrush;
DECL_SMART_PTR(IDOMBrush);
class IDOMGradientStop;
DECL_SMART_PTR(IDOMGradientStop);
class IDOMSolidColorBrush;
DECL_SMART_PTR(IDOMSolidColorBrush);
class IDOMLinearGradientBrush;
DECL_SMART_PTR(IDOMLinearGradientBrush);
class IDOMRadialGradientBrush;
DECL_SMART_PTR(IDOMRadialGradientBrush);
class IDOMImageBrush;
DECL_SMART_PTR(IDOMImageBrush);
class IDOMMaskedBrush;
DECL_SMART_PTR(IDOMMaskedBrush);
class IDOMVisualBrush;
DECL_SMART_PTR(IDOMVisualBrush);
class IDOMVisualRoot;
DECL_SMART_PTR(IDOMVisualRoot);
class IDOMSoftMaskBrush;
DECL_SMART_PTR(IDOMSoftMaskBrush);
class IDOMTilingPatternBrush;
DECL_SMART_PTR(IDOMTilingPatternBrush);
class IDOMShadingPatternBrush;
DECL_SMART_PTR(IDOMShadingPatternBrush);
class IDOMShadingPatternType1Brush;
DECL_SMART_PTR(IDOMShadingPatternType1Brush);
class IDOMShadingPatternType2Brush;
DECL_SMART_PTR(IDOMShadingPatternType2Brush);
class IDOMShadingPatternType3Brush;
DECL_SMART_PTR(IDOMShadingPatternType3Brush);
class IDOMShadingPatternType4567Brush;
DECL_SMART_PTR(IDOMShadingPatternType4567Brush);
class IDOMNullBrush;
DECL_SMART_PTR(IDOMNullBrush);

_END_EDL_DOM_NAMESPACE

#include <edl/edltypes.h>
#include <edl/idomnode.h>
#include <edl/iedlenum.h>
#include <edl/idomcolor.h>
#include <edl/idomfunction.h>
#include <edl/idomimageresource.h>
#include <edl/idomgroup.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
 * @class    IDOMBrush
 * @ingroup  IDom
 * @brief    Interface to the brush element
 *
 * Brushes are used to paint the interior of the geometric shapes defined by a Path and the
 * characters rendered by a Glyphs node. They are also used to define the alpha-transparency mask
 * in the IDOMCanvas::OpacityMask, IDOMPathNode::OpacityMask, and
 * IDOMGlyphs::OpacityMask properties.
 * 
 * All brushes are defined relative to a coordinate space. Most brushes (including image brushes,
 * visual brushes, linear gradient brushes, and radial gradient brushes) may specify a coordinate space
 * transform, in which the transform property is concatenated with the current effective
 * coordinate space to yield an effective coordinate space local to the brush. For image brushes and
 * visual brushes, the viewport is transformed using the local effective render transform. For linear
 * gradient brushes, the start point and end point are transformed. For radial gradient brushes, the
 * center, x-radius, y-radius, and gradient origin are transformed.
 * 
 * If there is an alpha component of the color, it is combined in a muliplicative way with the opacity
 * value.
 * @note Overprint information is stored in a property attached to each node that has overprint set -
 * the “Device Params". You can retrieve it with:
 * <p>
 * @code
		uint32 devParams = 0;
		PValue val;
		if (node->getProperty(“DeviceParams", val))
		{
			devParams = (uint32) val.getInt32();
		}
 * @endcode
 * <p>
 * It’s a set of flags. The most relevant values are:
 * <p>
 * @code
 
		#define OVERPRINT_MODE 	 1
		#define OVERPRINT_FILL 	 2
		#define OVERPRINT_STROKE 4
 * @endcode
 */
class IDOMBrush : public IEDLObject
{
public:
    /**
     * @enum     eBrushType
     * @ingroup  IDom
     * @brief    Brush type enumeration
     */
    typedef enum
    {
        eSolidColor,				//!< Solid color brush
        eLinearGradient,			//!< Linear gradient brush
        eRadialGradient,			//!< Radial gradient brush
        eImage,						//!< Image brush
        eMasked,					//!< Masked brush
        eVisual,					//!< Visual brush
        eSoftMask,					//!< Softmask brush
        eTilingPattern,				//!< Tiling pattern brush
        eType1ShadingPattern,		//!< Type 1 shading pattern brush
        eType2ShadingPattern,		//!< Type 2 shading pattern brush
        eType3ShadingPattern,		//!< Type 3 shading pattern brush
        eType4567ShadingPattern,	//!< Type 4, 5, 6 or 7 shading pattern brush
        eNull,						//!< Null brush
    } eBrushType;

    /**
     * @brief    Retrieves the type of the brush
     * @return   @b eBrushType the brush type
     */
    virtual eBrushType getBrushType() const = 0;
    
    /**
     * @brief    Retrieves the opacity value of the brush element
     * 
     * The opacity value defines the uniform transparency of the canvas. 
     * This is a number between 0 (fully transparent) and 1 (fully opaque).
     * Default value 1.0
     *
     * @return   @b float The function returns the brush's opacity value
     */
    virtual float getOpacity() const = 0;

    /**
     * @brief    Sets the opacity value of a brush element
     * @param    opc The opacity value
     * @return   @b bool The function returns true on success, false on failure.
     */
    virtual bool setOpacity(float opc) = 0;

};
#define edlobj2IDOMBrush(src) edl_cast((IDOMBrush *)NULL, src) 

/**
 * @class   IDOMTransformableBrush
 * @ingroup IDom
 * @brief   Abstract interface for a brush to which a render transform
 *          may be applied.
 *
 * IDOMTransformableBrush and its descendants can store a render transform and are therefore
 * capable of being used meaningfully within a node that itself has a render transform.  
 * A brush that is to be used within such a node has the node's render transform passed in to 
 * it, where it is used to modify any existing render transform that the brush may already have, 
 * such that using the brush in a transformed node will always provide the expected results.
 */
class IDOMTransformableBrush : public IDOMBrush
{
public:
    /**
     * @brief    Retrieves the render transform matrix
     * @param    matrix Render transform matrix
     * @return   @b bool Returns true on success, false if the call fails.
     */
    virtual bool getRenderTransform(FMatrix &matrix) const = 0;

    /**
     * @brief    Sets the render transform matrix
     * @param    matrix Render transform matrix
     * @return   @b bool Returns true on success, false if the call fails.
     */
    virtual bool setRenderTransform(const FMatrix &matrix) = 0;
};
#define edlobj2IDOMTransformableBrush(src) edl_cast((IDOMTransformableBrush *)NULL, src)
DECL_SMART_PTR(IDOMTransformableBrush);

/**
 * @class    IDOMGradientStop
 * @ingroup  IDOM
 * @brief    IDOMGradientStop defines the ramp of colors to use on a gradient.
 *
 * A gradient is a smooth transition from one color to the next.
 * The color range for a gradient can be composed of two or more colors.
 * Each color used in a gradient requires at least one gradient stop. A
 * gradient stop has two attributes: color and offset.  The offset defines a line along
 * which all points have the same color value; it determines the 
 * distance from the base line (for linear gradients) or the radial distance from the
 * starting point of the gradient (for radial gradients), relative to the total distance
 * the gradient covers.  The color value determines the color of all points on the 
 * line specified by the offset.  Between two adjacent gradient stops, the color shades
 * smoothly from the color specified at the first offset to that specified at the second.
 */
class IDOMGradientStop : public IEDLObject
{
#define IDOMGradientStopClassID 0x5c788994, 0xc0bb4387, 0x96ff4086, 0x874b16db
public:

    /**
     * @class    Data
     * @ingroup  IDom
     * @brief    Initialization data
     */
    class Data : public CClassParams
    {
    public:
        double offset;
        IDOMColorPtr color;
    };

    /**
     * @brief    Retrieves class id of IDOMGradientStop
     * @return   @b CClassID Class id of the element
     */
    static const CClassID &classID()
    {
        static CClassID id(IDOMGradientStopClassID);
        return id;
    };

    /**
     * @brief    Retrieves the color value of the gradient stop.
     * @param    color Reference parameter to receive the color value.
     * @return   @b bool Returns true on success, false if the call fails.
     */
    virtual bool getColor(IDOMColorPtr &color) const = 0;

    /**
     * @brief    Sets the color value of the gradient stop.
     * @param    color The new color value.
     * @return   @b bool Returns true on success, false if the call fails.
     */
    virtual bool setColor(const IDOMColorPtr &color) = 0;

    /**
     * @brief    Retrieves the offset value of the gradient stop.
     * @return   @b double Returns the offset value.
     */
    virtual double getOffset() const = 0;

    /**
     * @brief    Sets the offset value of the gradient stop.
     * @param    offset The new offset value.
     * @return   @b bool Returns true on success, false if the call fails.
     */
    virtual bool setOffset(double offset) = 0;

};
#define edlobj2IDOMGradientStop(src) edl_cast((IDOMGradientStop *)NULL, src)

typedef IInputEnumRC<IDOMGradientStopPtr> IDOMGradientStopCollectionEnum;
DECL_SMART_PTR(IDOMGradientStopCollectionEnum);


/**
 * @class    IDOMSolidColorBrush
 * @ingroup  IDom
 * @brief    A solid color brush is used to fill defined geometric regions with a solid color. If there is an alpha
 * component of the color, it is combined in a multiplicative way with the corresponding opacity attribute.
 */
class IDOMSolidColorBrush : public IDOMHashable, public IDOMBrush
{
#define IDOMSolidColorBrushClassID 0x9dc4c01e, 0x8fe545f7, 0xa51b15c6, 0xed751103
public:

    /**
     * @brief    Simplified solid color brush creation.
     *           Throws an IEDLError on failure.
     * @param    pFactory The EDL Class factory to use.
     * @param    color The color to use.
     * @param    opacity The alpha to use.
     * @return   @b IDOMSolidColorBrushPtr The new brush.
     */
    static EDL_API IDOMSolidColorBrushPtr create(IEDLClassFactory *pFactory, const IDOMColorPtr &color, float opacity = 1.0f);

    /**
     * @class    Data
     * @ingroup  IDom
     * @brief    Initialization data
     */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0)
        {}
        float opacity;
        IDOMColorPtr color;
    };

    /**
    * @brief    Retrieves class id of IDOMIDOMSolidColorBrush
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMSolidColorBrushClassID);
        return id;
    };

    /**
    * @brief    Retrieves the color value of the solid color brush.
    * @param    color Reference parameter to receive the color value
    * @return   @b bool Returns true on success, false if the call fails.
    */
    virtual bool getColor(IDOMColorPtr &color) const = 0;

    /**
    * @brief    Sets color value of the solid color brush.
    * @param    color The color value
    * @return   @b bool Returns true on success, false if the call fails.
    */
    virtual bool setColor(const IDOMColorPtr &color) = 0;

};
#define edlobj2IDOMSolidColorBrush(src) edl_cast((IDOMSolidColorBrush *)NULL, src)


/**
 * @enum     eColorInterpolationMode
 * @ingroup  IDom
 * @brief    Color interpolation mode type enumeration
 */
typedef enum {
    eSRgbLinearInterpolation,   //!< Linear interpolation for sRGB color space
    eSCRgbLinearInterpolation   //!< Linear interpolation for scRGB color space
} eColorInterpolationMode;

/**
 * @enum     eSpreadMethod
 * @ingroup  IDom
 * @brief    Spread Method type enumeration
 */
typedef enum {
    ePad,       //!< Fill the remaining area with the color specified by the final gradient stop.
    eReflect,   //!< Fill the remaining area by reflecting the gradient, such that the finish point becomes the start of the reflected gradient and the gradient is calculated by stepping back down through the original gradient points.
    eRepeat,    //!< Fill the remaining area by repeating the gradient from its first gradient stop, starting at the point defined by the last gradient stop.
    eNoSpread   //!< Do not fill the remaining area, but allow it to remain transparent.
} eSpreadMethod;

/**
* @class   IDOMGradientBrush
* @ingroup IDom
* @brief   A common interface for both IDOMLinearGradient and IDOMRadialGradient.
*          Provides straightforward access to common attributes.
*/
class IDOMGradientBrush : public IDOMTransformableBrush
{
  public:
    /**
    * @brief    Retrieves the color interpolation mode value of the radial gradient brush.
    *
    * This is the gamma function for color interpolation for sRGB colors. The gamma adjustment
    * should not be applied to the alpha component, if specified.
    * @see eColorInterpolationMode
    * @return   @b eColorInterpolationMode The color interpolation mode.
    */
    virtual eColorInterpolationMode getColorInterpolationMode() const = 0;

    /**
    * @brief    Sets the color interpolation mode value of the radial gradient brush.
    *
    * This is the gamma function for color interpolation for sRGB colors. The gamma adjustment
    * should not be applied to the alpha component, if specified.
    * @param    cim The color interpolation mode value
    * @see eColorInterpolationMode
    * @return   @b bool Returns true on success, false if the call fails.
    */
    virtual bool setColorInterpolationMode(eColorInterpolationMode cim) = 0;

    /**
    * @brief    Retrieves the spread method value of the RadialGradientBrush element
    *
    * The spread method describes how the brush should fill the content area outside of 
    * the primary gradient area.
    * @see eSpreadMethod
    * @return   @b SpreadMethod The spread method value.
    */
    virtual eSpreadMethod getSpreadMethod() const = 0;

    /**
    * @brief    Sets spread method value of the RadialGradientBrush element
    *
    * The spread method describes how the brush should fill the content area outside of 
    * the primary gradient area.
    * @param    sm The new spread method value.
    * @return   @b bool Returns true on success, false if the call fails.
    */
    virtual bool setSpreadMethod(eSpreadMethod sm) = 0;

    /**
    * @brief    Retrieves an enumerator for the list of the gradient stops for the brush.
    * @see      IDOMGradientStop
    * @return   @b IDOMGradientStopCollectionEnumPtr An enumerator for the gradient stop list.
    */
    virtual IDOMGradientStopCollectionEnumPtr getGradientStopCollectionEnum() = 0;

    /**
    * @brief    Retrieves the number of gradient stops in the gradient stop list.
    * @return   @b uint32 The number of defined gradient stops.
    */
    virtual uint32 getGradientStopsCount() = 0;

    /**
    * @brief    Clears the gradient stop list.
    */
    virtual void clearGradientStopCollection() = 0;

    /**
    * @brief    Append a gradient stop to the collection of gradient stops 
    * @param    ptrGradientStop The smart pointer to the gradient stop interface
    * @see      IDOMGradientStop
    * @return   @b bool Returns true on success, false if the call fails.
    */
    virtual bool addGradientStop(IDOMGradientStopPtr &ptrGradientStop) = 0;
    
    /**
    * @brief    Normalize the gradient stops to a single colour space,
    *           sort them and ensure there are stops at 0.0 and 1.0.
    * @param    pFactory The EDL class factory
    * @param    ptrColorSpace The desired colors pace. Pass NULL to use the
    *           color space of the first brush.
    * @param    intent The desired conversion intent if colour conversion
    *           is required.
    * @param    bpc The desired black point compensation treatment if colour
    *           conversion is required.
    * @return   @b bool Returns true on success, false if the call fails.
    */
    virtual bool normalizeStops(IEDLClassFactory *pFactory, const IDOMColorSpacePtr &ptrColorSpace = IDOMColorSpacePtr(NULL), eRenderingIntent intent = eRelativeColorimetric, eBlackPointCompensation bpc = eBPCDefault) = 0;
};
DECL_SMART_PTR(IDOMGradientBrush);
#define edlobj2IDOMGradientBrush(src) edl_cast((IDOMGradientBrush *)NULL, src) 

/**
* @class    IDOMLinearGradientBrush
* @ingroup  IDom
* @brief    IDOMLinearGradientBrush interface.  A linear gradient brush is used to specify a gradient along a vector.
*/
class IDOMLinearGradientBrush : public IDOMGradientBrush
{
#define IDOMLinearGradientBrushClassID 0x59493763, 0x87084cf5, 0x8033bfd1, 0x0740e0f2
public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0), colorInterpolationMode(eSRgbLinearInterpolation), spreadMethod(ePad)
        {}
        float opacity;
        FPoint startPoint;
        FPoint endPoint;
        eColorInterpolationMode colorInterpolationMode;
        eSpreadMethod spreadMethod;
        FMatrix renderTransform;
    };

    /**
    * @brief    Retrieves class id of IDOMLinearGradientBrush
    * @return   @b CClassID The class id of IDOMLinearGradientBrush.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMLinearGradientBrushClassID);
        return id;
    };

    /**
    * @brief    Retrieves the start point of the linear gradient.
    * @return   @b FPoint Start point of the linear gradient
    */
    virtual const FPoint &getStartPoint() const = 0;

    /**
    * @brief    Sets the start point of the linear gradient.
    * @param    pt The new start point.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStartPoint(const FPoint &pt) = 0;

    /**
    * @brief    Retrieves the end point of the linear gradient.
    * @return   @b FPoint The end point of the linear gradient.
    */
    virtual const FPoint &getEndPoint() const = 0;

    /**
    * @brief    Sets the end point of the linear gradient.
    * @param    pt The new end point
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setEndPoint(const FPoint &pt) = 0;
    
    /**
    * @brief   Create a Type2 Shading Pattern brush from this linear brush.
    * @brief   Only Pad repeat mode is supported; a plain Type 2 brush is
    *          supported. All alpha information in the gradient stops will
    *          be dropped.
    * @param   pFactory The EDL class factory
    * @param   ptrShadingBrush reference to receive the new shading brush.
    * @param   useFirstStopColorSpace If true the colour space in the first stop will be
    *          used for the shading. Otherwise either sRGB or scRGB will be used depending
    *          on the interpolation mode.
    * @return  @b bool True on success, false if the call fails.
    */
    virtual bool createShading(IEDLClassFactory *pFactory, IDOMShadingPatternType2BrushPtr &ptrShadingBrush, bool useFirstStopColorSpace = false) = 0;

};
#define edlobj2IDOMLinearGradientBrush(src) edl_cast((IDOMLinearGradientBrush *)NULL, src) 


/**
* @class    IDOMRadialGradientBrush
* @ingroup  IDom
* @brief    IDOMRadialGradientBrush interface.  A radial gradient brush defines an ellipse to be filled with the gradient.
* The ellipse is defined by its center, x radius, and y radius.  Independently, a gradient origin is specified for the brush.
* The gradient origin defines the center of the gradient; a gradient stop with an offset at 0.0 defines the color at the gradient origin.
* The outer bound of the ellipse defines the end "point" of the gradient; that is, a gradient stop with an offset at 1.0
* defines the color at the circumference of the ellipse, and all other gradient stops define their offsets relative to the
* radial distance between the gradient origin and the circumference.
*/
class IDOMRadialGradientBrush : public IDOMGradientBrush
{
#define IDOMRadialGradientBrushClassID 0xfd36410b, 0x387f4656, 0x9c2c276f, 0xcba05c91
public:
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0), spreadMethod(ePad), colorInterpolationMode(eSRgbLinearInterpolation)
        {}
        float opacity;
        eColorInterpolationMode colorInterpolationMode;
        eSpreadMethod spreadMethod;
        FPoint center;
        FPoint gradientOrigin;
        double radiusX;
        double radiusY;
        FMatrix renderTransform;
    };
    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMRadialGradientBrushClassID);
        return id;
    };


    /**
    * @brief    Retrieves the center point of the gradient brush ellipse.
    * @return   @b FPoint The center point of the gradient brush ellipse.
    */
    virtual const FPoint &getCenter() const = 0;

    /**
    * @brief    Sets the center point of the gradient brush ellipse.
    * @param    pt Reference parameter to receive the center point of the gradient brush ellipse.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setCenter(const FPoint &pt) = 0;

    /**
    * @brief    Retrieves the origin point of the radial gradient, corresponding to the gradient stop with an offset of 0.0.
    * @return   @b FPoint The origin point of the radial gradient.
    */
    virtual const FPoint &getGradientOrigin() const = 0;

    /**
    * @brief    Sets the origin point of the radial gradient, corresponding to the gradient stop with an offset of 0.0.
    * @param    pt The origin point of the radial gradient.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setGradientOrigin(const FPoint &pt) = 0;

    /**
    * @brief    Retrieves the x-radius of the gradient brush ellipse.
    * @return   @b double The x-radius of the gradient brush ellipse.
    */
    virtual double getRadiusX() const = 0;

    /**
    * @brief    Sets the x-radius of the gradient brush ellipse.
    * @param    r The x-radius of the ellipse
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setRadiusX(double r) = 0;

    /**
    * @brief    Retrieves the y-radius of the gradient brush ellipse.
    * @return   @b double The y-radius of the gradient brush ellipse.
    */
    virtual double getRadiusY() const = 0;

    /**
    * @brief    Sets the y-radius of the gradient brush ellipse.
    * @param    r The y-radius of the gradient brush ellipse.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setRadiusY(double r) = 0;

    /**
    * @brief   Create a Type3 Shading Pattern brush from this radial brush.
    * @brief   All alpha information in the gradient stops will be dropped.
    *          The resulting brush will be generated to fill the given 
    *          rectangle.
    * @param   pFactory The EDL class factory
    * @param   ptrShadingBrush reference to receive the new shading brush.
    * @param   fillArea The rectangular area that needs to be filled by the resulting brush.
    * @param   useFirstStopColorSpace If true the colour space in the first stop will be
    *          used for the shading. Otherwise either sRGB or scRGB will be used depending
    *          on the interpolation mode.
    * @return  @b bool True on success, false if the call fails.
    */
    virtual bool createShading(IEDLClassFactory *pFactory, IDOMShadingPatternType3BrushPtr &ptrShadingBrush, const FRect &fillArea, bool useFirstStopColorSpace = false) = 0;
    
    /**
    * @brief   Create a simplified radial gradient brush, where any repeat
    *          or reflect pad mode is converted to simple padding by repeating
    *          gradient stops as required. The resulting brush will be generated
    *          to fill the given rectangle.
    *          An exception of type IEDLError is thrown on failure.
    * @param   pFactory The EDL class factory
    * @param   fillArea The rectangular area that needs to be filled by the resulting brush.
    * @return  @b IDOMRadialGradientBrushPtr The simplified brush, or this brush if no
    *          action was required.
    */
    virtual IDOMRadialGradientBrushPtr getSimplifiedGradient(IEDLClassFactory *pFactory, const FRect &fillArea) = 0;

};
#define edlobj2IDOMRadialGradientBrush(src) edl_cast((IDOMRadialGradientBrush *)NULL, src) 


/**
* @enum     eViewUnits
* @ingroup  IDom
* @brief    View units type enumeration
*/
typedef enum
{
    eAbsolute      //!< Absolute units
} eViewUnits;

/**
* @enum     eTilingMode
* @ingroup  IDom
* @brief    Tiling mode type enumeration
*/
typedef enum
{
    eNoTile,    //!< No tiling.  If the area to be painted is larger than the image, just paint the image once (in the location specified by the brush's viewport), and leave the remaining area transparent.
    eTile,      //!< Tile image without any flipping or rotating of the image. A square image consisting of a single diagonal line between opposite corners would produce diagonal lines when tiled in this mode.
    eFlipX,     //!< Tile image such that alternate columns of tiles are flipped horizontally. A square image consisting of a single diagonal line between opposite corners would produce chevrons running horizontally across the area when tiled in this mode.
    eFlipY,     //!< Tile image such that alternate rows of tiles are flipped vertically. A square image consisting of a single diagonal line between opposite corners would produce chevrons running vertically across the area when tiled in this mode.
    eFlipXY     //!< Tile image such that alternate columns of tiles are flipped horizontally AND alternate rows of tiles are flipped vertically. A square image consisting of a single diagonal line between opposite corners would produce a grid of squares balanced on their points when tiled in this mode.
} eTilingMode;


/**
* @class    IDOMImageBrush
* @ingroup  IDom
* @brief    Provides an interface to a DOM image brush object.
*
* An image brush is used to fill a region with an image. The image is defined in a coordinate space
* specified by the resolution of the image.
*/
class IDOMImageBrush : public IDOMHashable, public IDOMTransformableBrush
{
#define IDOMImageBrushClassID 0xebe9f8fd, 0xc5c34d21, 0xae76465a, 0x4b4e64a0
public:

    /**
    * @brief  Simplified creator for an image brush.
    *         Throws an IEDLError on failure.
    * @param  pFactory The factory to use.
    * @param  image The image to use.
    * @param  viewBox The desired view box. If empty, the viewPort will be set to the
    *         full area of the image.
    * @param  viewPort The desired view port.
    * @param  renderTransform The desired render transform.
    * @param  opacity The desired brush opacity.
    * @param  tileMode The desired tile mode.
    * @param  alternateImages The PDF Alternate image brushes, if present.
    * @param  optionalContentDetails The optional content details. Ignored for tiling images
    * @return IDOMImageBrushPtr The new brush.
    */
    static EDL_API IDOMImageBrushPtr create(IEDLClassFactory *pFactory,
                                            const IDOMImagePtr &image,
                                            const FRect &viewBox, const FRect &viewPort,
                                            const FMatrix &renderTransform = FMatrix(),
                                            float opacity = 1.0f,
                                            eTilingMode tileMode = eNoTile,
                                            const CDOMAlternateImageVect &alternateImages = CDOMAlternateImageVect(),
                                            const JawsMako::IOptionalContentDetailsPtr &optionalContentDetails =         JawsMako::IOptionalContentDetailsPtr());

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0), tileMode(eNoTile), viewBoxUnits(eAbsolute), viewPortUnits(eAbsolute)
        {
        }
        float                                opacity;
        FRect                                viewBox;
        FRect                                viewPort;
        eTilingMode                          tileMode;
        eViewUnits                           viewBoxUnits;
        eViewUnits                           viewPortUnits;
        IDOMImagePtr                         ptrImageSource;
        IDOMICCProfilePtr                    ptrICCProfile;
        FMatrix                              renderTransform;
        JawsMako::IOptionalContentDetailsPtr optionalContentDetails;
        CDOMAlternateImageVect               alternateImages;
    };
    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMImageBrushClassID);
        return id;
    };

    /**
    * @brief    Retrieves the tiling mode value of the image brush.
    * @see      eTilingMode
    * @return   @b eTilingMode The image tiling mode.
    */
    virtual eTilingMode getTileMode() const = 0;

    /**
    * @brief    Sets the tiling mode of the image brush.
    * @param    tm The tiling mode value
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setTileMode(eTilingMode tm) = 0;

    /**
    * @brief    Retrieves the viewbox units used by the image brush.  Currently, only absolute units are supported.
    * @see      getViewBox()
    * @see      setViewBox()
    * @return   @b eViewUnits The viewbox units.
    */
    virtual eViewUnits getViewBoxUnits() const = 0;

    /**
    * @brief    Sets the viewbox units value of the image brush.  Currently, only absolute units are supported.
    * @see      getViewBox()
    * @see      setViewBox()
    * @param    vu The viewbox units value
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setViewBoxUnits(eViewUnits vu) = 0;

    /**
    * @brief    Retrieves the viewport units value of the image brush.  Currently, only absolute units are supported.
    * @see      getViewPort()
    * @see      setViewPort()
    * @return   @b eViewUnits The viewport units.
    */
    virtual eViewUnits getViewPortUnits() const = 0;

    /**
    * @brief    Sets the viewport units used for the image brush.  Currently, only absolute units are supported.
    * @see      getViewPort()
    * @see      setViewPort()
    * @param    vu The new viewport units value
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setViewPortUnits(eViewUnits vu) = 0;

    /**
    * @brief    Retrieves the viewbox rectangle
    *
    * The viewbox specifies the portion of a source image or visual to be rendered to the page as a tile, 
    * whose size and location are determined by the image brush's viewport.
    * The tile is then used to fill the geometry specified by the parent element according to the image brush's tile mode.
    * The ViewBox can specify a region larger than the image itself, including negative values. The view box specifies 
    * the position and dimension of the brush's source content. It is specified by four comma-separated real numbers
    * (x, y, width, height) where width and height are non-negative.
    * @param    vb The viewbox rectangle
    * @return   @b bool True on success
    */
    virtual bool getViewBox(FRect &vb) const = 0;

    /**
    * @brief    Sets viewbox rectangle
    *
    * The viewbox specifies the portion of a source image or visual to be rendered to the page as a tile.
    * The tile is then used to fill the geometry specified by the parent element according to the TileMode() function.
    * The viewbox can specify a region larger than the image itself, including negative values. The viewbox specifies 
    * the position and dimension of the brush's source content. It is specified by four comma-separated real numbers
    * (x, y, width, height) where width and height are non-negative.
    * @param    vb The viewbox rectangle
    * @return   @b bool True on success
    */
    virtual bool setViewBox(const FRect &vb) = 0;

    /**
    * @brief    Retrieves the viewport rectangle.
    *
    * The viewport specifies the dimensions and location of the initial tile that will be filled with the specified
    * image or visual fragment.  It is defined in the current effective coordinate space.
    * It is specified by four comma separated real numbers (x, y, width, height) where width and height
    * are non-negative.
    * @param    vp Reference parameter to receive the viewport rectangle.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getViewPort(FRect &vp) const = 0;

    /**
    * @brief    Sets the viewport rectangle.
    *
    * The viewport specifies the dimensions and location of the initial tile that will be filled with the specified
    * image or visual fragment.  It is defined in the current effective coordinate space.
    * It is specified by four comma separated real numbers (x, y, width, height) where width and height
    * are non-negative.
    * @param    vp The viewport rectangle
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setViewPort(const FRect &vp) = 0;

    /**
    * @brief    Retrieves the ICC profile of the brush.
    * @param    icc Reference parameter to receive the ICC profile.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getICCProfile(IDOMICCProfilePtr &icc) const = 0;

    /**
    * @brief    Sets ICC profile
    * @param    icc The ICC profile
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setICCProfile(const IDOMICCProfilePtr &icc) = 0;

    /**
    * @brief    Retrieves a smart pointer to the image resource.
    * @param    ptrImageSource Smart pointer to receive the image resource.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getImageSource(IDOMImagePtr &ptrImageSource) const = 0;

    /**
    * @brief    Sets the image resource for the brush.
    * @param    ptrImageSource Smart pointer to the new image resource.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setImageSource(const IDOMImagePtr &ptrImageSource) = 0;
    
    /**
    * @brief       Retrieves any alternate images associated with this brush.
    * @return      CDOMAlternateImageVect The alternate images, or an empty vector if there are no alternates.
    */
    virtual CDOMAlternateImageVect getAlternateImages() const = 0;

    /**
    * @brief       Set the alternate images associated with this brush.
    * @param       alternates The alternate images.
    */
    virtual void setAlternateImages(const CDOMAlternateImageVect &alternates) = 0;

    /**
    * @brief     Returns any optional content information that applies to this brush
    * @return    @b JawsMako::IOptionalContentDetailsPtr The optional content details, or NULL if optional content details do not apply
    */
    virtual JawsMako::IOptionalContentDetailsPtr getOptionalContentDetails() const = 0;
    
    /**
    * @brief     Set the optional content details that apply to this brush, or NULL to remove
    * @param     @b details The optional content details, or NULL to remove
    */
    virtual void setOptionalContentDetails(const JawsMako::IOptionalContentDetailsPtr &details) = 0;

    /**
    * @brief    Gets an equivalent IDOMTilingPattern brush. If the receiver
    *           has a tile mode of eNoTile, this call will fail.
    * @param    pFactory A pointer to an EDL class factory
    * @param    tiling A reference to receive the equivalent tiling brush
    * @return   bool True on success.
    */
    virtual bool getEquivalentTilingBrush(IEDLClassFactory *pFactory, IDOMTilingPatternBrushPtr &tiling) = 0;
    
    /**
     * @brief   For tiled images, returns an equivalent visual brush containing
     *          the image without tiling. Will throw an exception if the image
     *          is not tiled. Exceptions of type IEDLError are thrown on failure.
     *          Not available for masked brushes as tiling is not supported for those
     *          brushes.
     * @param   pFactory A pointer to an EDL class factory
     * @return  IDOMVisualBrushPtr The created visual brush.
     */
    virtual IDOMVisualBrushPtr getEquivalentVisualBrush(IEDLClassFactory *pFactory) = 0;
};
#define edlobj2IDOMImageBrush(src) edl_cast((IDOMImageBrush *)NULL, src) 

/**
* @class    IDOMMaskedBrush
* @ingroup  IDom
* @brief    IDOMMaskedBrush interface, this describes a generalization of
*           a masked image. The sub-brush (set by getBrush()/setBrush())
*           is painted through a mask specified by the image.
*           Importantly, the sub-brush is not subject to the IDOMImageBrush
*           render transform.
*           Tiling is not supported for this brush type.
*/
class IDOMMaskedBrush : public IDOMImageBrush
{
#define IDOMMaskedBrushClassID 0x8EC7F5C6, 0xC56049d1, 0x9E2864EB, 0x0E7A90A6
public:

    /**
    * @brief  Simplified creator for a masked brush.
    *         Throws an IEDLError on failure.
    * @param  pFactory The factory to use.
    * @param  mask The image mask to use.
	* @param  brush The brush to use
    * @param  viewBox The desired view box. If empty, the viewPort will be set to the
    *         full area of the image.
    * @param  viewPort The desired view port.
    * @param  renderTransform The desired render transform.
    * @param  opacity The desired brush opacity.
    * @param  optionalContentDetails Any optional content details to be applied to this masked case
    * @return @b IDOMMaskedBrushPtr The new brush.
    */
    static EDL_API IDOMMaskedBrushPtr create(IEDLClassFactory *pFactory,
                                             const IDOMImagePtr &mask,
                                             const IDOMBrushPtr &brush,
                                             const FRect &viewBox, const FRect &viewPort,
                                             const FMatrix &renderTransform = FMatrix(),
                                             float opacity = 1.0f,
                                             const CDOMAlternateImageVect &alternateImages = CDOMAlternateImageVect(),
                                             const JawsMako::IOptionalContentDetailsPtr &optionalContentDetails =         JawsMako::IOptionalContentDetailsPtr());
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0), viewBoxUnits(eAbsolute), viewPortUnits(eAbsolute)
        {
        }
        float                                opacity;
        FRect                                viewBox;
        FRect                                viewPort;
        eViewUnits                           viewBoxUnits;
        eViewUnits                           viewPortUnits;
        IDOMImagePtr                         imageMask;
        IDOMBrushPtr                         brush;
        FMatrix                              renderTransform;
        JawsMako::IOptionalContentDetailsPtr optionalContentDetails;
        CDOMAlternateImageVect               alternateImages;
    };
    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMMaskedBrushClassID);
        return id;
    };

    /**
    * @brief    Retrieves smart pointer to the brush to be painted through the image.
    * @param    brush Smart pointer to the brush
    * @return   @b bool True on success
    */
    virtual bool getBrush (IDOMBrushPtr &brush) const = 0;

    /**
    * @brief    Sets brush
    * @param    brush Smart pointer to the brush resource to be painted through the image.
    * @return   @b bool True on success
    */
    virtual bool setBrush (const IDOMBrushPtr &brush) = 0;
    
    /**
    * @brief    Is the mask a soft mask?
    *           That is, is the mask not a single bit image?
    *           An outright error will result in an exception of type IEDLError.
    * @param    pFactory A pointer to an EDL class factory
    * @return   @b bool True if the mask is soft.
    */
    virtual bool getIsSoftMask(IEDLClassFactory *pFactory) = 0;

    /**
    * @brief    Gets an equivalent image or visual brush that can be used to express this
    *           masked brush in XPS compatible form. This does not guarantee that the sub
    *           brush can be represented in XPS; these ought to be processed first.
    *           To do this, the bounds of the enclosing object in internal space must
    *           be provided (that is, the coordinate space effective inside that object).
    * @param    pFactory A pointer to an EDL class factory
    * @param    xpsForm A reference to receive the xps compatible form
    * @param    enclosedBounds The bounds in internal object space of the object that has
    *           this brush.
    * @return   @b bool True on success.
    */
    virtual bool getEquivalentXPSBrush(IEDLClassFactory *pFactory, IDOMBrushPtr &xpsForm, const FRect &enclosedBounds) = 0;
    
    /**
     * @brief    Attempts to create a single IDOMImageBrush that representes the masked
     *           result. This can be achieved if the brush masked by the image is a
     *           solid color brush, or if the brush masked by the image is an image with
     *           the same dimensions.
     *           Returns NULL if such a brush isn't possible, and throws an IEDLError
     *           exception on outright failures.
	 *  @param   pFactory pFactory A pointer to an EDL class factory
	 *  @return  @b IDOMImageBrushPtr The image brush
	 */	
	virtual IDOMImageBrushPtr getSimpleImageBrush(IEDLClassFactory *pFactory) = 0;

};
#define edlobj2IDOMMaskedBrush(src) edl_cast((IDOMMaskedBrush *)NULL, src) 

/**
* @class    IDOMVisualBrush
* @ingroup  IDom
* @brief    A visual brush is used to fill a region with a vector drawing.
*
* The drawing may be specified as either a visual brush property or as a resource reference. 
* The drawing content may include exactly one Canvas, Path, or Glyphs node and that node's 
* child and descendant nodes.
*
* Visual brushes share a number of tile-related properties with image brushes.
* @see IDOMImageBrush
*/
class IDOMVisualBrush : public IDOMTransformableBrush
{
#define IDOMVisualBrushClassID 0xa811d50a, 0xfebd4be7, 0xa3901ac9, 0x0bd52265
public:

    /**
    * @brief  Simplified creator for a visual brush.
    *         Throws an IEDLError on failure.
    * @param  pFactory The factory to use.
    * @param  visual The node to use as the visual.
    * @param  viewBox The desired view box.
    * @param  viewPort The desired view port.
    * @param  renderTransform The desired render transform.
    * @param  opacity The desired brush opacity.
    * @param  tileMode The desired tile mode.
    * @return @b IDOMVisualBrushPtr The new visual brush.
    */
    static EDL_API IDOMVisualBrushPtr create(IEDLClassFactory *pFactory,
                                             const IDOMNodePtr &visual,
                                             const FRect &viewBox, const FRect &viewPort,
                                             const FMatrix &renderTransform = FMatrix(),
                                             float opacity = 1.0f,
                                             eTilingMode tileMode = eNoTile);

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0), tileMode(eNoTile), viewBoxUnits(eAbsolute), viewPortUnits(eAbsolute)
        {
        }
        float       opacity;
        FRect       viewBox;
        FRect       viewPort;
        eTilingMode tileMode;
        eViewUnits  viewBoxUnits;
        eViewUnits  viewPortUnits;
        IDOMNodePtr ptrVisual;
        FMatrix     renderTransform;
    };

    /**
    * @brief    Retrieves the class id of IDOMVisualBrush.
    * @return   @b CClassID The class id of IDOMVisualBrush.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMVisualBrushClassID);
        return id;
    };

    /**
    * @brief    Retrieves the tiling mode value of the visual brush.
    * @return   @b eTilingMode The tiling mode
    * @see      eTilingMode
    */
    virtual eTilingMode getTileMode() const = 0;

    /**
    * @brief    Sets tiling mode value of the visual brush.
    * @param    tm The tiling mode value
    * @return   @b bool True on success, false if the call fails.
    * @see      eTilingMode
    */
    virtual bool setTileMode(eTilingMode tm) = 0;

    /**
    * @brief    Retrieves the viewbox units used by the image brush.  Currently, only absolute units are supported.
    * @see      getViewBox()
    * @see      setViewBox()
    * @return   @b eViewUnits The viewbox units.
    */
    virtual eViewUnits getViewBoxUnits() const = 0;

    /**
    * @brief    Sets the viewbox units value of the image brush.  Currently, only absolute units are supported.
    * @see      getViewBox()
    * @see      setViewBox()
    * @param    vu The viewbox units value
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setViewBoxUnits(eViewUnits vu) = 0;

    /**
    * @brief    Retrieves the viewport units value of the image brush.  Currently, only absolute units are supported.
    * @see      getViewPort()
    * @see      setViewPort()
    * @return   @b eViewUnits The viewport units.
    */
    virtual eViewUnits getViewPortUnits() const = 0;

    /**
    * @brief    Sets the viewport units used for the image brush.  Currently, only absolute units are supported.
    * @see      getViewPort()
    * @see      setViewPort()
    * @param    vu The new viewport units value
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setViewPortUnits(eViewUnits vu) = 0;

    /**
    * @brief    Retrieves the viewbox rectangle
    *
    * The viewbox specifies the portion of a source image or visual to be rendered to the page as a tile, 
    * whose size and location are determined by the image brush's viewport.
    * The tile is then used to fill the geometry specified by the parent element according to the image brush's tile mode.
    * The ViewBox can specify a region larger than the image itself, including negative values. The view box specifies 
    * the position and dimension of the brush's source content. It is specified by four comma-separated real numbers
    * (x, y, width, height) where width and height are non-negative.
    * @param    vb The viewbox rectangle
    * @return   @b bool True on success
    */
    virtual bool getViewBox(FRect &vb) const = 0;

    /**
    * @brief    Sets viewbox rectangle
    *
    * The viewbox specifies the portion of a source image or visual to be rendered to the page as a tile.
    * The tile is then used to fill the geometry specified by the parent element according to the TileMode() function.
    * The viewbox can specify a region larger than the image itself, including negative values. The viewbox specifies 
    * the position and dimension of the brush's source content. It is specified by four comma-separated real numbers
    * (x, y, width, height) where width and height are non-negative.
    * @param    vb The viewbox rectangle
    * @return   @b bool True on success
    */
    virtual bool setViewBox(const FRect &vb) = 0;

    /**
    * @brief    Retrieves the viewport rectangle.
    *
    * The viewport specifies the dimensions and location of the initial tile that will be filled with the specified
    * image or visual fragment.  It is defined in the current effective coordinate space.
    * It is specified by four comma separated real numbers (x, y, width, height) where width and height
    * are non-negative.
    * @param    vp Reference parameter to receive the viewport rectangle.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getViewPort(FRect &vp) const = 0;

    /**
    * @brief    Sets the viewport rectangle.
    *
    * The viewport specifies the dimensions and location of the initial tile that will be filled with the specified
    * image or visual fragment.  It is defined in the current effective coordinate space.
    * It is specified by four comma separated real numbers (x, y, width, height) where width and height
    * are non-negative.
    * @param    vp The viewport rectangle
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setViewPort(const FRect &vp) = 0;

    /**
    * @brief    Retrieves smart pointer to the visual node (path, glyphs or canvas node) used to specify the source for the visual brush.
    * @param    ptrVisual Smart pointer to receive the visual node.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getVisual(IDOMNodePtr &ptrVisual) const = 0;

    /**
    * @brief    Sets the visual node (path, glyphs or canvas node) used to specify the source for the visual brush.
    * @param    ptrVisual  Pointer to the visual node (path, glyphs or canvas node) used to specify the source for the visual brush.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setVisual(const IDOMNodePtr &ptrVisual) = 0;

    /**
    * @brief    Gets an equivalent IDOMTilingPattern brush. If the receiver
    *           has a tile mode of eNoTile, this call will fail.
    * @param    pFactory A pointer to an EDL class factory
    * @param    tiling A reference to receive the equivalent tiling bruhs
    * @return   @b bool True on success.
    */
    virtual bool getEquivalentTilingBrush(IEDLClassFactory *pFactory, IDOMTilingPatternBrushPtr &tiling) = 0;
    
    /**
    * @brief    Gets an equivalent visual brush where any flip tile mode is simplified to simple tiling.
    *           Useful for situations where flipping cannot be handled, but simple visual brushes can.
    *           An exception of type IEDLError is thrown on failure.
    * @param    pFactory A pointer to an EDL class factory.
    * @return   @b IDOMVisualBrushPtr The equivalent visual brush, or this brush if no changes are required.
    */
    virtual IDOMVisualBrushPtr getEquivalentSimpleVisualBrush(IEDLClassFactory *pFactory) = 0;
};

#define edlobj2IDOMVisualBrush(src) edl_cast((IDOMVisualBrush *)NULL, src) 


/**
* @class    IDOMVisualRoot
* @ingroup  IDom
* @brief    IDOMVisualRoot interface
*/
class IDOMVisualRoot : public IDOMNode
{
#define IDOMVisualRootClassID 0x2309ff5c, 0x29c64c63, 0xbf58de4d, 0x7819ee02
public:

    /**
    * @brief    Retrieves the class ID of IDOMVisualRoot
    * @return   @b CClassID The class ID of IDOMVisualRoot.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMVisualRootClassID);
        return id;
    }

};
#define edlobj2IDOMVisualRoot(src) edl_cast((IDOMVisualRoot *)NULL, src) 


class IDOMSoftMaskBrush;
DECL_SMART_PTR(IDOMSoftMaskBrush);

/**
* @class    IDOMSoftMaskBrush
* @ingroup  IDom
* @brief    IDOMSoftMaskBrush provides a way of representing a PDF style
*           soft mask in it's entirity. The soft mask brush contains a suitable
*           IDOMTransparency group, as well as the necessary soft mask details.
*           See section 7.5.4 of the PDF 1.7 specification.
*           These are only allowed for OpacityMask entries.
*/
class IDOMSoftMaskBrush : public IDOMTransformableBrush
{
#define IDOMSoftMaskBrushClassID 0x27315fbf, 0x8f3246a9, 0x929eb580, 0x13f7955d
public:
    /**
    * @brief Enum for soft mask interpretation.
    */
    typedef enum
    {
        eAlpha,
        eLuminosity
    } eSoftMaskType;

    /**
    * @brief  Simplified creator for a soft mask brush.
    *         Throws an IEDLError on failure.
    * @param  pFactory The factory to use.
    * @param  group The group to form the content of the soft mask.
    * @param  type  The type of the soft mask.
    * @param  renderTransform The desired render transform.
    * @param  backdropColor The backdrop colour (must have the same
    *         colour space as the group). Optional.
    * @param  transfer The soft mask transfer function. Optional.
    * @return @b IDOMSoftMaskBrushPtr The new brush.
    */
    static EDL_API IDOMSoftMaskBrushPtr create(IEDLClassFactory *pFactory,
                                               const IDOMTransparencyGroupPtr &group,
                                               eSoftMaskType type,
                                               const FMatrix &renderTransform = FMatrix(),
                                               const IDOMColorPtr &backdropColor = IDOMColorPtr(),
                                               const IDOMFunctionPtr &transfer = IDOMFunctionPtr());

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : type(eAlpha), opacity(1.0)
        {
        }
        eSoftMaskType            type;
        float                    opacity;
        IDOMTransparencyGroupPtr group;
        IDOMColorPtr             backdropColor;
        IDOMFunctionPtr          transfer;
        FMatrix                  renderTransform;
    };

    /**
     * @brief    Retrieves class id of IDOM
     * @return   @b CClassID Class id of the element
     */
    static const CClassID &classID()
    {
        static CClassID id(IDOMSoftMaskBrushClassID);
        return id;         
    };

    /**
     * @brief    Retrieves the soft mask type.
     * @return   @b eSoftMaskType The soft mask type.
     */
    virtual eSoftMaskType getSoftMaskType() = 0;

    /**
     * @brief    Retrieves the transparency group used for the mask.
     * @param    group Smart pointer to receive the group.
     * @return   @b bool True on success.
     */
    virtual bool getGroup(IDOMTransparencyGroupPtr &group) = 0;

    /**
     * @brief    Retrieves the color used for luminosity backdrop composition.
     * @param    backdropColor Smart pointer to receive the color.
     * @return   @b bool True on success. Will return false
     *                if there is no color, or if the mask is eAlpha type.
     */
    virtual bool getBackdropColor(IDOMColorPtr &backdropColor) = 0;

    /**
     * @brief    Retrieves the function used for luminosity used for deriving mask values.
     * @param    transfer Smart pointer to receive the function.
     * @return   @b bool True on success. Will return false
     *               if there is no function.
     */
    virtual bool getTransferFunction(IDOMFunctionPtr &transfer) = 0;

};

#define edlobj2IDOMSoftMaskBrush(src) edl_cast((IDOMSoftMaskBrush *)NULL, src)


/**
* @class    IDOMTilingPatternBrush
* @ingroup  IDom
* @brief    IDOMTilingPatternBrush provides a way of representing a PS style tiling pattern. 
*/
class IDOMTilingPatternBrush : public IDOMTransformableBrush
{
#define IDOMTilingPatternBrushClassID 0xE987B191, 0x552840e1, 0xAFA8EC97, 0x2C6EE3DC
public:

    /**
     * @brief  Simplified creator for a tiling brush.
     *         Throws an IEDLError on failure.
     * @param  pFactory The factory to use.
     * @param  visual The node or node tree representing the pattern's content.
     * @param  bBox the bounding box of the pattern's content.
     * @param  paintType The paint type, either 1 (coloured) or 2 (uncoloured)
     * @param  color The color to use. Must be NULL for paintType 1. For paintType 2,
     *         if NULL, A DeviceGray black will be used.
     * @param  xStep The tiling step in X
     * @param  yStep The tiling step in Y
     * @param  tilingType The tiling type; 1 for constant spacing, 2 for no distortion,
     *         3 for constant spacing and faster tiling.
     * @param  renderTransform The desired render transform.
     * @return @b IDOMTilingPatternBrushPtr The new brush.
     */
    static EDL_API IDOMTilingPatternBrushPtr create(IEDLClassFactory *pFactory,
                                                    const IDOMNodePtr &visual, const FRect &bBox,
                                                    uint32 paintType, const IDOMColorPtr &color,
                                                    float xStep, float yStep,
                                                    uint32 tilingType = 1,
                                                    const FMatrix &renderTransform = FMatrix());

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data()
        {
            opacity     = 1.0f;
            patternType = 1;
            tilingType  = 1;
            opacity     = 1.0;
            bbox [0]    = 0.0;
            bbox [1]    = 0.0;
            bbox [2]    = 1.0;
            bbox [3]    = 1.0;
            xstep       = 1;
            ystep       = 1;
            renderTransform = FMatrix (1.0, 0, 0, 1.0, 0, 0);
        }
        float           opacity;
        int             patternType;
        float           bbox [4];
        float           xstep;
        float           ystep;
        int             paintType;
        int             tilingType;
        FMatrix         renderTransform;

        /* for uncolored patterns, these are the color to use for the 
        * specific invocation of setpattern
        */
        IDOMColorPtr    patternColor;

        IDOMNodePtr     visual;
    };

    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id (IDOMTilingPatternBrushClassID);
        return id;
    };

    /**
    * @brief    Retrieves the pattern type
    * @return   @b int Returns pattern type, 1 for tiling pattern
    */
    virtual int getPatternType () = 0;

    /**
    * @brief    Retrieves the bounding box for the pattern
    * @param    bbox Reference to bbox
    * @return   @b bool True on success.
    */
    virtual bool getBBox (float *bbox) = 0;

    /**
    * @brief    Sets the bounding box for the pattern
    * @param    bbox Reference to bbox
    * @return   @b bool True on success.
    */
    virtual bool setBBox (float *bbox) = 0;

    /**
    * @brief    Retrieves the tiling step for the pattern
    * @param    xstep Reference to xstep
    * @param    ystep Reference to ystep
    * @return   @b bool True on success.
    */
    virtual bool getTilingStep (float &xstep, float &ystep) = 0;

    /**
    * @brief    Sets the tiling step for the pattern
    * @param    xstep Reference to xstep
    * @param    ystep Reference to ystep
    * @return   @b bool True on success.
    */
    virtual bool setTilingStep (float xstep, float ystep) = 0;

    /**
    * @brief    Sets the paint type for the pattern
    * @param    paintType The paint type (1 for colored tiling pattern, 2 for uncolored)
    * @return   bool True on success.
    */
    virtual bool setPaintType (int paintType) = 0;

    /**
    * @brief    Gets the paint type for the pattern
    * @param    paintType The paint type (1 for colored tiling pattern, 2 for uncolored)
    * @return   @b bool True on success.
    */
    virtual bool getPaintType (int &paintType) = 0;

    /**
    * @brief    Sets the tiling type for the pattern
    * @param    tilingType The tiling type (1 for constant spacing, 2 for no distortion, 3 for constant spacing and faster tiling)
    * @return   @b bool True on success.
    */
    virtual bool setTilingType (int tilingType) = 0;

    /**
    * @brief    Gets the tiling type for the pattern
    * @param    tilingType The tiling type (1 for constant spacing, 2 for no distortion, 3 for constant spacing and faster tiling)
    * @return   @b bool True on success.
    */
    virtual bool getTilingType (int &tilingType) = 0;

    /**
    * @brief    Sets the pattern color for uncolored pattern (paint type is 2)
    * @param    color Reference to the desired color to use.
    * @return   @b bool True on success.
    */
    virtual bool setPatternColor (IDOMColorPtr &color) = 0;

    /**
    * @brief    Gets the pattern color for uncolored pattern (paint type is 2)
    * @param    color Reference to receive the color.
    * @return   @b bool True on success.
    */
    virtual bool getPatternColor (IDOMColorPtr &color) const = 0;

    /**
    * @brief    Retrieves smart pointer to visual (path, glyphs, canvas) node
    * @param    ptrVisual Smart pointer to the visual node
    * @return   @b bool True on success
    */
    virtual bool getVisual(IDOMNodePtr &ptrVisual) const = 0;

    /**
    * @brief    Sets visual node
    * @param    ptrVisual Smart pointer to the visual node
    * @return   @b bool True on success
    */
    virtual bool setVisual(const IDOMNodePtr &ptrVisual) = 0;

    /**
    * @brief    Retrieves render transform matrix
    * @param    matrix Render transform matrix
    * @return   @b bool True on success
    */
    virtual bool getRenderTransform(FMatrix &matrix) const = 0;

    /**
    * @brief    Sets render transform matrix
    * @param    matrix Render transform matrix
    * @return   @b bool True on success
    */
    virtual bool setRenderTransform(const FMatrix &matrix) = 0;

   /**
    * @brief    Gets an equivalent IDOMVisualBrush brush.
    *           If the brush has overlapping tiles, this call will fail.
    * @param    pFactory A pointer to an EDL class factory
    * @param    visualBrush A reference to receive the equivalent visual brush
    * @return   @b bool True on success.
    */
    virtual bool getEquivalentVisualBrush(IEDLClassFactory *pFactory, IDOMVisualBrushPtr &visualBrush) = 0;

};
#define edlobj2IDOMTilingPatternBrush(src) edl_cast((IDOMTilingPatternBrush *)NULL, src)


/**
* @class    IDOMShadingPatternBrush
* @ingroup  IDom
* @brief    IDOMShadingBrush provides a way of representing a PS style shading pattern
*/
class IDOMShadingPatternBrush : public IDOMTransformableBrush
{
public:

    /**
    * @brief    Retrieves the shading type
    * @return   @b int Returns 1 to 7 for the different PS shading types
    */
    virtual int getShadingType () = 0;

    /**
    * @brief    Retrieves the bounding box for the shade 
    * @param    bbox Reference to bbox
    * @return   @b bool True on success.
    */
    virtual bool getBBox (float *bbox) = 0;

    /**
    * @brief    Sets the bounding box for the shade 
    * @param    bbox Reference to the bounding box
    * @return   @b bool True on success.
    */
    virtual bool setBBox (float *bbox) = 0;

    /**
    * @brief    Sets the background color to use before painting the shade
    * @param    color The smart pointer to the color object. 
    * @return   @b bool True on success.
    */
    virtual bool setBackgroundColor (const IDOMColorPtr &color) = 0;

    /**
    * @brief    Gets the background color to use before painting the shade.
    *           If the return is false then the background is not painted before
    *           applying the shade.
    * @param    color The smart pointer to the color object. 
    * @return   @b bool True on success.
    */
    virtual bool getBackgroundColor (IDOMColorPtr &color) = 0;

    /**
    * @brief    Sets the colorspace to use for painting the shade
    * @param    colorSpace The smart pointer to the colorspace object.
    * @return   @b bool True on success.
    */
    virtual bool setColorSpace (const IDOMColorSpacePtr &colorSpace) = 0;

    /**
    * @brief    Gets the colorspace object to be used when painting the shading
    * @param    colorSpace The smart pointer to the colorspace object.
    * @return   @b bool True on success.
    */
    virtual bool getColorSpace (IDOMColorSpacePtr &colorSpace) = 0;
    
    /**
     * @brief    Set the colorspace of the shading brush, performing color
     *           conversion to that target space. This will also convert the
     *           background color if present, whereas the alternate setColorSpace()
     *           above will not.
     *           Throws exceptions of type IEDLError on failure.
     * @param    pFactory A pointer to an EDL class factory
     * @param    colorSpace The smart pointer to the color space. Must not be
     *           a complex or composite color space such as Indexed or DeviceN.
     * @param    intent The rendering intent to use for conversion.
     * @param    bpc Black point compensation treatment. If in doubt, use eBPCDefault.
     */
    virtual void setColorSpace (IEDLClassFactory *pFactory, const IDOMColorSpacePtr &colorSpace, eRenderingIntent intent, eBlackPointCompensation bpc) = 0;

    /**
    * @brief    Sets anti aliasing flag
    * @param    antiAlias New value of anti aliasing flag
    * @return   @b bool True on success.
    */
    virtual bool setAntiAlias (bool antiAlias) = 0;

    /**
    * @brief    Gets anti aliasing flag
    * @param    antiAlias Reference to anti aliasing indicator
    * @return   @b bool True on success.
    */
    virtual bool getAntiAlias (bool &antiAlias) = 0;

};
#define edlobj2IDOMShadingPatternBrush(src) edl_cast((IDOMShadingPatternBrush *)NULL, src)


/**
* @class    IDOMShadingPatternType1Brush
* @ingroup  IDom
* @brief    IDOMShadingBrush provides a way of representing a PS style type 1 shading pattern. 
*/

class IDOMShadingPatternType1Brush : public IDOMShadingPatternBrush
{
#define IDOMShadingPatternType1BrushClassID 0x599BD13A, 0xB5B74368, 0xA728CDF8, 0x621F8CC0
public:

    /**
    * @class    Data
    * @ingroup      IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data()
        {
            opacity     = 1.0f;
            shadingType = 1;
            antiAlias   = false;
            hasBBox     = false;

            bbox [0]    = 0.0;
            bbox [1]    = 0.0;
            bbox [2]    = 1.0;
            bbox [3]    = 1.0;

            domain [0]  = 0.0;
            domain [1]  = 1.0;
            domain [2]  = 0.0;
            domain [3]  = 1.0;

            function    = NULL;
        }

        /* common to all shading types */
        float                       opacity;
        int                         shadingType;
        float                       bbox [4];
        bool                        hasBBox;
        bool                        antiAlias;
        IDOMColorSpacePtr           colorSpace;
        IDOMColorPtr                background;
        FMatrix                     renderTransform;


        /* type 1 info */
        float                       domain [4];
        FMatrix                     matrix;
        IDOMFunctionPtr             function;
    };

    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id (IDOMShadingPatternType1BrushClassID);
        return id;
    };

    /**
    * @brief    Sets the domain range
    * @param    domain Array of 4 floats
    * @return   @b bool True on success.
    */
    virtual bool setDomain (float *domain) = 0;

    /**
    * @brief    Gets the domain range
    * @param    domain Array of 4 floats
    * @return   @b bool True on success.
    */
    virtual bool getDomain (float *domain) = 0;

    /**
    * @brief    Sets the Type 1 shade matrix
    * @param    matrix Reference to a matrix object
    * @return   @b bool True on success.
    */
    virtual bool setMatrix (FMatrix &matrix) = 0;

    /**
    * @brief    Gets the Type 1 shade matrix
    * @param    matrix Reference to matrix object
    * @return   @b bool True on success.
    */
    virtual bool getMatrix (FMatrix &matrix) = 0;
    /**
    * @brief    Sets the shade function
    * @param    function Pointer to a function object
    * @return   @b bool True on success.
    */
    virtual bool setFunction (IDOMFunctionPtr &function) = 0;

    /**
    * @brief    Gets the shade function object
    * @param    function Reference to a pointer to a function object
    * @return   @b bool True on success.
    */
    virtual bool getFunction (IDOMFunctionPtr &function) = 0;
};
#define edlobj2IDOMShadingPatternType1Brush(src) edl_cast((IDOMShadingPatternType1Brush *)NULL, src)


/**
* @class    IDOMShadingPatternType2Brush
* @ingroup  IDom
* @brief    IDOMShadingBrush provides a way of representing a PS style type 2 shading pattern. 
*/
class IDOMShadingPatternType2Brush : public IDOMShadingPatternBrush
{
#define IDOMShadingPatternType2BrushClassID 0xDDD7C873, 0x2C0045b6, 0xB67E54BE, 0x04AD6190
public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data()
        {
            opacity     = 1.0f;
            shadingType = 2;
            antiAlias   = false;
            hasBBox     = false;

            bbox [0]    = 0.0;
            bbox [1]    = 0.0;
            bbox [2]    = 1.0;
            bbox [3]    = 1.0;

            coords [0]  = 0.0;
            coords [1]  = 0.0;
            coords [2]  = 0.0;
            coords [3]  = 0.0;

            domain [0]  = 0.0;
            domain [1]  = 1.0;

            function    = NULL;

            extend [0]  = false;
            extend [1]  = false;
        }

        /* common to all shading types */
        float                       opacity;
        int                         shadingType;
        float                       bbox [4];
        bool                        hasBBox;
        bool                        antiAlias;
        IDOMColorSpacePtr           colorSpace;
        IDOMColorPtr                background;
        FMatrix                     renderTransform;


        /* type 2 info */
        float                       coords [4];
        float                       domain [2];
        IDOMFunctionPtr             function;
        bool                        extend [2];
    };

    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id (IDOMShadingPatternType2BrushClassID);
        return id;         
    };

    /**
    * @brief    Sets the domain range
    * @param    domain Array of 4 floats
    * @return   @b bool True on success.
    */
    virtual bool setDomain (float *domain) = 0;

    /**
    * @brief    Gets the domain range
    * @param    domain Array of 4 floats
    * @return   @b bool True on success.
    */
    virtual bool getDomain (float *domain) = 0;

    /**
    * @brief    Sets the coords
    * @param    coord An array of 4 floats that represent coordinates
    * @return   @b bool True on success.
    */
    virtual bool setCoords (float *coord) = 0;

    /**
    * @brief    Gets the shade coordinate
    * @param    coord An array of 4 floats that represent coordinates
    * @return   @b bool True on success.
    */
    virtual bool getCoords (float *coord) = 0;

    /**
    * @brief    Sets the shading Extend flag to represent whether or not to 
    *           extend beyond the start and end points for each axis.
    * @param    extendX parameter representing the flags for the x axis
    * @param    extendY parameter representing the flags for the y axis
    * @return   @b bool True on success.
    */
    virtual bool setExtend (bool extendX, bool extendY) = 0;

    /**
    * @brief    Gets the shading Extend flag to represent whether or not to 
    *           extend beyond the start and end points for each axis.
    * @param    extendX Reference to bool parameter representing the flags for the x axis
    * @param    extendY Reference to bool parameter representing the flags for the y axis
    * @return   @b bool True on success.
    */
    virtual bool getExtend (bool &extendX, bool &extendY) = 0;

    /**
    * @brief    Sets the shade function
    * @param    function Pointer to a function object
    * @return   @b bool True on success.
    */
    virtual bool setFunction (IDOMFunctionPtr &function) = 0;

    /**
    * @brief    Gets the shade function object
    * @param    function Reference to a pointer to a function object
    * @return   @b bool True on success.
    */
    virtual bool getFunction (IDOMFunctionPtr &function) = 0;
    
    /**
    * @brief    Gets an equivalent Linear gradient or Visual brush, which may involve
    *           sampling the functions. A Visual brush is required if the shading pattern
    *           has a bounding box, in order to apply a clip. This is intended to be
    *           used to generate a brush that can be expressed directly in XPS.
    * @param    pFactory A pointer to an EDL class factory
    * @param    simple A reference to receive the equivalent gradient
    * @param    maxSamples The maximum number of samples that should be taken from the function to
    *           generate gradient stops.
    * @return   @b bool True on success.
    */
    virtual bool getEquivalentSimpleBrush(IEDLClassFactory *pFactory, IDOMBrushPtr &simple, uint32 maxSamples = 255) = 0;
};

#define edlobj2IDOMShadingPatternType2Brush(src) edl_cast((IDOMShadingPatternType2Brush *)NULL, src)


/**
* @class    IDOMShadingPatternType3Brush
* @ingroup  IDom
* @brief    IDOMShadingPatternType3Brush provides a way of representing a PS style type 2 shading pattern. 
*/

class IDOMShadingPatternType3Brush : public IDOMShadingPatternBrush
{
#define IDOMShadingPatternType3BrushClassID 0x383008FE, 0xB7D64314, 0x9B715287, 0xA57ACBDA
public:

    /**
    * @class    Data
    * @ingroup      IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data()
        {
            opacity     = 1.0f;
            shadingType = 3;
            antiAlias   = false;
            hasBBox     = false;

            bbox [0]    = 0.0;
            bbox [1]    = 0.0;
            bbox [2]    = 1.0;
            bbox [3]    = 1.0;

            coords [0]  = 0.0;
            coords [1]  = 0.0;
            coords [2]  = 0.0;
            coords [3]  = 0.0;
            coords [4]  = 0.0;
            coords [5]  = 0.0;

            domain [0]  = 0.0;
            domain [1]  = 1.0;

            function    = NULL;

            extend [0]  = false;
            extend [1]  = false;
        }

        /* common to all shading types */
        float                       opacity;
        int                         shadingType;
        float                       bbox [4];
        bool                        hasBBox;
        bool                        antiAlias;
        IDOMColorSpacePtr           colorSpace;
        IDOMColorPtr                background;
        FMatrix                     renderTransform;


        /* type 3 info */
        float                       coords [6];
        float                       domain [2];
        IDOMFunctionPtr             function;
        bool                        extend [2];
    };

    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id (IDOMShadingPatternType3BrushClassID);
        return id;         
    };

    /**
    * @brief    Sets the domain range
    * @param    domain Array of 4 floats
    * @return   @b bool True on success.
    */
    virtual bool setDomain (float *domain) = 0;

    /**
    * @brief    Gets the domain range
    * @param    domain Array of 4 floats
    * @return   @b bool True on success.
    */
    virtual bool getDomain (float *domain) = 0;

    /**
    * @brief    Sets the coords
    * @param    coord An array of 6 floats that represent coordinates
    * @return   @b bool True on success.
    */
    virtual bool setCoords (float *coord) = 0;

    /**
    * @brief    Gets the shade coordinate
    * @param    coord An array of 6 floats that represent coordinates
    * @return   @b bool True on success.
    */
    virtual bool getCoords (float *coord) = 0;

    /**
    * @brief    Sets the shading Extend flag to represent whether or not to 
    *           extend beyond the start and end points for each axis.
    * @param    extendX Boolean parameter representing the flags for the x axis
    * @param    extendY Boolean parameter representing the flags for the y axis
    * @return   @b bool True on success.
    */
    virtual bool setExtend (bool extendX, bool extendY) = 0;

    /**
    * @brief    Gets the shading Extend flag to represent whether or not to 
    *           extend beyond the start and end points for each axis.
    * @param    extendX Reference to boolean parameter representing the flags for the x axis
    * @param    extendY Reference to boolean parameter representing the flags for the y axis
    * @return   @b bool True on success.
    */
    virtual bool getExtend (bool &extendX, bool &extendY) = 0;

    /**
    * @brief    Sets the shade function
    * @param    function Pointer to a function object
    * @return   @b bool True on success.
    */
    virtual bool setFunction (IDOMFunctionPtr &function) = 0;

    /**
    * @brief    Gets the shade function object
    * @param    function Reference to a pointer to a function object
    * @return   @b bool True on success.
    */
    virtual bool getFunction (IDOMFunctionPtr &function) = 0;

    /**
    * @brief    Gets an equivalent Radial gradient or Visual brush, which may involve
    *           sampling the functions. A Visual brush is required if the shading pattern
    *           has a bounding box, in order to apply a clip. This is intended to be
    *           used to generate a brush that can be expressed directly in XPS.
    *           This is not possible for all Type 3 shading brushes.
    * @param    pFactory A pointer to an EDL class factory
    * @param    simple A reference to receive the equivalent gradient
    * @param    maxSamples The maximum number of samples that should be taken from the function to
    *           generate gradient stops.
    * @return   @b bool True on success.
    */
    virtual bool getEquivalentSimpleBrush(IEDLClassFactory *pFactory, IDOMBrushPtr &simple, uint32 maxSamples = 255) = 0;
};

#define edlobj2IDOMShadingPatternType3Brush(src) edl_cast((IDOMShadingPatternType3Brush *)NULL, src)


/**
* @class    IDOMShadingPatternType4567Brush
* @ingroup  IDom
* @brief    IDOMShadingPatternType4567Brush provides a way of representing a PS style type 4 shading pattern. 
*/

class IDOMShadingPatternType4567Brush : public IDOMShadingPatternBrush
{
#define IDOMShadingPatternType4567BrushClassID 0x7204F554, 0x95984d30, 0xA090074B, 0xC03337EE
public:

    /**
    * @class        Data
    * @ingroup      IDom
    * @brief        Initialization data
    */
    class Data : public CClassParams
    {
    public:
            Data()
            {
                opacity     = 1.0f;
                shadingType = 4;
                antiAlias   = false;
                hasBBox     = false;

                bbox [0]    = 0.0;
                bbox [1]    = 0.0;
                bbox [2]    = 1.0;
                bbox [3]    = 1.0;

                bitsPerCoordinate   = 0;
                bitsPerFlag         = 0;
                bitsPerComponent    = 0;
                verticesPerRow      = 0;
                decode              = NULL;
                decodeSize          = 0;
                function            = NULL;
                dataSource          = NULL;
            }

            virtual ~Data()
            {
                if (decode)
                    delete [] decode;
            }

            /* common to all shading types */
            float                       opacity;
            int                         shadingType;
            float                       bbox [4];
            bool                        hasBBox;
            bool                        antiAlias;
            IDOMColorSpacePtr           colorSpace;
            IDOMColorPtr                background;
            FMatrix                     renderTransform;

            /* type 4, 5, 6, 7 info */
            int                         bitsPerCoordinate;
            int                         bitsPerFlag;
            int                         bitsPerComponent;
            int                         verticesPerRow;
            IEDLObjectPtr               dataSource;
            float                       *decode;
            int                         decodeSize;
            IDOMFunctionPtr             function;
    };

    /**
    * @brief        Retrieves class id of IDOM
    * @return       @b CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id (IDOMShadingPatternType4567BrushClassID);
        return id;         
    };

    /**
     * @brief        Sets the shading type
     * @param        shadingType An integer which represents the PS style shading type (one of 4, 5, 6 or 7)
     * @return       @b bool True on success.
     */
    virtual void setShadingType (int shadingType) = 0;

    /**
     * @brief        Sets the data source property
     * @param        dataSource A reference to data source object
     * @return       @b bool True on success.
     */
    virtual bool setDataSource (IEDLObjectPtr &dataSource) = 0;

    /**
     * @brief        Gets the data source property
     * @param        dataSource A reference to a smart pointer which receives a data source object
     * @return       @b bool True on success.
     */
    virtual bool getDataSource (IEDLObjectPtr &dataSource) = 0;

    /**
     * @brief        Sets the bits per coordinate 
     * @param        bitsPerCoordinate Bits per coordinate value to set  
     * @return       @b bool True on success.
     */
    virtual bool setBitsPerCoordinate (int bitsPerCoordinate) = 0;

    /**
     * @brief        Gets the bits per coordinate parameter 
     * @param        bitsPerCoordinate A reference to an int which receives the bits per coordinate value 
     * @return       @b bool True on success.
     */
    virtual bool getBitsPerCoordinate (int &bitsPerCoordinate) = 0;

    /**
     * @brief        Sets the bits per component
     * @param        bitsPerComponent Bits per component value to set 
     * @return       @b bool True on success.
     */
    virtual bool setBitsPerComponent (int bitsPerComponent) = 0;

    /**
     * @brief        Gets the bits per component parameter 
     * @param        bitsPerComponent A reference to an int which receives the bits per component value 
     * @return       @b bool True on success.
     */
    virtual bool getBitsPerComponent (int &bitsPerComponent) = 0;

    /**
     * @brief        Sets the bits per flag 
     * @param        bitsPerFlag Bits per flag value to set  
     * @return       @b bool True on success.
     */
    virtual bool setBitsPerFlag (int bitsPerFlag) = 0;

    /**
     * @brief        Gets the bits per flag parameter 
     * @param        bitsPerFlag A reference to an int which receives the bits per flag value 
     * @return       @b bool True on success.
     */
    virtual bool getBitsPerFlag (int &bitsPerFlag) = 0;

    /**
    * @brief        Sets the vertices per row flag
    * @param        verticesPerRow Vertices per row value to set 
    * @return       @b bool True on success.
    */
    virtual bool setVerticesPerRow (int verticesPerRow) = 0;

    /**
    * @brief        Gets the vertices per row parameter 
    * @param        verticesPerRow A reference to an int which receives the vertices per row value 
    * @return       @b bool True on success.
    */
    virtual bool getVerticesPerRow (int &verticesPerRow) = 0;

    /**
    * @brief        Sets the decode array
    * @param        decodeSize number of floats in the array
    * @param        decodeArray array of floats
    * @return       @b bool True on success.
    */
    virtual bool setDecode (int decodeSize, float *decodeArray) = 0;

    /**
    * @brief        Gets the size of the decode array
    * @param        decodeSize A reference to an int which receives number of floats in the array
    * @return       @b bool True on success.
    */
    virtual bool getDecodeSize (int &decodeSize) = 0;

    /**
    * @brief        Gets the decode array
    * @param        decodeArray An array of floats; it is assumed that the array is big enough.
    * @return       @b bool True on success.
    */
    virtual bool getDecode (float *decodeArray) = 0;

    /**
    * @brief        Sets the shade function
    * @param        function Smart pointer to a function object
    * @return       @b bool True on success.
    */
    virtual bool setFunction (IDOMFunctionPtr &function) = 0;

    /**
    * @brief        Gets the shade function object
    * @param        function A reference to a smart pointer which receives the function object
    * @return       @b bool True on success.
    */
    virtual bool getFunction (IDOMFunctionPtr &function) = 0;
};
#define edlobj2IDOMShadingPatternType4567Brush(src) edl_cast((IDOMShadingPatternType4567Brush *)NULL, src)

/**
* @class    IDOMNullBrush
* @ingroup  IDom
* @brief    IDOMNullBrush provides a way of representing the default marking brush in a Type3 postscript
*           glyph definition or a tiling pattern with paintType 2. This is more of a placeholder that gets
*           replaced when the Type3 glyph or paintType 2 tiling pattern is actually invoked.
*/

class IDOMNullBrush : public IDOMBrush
{
#define IDOMNullBrushClassID 0x98F3706B, 0x403E4a41, 0xA975A9B6, 0xD52C99CC
public:

    /**
    * @brief  Simplified creator for a Null brush.
    *         Throws an IEDLError on failure.
    * @param  pFactory The factory to use.
    * @return @b IDOMNullBrushPtr The new brush.
    */
    static EDL_API IDOMNullBrushPtr create(IEDLClassFactory *pFactory);

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0)
        {}
        float opacity;
    };

    /**
    * @brief    Retrieves class id of IDOMNullBrush
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMNullBrushClassID);
        return id;
    };
};
#define edlobj2IDOMNullBrush(src) edl_cast((IDOMNullBrush *)NULL, src)

_END_EDL_DOM_NAMESPACE

#endif /* __EDLIDOMBRUSH_H__ */

