/*
* Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomcanvas.h
* @ingroup  IDom
* @brief    IDOMCanvas interface 
*
* Interface to DOM node representing \<Canvas\> element
*/

#ifndef __EDLIDOMCANVAS_H__
#define __EDLIDOMCANVAS_H__

#include <edl/idomgroup.h>
#include <edl/idomid.h>
#include <edl/idomresources.h>
#include <edl/edlstring.h>
#include <edl/idomtarget.h>
#include <edl/idomedgemode.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMCanvas                                                                          
* @ingroup  IDom
* @brief    A canvas is a special form of an isolated, non-knockout, normal blended
*           transparency group.
*
* A Canvas groups other elements of a page together. For example, Glyphs and Paths can be 
* grouped in a Canvas in order to be identified as a unit or to
* apply a composed property value to each child and its ancestor.
*
* Some properties of the Canvas element, including the coordinate space of the canvas,
* are composable and affect the rendering of child elements.
*
* The RenderOptionsEdgeMode property can be set in the Canvas node to instruct anti-aliasing consumers
* to render the contents of the Canvas node and all child and descendant nodes without
* performing anti-aliasing.
*/
class IDOMCanvas : public IDOMTransparencyGroup
{
#define IDOMCanvasClassID 0x91974df3, 0x24764464, 0x9cb40b30, 0xb0d2f71f
public:

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0), edgeMode(eEMDefault)
        {}
        float opacity;
        EDLString language;
        eEdgeMode edgeMode;
        EDLString automationPropertiesName;
        EDLString automationPropertiesHelpText;
        FMatrix renderTransform;
        IDOMTargetPtr ptrNavigateLink;
        IDOMBrushPtr ptrOpacityMask;
        IDOMPathGeometryPtr ptrClip;
        IDOMResourceDictionaryPtr ptrResources;
    };

    /**
    * @brief    Retrieves the class id of IDOMCanvas
    * @return   CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMCanvasClassID);
        return id;
    };

    /**
    * @brief    Retrieves the default language of the \<Canvas\> element and any of its children.
    *
    * English is defined as en_GB and American English as en_US. There is no default setting. 
    * If the language is not known it is set to und (undetermined). For further information see
    * http://www.w3.org/International/articles/language-tags/.
    *
    * The language is specified according to RFC 3066.
    *
    * @param    lang The language setting of the \<Canvas\> element
    * @return   bool. Returns true on success, false on failure.
    */
    virtual bool getLanguage (EDLString &lang) const = 0;

    /**
    * @brief    Sets the default language of the \<Canvas\> element and any of its children
    * @param    lang The new language setting for the canvas.
    *
    * English is defined as en_GB and American English as en_US. There is no default setting. 
    * If the language is not known it is set to und (undetermined). For further information see
    * http://www.w3.org/International/articles/language-tags/.
    *
    * The language is specified according to RFC 3066.
    * @return   bool. Returns true on success, false on failure.
    */
    virtual bool setLanguage (const EDLString &lang) = 0;

    /**
    * @brief    Retrieves the automation properties name of the \<Canvas\> element
    * 
    * The automation properties name is a brief description of the \<Canvas\> content
    * for accessibility purposes, particularly if the canvas is filled with a set of 
    * vector graphics and text elements intended to comprise a single vector graphic.
    *
    * @param    propname The automation properties name.
    * @return   bool. Returns true on success, false on failure.
    */
    virtual bool getAutomationPropertiesName (EDLString &propname) const = 0;

    /**
    * @brief    Sets the automation properties name of the \<Canvas\> element.
    *
    * The automation properties name is a brief description of the \<Canvas\> content
    * for accessibility purposes, particularly if the canvas is filled with a set of 
    * vector graphics and text elements intended to comprise a single vector graphic.
    *
    * @param    propname The new automation properties name.
    * @return   bool. Returns true on success, false on failure.
    */
    virtual bool setAutomationPropertiesName (const EDLString &propname) = 0;

    /**
    * @brief    Retrieves the automation properties help text of the \<Canvas\> element
    * 
    * The automation properties help text of a canvas is a more detailed description of 
    * the canvas content for accessibility purposes.  This is particularly useful if 
    * the canvas is filled with a set of vector graphics and text elements intended to 
    * comprise a single vector graphic. For example, "This is a picture of the Earth."
    * @param    helptext The automation properties help text.
    * @return   bool. Returns true on success, false on failure.
    */
    virtual bool getAutomationPropertiesHelpText (EDLString &helptext) const = 0;

    /**
    * @brief    Sets automation properties help text of the \<Canvas\> element
    * 
    * The automation properties help text of a canvas is a more detailed description of 
    * the canvas content for accessibility purposes.  This is particularly useful if 
    * the canvas is filled with a set of vector graphics and text elements intended to 
    * comprise a single vector graphic. For example, "This is a picture of the Earth."
    * @param    helptext The new automation properties help text.
    * @return   bool. Returns true on success, false on failure.
    */
    virtual bool setAutomationPropertiesHelpText (const EDLString &helptext) = 0;

    /**
    * @brief    Retrieves render options edge mode of the \<Canvas\> element
    * 
    * Render options edge mode controls how edges of paths within the canvas are rendered.
    * The only valid value for render options edge mode is Aliased. 
    * Omitting this attribute causes the edges to be rendered in the consumer's default manner
    *
    * @return   eEdgeMode. The function returns the edge mode of the canvas.
    */
    virtual eEdgeMode getEdgeMode () const = 0;

    /**
    * @brief    Sets render the options edge mode of the \<Canvas\> element.
    * 
    * The EdgeMode property controls how the edges of paths within the canvas are rendered.
    * The only valid value is Aliased. Omitting this attribute causes the edges to be rendered in the 
    * consumers default manner. The EdgeMode property can be set in a Canvas to 
    * instruct anti-aliasing consumers to render the contents of the Canvas and all child and descendant
    * nodes without performing antialiasing.
    *
    * @param    em The new edge mode for the canvas.
    * @return   bool. Returns true on success, false on failure.
    */
    virtual void setEdgeMode (eEdgeMode em) = 0;

    /**
    * @brief    Retrieves the target of a hyperlink.
    * @param    target Smart pointer to receive the target of the hyperlink
    * @return   bool. Returns true on success, false on failure.
    */
    virtual bool getNavigateLink (IDOMTargetPtr &target) const = 0;

    /**
    * @brief    Sets the target of a hyperlink.
    * @param    target The new target for the hyperlink.
    * @return   bool. Returns true on success, false on failure.
    */
    virtual bool setNavigateLink (const IDOMTargetPtr &target) = 0;

    /**
    * @brief    Retrieves a smart pointer to the resource dictionary.
    *
    * The resource dictionary contains the IDs of resources stored at this level of the document.
    * @param    ptrResourceDictionary A smart pointer to receive the reference to the resource dictionary.
    * @return   bool. Returns true on success, false on failure.
    */
    virtual bool getResourceDictionary(IDOMResourceDictionaryPtr &ptrResourceDictionary) const = 0;

    /**
    * @brief    Sets the resource dictionary.
    *
    * The resource dictionary contains the IDs of resources stored at this level of the document.
    * @param    ptrResourceDictionary Smart pointer to the resource dictionary
    * @return   bool. Returns true on success, false on failure.
    */
    virtual bool setResourceDictionary(const IDOMResourceDictionaryPtr &ptrResourceDictionary) = 0;

};

DECL_SMART_PTR(IDOMCanvas);
#define edlobj2IDOMCanvas(src) edl_cast((IDOMCanvas *)NULL, src) 


_END_EDL_DOM_NAMESPACE

#endif /* __EDLIDOMCANVAS_H__ */
