/*
* Copyright (C) 2008 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomcharpathgroup.h
* @ingroup  IDom
* @brief    IDOMCharPathGroup interface 
*
* Interface to DOM node representing Group Elements that consist of stroked text
*/


#ifndef __EDLIDOMCHARPATHGROUP_H__
#define __EDLIDOMCHARPATHGROUP_H__

#include <edl/idomnode.h>
#include <edl/edlnamespaces.h>
#include <edl/idomgroup.h>
#include <edl/edlblend.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMCharPathGroup
* @ingroup  IDom
* @brief    IDOMCharPathGroup interface
*/
class IDOMCharPathGroup : public IDOMGroup
{
#define IDOMCharPathGroupClassID 0x29D7FB4A, 0xFBBF4ac6, 0xAF6A3EA9, 0x4448EA48

public:

    virtual ~IDOMCharPathGroup() {}

    typedef enum
    {
        eCharPath_Stroke    = 0,
        eCharPath_Clip      = 1,
        eCharPath_Fill      = 2
    } eCharPathType;

    /**
    * @brief    Retrieves class id of IDOMGroup
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMCharPathGroupClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        FMatrix             renderTransform;
        IDOMPathGeometryPtr ptrClip;
        IDOMPathNodePtr     strokePath;
        eCharPathType       charPathType;
        IDOMGroupPtr        clippedGroup;
        IDOMBrushPtr        brush;
        eBlendMode          blendMode;

        Data ()
        {
            charPathType = eCharPath_Stroke;
            blendMode = eBlendModeNormal;
        }
    };

    /**
    * @brief    Retrieves the type of charpath rendering needed which can be Stroke, Clip or Fill
    * @return   eCharPathType Returns the type of charpath rendering 
    */
    virtual eCharPathType getCharPathType () = 0;

    /**
    * @brief    Sets the type of charpath rendering needed which can be Stroke, Clip or Fill
    * @param    charPathType The type of charpath
    * @return   bool Returns true on success 
    */
    virtual bool setCharPathType (eCharPathType charPathType) = 0;

    /**
    * @brief    Gets the equivalent path that the charpath makes. 
    * @param    strokePath Smart pointer to stroke path of the charpath
    * @return   bool Returns true on success 
    */
    virtual bool getStrokePath (IDOMPathNodePtr &strokePath) = 0;

    /**
    * @brief    Sets the equivalent path that the charpath makes. 
    * @param    strokePath Smart pointer to stroke path of the charpath
    * @return   bool Returns true on success 
    */
    virtual bool setStrokePath (IDOMPathNodePtr &strokePath) = 0;

    /**
    * @brief    Gets the objects that need to be drawn that must be clipped by the charpath.
    *           Only used when the charpath type is Clip 
    * @param    clippedGroup Smart pointer to group of objects that should be clipped
    * @return   bool Returns true on success 
    */
    virtual bool getClippedGroup (IDOMGroupPtr &clippedGroup) = 0;

    /**
    * @brief    Sets the objects that need to be drawn that must be clipped by the charpath 
    *           Only used when the charpath type is Clip 
    * @param    clippedGroup Smart pointer to group of objects that should be clipped
    * @return   bool Returns true on success 
    */
    virtual bool setClippedGroup (IDOMGroupPtr &clippedGroup) = 0;

    /**
    * @brief    Gets the brush that is used to fill the charpath when the charpath type is Fill
    * @param    brush Smart pointer to the brush used to fill the charpath
    * @return   bool Returns true on success 
    */
    virtual bool getFill (IDOMBrushPtr &brush) = 0;

    /**
    * @brief    Sets the brush that is used to fill the charpath when the charpath type is Fill
    * @param    brush Smart pointer to the brush used to fill the charpath
    * @return   bool Returns true on success 
    */
    virtual bool setFill (IDOMBrushPtr &brush) = 0;

    /**
    * @brief    Get the blend mode to be used for rendering this char path group.
    * @return   blendMode. The blend mode.
    */
    virtual eBlendMode getBlendMode() const = 0;

    /**
    * @brief    Set the blend mode to be used for rendering this char path group.
    * @param    blendMode The desired blend mode.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setBlendMode(eBlendMode blendMode) = 0;
};
DECL_SMART_PTR(IDOMCharPathGroup);
#define edlobj2IDOMCharPathGroup(src) edl_cast((IDOMCharPathGroup *)NULL, src) 


_END_EDL_DOM_NAMESPACE


#endif /* __EDLIDOMCHARPATHGROUP_H__ */


