/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file    idomfont.h
* @ingroup IDom
* @brief   IDOMFont interface
*/

#ifndef __IDOMFONT_H__
#define __IDOMFONT_H__

#include <edl/edltypes.h>
#include <edl/edlstream.h>
#include <edl/idomnode.h>
#include <edl/idomglyph.h>
#include <edl/isession.h>
#include <edl/edlvector.h>
#include <edl/idomhashable.h>


_BEGIN_EDL_DOM_NAMESPACE


/**
* @class    IDOMFontSource
* @ingroup  IDom
* @brief    The font source for the class IDOMFont.
* This class describes the different ways fonts are constructed.
*/

class IDOMFontSource : public IDOMHashable, public IEDLObject
{
#define IDOMFontSourceClassID 0x2eed7bfe, 0x821011dc, 0x83140800, 0x200c9a66
public:
    /**
    * @enum     eFontSourceType
    * @ingroup  IDom
    * @brief    type used to uniquely identify the source type of a font
    */
    typedef enum
    {
        eFontSourceTypeNone,            //!< Not specified
        eFontSourceTypeStreamFilter,    //!< Font filter (converter)
        eFontSourceTypeStream,          //!< Font data stream
        eFontSourceTypeFont,            //!< Derived from a font

    } eFontSourceType;

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:

        eFontSourceType sourceType;
    };

    /**
    * @brief    Retrieves the class id of IDOMFontSource.
    * @return   @b CClassID The class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFontSourceClassID);
        return id;
    };

    /**
    * @brief    Gets the font source type.
    * @return   @b eFontSourceType The font source type.
    */
    virtual eFontSourceType getFontSourceType() const = 0;

    /**
    * @brief    Determines the URI based on the font source (underlying font sources may be searched)
    * @return   @b EDLSysString The uri, which will have a size of zero on failure.
    */
    virtual const EDLSysString &determineUri() const = 0;

   /**
    *  Note:   This is an alternate version of determineUri() given above.
    *               This version has been kept in source for backward compatibility. 
    *
    * @brief    Determines the URI based on the font source (backwards compatible version)
    * @param    uri Reference parameter to receive the determined URI.
    * @return   @b bool True on success, or false if the call fails.
    */
    virtual bool determineUri(EDLSysString &uri) const
    {
        uri = determineUri();
        
        return (uri.size() > 0);
    }
};
DECL_SMART_PTR(IDOMFontSource);
#define edlobj2IDOMFontSource(src) edl_cast((IDOMFontSource *)NULL, src) 



/**
* @class    IDOMFontSourceStreamFilter
* @ingroup  IDom
* @brief    An abstract interface for fonts sourced from a font stream filter
*
*/
class 
IDOMFontSourceStreamFilter : public IDOMFontSource
{
public:

    /**
    * @enum     eFontStreamFilterType
    * @ingroup  IDom
    * @brief    An enumeration type used to identify the source type of a font.
    */
    typedef enum
    {
        eFontStreamFilterTypeNone,            //!< Not specified
        eFontStreamFilterTypeObfuscation,    //!< Obfuscation filter

    } eFontStreamFilterType;

    /**
    * @brief    Get the font stream filter type
    * @return   @b eFontStreamFilterType The font stream filter type
    */
    virtual eFontStreamFilterType getFontStreamFilterType() const = 0;

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMFontSource::Data
    {
    public:
        IDOMFontSourcePtr inputFontSource;        //The input font source
    };

    /**
    * @brief    Retrieves the output stream of the processor.
    * @param    stream Reference parameter to receive a smart pointer to the output stream.
    * @return   @b bool True on success, false if the function fails.
    */
    virtual bool getStream(IInputStreamPtr &stream) const = 0;


     /**
     * @brief    Determines the URI based on the font source (underlying font sources may be searched)
     * @return   @b EDLSysString The uri, which will have a size of zero on failure.
     */
     virtual const EDLSysString &determineUri() const = 0;


   /**
    *  Note:   This is an alternate version of determineUri() given above.
    *               This version has been kept in source for backward compatibility. 
    *
    * @brief    Determines the URI based on the font source (backwards compatible version)
    * @param    uri Reference parameter to receive the determined URI.
    * @return   @b bool True on success, or false if the call fails.
    */
    virtual bool determineUri(EDLSysString &uri) const
    {
        uri = determineUri();
        
        return (uri.size() > 0);
    }

    /**
    * @brief    Get the input font source of this font stream processor
    * @param    fontSource The font Source to get retrieved
    * @return   @b bool True on success
    */
    virtual bool getInputFontSource(IDOMFontSourcePtr &fontSource) const = 0;

};
DECL_SMART_PTR(IDOMFontSourceStreamFilter);
#define edlobj2IDOMFontSourceStreamFilter(src) edl_cast((IDOMFontSourceStreamFilter *)NULL, src) 


/**
* @class    IDOMFontSourceFromStream
* @ingroup  IDom
* @brief    The source for IDOMFont when sourced from an existing stream.
*/
class 
IDOMFontSourceFromStream : public IDOMFontSource
{
#define IDOMFontSourceFromStreamClassID 0x6d2e3ca8, 0x821511dc, 0x83140800, 0x200c9a66
public:
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMFontSourceStreamFilter::Data
    {
    public:
        Data() : streamLength(0)
        {}
        EDLSysString uri;
        IInputStreamPtr stream;
        uint64 streamLength;
    };

    /**
    * @brief    Retrieves the class id of IDOMFontSourceFromStream.
    * @return   @b CClassID Returns the class id of the element.
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFontSourceFromStreamClassID);
        return id;
    };

    /**
    * @brief    Return the actual stream
    * @param    stream Reference parameter to receive a smart pointer to the actual stream.
    * @return   @b bool True on success, or false if the call fails.
    */
    virtual bool getStream(IInputStreamPtr &stream) const = 0;

    /**
    * @brief    Retrieves the stream length, if possible.
    * @param    length Reference parameter to receive the stream length.
    * @return   @b bool True on success, or false if the call fails.
    */
    virtual bool getStreamLength(uint64 &length) const = 0;

     /**
     * @brief    Determines the URI based on the font source (underlying font sources may be searched)
     * @return   EDLSysString. Returns the uri, which will have a size of zero on failure.
     */
     virtual const EDLSysString &determineUri() const = 0;


   /**
    *  Note:   This is an alternate version of determineUri() given above.
    *               This version has been kept in source for backward compatibility. 
    *
    * @brief    Determines the URI based on the font source (backwards compatible version)
    * @param    uri Reference parameter to receive the determined URI.
    * @return   @b bool True on success, or false if the call fails.
    */
    virtual bool determineUri(EDLSysString &uri) const
    {
        uri = determineUri();
        
        return (uri.size() > 0);
    }
};
DECL_SMART_PTR(IDOMFontSourceFromStream);
#define edlobj2IDOMFontSourceFromStream(src) edl_cast((IDOMFontSourceFromStream *)NULL, src) 


/**
* @class    IDOMFontSourceObfuscationConverter
* @ingroup  IDom
* @brief    Interface for a font sourced from a converter that performs obfuscation and deobfuscation.
*
*/
class 
IDOMFontSourceObfuscationConverter : public IDOMFontSourceStreamFilter
{
#define IDOMFontSourceObfuscationConverterClassID 0x51e24e4a, 0x831a11dc, 0x83140800, 0x200c9a66
public:

    /**
    * @enum     eOperation
    * @ingroup  IDom
    * @brief    type used to uniquely identify the conversion direction
    */
    typedef enum
    {
        eObfuscate,        //!< Obfuscation
        eDeobfuscate,    //!< Deobfuscation

    } eOperation;

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMFontSource::Data
    {
    public:
        IDOMFontSourcePtr inputFontSource;        //!< The input font source for the converter
        eOperation operation;                    //!< The obfuscation or deobfuscation operation
    };

    /**
    * @brief    Retrieves the class id of IDOMFontSourceObfuscationConverter
    * @return   @b CClassID The class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFontSourceObfuscationConverterClassID);
        return id;
    };

    /**
    * @brief    Get the output stream of the processor.
    * @param    stream A reference parameter to receive a smart pointer to the output stream.
    * @return   @b bool True on success, or false if the call fails.
    */
    virtual bool getStream(IInputStreamPtr &stream) const = 0;

     /**
     * @brief    Determines the URI based on the font source (underlying font sources may be searched)
     * @return   EDLSysString. Returns the uri, which will have a size of zero on failure.
     */
     virtual const EDLSysString &determineUri() const = 0;

   /**
    *  Note:   This is an alternate version of determineUri() given above.
    *               This version has been kept in source for backward compatibility. 
    *
    * @brief    Determines the URI based on the font source (backwards compatible version)
    * @param    uri Reference parameter to receive the determined URI.
    * @return   @b bool True on success, or false if the call fails.
    */
    virtual bool determineUri(EDLSysString &uri) const
    {
        uri = determineUri();
        
        return (uri.size() > 0);
    }

    /**
    * @brief    Get the underlying font source of this font stream processor.
    * @param    fontSource A reference parameter to receive a smart pointer to the font source.
    * @return   @b bool True on success, or false if the call fails.
    */
    virtual bool getInputFontSource(IDOMFontSourcePtr &fontSource) const = 0;

};
DECL_SMART_PTR(IDOMFontSourceObfuscationConverter);
#define edlobj2IDOMFontSourceObfuscationConverter(src) edl_cast((IDOMFontSourceFromStream *)NULL, src) 


/**
* @class    IDOMFont
* @ingroup  IDom
* @brief    IDOMFont Base Class
*
* Requesting the stream from this resource will give you
* a useable font file related to the font type.
*/
class 
IDOMFont : public IDOMFontSource
{
#define IDOMFontClassID 0x4330123e, 0x7e6211dc, 0x83140800, 0x200c9a66
public:

    /**
    * @typedef  CharCode
    * @ingroup  IDom
    * @brief    type used to uniquely identify a character code
    */
    typedef uint32    CharCode;

    /**
    * @typedef  eFontType
    * @ingroup  IDom
    * @brief    type used to uniquely identify the type of font
    */

    typedef enum
    {
        eFontTypeUnknown,
        eFontTypeOpenType,
        eFontType3,
        eFontTypePCLXL,
        eFontTypePCL5
    } eFontType;

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        IDOMFontSourcePtr fontSource;
    };

    /**
    * @brief    Retrieves class id of IDOMFont
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFontClassID);
        return id;
    };

    /**
    * @brief    Gets the font type.
    * See eFontType for more information on font types.
    * @return   eFontType. Returns the font type.
    */
    virtual eFontType getFontType() const = 0;

    /**
    * @brief    Get the font source of this font.
    * @param    fontSource Reference parameter to be populated with the retrieved font source.
    * @return   @b bool Function True on success, false if the call fails.
    */
    virtual bool getFontSource(IDOMFontSourcePtr &fontSource) const = 0;

    /**
    * @brief    Sets the font source for this font.
    * @param    fontSource The new font source
    * @return   @b bool Function True on success, false if the call fails.
    */
    virtual bool setFontSource(IDOMFontSourcePtr &fontSource) = 0;

    /**
    * @brief    Return the base stream for this font, obtaining it from the font source.
    * @param    stream Reference parameter to be populated with the retrieved font base stream.
    * @return   @b bool Function True on success, false if the call fails.
    */
    virtual bool getFontBaseStream(IInputStreamPtr &stream) const = 0;

};
DECL_SMART_PTR(IDOMFont);
#define edlobj2IDOMFont(src) edl_cast((IDOMFont *)NULL, src) 


/**
* @typedef TableSignatureID
* @ingroup IDOM
* @brief type used to uniquely identify an opentype font signature
*/
typedef uint32    TableSignatureID;


class IFontOpenTypeTableSignatureEnumerator  :  public IEDLObject
{
#define IFontOpenTypeTableEnumeratorClassID    0x2c0da580, 0xdb0a11dc, 0x95ff0800, 0x200c9a66
public:

    /**
    * @brief    Retrieves class id of IFontOpenTypeTableEnumerator
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IFontOpenTypeTableEnumeratorClassID);
        return id;
    };

    /**
    * @brief     Get the table signature ID of the current item
    * @return    TableSignatureID The table font signature ID
    */
    virtual TableSignatureID getSignatureID() = 0;

    /**
    * @brief    Indicate whether there are more items in the enumeration.
    * @return   @b bool Whether there are more items in the enumeration.
    */
    virtual bool haveMoreEnumerationItems() = 0;

    /**
    * @brief    Get the next enumeration item
    * @return   @b bool True on success
    */
    virtual bool nextEnumerationItem() = 0;
};


class IFontOpenTypeTableAccessor  :  public IEDLObject
{
#define IFontOpenTypeTableAccessorClassID    0x06e3b890, 0xda2311dc, 0x95ff0800, 0x200c9a66
public:

    /**
    * @brief    Retrieves class id of IFontOpenTypeTableAccessor
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IFontOpenTypeTableAccessorClassID);
        return id;
    };

    /**
    * @brief    Returns an indicator of the existance of a font table with the given signature.
    * @param    tableSignatureId The ID of the opentype table 
    * @return   @b bool Indicator of table existance
    */
    virtual bool hasTable(TableSignatureID tableSignatureID) = 0;

    /**
    * @brief    Returns the size of the table given by the signature
    * @param    tableSignatureId The ID of the opentype table
    * @param    withPadding Indicate whether the size includes 32 bit padding.    
    * @return   int32 The table size
    */
    virtual int32 getTableSize(TableSignatureID tableSignatureID, bool withPadding = true) = 0;

    /**
    * @brief    Returns the check sum of the table given by the signature
    * @param    tableSignatureId The ID of the opentype table
    * @return   uint32 The table data checksum
    */
    virtual uint32 getTableCheckSum(TableSignatureID tableSignatureID) = 0;

    /**
    * @brief    Writes the table to the given output stream
    * @ param   outStream The output stream
    * @param    tableSignatureId The ID of the opentype table
    * @param    withPadding Indicate whether the size includes 32 bit padding.    
    * @return   int32 Byte count of the data written.
    */
    virtual int32 tableWrite(IOutputStreamPtr &outStream, TableSignatureID tableSignatureID, int32 startPos = 0, int32 endPos = -1, bool withPadding = true) = 0;

    /**
    * @brief    Copies the table to the given buffer
    * @param    buf The destination buffer
    * @param    bufSize Size of the destination buffer
    * @param    tableSignatureId The ID of the opentype table
    * @return   int32 Byte size of the table
    */
    virtual int32 tableCopy(char *buf, int32 bufSize, TableSignatureID tableSignatureId, bool withPadding = true) = 0;
};

DECL_SMART_PTR(IFontOpenTypeTableAccessor);
#define edlobj2IFontOpenTypeTableAccessor(src) edl_cast((IFontOpenTypeTableAccessor *)NULL, src) 


class IFontTrueTypeGlyphIDEnumerator  :  public IEDLObject
{
#define IFontTrueTypeGlyphIDEnumeratorClassID   0xa8c1c370, 0xdb1111dc, 0x95ff0800, 0x200c9a66
public:

    /**
    * @brief    Retrieves class id of IFontOpenTypeTableEnumerator
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IFontTrueTypeGlyphIDEnumeratorClassID);
        return id;
    };

    /**
    * @brief    Get the glyph ID of the current item
    * @return   IDOMGlyph::GlyphID The glyph ID
    */
    virtual IDOMGlyph::GlyphID getGlyphID() = 0;

    /**
    * @brief    Indicate whether there are more items in the enumeration.
    * @return   @b bool Whether there are more items in the enumeration.
    */
    virtual bool haveMoreEnumerationItems() = 0;

    /**
    * @brief    Get the next enumeration item
    * @return   @b bool True on success
    */
    virtual bool nextEnumerationItem() = 0;
};


class IFontTrueTypeGlyphAccessor :  public IEDLObject
{
#define IFontTrueTypeGlyphAccessorClassID 0x114cab20, 0xda2311dc, 0x95ff0800, 0x200c9a66
public:

    /**
    * @brief    Retrieves class id of IFontTrueTypeGlyphAccessor
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IFontTrueTypeGlyphAccessorClassID);
        return id;
    };

    /**
    * @brief    Indicate existance of glyph ID in the font
    * @param    glyphID The glyph ID
    * @return   @b bool Inidicator of existence
    */
    virtual bool hasGlyph(IDOMGlyph::GlyphID glyphID) = 0;

    /**
    * @brief    Gets the size of the glyph data
    * @param    glyphID The glyph ID
    * @return   int32 The size of the glyph data
    */
    virtual int32 getGlyphDataSize(IDOMGlyph::GlyphID glyphID) = 0;

    /**
    * @brief    Writes the glyph associated with the glyphID to the given output stream
    * @ param   outStream The output stream
    * @param    glyphID The glyph ID
    * @return   @b bool True on success
    */
    virtual int32 glyphDataWrite(IOutputStreamPtr &outStream, IDOMGlyph::GlyphID glyphID) = 0;

    /**
    * @brief    Gets a glyphID enumerator which lists the dependant glyph IDs of the given glyphID
    * @param    glyphID The given glyph ID
    * @param    ienumerator The returned enumerator of dependant glyph IDs
    * @return   @b bool True on success
    */
    virtual bool getDependantGlyphIDEnumerator(IDOMGlyph::GlyphID glyphID, IDOMGlyphIDEnumeratorPtr &ienumerator) = 0;
};

DECL_SMART_PTR(IFontTrueTypeGlyphAccessor);
#define edlobj2IFontTrueTypeGlyphAccessor(src) edl_cast((IFontTrueTypeGlyphAccessor *)NULL, src) 

class IDOMFontOpenType;
DECL_SMART_PTR(IDOMFontOpenType);

class IDOMFontOpenTypeTT ;
DECL_SMART_PTR(IDOMFontOpenTypeTT);

/**
* @class IDOMFontOpenType
* @ingroup IDom
* @brief IDOMFontOpenType interface
*/
class IDOMFontOpenType : public IDOMFont
{
#define IDOMFontOpenTypeClassID 0x287baf3f, 0x307e4aa0, 0xa66f981f, 0xd4ac090a
public:
    /**
    * @brief    Simplified creator of a font from a stream.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory EDL class factory to use.
    * @param    stream The font stream.
    * @return   IDOMFontOpenTypePtr the new font
    */
    static EDL_API IDOMFontOpenTypePtr create(IEDLClassFactory *pFactory, const IInputStreamPtr &stream);

    /**
    * @typedef eOpenTypeFontType
    * @ingroup IDom
    * @brief Type used to uniquely identify the type of OpenType font
    */
    typedef enum
    {
        eOpenTypeFontTypeUnknown,            //!< Unknown font type
        eOpenTypeFontTypeTTF,                //!< Truetype font
        eOpenTypeFontTypeCFF,                //!< Compact font format
        eOpenTypeFontTypeTTC,                //!< Truetype collections

    } eOpenTypeFontType;

    /**
    * @typedef eOriginalFontType
    * @ingroup IDom
    * @brief Type used to uniquely identify the original type of Font.
    *        In the PostScript/PDF input filter, most font types are
    *        converted to OpenType before insertion into the DOM.
    *        This allows the ability to discover what the original type was.
    */
    typedef enum
    {
        eOriginalTypeOpenType,            //!< Originally OpenType format
        eOriginalType1,                   //!< Type 1 font
        eOriginalType2,                   //!< Type 2 font - Usually a Type 1 encoded in a PDF in CFF form
        eOriginalType42,                  //!< Type42/Truetype from PS/PDF
        eOriginalType9,                   //!< CIDFontType0 (CIDFont based on Type 1 CharStrings)
        eOriginalType11                   //!< CIDFontType2 (CIDFont based on TrueType glyph descriptions)
    } eOriginalFontType;

    /**
    * @class SignatureID
    * @ingroup IDom
    * @brief Opentype table signatures
    *
    */
    class Signature
    {
    public:

        /**
        * @typedef eSignature
        * @ingroup IDom
        * @brief Type used to uniquely identify the known open type signatures
        */
        typedef enum
        {
            eSignatureNone    = -1,

            // Font type signatures
            eSignatureTTF = 0,                  //!< Truetype font signature
            eSignatureCFF,                      //!< Opentype CFF font signature
            eSignatureTTC,                      //!< Truetype collections font signature

            // OpenType TrueType Signatures

            eSignatureBASE,                     //!< The Baseline table for glyph alignment
            eSignatureCMAP,                     //!< Character to glyph mapping
            eSignatureGSUB,                     //!< The glyph subsititution table.
            eSignatureLTSH,                     //!< The Linear Threahold table for glyph positioning
            eSignatureOS2,                      //!< OS/2 and Windows metrics
            eSignatureHead,                     //!< Font header 
            eSignatureHhea,                     //!< Horizontal header 
            eSignatureVhea,                     //!< Vertical header
            eSignatureMAXP,                     //!< Maximum profile 
            eSignatureName,                     //!< Naming table 
            eSignatureCvt,                      //!< Control Value Table 
            eSignatureFpgm,                     //!< Font program 
            eSignatureGasp,                     //!< Grid fitting and scan conversion procedure table
            eSignatureGdir,                     //!< Indicates that this is an incremental font
            eSignatureGlyf,                     //!< Glyph data 
            eSignatureLoca,                     //!< Index to location 
            eSignaturePost,                     //!< PostScript table
            eSignaturePrep,                     //!< CVT Program 
            eSignatureMort,                     //!< The metamorphsis table for ligatures, contextual or non-contextual substitutions, insertions, and rearrangements.
            eSignatureHmtx,                     //!< Horizontal metrics 
            eSignatureVmtx,                     //!< Vertical metrics
            eSignatureVORG,                     //!< Vertical origin table (for CFF based fonts)
			eSignatureEnd						// End of list marker

        } eSignature;

        Signature()
        {
            set(eSignatureNone);
        }

        Signature (const char *str)
        {
            set(str);
        }

        Signature(eSignature sSignature)
        {
            set(sSignature);
        }

        Signature(TableSignatureID signature)
        {
            set(signature);
        }

        void set (const char *str);

        void set(TableSignatureID signature)
        {
            m_signatureID = signature;
        }

        void set(eSignature signature);

        TableSignatureID getSignatureID()
        {
            return m_signatureID;
        }
        
        eSignature getSignatureType()
        {
            return m_eSignature;
        }
        
        void getStr(char *str);

        static const char *signatureToStr(TableSignatureID signatureID)
        {
            static char sigStr[5];

            Signature   sig(signatureID);

            sig.getStr(sigStr);
            sigStr[4] = '\0';

            return sigStr;
        }

        bool equals(TableSignatureID signature)
        {
            return (signature == m_signatureID);
        }

        bool equals(Signature &signature)
        {
            return (m_signatureID == signature.m_signatureID);
        }

        bool isLessThan(TableSignatureID signature)
        {
            return (m_signatureID < signature);
        }

        bool isGreaterThan(TableSignatureID signature)
        {
            return (m_signatureID > signature);
        }

        bool equals(eSignature signature)
        {
            Signature sig(signature);

            return equals(sig);
        }

    private:

        TableSignatureID m_signatureID;
        eSignature m_eSignature;
    };


    /**
    * @class    Data
    * @ingroup    IDom
    * @brief    Initialization data
    */
    class Data : public IDOMFont::Data
    {
    public:
        Data()
        {
            obfuscated          = false;
            embedded            = true;
            originalFontType    = eOriginalTypeOpenType;
            psStandardFont      = false;
            pdfStandardFont     = false;
        }
        bool                embedded;
        bool                obfuscated;
        EDLSysString        requestedFontName;
        bool                psStandardFont;
        bool                pdfStandardFont;
        eOriginalFontType   originalFontType;
    };
    /**
    * @brief    Retrieves the class id of IDOMFontOpenType
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFontOpenTypeClassID);
        return id;
    };

    /**
    * @brief    Returns true if font is obfuscated.
    *           Obfuscated fonts are only found in XPS Documents.
    * @details  Embedded font obfuscation is a means of preventing end users from using 
    *           standard ZIP utilities to extract fonts from XPS format documents and 
    *           install them on their own systems.
    * @return   @b bool Returns true if the font is obfuscated, false if it is not obfuscated
    */
    virtual bool getObfuscated() const = 0;
    
    /**
     * @brief    Establishes whether the font is a standard PostScript font
     * @return   @b bool True if the font was selected when the input requested a
     *           standard PostScript font
     */
    virtual bool getIsPSStandardFont() const = 0;
    
    /**
     * @brief    Establishes whether the font is a standard PDF font
     * @return   @b bool True if the font was selected when the input requested a
     *           standard PDF font
     */
    virtual bool getIsPDFStandardFont() const = 0;

    /**
    * @brief    Establishes whether the font is flagged for embeddeding.
    * @details  Inputs will set this to true if the font was embedded in the input
    *           or false otherwise. Outputs will honour this setting, if possible,
    *           or not overridden. Currently this will only occur for PDF and PS
    *           output.
    *
    * @return   @b bool True if the font is flagged for embedding
    */
    virtual bool getEmbedded() const = 0;

    /**
    * @brief    Sets whether of not the font is flagged for embeddeding.
    * @details  Outputs will honour this setting, if possible, or not overridden.
    *           In particular, if a font is a standard font in the output format
    *           being used, this setting will be ignored.
    *           Currently this will only occur for PDF and PS output.
    * @param    embedded Set to true to flag the font for embedding
    *
    * @return   @b bool True if font data is permitted to be embedded in the output
    */
    virtual void setEmbedded(bool embedded) = 0;

    /**
    * @brief    Get the name of the font as requested by the input document.
    *           This is only useful for PDF and PostScript input. If a font
    *           has to be substituted for another requested font, this will return
    *           the name of the font the input desired.
    * @return   @b EDLSysString The requested font name if applicable, an empty string otherwise.
    */
    virtual EDLSysString getRequestedFontName() const = 0;

    /**
    * @brief    Get the "PostScript" Name of the font, from the font data itself.
    *           This is usually extracted from the 'name' OpenType table, but
    *           may be synthesized if the name cannot be obtained.
    * @param    pFactory A pointer to an EDL Class factory.
    * @param    fontIndex The index of the desired font (if a TrueType collection)
    * @param    postScriptName A reference parameter to receive the font name.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getPostScriptName(IEDLClassFactory *pFactory, int32 fontIndex, EDLSysString &postScriptName) = 0;
     
    /**
    * @brief    Get the "Full" Name of the font, from the font data itself.
    *           This is usually extracted from the 'name' OpenType table, but
    *           may be synthesized if the name cannot be obtained.
    * @param    fontIndex The index of the desired font (if a TrueType collection)
    * @param    pFactory A pointer to an EDL Class factory.
    * @param    fullName A reference parameter to receive the font name.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getFullName(IEDLClassFactory *pFactory, int32 fontIndex, EDLSysString &fullName) = 0;
    
    /**
    * @brief    Returns the sub font type for this opentype font
    * @return   @b eOpenTypeFontType The opentype font type
    */
    virtual eOpenTypeFontType getOpenTypeFontType() = 0;

    /**
    * @brief    Returns the original font type for this opentype font
    * @return   @b eOriginalFontType The original font type.
    */
    virtual eOriginalFontType getOriginalFontType() = 0;

    /**
    * @brief    Creates an OpenType font table accessor.
    * @param    accessor A reference parameter to receive the font table accessor.
    * @param    fontIndex The index of the font - only used for TTC fonts.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getFontOpenTypeTableAccessor(IFontOpenTypeTableAccessorPtr &accessor, uint32 fontIndex = 0) = 0;

    /**
    * @brief    Creates a TrueType glyph accessor.
    * @param    accessor A reference parameter to receive the glyph accessor
	* @param    stripInstructions Determines whether instructions are stripped. Default is false
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getFontTrueTypeGlyphAccessor(IFontTrueTypeGlyphAccessorPtr &accessor, bool stripInstructions = false) = 0;

    /**
    * @brief    Gets the fsType field (embedded licensing information) from the OS/2 table
    * @return   @b uint16 The function returns the fsType field from the OS/2 table, or zero if the table does not exist in the font.
    */
    virtual uint16 getFontLicenseFromOS2Table() = 0;
    
    /**
    * @brief    Create a TrueType only font from this OpenType font that may contain CFF fonts
    * @param    ptrSession A pointer to the EDL session.
    * @param    fontOpenTypeTT The returned IDOMFontOpenTypeTT True-Type only Opentype font
	* @param    regenerateStream Determines whether the stream should be regenerated
    * @return   @b bool  Function returns false on error.
    */
    virtual bool createTrueTypeOnlyFontVersion(const ISessionPtr &ptrSession, IDOMFontOpenTypeTTPtr &fontOpenTypeTT, bool regenerateStream) = 0;
    
    /**
    * @brief   Create a Subsetted version if this font.
    * @details Provide a vector of glyph IDs and/or unicode codepoints, and a subset font
    *          containing only those glyphs required will be generated. Currently only fonts
    *          with 3,0 or 3,1 format 4 subtables or a 3,10 format 12 subtable are supported.
    *          Glyphs used by composite glyphs that are to be retained in the subfont will be
    *          automatically included. Glyphs or codepoints that are out of range will be
    *          ignored.
    * @param   ptrSession A pointer to the EDL session.
    * @param   usedGlyphs A vector of glyph IDs that should be retained in the subset font.
    * @param   usedUnicode A vector of unicode code points whose underlying glyphs should be retained in the subset font.
    * @param   subsetFont A reference to receive the new subset font
    * @return   @b bool True on success, false if the call fails.	
    */
    virtual bool createSubsetFont(const ISessionPtr &ptrSession, CEDLVector<uint16> &usedGlyphs, CEDLVector<uint32> &usedUnicode,
                                  IDOMFontOpenTypePtr &subsetFont) = 0;

    /**
    * @brief   Get a renamed version of this font.
    *          An exception of type IEDLError is thrown on failure.
    * @param   ptrSession A pointer to the EDL session.
    * @param   fontName The desired font name
    * @param   fontIndex The index of the font, if the font being renamed is a TrueType collection. The default is zero.
    * @return  @b IDOMFontOpenTypePtr The renamed font.
    */
    virtual IDOMFontOpenTypePtr createRenamedFont(const ISessionPtr &ptrSession, const EDLSysString &fontName, uint32 fontIndex = 0) = 0;

};
#define edlobj2IDOMFontOpenType(src) edl_cast((IDOMFontOpenType *)NULL, src) 



/**
* @class IDOMFontOTFTrueType
* @ingroup IDom
* @brief Opentype Font 
*/

class IDOMFontOpenTypeTT : public IDOMFontOpenType
{
#define IDOMFontOpenTypeTTClassID 0x47fda935, 0xb25944b3, 0xbdc1061b, 0x7356cf50

public:

    /**
    * @brief    Retrieves class id of IDOM
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFontOpenTypeTTClassID);
        return id;
    };
    
     /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMFontOpenType::Data
    {
      public:
        Data()
        {
            fontIndex = 0;
        }
        uint32 fontIndex;
    };
	
    /**
     * @brief    Write the converted OpenType font to stream (CFF style Opentype fonts are converted to TTF style).
     * @param    outStream The output stream.
     * @return   @b int32  The number of bytes written. On failure returns -1
     */    
     virtual int32 writeFont(IOutputStreamPtr &outStream) = 0;
    
    /**
     * @brief    Write the converted OpenType font to a file (CFF style Opentype fonts are converted to TTF style).
     *           This method is normally used to debug and test OpenType CFF->TTF converted fonts.
     * @param    filePath The file path or a file template with six trailing X's (e.g "/tmp/fontdump-XXXXXX").
     * @param    isTemplate Indicates whether the "filePath" argument is to be treated as a path template instead of the intended path.
     * @return   @b int32 The number of bytes written. On failure returns -1
     */    
     int32 writeFont(EDLSysString &filePath, bool isTemplate = false);
    
};
DECL_SMART_PTR(IDOMFontOpenTypeTT);
#define edlobj2IDOMFontOpenTypeTT(src) edl_cast((IDOMFontOpenTypeTT *)NULL, src) 


/**
* @class IFontHeaderWriteSegmentBlockEnumerator
* @ingroup IDom
* @brief IFontHeaderWriteSegmentBlockEnumerator Enumerates over the PCLXL Font Header block items for the XL ReadFontHeader operator.
*/
class IFontHeaderWriteSegmentBlockEnumerator :  public IEDLObject
{
#define IFontHeaderWriteSegmentBlockEnumeratorClassID 0x506603ee, 0x8de311dc, 0x83140800, 0x200c9a66
public:

    /**
    * @typedef eFontHeaderWriteSegmentItem
    * @ingroup IDom
    * @brief the enumeration of the segment types to be enumerated over
    */
    typedef enum {

        eItemGTSegment,         // The Global TrueType Segment
        eItemGCSegment,         // The Galley Character Segment
        eItemNullSegment,       // The Null Segment
        eItemFinished

    } eFontHeaderWriteSegmentItem;

    /**
    * @brief    Retrieves class id of IDOM
    * @return   CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IFontHeaderWriteSegmentBlockEnumeratorClassID);
        return id;
    };

    /**
    * @brief    Returns the current segment item in the enumeration
    * @return   @b eFontHeaderWriteSegmentItem The segment item
    */
    eFontHeaderWriteSegmentItem getSegmentItem();

    /**
    * @brief    Returns the current block item of segment item in the enumeration
    * @return   @b int32 The segment block number
    */
    int32 getSegmentBlockItem();

    /**
    * @brief    Returns the current enumeration item segment block size
    * @return   @b int32 The segment block size
    */
    virtual int32 getSegmentBlockSize() = 0;

    /**
    * @brief    Writes the current enumeration item block
    * @param    outStream The output stream
    * @param    endian The endian of the data to write out
    * @return   @b int32 The size of data writen to the output stream
    */
    virtual int32 writeSegmentBlock(IOutputStreamPtr &outStream, eEDLEndian endian) = 0;

    /**
    * @brief    Indicate whether there are more items in the enumeration.
    * @return   @b bool True when there are more items in the enumeration.
    */
    virtual bool haveMoreEnumerationItems() = 0;

    /**
    * @brief    Get the next enumeration item
    * @return   @b bool True on success
    */
    virtual bool nextEnumerationItem() = 0;

};

DECL_SMART_PTR(IFontHeaderWriteSegmentBlockEnumerator);
#define edlobj2IFontHeaderWriteSegmentBlockEnumerator(src) edl_cast((IFontHeaderWriteSegmentBlockEnumerator *)NULL, src) 



/**
* @class IFontPCL5WriteSegmentBlockEnumerator
* @ingroup IDom
* @brief IFontPCL5WriteSegmentBlockEnumerator Enumerates over the PCL5  font blocks.
*/

class IFontPCL5WriteSegmentBlockEnumerator :  public IEDLObject
{

#define IFontPCL5WriteSegmentBlockEnumeratorClassID 0x41108aa0, 0x4e5611dd, 0xae160800, 0x200c9a66

public:

    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IFontHeaderWriteSegmentBlockEnumeratorClassID);
        return id;
    };

   /**
    * @brief    Return the total number of enumeration blocks that will be iterated through
    * @return   @b int32 The enumeration count
    */
    virtual int32 getEnumerationCount() = 0;

    /**
    * @brief    Return the current enumeration item block size
    * @return   @b int32 The segment block size
    */
    virtual int32 getEnumerationItemBlockSize() = 0;

    /**
    * @brief    Writes the current enumeration item block
    * @param    outStream The output stream
    * @param    endian The endian of the data to write out
    * @return   @b int32 The size of data writen to the output stream
    */
    virtual int32 writeEnumerationItemBlock(IOutputStreamPtr &outStream, eEDLEndian endian) = 0;

    /**
    * @brief    Indicate whether there are more items in the enumeration.
    * @return   @b bool Whether there are more items in the enumeration.
    */
    virtual bool haveMoreEnumerationItems() = 0;

    /**
    * @brief    Get the next enumeration item
    * @return   @b bool True on success
    */
    virtual bool nextEnumerationItem() = 0;

};

DECL_SMART_PTR(IFontPCL5WriteSegmentBlockEnumerator);
#define edlobj2IFontPCL5WriteSegmentBlockEnumerator(src) edl_cast((IFontPCL5WriteSegmentBlockEnumerator *)NULL, src) 



class IFontPCLXLTrueTypeGlyphAccessor :  public IEDLObject
{
#define IFontPCLXLTrueTypeGlyphAccessorClassID  0xdd04e870, 0xdae311dc, 0x95ff0800, 0x200c9a66
public:

    /**
    * @brief    Retrieves class id of IFontPCLXLTrueTypeGlyphAccessor
    * @return   @b CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IFontPCLXLTrueTypeGlyphAccessorClassID);
        return id;
    };

    /**
    * @brief    Indicate existance of glyph ID in the font
    * @param    glyphID The glyph ID
    * @return   @b bool Inidicator of existence
    */
    virtual bool hasGlyph(IDOMGlyph::GlyphID glyphID) = 0;

    /**
    * @brief    Gets the size of the glyph data
    * @param    glyphID The glyph ID
    * @param    characterClass The PCLXL ReadChar character class
    * @return   @b int32 The size of the glyph data
    */
    virtual int32 getGlyphDataSize(IDOMGlyph::GlyphID glyphID, uint8 characterClass) = 0;

    /**
    * @brief    Writes the glyph associated with the glyphID to the given output stream
    * @ param   outStream The output stream
    * @param    glyphID The glyph ID
    * @param    endian The endian of the data to write out
    * @param    characterClass The PCLXL ReadChar character class
    * @return   @b int32 Returns the actual number of bytes written to the stream
    */
    virtual int32 glyphDataWrite(IOutputStreamPtr &outStream, IDOMGlyph::GlyphID glyphID, eEDLEndian endian, uint8 characterClass) = 0;

    /**
    * @brief    Gets a glyphID enumerator which lists the dependant glyph IDs of the given glyphID
    * @param    glyphID The given glyph ID
    * @param    ienumerator The returned enumerator of dependant glyph IDs
    * @return   @b bool True on success
    */
    virtual bool getDependantGlyphIDEnumerator(IDOMGlyph::GlyphID glyphID, IDOMGlyphIDEnumeratorPtr &ienumerator) = 0;

    /**
    * @brief    Returns the coresponding character code of the glyphID
    * @param    glyphID The glyph ID
    * @return   @b IDOMFont::CharCode The character code
    */
    virtual IDOMFont::CharCode glyphIDToCharCode(IDOMGlyph::GlyphID glyphID) = 0;

    /**
    * @brief    Returns the coresponding glyphID of the character code
    * @param    charCode The character code
    * @return   @b IDOMGlyph::GlyphID The glyph ID
    */
    virtual IDOMGlyph::GlyphID charCodeToGlyphID(IDOMFont::CharCode charCode) = 0;

};

DECL_SMART_PTR(IFontPCLXLTrueTypeGlyphAccessor);
#define edlobj2IFontPCLXLTrueTypeGlyphAccessor(src) edl_cast((IFontPCLXLTrueTypeGlyphAccessor *)NULL, src) 


/**
* @class IDOMType3Font
* @ingroup IDom
* @brief Representation of a PostScript/PDF Type 3 Font.
*        At present, the stream cannot be set, only retrieved.
*/
class IDOMType3Font : public IDOMFont
{
#define IDOMType3FontClassID 0xb885316f, 0x7b114d80, 0x8fbe958c, 0xf1b34ab3
public:
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        IEDLObjectPtr     internal;
        IEDLTempStorePtr  tempStore;
        IEDLClassFactory  *pFactory;
    };

    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMType3FontClassID);
        return id;
    };
    
    /**
    *  @brief   Retrieves the unique ID of this font.
    *           This ID is allocated on creation. Clones of this font
    *           will receive a new id
    *  @return  DOMid The DOM ID
    */
    virtual DOMid getId() const = 0;

    /**
    * @brief    Retrieves a glyph of the given name, if it exists.
    * @param    glyphName The glyph name
    * @param    glyph The glyph
    * @return   @b bool True on success
    */
    virtual bool getGlyph(EDLSysString &glyphName, IDOMGlyphPtr &glyph) = 0;

    /**
    * @brief    Retrieves a glyph of the given unicode codepoint, if it exists.
    * @param    codePoint The unicode codepoint
    * @param    glyph The glyph
    * @return   @b bool True on success
    */
    virtual bool getGlyph(uint32 codePoint, IDOMGlyphPtr &glyph) = 0;

    /**
    * @brief    Retrieves a glyph of the given GlyphID, if it exists.
    * @param    glyphID The glyphID
    * @param    glyph The glyph
    * @return   @b bool True on success
    */
    virtual bool getGlyph(IDOMGlyph::GlyphID glyphID, IDOMGlyphPtr &glyph) = 0;
    
    /**
     * @brief   Delete the glyph collection from the font.
     * @return  bool True on success
     */
    virtual bool deleteGlyphs() = 0;

    /**
    * @brief    Adds a glyph of the given name to the font
    * @param    glyph The glyph
    * @return   @b bool True on success
    */
    virtual bool addGlyph(IDOMGlyphPtr &glyph) = 0;

    /**
    * @brief    Returns font's bounding box
    * @param    bBox Bounding box
    * @return   @b bool True on success
    */
    virtual bool getBBox(IntRect & bBox) = 0;
};
DECL_SMART_PTR(IDOMType3Font);
#define edlobj2IDOMType3Font(src) edl_cast((IDOMType3Font *)NULL, src) 


/**
* @class IEDLFontSystemFont
* @ingroup IDom
* @brief Representation of fonts installed on the target system (OS dependant).
*/

class IEDLFontSystemFont : public IDOMFont
{
public:
    
#define IEDLFontSystemFontClassID 0xb8e4db61, 0x81774a41, 0x9416a4b2, 0x09f86233

    /**
    * @brief    Retrieves class id of IDOM
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IEDLFontSystemFontClassID);
        return id;
    };
    
    /**
    * @typedef eFontFamily
    * @ingroup IDom
    * @brief Type used to uniquely identify the font face-family.
    */
    
    typedef enum { eFontFamilyUnknown, eFontFamilyLatinText, eFontFamilyLatinHand, eFontFamilyLatinDecorative } eFontFamily;

    /**
    * @typedef eFontDecoration
    * @ingroup IDom
    * @brief Type used to define a set of bit flag values that describe the visual decorations designed into the font.
    */
    
    typedef enum { eFontDecorationUnknown = 0x0, eFontDecorationNormal = 0x1, eFontDecorationItalic = 0x2, eFontDecorationUnderlined = 0x4, eFontDecorationStrikeOut = 0x8 } eFontDecoration;
    
   /*
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    
    class Data: public IDOMFontOpenType::Data
    {
    public:
   
        EDLString fontName;
        EDLString fontFilePath;
        
        eFontFamily fontFamily;
        int16 weight;
        uint32 fontDecorationFlags;
        float ascenderDescenderRatio;
        float averageCharAspectRatio;
    };


   /**
    * @brief    Gets the font file path.
    * @param    fontFilePath Reference parameter to return the font file path.
    * @return   @b bool True on success, false if the call fails.
    */
    
    virtual bool getFontFilePath(EDLString &fontFilePath) = 0;

   /**
    * @brief    Sets the font file path.
    * @param    fontFilePath The font file path.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setFontFilePath(EDLString &fontFilePath) = 0;

    /**
    * @brief    Gets the font name.
    * @param    fontName Reference parameter to return the font name.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getFontName(EDLString &fontName) = 0;
    
    /**
     * @brief    Gets the font weight
     * @details  'Font Weight' describes the font stroke weight. The convention of weight values is given below:
     *
     * | Weight      | Value |
     * |-------------|:-----:|
     * | UNKNOWN     |   0   |
     * | THIN        | 100   |
     * | EXTRALIGHT  | 200   |
     * | ULTRALIGHT  | 200   |
     * | LIGHT       | 300   |
     * | NORMAL      | 400   |
     * | REGULAR     | 400   |
     * | MEDIUM      | 500   |
     * | SEMIBOLD    | 600   |
     * | DEMIBOLD    | 600   |
     * | BOLD        | 700   |
     * | EXTRABOLD   | 800   |
     * | ULTRABOLD   | 800   |
     * | HEAVY       | 900   |
     * | BLACK       | 900   |
     *
     *  @param    weight Reference parameter to return the font weight.
     *  @return   @b bool True on success, false if the call fails.
     */
    virtual bool getFontWeight(uint16 & weight) = 0;
    
    /**
    * @brief    Gets the font family.
    * @return   @b eFontFamily The font family.
    */
    
    virtual eFontFamily getFontFamily() = 0;
    
    /**
    * @brief    Gets the font decoration flags.
    * @return   @b uint32 The font decoration flags, defined in typedef eFontDecoration.
    */
    
    virtual uint32 getFontDecorationFlags() = 0;
    
    /**
     * @brief    Gets the ratio of the font ascent to descent, if available.
     * @param    ascendDescentRatio Reference parameter to return the ratio.
     * @return   @b bool The function returns an indication of whether the ratio is available.
     */
       
    virtual bool getAscentDescentRatio(float &ascendDescentRatio) = 0;
    
    /**
    * @brief    Gets the average character aspect ratio, if available.
    * @param    averageCharAspectRatio Reference parameter to return the ratio.
    * @return   @b bool The function returns an indication of whether the ratio is available.
    */

    virtual bool getAverageCharAspectRatio(float &averageCharAspectRatio) = 0;

     /**
    * @brief    Loads the font from the system and makes it available as a regular DOM Font.
    *           Only TrueType and Opentype fonts are currently supported.
    * @param    font Reference parameter to return the loaded DOM Font.
    * @return   @b bool True on success, false if the call fails.
    */    
    
    virtual bool loadFont(IDOMFontPtr &font) = 0;
};
DECL_SMART_PTR(IEDLFontSystemFont);
#define edlobj2IEDLFontSystemFont(src) edl_cast((IEDLFontSystemFont *)NULL, src) 


/**
* @class IDOMFontPCLXL
* @ingroup IDom
* @brief This class models PCL XL TrueType and bitmap fonts derived from an OpenType font source.
*/
class IDOMFontPCLXL : public IDOMFontOpenTypeTT
{
#define IDOMFontPCLXLClassID 0x2c1b3204, 0x821811dc, 0x83140800, 0x200c9a66
public:

    /**
    * @typedef ePCLXLFontType
    * @ingroup IDom
    * @brief The enumeration used to identify the type of the PCL XL font
    */
    typedef enum
    {
        ePCLXLFontTypeTTF,        //!< TrueType based font
        ePCLXLFontTypeBitmap,    //!< Bitmap font

    } ePCLXLFontType;

    /**
    * @typedef SymbolSet
    * @ingroup IDom
    * @brief Type represents the PCL XL Font Smbol Set
    */
    typedef uint16  SymbolSet;

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMFontOpenTypeTT::Data
    {
        // Data inherited from IDOMFontOpenTypeTT::Data
    };

    /**
    * @brief    Retrieves the class id of IDOMFontPCLXL
    * @return   @b CClassID The class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFontPCLXLClassID);
        return id;
    };

    /**
    * @brief    Returns the sub font type for this PCL XL font
    * @return   IDOMFontPCLXL::ePCLXLFontType. Returns the sub-font type
    */
    virtual IDOMFontPCLXL::ePCLXLFontType getPCLXLFontType() const = 0;

    /**
    * @brief    Returns the XL font name.
    * For example, "Times New Roman".
    * @param    fontName Reference parameter to receive a pointer to the font name.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getFontName(EDLSysString &fontName) = 0;

    /**
    * @brief    Creates a PCL XL FontHeader enumerator of the font for the PCL XL ReadFontHeader operator.
    * @param    enumerator Reference parameter to receive a smart pointer to the enumerator.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getFontHeaderSegmentBlockEnumerator(IFontHeaderWriteSegmentBlockEnumeratorPtr &enumerator) = 0;

    /**
    * @brief    Creates a TrueType glyph accessor.
    * @param    accessor A reference parameter to receive the glyph accessor.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getFontTrueTypeGlyphAccessor(IFontPCLXLTrueTypeGlyphAccessorPtr &accessor) = 0;

};
DECL_SMART_PTR(IDOMFontPCLXL);
#define edlobj2IDOMFontPCLXL(src) edl_cast((IDOMFontPCLXL *)NULL, src) 


class IFontPCL5TrueTypeGlyphAccessor :  public IEDLObject
{

#define IFontPCL5TrueTypeGlyphAccessorClassID 0xb5da9de0, 0x3d3411dd, 0xae160800, 0x200c9a66

public:

    /**
    * @brief    Retrieves class id of IFontPCL5TrueTypeGlyphAccessor
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IFontPCL5TrueTypeGlyphAccessorClassID);
        return id;
    };

    /**
    * @brief    Indicate existance of glyph ID in the font
    * @param    glyphID The glyph ID
    * @return   @b bool Inidicator of existence
    */
    virtual bool hasGlyph(IDOMGlyph::GlyphID glyphID) = 0;

    /**
    * @brief    Gets the size of the glyph data
    * @param    glyphID The glyph ID
    * @param    characterClass The PCL5 ReadChar character class
    * @return   int32 The size of the glyph data
    */
    virtual int32 getGlyphDataSize(IDOMGlyph::GlyphID glyphID) = 0;

	/**
	* @brief	Writes the glyph associated with the glyphID to the given output stream
	* @ param   outStream The output stream
	* @param	glyphID The glyph ID
	* @param	endian The endian of the data to write out
	* @return   int32 Returns the actual number of bytes written to the stream
	*/
	virtual int32 glyphDataWrite(IOutputStreamPtr &outStream, IDOMGlyph::GlyphID glyphID, eEDLEndian endian) = 0;

    /**
    * @brief    Gets a glyphID enumerator which lists the dependant glyph IDs of the given glyphID
    * @param    glyphID The given glyph ID
    * @param    ienumerator The returned enumerator of dependant glyph IDs
    * @return   @b bool True on success
    */
    virtual bool getDependantGlyphIDEnumerator(IDOMGlyph::GlyphID glyphID, IDOMGlyphIDEnumeratorPtr &ienumerator) = 0;

    /**
    * @brief    Returns the coresponding character code of the glyphID
    * @param    glyphID The glyph ID
    * @return   IDOMFont::CharCode The character code
    */
    virtual IDOMFont::CharCode glyphIDToCharCode(IDOMGlyph::GlyphID glyphID) = 0;

    /**
    * @brief    Returns the coresponding glyphID of the character code
    * @param    charCode The character code
    * @return   IDOMGlyph::GlyphID The glyph ID
    */
    virtual IDOMGlyph::GlyphID charCodeToGlyphID(IDOMFont::CharCode charCode) = 0;

};

DECL_SMART_PTR(IFontPCL5TrueTypeGlyphAccessor);
#define edlobj2IFontPCL5TrueTypeGlyphAccessor(src) edl_cast((IFontPCL5TrueTypeGlyphAccessor *)NULL, src) 



/**
* @class IDOMFontPCL5
* @ingroup IDom
* @brief IDOMFontPCL5 (PCL5 Truetype) derived from an OpenType font source
*/
class IDOMFontPCL5 : public IDOMFontOpenTypeTT
{

#define IDOMFontPCL5ClassID 0x2964f9a0, 0x3d3411dd, 0xae160800, 0x200c9a66

public:

    /**
    * @enum     ePCL5FontType
    * @ingroup  IDom
    * @brief    Type used to uniquely identify the type of PCL5 font (Currently, only TTF supported).
    */
    typedef enum
    {
        ePCL5FontTypeTTF,
        ePCL5FontTypeBitmap,

    } ePCL5FontType;

    /**
    * @typedef SymbolSetIDCode
    * @ingroup IDom
    * @brief Type represents the PCL5 Font Symbol Set
    */
    typedef uint16  SymbolSetIDCode;

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public IDOMFontOpenTypeTT::Data
    {
        // Data inherited from IDOMFontOpenTypeTT::Data
    };

    /**
    * @brief    Retrieves class id of IDOMFontPCL5
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFontPCL5ClassID);
        return id;
    };

    /**
    * @brief    Returns the sub font type for this PCLXL font
    * @return   IDOMFontPCL5::ePCL5FontType Returns the sub-font type. Currently only TTF is supported.
    */
    virtual IDOMFontPCL5::ePCL5FontType getPCL5FontType() const = 0;
    
    /**
    * @brief    Returns the XL font name
    * @param    fontName The font name returned 
    * @return   @b bool True on success
    */
    virtual bool getFontName(EDLSysString &fontName) = 0;


    /**
    * @brief    Set the font symbol set to be used in the generated font
    * @param    symbolSet The font symbol set
    * @return   @b bool True on success
    */
    virtual bool setSymbolSetIDCode(IDOMFontPCL5::SymbolSetIDCode symbolSet) = 0;
    
    
    /**
    * @brief    Returns the font symbol set ID as an ID Code
    * @return   IDOMFontPCL5::SymbolSet Returns the symbol set
    */
    virtual IDOMFontPCL5::SymbolSetIDCode getSymbolSetIDCode() const = 0;


	/**
	* @brief	Returns the font symbol set ID
	* @param	symbolSetID The font symbol set ID
	* @return   @b bool True on success
	*/
	virtual bool getSymbolSetID(EDLSysString &symbolSetID) const = 0;
	
	
	/**
	* @brief	Creates a Format 15 font block enumerator
	* @param	enumerator Returns the enumerator
	* @return   @b bool True on success
	*/
	virtual bool getFormat15FontBlockEnumerator(IFontPCL5WriteSegmentBlockEnumeratorPtr &enumerator) = 0;


	/**
	* @brief	Creates a Format 16 font block enumerator
	* @param	enumerator Returns the enumerator
	* @return   @b bool True on success
	*/
	virtual bool getFormat16FontBlockEnumerator(IFontPCL5WriteSegmentBlockEnumeratorPtr &enumerator) = 0;
	
	/**
	* @brief	Creates a TrueType glyph accessor.
	* @param	accessor Returns the accessor.
	* @return   @b bool True on success.
	*/
	virtual bool getFontTrueTypeGlyphAccessor(IFontPCL5TrueTypeGlyphAccessorPtr &accessor) = 0;

};
DECL_SMART_PTR(IDOMFontPCL5);
#define edlobj2IDOMFontPCL5(src) edl_cast((IDOMFontPCL5 *)NULL, src) 

_END_EDL_DOM_NAMESPACE


#endif /* __IDOMFONT_H__ */

