/*
* Copyright (C) 2007-2018 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomform.h
* @ingroup  IDom
* @brief    IDOMGroup interface 
*
* Interface to DOM node representing PDF-style Form Elements
*/


#ifndef __EDLIDOMFORM_H__
#define __EDLIDOMFORM_H__

#include <edl/edlblend.h>
#include <edl/idomnode.h>
#include <edl/idombrush.h>
#include <edl/edlnamespaces.h>
#include <jawsmako/types.h>

_BEGIN_EDL_DOM_NAMESPACE

class IDOMForm;
DECL_SMART_PTR(IDOMForm);

/**
* @class    IDOMForm
* @ingroup  IDom
* @brief    IDOMForm interface. The children of this node type comprise the
*           contents of a PDF/PS style form.
*           This includes the /Matrix and /BBox (bounds) entries that are normally present
*           in form dictionaries. Here, bounds (if non-empty) is used in preference to
*           calculating the bounds of any children.
*           This node should not be present in the DOM tree as a general node. It must only
*           be used as the contents of an IDOMFormInstance
*/
class IDOMForm : public IDOMNode
{
#define IDOMFormClassID 0x62c7232c, 0x75c94156, 0x98f44bff, 0xef34d062
public:

    /**
    * @brief    Simplified creation function for IDOMForm.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The EDL class factory
    * @param    matrix The desired form matrix
    * @param    bounds The desired form bounds
	* @return   IDOMFormPtr A smart pointer to the IDOMForm
    */
    static EDL_API IDOMFormPtr create(IEDLClassFactory *pFactory, const FMatrix &matrix = FMatrix(), const FRect &bounds = FRect());

    /**
    * @brief    Retrieves class id of IDOMForm
    * @return   CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFormClassID);
        return id;
    };
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : matrix(1.0, 0.0, 0.0, 1.0, 0.0, 0.0), bounds(0, 0, 0, 0) {}
        FMatrix matrix;
        FRect   bounds;
    };
    
    /**
     * @brief   Obtain the uique DOMid of this form.
     *
     *          This is distinct from that received from IDOMNode::getDOMid() which
     *          is user set and is copied when the node is cloned.
     *
     *          This DOMid is unique to the form object, and is allocated when
     *          the form is allocated. Forms cloned from this object will receive
     *          a new DOMid.
     * @return  DOMid The DOMid
     */
    virtual DOMid getFormId() const = 0;

    /**
    * @brief    Retrieves the Matrix of the form contents (equivalent to the
    *           /Matrix entry in a PDF/PS form dictionary).
    * @param    matrix Reference to receive the matrix
    * @return   @b bool True on success
    */
    virtual bool getMatrix(FMatrix &matrix) const = 0;

    /**
    * @brief    Sets the Matrix of the form contents (equivalent to the
    *           /Matrix entry in a PDF/PS form dictionary).
    * @param    matrix Reference to receive the matrix
    * @return   @b bool True on success
    */
    virtual bool setMatrix(const FMatrix &matrix) = 0;

    /**
    * @brief    Sets the form bounding box
    * @param    bounds Reference parameter to receive the bounding box.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setBounds(const FRect &bounds) = 0;
    
    /**
    * @brief    Get the JawsMako Logical structure element reference
    *           associated with this form.
    * @return   @b JawsMako::IStructureElementReferencePtr A smart pointer to the structure element reference
    */
    virtual JawsMako::IStructureElementReferencePtr getStructureElementReference() const = 0;
    
    /**
     * @brief    Set the JawsMako logical structure element reference
     *           to be associated with this form, or NULL to clear
     * @param    elementRef The structure element reference to set
     */
    virtual void setStructureElementReference(const JawsMako::IStructureElementReferencePtr &elementRef) = 0;

};

DECL_SMART_PTR(IDOMForm);
#define edlobj2IDOMForm(src) edl_cast((IDOMForm *)NULL, src) 

/**
* @class    IDOMFormInstance
* @ingroup  IDom
* @brief    IDOMFormInstance interface. This describes an instance of an IDOMForm in a DOM tree.
*/
class IDOMFormInstance : public IDOMNode
{
#define IDOMFormInstanceClassID 0x2748dfc4, 0x3a0c43e2, 0x904e135b, 0xbec65b36
public:

    /**
    * @brief    Retrieves class id of IDOMForm
    * @return   @b CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMFormInstanceClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0), blendMode(eBlendModeNormal) {}
        FMatrix             renderTransform;
        IDOMFormPtr         form;
        IDOMBrushPtr        opacityMask;
        eBlendMode          blendMode;
        float               opacity;
    };

    /**
    * @brief  Get the form alpha/opacity.
    * @return @b float The opacity.
    */
    virtual float getOpacity() const = 0;

    /**
    * @brief  Set the form alpha/opacity
	* @param  opacity The desired opacity, a value between 0.0 and 1.0
    * @return @b bool True on success
    */
    virtual bool setOpacity(float opacity) = 0;

    /**
    * @brief    Retrieves render transform applied to this instance of the form.
    * @param    renderTransform Reference to receive the render transform
    * @return   @b bool True on success
    */
    virtual bool getRenderTransform(FMatrix &renderTransform) const = 0; 
    
    /**
    * @brief    Retrieves render transform applied to this instance of the form.
    * @param    renderTransform Reference to the desired transform
    * @return   @b bool True on success
    */
    virtual bool setRenderTransform(FMatrix &renderTransform) = 0; 
    
    /**
    * @brief    Retrieves the Form associated with this instance.
    * @param    form Reference to receive the form.
    * @return   @b bool True on success. False if there are no contents.
    */
    virtual bool getForm(IDOMFormPtr &form) const = 0;

    /**
    * @brief    Sets the Form associated with this instance.
    * @param    form Reference to the desired form.
    * @return   @b bool True on success.
    */
    virtual bool setForm(IDOMFormPtr &form) = 0;

    /**
    * @brief  Get the blend mode to be used for compositing this form.
    * @param  blendMode Reference to receive the blend mode
    * @return @b bool True on success
    */
    virtual bool getBlendMode(eBlendMode &blendMode) const = 0;

    /**
    * @brief  Set the blend mode to be used for compositing this form.
    * @param  blendMode The desired blend mode
    * @return @b bool True on success
    */
    virtual bool setBlendMode(eBlendMode blendMode) = 0;

    /**
    * @brief    Retrieves smart pointer to opacity mask
    * @param    ptrOpacityMask Smart pointer to opacity mask
    * @return   @b bool True on success
    */
    virtual bool getOpacityMask(IDOMBrushPtr &ptrOpacityMask) const = 0;

    /**
    * @brief    Sets the opacity mask
    * @param    ptrOpacityMask Smart pointer to desired opacity mask
    * @return   @b bool True on success
    */
    virtual bool setOpacityMask(IDOMBrushPtr &ptrOpacityMask) = 0;
};
DECL_SMART_PTR(IDOMFormInstance);
#define edlobj2IDOMFormInstance(src) edl_cast((IDOMFormInstance *)NULL, src) 

_END_EDL_DOM_NAMESPACE

#endif /* __EDLIDOMFORM_H__ */


