/*
* Copyright (C) 2007-2014 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomglyphs.h
* @ingroup  IDom
* @brief    IDOMGlyphs Interface
*/

#ifndef __EDLIDOMGLYPHS_H__
#define __EDLIDOMGLYPHS_H__

#include <edl/edltypes.h>
#include <edl/edlgeom.h>
#include <edl/idombrush.h>
#include <edl/idompathgeometry.h>
#include <edl/idomgroup.h>
#include <edl/idomfont.h>
#include <edl/idomglyph.h>
#include <edl/idomnode.h>
#include <edl/idomresources.h>
#include <edl/idomtarget.h>
#include <edl/idompath.h>

_BEGIN_EDL_DOM_NAMESPACE

class IDOMGlyphs;
DECL_SMART_PTR(IDOMGlyphs);

/**
* @class    IDOMGlyphs
* @ingroup  IDom
* @brief    An abstract class providing an interface to a "Glyphs" node.
* Glyphs nodes are used to represent a run of uniformly formatted text from a single font. Text runs are
* broken by line advances and formatting changes. When a text run is broken, a new Glyphs node 
* will be created to describe the text from the change point onwards.
* 
* A Glyphs node may describe an extended run of text or a single character,
* depending on the document formatting. The set of properties of a Glyphs node allows for a complete description
* of the glyph characteristics, such as the fill and opacity of the text, as well as clipping information.
* 
* A Glyphs node also allows the specification of a Unicode string and supports bidirectional and vertical text.
* 
* Some properties of the Glyphs node are composable, meaning that the markings rendered to the
* page are determined by a combination of the property and all the like-named properties of the
* Glyphs node's parent and child nodes.
*/
class IDOMGlyphs : public IDOMNode
{
#define IDOMGlyphsClassID 0x26AA00B9, 0x290A4048, 0xA0D141C1, 0xEFCB15CE
public:
    
    /**
    * @enum     eStyleSimulations
    * @ingroup  IDom
    * @brief    Specifies a style simulation for a glyphs node.
    */
    typedef enum
    {
        eStyleSimulationsNone       = 0x00,
        eStyleSimulationsItalic     = 0x01,
        eStyleSimulationsBold       = 0x02,
        eStyleSimulationsBoldItalic = eStyleSimulationsItalic | eStyleSimulationsBold
    } eStyleSimulations;

    /**
    * @brief    Simplified creator for a glyphs object
    *           Throws an IEDLError on failure.
    * @param    pFactory The factory to use.
    * @param    position The desired position for the glyphs.
    * @param    unicodeString The text to be used.
    * @param    size The font size to use, in ems.
    * @param    brush The fill brush to use.
    * @param    font The font to use.
    * @param    fontIndex The index of the font to use, if the font is within
    *           a truetype collection.
    * @param    styleSimulations The desired style simulations effect.
    * @param    renderTransform The render transform to use.
    * @param    clip The geometry to use for clipping. NULL if no clip.
    * @param    opacity The opacity to use.
    * @return   @b IDOMPathNodePtr The new path.
    */
    static EDL_API IDOMGlyphsPtr create(IEDLClassFactory *pFactory,
                                        const EDLString unicodeString,
                                        double size,
                                        const FPoint &position,
                                        const IDOMBrushPtr &brush,
                                        const IDOMFontPtr  &font,
                                        uint32 fontIndex = 0,
                                        eStyleSimulations styleSimulations = eStyleSimulationsNone,
                                        const FMatrix renderTransform = FMatrix(),
                                        const IDOMPathGeometryPtr &clip = IDOMPathGeometryPtr(),
                                        float opacity = 1.0f);

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : styleSimulations(eStyleSimulationsNone), opacity(1.0), bidiLevel(0), isSideways(false), fontIndex(0), blendMode(eBlendModeNormal)
        {
            isCharPath  = false;
        }
        uint8               bidiLevel;
        EDLSysString        caretStops;
        EDLString           deviceFontName;
        double              fontRenderingEmSize;
        uint32              fontIndex; // >= 0, number identifying font face number in TTC
        double              originX, originY;
        bool                isSideways;
        EDLSysString        indices;
        EDLString           unicodeString;
        eStyleSimulations   styleSimulations;
        FMatrix             renderTransform;
        float               opacity;
        EDLString           language;
        IDOMFontPtr         font;
        IDOMTargetPtr       ptrNavigateLink;
        IDOMBrushPtr        ptrOpacityMask;
        IDOMPathGeometryPtr ptrClip;
        IDOMBrushPtr        ptrFill;
        eBlendMode          blendMode;
        bool                isCharPath;
    };

    /**
    * @brief    Retrieves class id of IDOM
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMGlyphsClassID);        
        return id;
    };

    /**
    * @brief    Retrieves the bidiLevel of the text run.
    * @details The bidi (bi-directional) level specifies the Unicode bi-directional nesting level.
    *           A value of 0 implies left-to-right text, while a value of 1 implies right-to-left text.
    *           Right-to-left text places the run origin at the right side of the first glyph, with positive advance
    *           widths representing advances to the left, so that subsequent glyphs are placed to the left of 
    *           the previous glyph.
    * @return  @b uint8 The bidiLevel value.
    */
    virtual uint8 getBidiLevel () const = 0;

    /**
    * @brief    Sets the bidiLevel for this run of text.
    * @details The bidi (bi-directional) level specifies the Unicode bi-directional nesting level.
    *           A value of 0 implies left-to-right text, while a value of 1 implies right-to-left text.
    *           Right-to-left text places the run origin at the right side of the first glyph, with positive advance
    *           widths representing advances to the left, so that subsequent glyphs are placed to the left of 
    *           the previous glyph.
    * @param    bidiLevel The new bidiLevel value.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setBidiLevel (uint8 bidiLevel) = 0;

    /**
    * @brief    Retrieves the caret stop specification for a run of glyphs.
    *
    * @details Caret stops identify the positions within the sequence of Unicode characters at which a text-selection
    *          tool may place a text-editing caret. Potential caret-stop positions are identified by their indices
    *          into the UTF-16 code units represented by the unicodeString attribute value. When this value
    *          is missing, the test in the UnicodeString attribute must be interpreted as having a caret stop
    *          between every Unicode UTF-16 code unit and at the beginning and end of the text.
    *          
    *          Because it is legal to place the caret before any of the UTF-16 code units in the Unicode string
    *          if the caretStops attribute is omitted, omitting the caretStops attribute is equivalent to
    *          specifying a string that has all the bits set to 1.
    *          
    *          If there are insufficient flags in the caretStops string to correspond to all the UTF-16 code units in
    *          the Unicode string, all remaining UTF-16 code units in the Unicode string MUST be considered
    *          valid caret stops.
    * @param    caretStops A reference parameter to recieve the caret stops specification string.
    * See setCaretStops() for a description of the format of the specification string.
    * @return   @b bool True on success
    */
    virtual bool getCaretStops (EDLSysString& caretStops) const = 0;

    /**
    * @brief    Sets the caret stop specification for a run of glyphs.
    *
    * @details Caret stops identify the positions within the sequence of Unicode characters at which a text-selection
    *          tool may place a text-editing caret. Potential caret-stop positions are identified by their indices
    *          into the UTF-16 code units represented by the unicodeString attribute value. When this value
    *          is missing, the test in the UnicodeString attribute must be interpreted as having a caret stop
    *          between every Unicode UTF-16 code unit and at the beginning and end of the text.
    *          
    *          Because it is legal to place the caret before any of the UTF-16 code units in the Unicode string
    *          if the caretStops attribute is omitted, omitting the caretStops attribute is equivalent to
    *          specifying a string that has all the bits set to 1.
    *          
    *          If there are insufficient flags in the caretStops string to correspond to all the UTF-16 code units in
    *          the Unicode string, all remaining UTF-16 code units in the Unicode string MUST be considered
    *          valid caret stops.
    * @param    caretStops The caret stops specification expressed as a string.
    * @details The caretStops attribute contains an array of boolean bit-flags, which is
    *          represented as a string of hexadecimal characters. The caretStops
    *          attribute includes a final flag for placement of the caret following the
    *          final UTF-16 code unit in the Unicode string. Each hexadecimal character
    *          in the caretStops value represents the flags for four UTF-16 code
    *          units in the Unicode string, with the highest-order bit representing the
    *          first UTF-16 code unit. Any unused bits in the last UTF-16 code unit must be 0.
    *          For example if caretStops is set to "f7" (that is, 1111 0111), in reference to 
    *          the run of glyphs, "Not here.", it means that it is valid to place the caret
    *          stop in any position in the run of text except for directly before the "h" of
    *          "here".
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setCaretStops (const EDLSysString& caretStops) = 0;

    /**
    * @brief    Retrieves the device font name.
    * The device font name, which uniquely identifies a specific device font. The identifier is
    * typically defined by a hardware vendor or font vendor.
    * @param    name The device font name
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getDeviceFontName (EDLString& name) const = 0;

    /**
    * @brief    Sets the device font name
    * The device font name, which uniquely identifies a specific device font. The identifier is
    * typically defined by a hardware vendor or font vendor.
    * @param    name The new device font name.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setDeviceFontName (const EDLString& name) = 0;

    /**
    * @brief    Retrieves the font rendering em size in drawing surface units.
    * The font rendering em size is expressed as a floating point value in units of the effective coordinate
    * space. A value of 0 results in no visible text.
    * @return   @b double The font rendering em size.
    */
    virtual double getFontRenderingEmSize () const = 0;
	
    /**
    * @brief    Sets the font rendering em size in drawing surface units.
    * The font rendering em size is expressed as a floating point value in units of the effective coordinate
    * space. A value of 0 results in no visible text.
    * @param    emSize The new font rendering em size value.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setFontRenderingEmSize (double emSize) = 0;

    /**
    * @brief    Retrieves a pointer to the font object describing the font in which the text is displayed.
    * @return   @b IDOMFontPtr A smart pointer to the text's font object.
    */
    virtual IDOMFontPtr getFont () const = 0;

    /**
    * @brief    Sets the font object to be used to display the text.
    * @param    font Smart pointer to the new font object to use.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setFont (IDOMFontPtr &font) = 0;

    /**
    * @brief    Retrieves the font index.
    * @return   @b uint32 The font index.
    */
    virtual uint32 getFontIndex () const = 0;
    /**
    * @brief    Sets the font index.
    * @param    index The new font index value.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setFontIndex (uint32 index) = 0;

    /**
    * @brief    Retrieves the x-coordinate of the first glyph in the run, 
    *           in units of the effective coordinate space.
    * The first glyph is placed so that the leading edge of its advance vector and its baseline
    * intersect with the point defined by the originX and originY coordinates.
    * @return   @b double The first glyph's x-coordinate.
    */
    virtual double getOriginX () const = 0;

    /**
    * @brief    Sets the x-coordinate of the first glyph in the run, 
    *           in units of the effective coordinate space.
    * The first glyph is placed so that the leading edge of its advance vector and its baseline
    * intersect with the point defined by the originX and originY coordinates.
    * @param    originX The new x-coordinate for the first glyph in the text run.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setOriginX (double originX) = 0;

    /**
    * @brief    Retrieves the y-coordinate of the first glyph in the run, 
    *           in units of the effective coordinate space.
    * The first glyph is placed so that the leading edge of its advance vector and its baseline
    * intersect with the point defined by the originX and originY coordinates.
    * @return   @b double The first glyph's y-coordinate.
    */
    virtual double getOriginY () const = 0;

    /**
    * @brief    Sets the y-coordinate of the first glyph in the run, 
    *           in units of the effective coordinate space.
    * The first glyph is placed so that the leading edge of its advance vector and its baseline
    * intersect with the point defined by the originX and originY coordinates.
    * @param    originY The new y-coordinate for the first glyph in the text run.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setOriginY (double originY) = 0;

    /**
    * @brief    Indicates that a glyph is turned on its side, with the origin being 
    *           defined as the top center of the unturned glyph.
    * @details  Glyphs for text in vertical writing are normally represented by rotating the co-ordinate system
    *           and using the isSideways attribute. Glyph runs with isSideways set to true will be rotated 90 degrees 
    *           counter-clockwise and placed so that the sideways baseline origin is coincident with the normal 
    *           origin of the character, as modified by the offset vector in the indices attribute. The advance 
    *           setting places the nominal origin of the next character a distance along the direction of the 
    *           progression of the run. The direction of the advance setting is unaffected by isSideways, however 
    *           the method by which the size of the advance vector is chosen is different.
    *           
    *           For example, to represent a run of characters top to bottom of a page, a render transform can be
    *           used to rotate the coordinate system 90 degrees clockwise. originX and originY can be used to specify a
    *           position at the top of the column of text. Text from a vertical writing system can then be written
    *           using a glyphs run with isSideways set to true. The individual glyphs appear in the normal orientation
    *           because the rotation is effected by the isSideways attribute undoes the effect of the
    *           render transform.
    * @return   @b bool The IsSideways value
    */
    virtual bool getIsSideways () const = 0;
	
    /**
    * @brief    Sets the IsSideways value. See getIsSideways for a description of the effects of IsSideways.
    * @param    isSideways The new IsSideways value.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setIsSideways (bool isSideways) = 0;

    /**
    * @brief    Retrieves the indices for a rendering a run of glyphs.
    * The indices may be used specify a series of glyphs, complex character to 
    * glyph mappings, or a combination of both.
    * @param    indices A string specifying the glyph indices-that is, metrics and clustering information.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getIndices (EDLSysString& indices) const = 0;
    
    /**
    * @brief    Retrieves a Unicode string that represents the text of the run of glyphs.
    *           Throws an IEDLError exception on failure. Here an empty indices is not considered a failure.
    * @return   @b EDLSysString The indices.
    */
    virtual EDLSysString getIndices () const = 0;
    
    /**
    * @brief    Sets the glyphs indices.
    * @param    indices The string specifying the indices. Within the string,
    * each glyph specification is separated by a semicolon.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setIndices (const EDLSysString& indices)= 0;

    /**
    * @brief    Retrieves a Unicode string that represents the text of the run of glyphs.
    * @param    unicodeString A reference to receive the string.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getUnicodeString (EDLString& unicodeString) const = 0;
    
    /**
     * @brief    Retrieves a Unicode string that represents the text of the run of glyphs.
     *           Throws an IEDLError exception on failure. Here an empty string is not considered a failure.
     * @return   @b EDLString The unicode string.
     */
    virtual EDLString getUnicodeString () const = 0;
    
    /**
    * @brief    Sets the text value of the run of glyphs.
    * The text is specified as Unicode code points.
    * @param    unicodeString A string of UTF-8 characters representing the text to be rendered by this Glyphs node.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setUnicodeString (const EDLString& unicodeString)= 0;

    /**
    * @brief    Retrieves the style simulation value.
    * @details  Synthetic style simulations can be applied to the shape of the glyphs by using the Style Simulations
    *           function. Style simulations can be applied to the designed style of a font. The default value
    *           for Style Simulations is eStyleSimulationsNone, that is, the shapes of the glyphs are not modified.
    *           When eStyleSimulationsBold is set, the glyphs are geometrically widened
    *           by 1% of the em size, so that the centres of strokes remain at the same position. This leaves the
    *           baseline origin unmodified. The bounding box grows 1% all around for a total of 2% horizontal
    *           and 2% vertical. As a result the character height, and the advance width of each glyph are
    *           increased by 2% of the em size.
    *           
    *           When eStyleSimulationsItalic is set, italicization is applied to glyphs with    an IsSideways value of false.
    *           Italicization is applied by skewing the top edge of the bounding box by 20 degrees to the right,
    *           relative to the baseline of the character. Glyphs with an IsSideways value of true are italicized by
    *           skewing the right edge of the bounding box by 20 degrees down, relative to the baseline origin of the
    *           glyph. The character height and advance width are not modified.
    *           
    *           It is possible to apply both bold and italic simulations to the glyphs by setting eStyleSimulationsBoldItalic.
    * @return   @b eStyleSimulations The current style simulation setting.
    */
    virtual eStyleSimulations getStyleSimulations () const = 0;
    /**
    * @brief    Retrieves the style simulation value.
    * @details  Synthetic style simulations can be applied to the shape of the glyphs by using the Style Simulations
    *           function. Style simulations can be applied to the designed style of a font. The default value
    *           for Style Simulations is eStyleSimulationsNone, that is, the shapes of the glyphs are not modified.
    *           When eStyleSimulationsBold is set, the glyphs are geometrically widened
    *           by 1% of the em size, so that the centres of strokes remain at the same position. This leaves the
    *           baseline origin unmodified. The bounding box grows 1% all around for a total of 2% horizontal
    *           and 2% vertical. As a result the character height, and the advance width of each glyph are
    *           increased by 2% of the em size.
    *           
    *           When eStyleSimulationsItalic is set, italicization is applied to glyphs with    an IsSideways value of false.
    *           Italicization is applied by skewing the top edge of the bounding box by 20 degrees to the right,
    *           relative to the baseline of the character. Glyphs with an IsSideways value of true are italicized by
    *           skewing the right edge of the bounding box by 20 degrees down, relative to the baseline origin of the
    *           glyph. The character height and advance width are not modified.
    *           
    *           It is possible to apply both bold and italic simulations to the glyphs by setting eStyleSimulationsBoldItalic.
    * @param    styleSimulations The new style simulation setting.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setStyleSimulations (eStyleSimulations styleSimulations) = 0;

    /**
    * @brief    Retrieves the render transform matrix of the Glyphs node and its children.
    *
    * The render transform establishes a new coordinate frame for the child and descendants 
    * of the Glyphs node. The render transform also affects clip and opacity masks, fill, 
    * x-origin, y-origin, the actual shape of the individual glyphs and the advance widths. 
    * The render transform also affects the font size and values specified by the indices setting.
    * @param    matrix A smart pointer to recieve the render transform matrix.
    */
    virtual void getRenderTransform(FMatrix &matrix) const = 0; 
    /**
    * @brief    Sets the render transform matrix of the Glyphs node and its children.
    *
    * The render transform establishes a new coordinate frame for the child and descendants 
    * of the Glyphs node. The render transform also affects clip and opacity masks, fill, 
    * x-origin, y-origin, the actual shape of the individual glyphs and the advance widths. 
    * The render transform also affects the font size and values specified by the indices setting.
    * @param    matrix the new render transform matrix.
    */
    virtual void setRenderTransform(const FMatrix &matrix) = 0;
    /**
    * @brief    Retrieves the opacity value of the Glyphs node.
    * The opacity value defines the uniform transparency of the canvas. 
    * The opacity value is a number between 0 (fully transparent) and 1 (fully opaque). 
    * Values outside this range are invalid. The default value is 1.0.
    *
    * @return   @b double Returns the opacity value.
    */
    virtual float getOpacity() const = 0;   
    /**
    * @brief    Sets the opacity value of the Glyphs node.
    * The opacity value defines the uniform transparency of the canvas. 
    * The opacity value is a number between 0 (fully transparent) and 1 (fully opaque). 
    * Values outside this range are invalid. The default value is 1.0.
    * @param    opc The new opacity value.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setOpacity(float opc) = 0;
	
    /**
    * @brief    Gets the blend mode to be used for rendering this node.
    * @return   @b eBlendMode The current blend mode.
    */
    virtual eBlendMode getBlendMode() const = 0;
	
    /**
    * @brief    Sets the blend mode to be used for rendering this node.
    *           Note: modes other than Normal are not directly
    *           representable in XPS.
    * @param    blendMode The new blend mode.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setBlendMode(eBlendMode blendMode) = 0;
    /**
    * @brief    Retrieves a hyperlink URI associated with the node.
    * The URI may be a relative reference or a URI that addresses a 
    * resource that is internal to or external to the package.
    * @param    target A smart pointer to receive the URI.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getNavigateLink (IDOMTargetPtr &target) const = 0;
    /**
    * @brief    Associates a hyperlink URI with the node.
    * The URI may be a relative reference or URI that addresses a
    * resource that is internal to or external to the package.
    * @param    target The URI to associate with the node.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setNavigateLink (const IDOMTargetPtr &target) = 0;
    /**
    * @brief    Retrieves the default language of the Glyphs element and any of its children.
    *
    * The default language is specified according to RFC 3066. English is defined as en_GB and American
    * English as en_US. There is no default setting. If the language is not known it is 
    * set to und (undetermined). For further information see 
    * http://www.w3.org/International/articles/language-tags/
    *
    * @param    lang Smart pointer to receive the default language string.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getLanguage (EDLString &lang) const = 0;
    /**
    * @brief    Sets the default language of the Glyphs element and any of its children.
    *
    * The default language is specified according to RFC 3066. English is defined as en_GB and American
    * English as en_US. There is no default setting. If the language is not known it is 
    * set to und (undetermined). For further information see 
    * http://www.w3.org/International/articles/language-tags/
    * @param    lang The new default language string.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setLanguage (const EDLString &lang) = 0;
    /**
    * @brief    Retrieves the opacity mask for the node.
    * The opacity mask specifies a mask of alpha values that is applied to the 
    * glyphs in the same fashion as the opacity setting, but allowing different 
    * alpha values for different areas of the canvas.
    * @param    ptrOpacityMask Smart pointer to receive the opacity mask.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getOpacityMask(IDOMBrushPtr &ptrOpacityMask) const = 0;
    /**
    * @brief    Sets the opacity mask for the node.
    * The opacity mask specifies a mask of alpha values that is applied to the 
    * glyphs in the same fashion as the opacity setting, but allowing different 
    * alpha values for different areas of the canvas.
    * @param    ptrOpacityMask Smart pointer to a brush representing the new opacity mask for the node.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setOpacityMask(const IDOMBrushPtr &ptrOpacityMask) = 0;
    /**
    * @brief    Retrieves the clip for the Glyphs node.
    * The clip limits the rendered region of the glyph.
    * @param    ptrClip Smart pointer to receive the clip information.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getClip(IDOMPathGeometryPtr &ptrClip) const = 0;
    /**
    * @brief    Sets the clip for the Glyphs node.
    * The clip limits the rendered region of the glyph.
    * @param    ptrClip Smart pointer to the new clip information.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setClip(const IDOMPathGeometryPtr &ptrClip) = 0;
    /**
    * @brief    Retrieves the fill brush used to fill the shape of the rendered glyphs.
    * @param    ptrFill Smart pointer to receive a reference to the fill brush.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getFill(IDOMBrushPtr &ptrFill) const = 0;
    /**
    * @brief    Sets the fill brush used to fill the shape of the rendered glyphs.
    * @param    ptrFill Smart pointer to the new fill brush.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setFill(const IDOMBrushPtr &ptrFill) = 0;
    /**
    * @brief    Retrieves the enumerator of the glyph information collection.
    * @return   @b IDOMGradientStopCollectionEnumPtr The glyph information collection enumerator
    */
    virtual IDOMGlyphCollectionEnumPtr getGlyphInfoCollectionEnum() = 0;
	
    /**
    * @brief    Retrieves the flattened glyph information, similarly to getGlyphInfoCollectionEnum().
    *           However, the results are provided as a simple vector of IDOMGlyph::Data objects, which
    *           is faster. Each glyph is laid out according to the glyphs node. Throws IEDLError on
    *           failure.
    * @param    flattenedGlyphs A reference to receive the data  
    */
    virtual void getFlattenedGlyphInfo(IDOMGlyph::CDOMGlyphDataVect &flattenedGlyphs) const = 0;
	
    /**
    * @brief    Retrieves a Boolean value which indicates if this Glyphs node is intended to draw a path.
    * @return   @b bool True if this node is a charpath , false otherwise.
    */
    virtual bool getIsCharPath () = 0;
	
    /**
    * @brief    Sets the flag to indicate if this glyph is intended to draw a path 
    * @param    isCharPath True if this node is a charpath , false otherwise.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool setIsCharPath (bool isCharPath) = 0;

    /**
    * @brief    Get a graphically equivalent path node for the glyphs node,
    *           with all glyphs converted to path geometry.
    * @param    path A reference to receive the path.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getEquivalentPath(IDOMPathNodePtr &path) = 0;
    
    /**
    * @brief    Split the glyphs node into a series of glyph nodes each consisting
    *           of the smallest possible unit.
    *
    * @details  The resulting glyphs nodes may still consist of multiple unicode
    *           codepoints per node if the combinations of code points results in
    *           a single glyph (such as for ligatures). Similarly the resulting
    *           glyphs nodes may still consist of multiple individual glyphs with
    *           a single code point (such as for combining characters). Regardless,
    *           the resulting glyphs nodes are split to the smallest form that
    *           can be interpreted on their own.
    *
    *           If the glyphs node cannot be further split (for example if it is
    *           already in it's simplest form), then the returned result will be
    *           this glyphs node.
    *
    *           If the glyphs node can be split, the returned node will be an
    *           IDOMGroupNode or one of it's subclasses, with the split glyphs
    *           present as children of that group.
    *
    *           Depending on the brushes or transparency that may be used with the
    *           glyphs node, the group may be a simple IDOMGroup, an IDOMCanvas or
    *           alternatively an IDOMTransparency group. In any case, the simplest
    *           form that can be used will be used. The transform for the glyphs
    *           node will be moved to the group.
    *
    *           An exception of type IEDLObject will be thrown on error.
    *
    * @return   @b IDOMNodePtr The result of splitting the glyphs node.
    */
    virtual IDOMNodePtr split() const = 0;
};
#define edlobj2IDOMGlyphs(src) edl_cast((IDOMGlyphs *)NULL, src) 

_END_EDL_DOM_NAMESPACE

#endif /* __EDLIDOMGLYPHS_H__ */
