/*
* Copyright (C) 2007-2018 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomgroup.h
* @ingroup  IDom
* @brief    IDOMGroup interface 
*
* Interface to DOM node representing Group Elements
*/


#ifndef EDLIDOMGROUP_H
#define EDLIDOMGROUP_H

#include <edl/idomnode.h>
#include <edl/edlnamespaces.h>
#include <edl/edlfwd.h>

#include <edl/edlblend.h>
#include <edl/idombrush.h>
#include <edl/idomcolorspace.h>
#include <edl/idompathgeometry.h>
#include <jawsmako/types.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMGroup
* @ingroup  IDom
* @brief    IDOMGroup interface
*/
class IDOMGroup : public IDOMNode
{
#define IDOMGroupClassID 0x4535624b, 0xc4c04966, 0x9a248943, 0x89527f98
public:

    virtual ~IDOMGroup () {}
    
    /**
    * @brief    Simplified creation function for IDOMGroup.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The EDL class factory
    * @param    transform The desired render transform
    * @param    clip The desired clip
	* @return   @b IDOMGroupPtr The IDOMGroup
    */
    static EDL_API IDOMGroupPtr create(IEDLClassFactory *pFactory, const FMatrix &transform = FMatrix(), const IDOMPathGeometryPtr &clip = IDOMPathGeometryPtr(NULL));

    /**
    * @brief    Retrieves class id of IDOMGroup
    * @return   @b CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMGroupClassID);
        return id;
    };
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
      public:
        FMatrix             renderTransform;
        IDOMPathGeometryPtr ptrClip;
    };

    /**
    * @brief    Retrieves render transform matrix of the Group and its children
    * @param    matrix Render transform matrix
    * @return   @b bool True on success
    */
    virtual bool getRenderTransform(FMatrix &matrix) const = 0; 
    /**
    * @brief    Sets render transform matrix of the Group and its children
    * @param    matrix Render transform matrix
    * @return   @b bool True on success
    */
    virtual bool setRenderTransform(const FMatrix &matrix) = 0;
    /**
    * @brief    Retrieves smart pointer to clip
    * @param    ptrClip Smart pointer to clip
    * @return   @b bool True on success
    */
    virtual bool getClip(IDOMPathGeometryPtr &ptrClip) const = 0;
    /**
    * @brief    Sets clip
    * @param    ptrClip Smart pointer to clip
    * @return   @b bool True on success
    */
    virtual bool setClip(const IDOMPathGeometryPtr &ptrClip) = 0;
    
    /**
    * @brief    Get the JawsMako Optional Content details, or NULL if
    *           the group is not subject to optional content.
    * @return   @b JawsMako::IOptionalContentDetailsPtr A smart pointer to the optional content details
    */
    virtual JawsMako::IOptionalContentDetailsPtr getOptionalContentDetails() const = 0;
    
    /**
    *  @brief   Set the JawsMako Optional Content details for the group,
    *           or NULL to remove. Note that this is only allowed for bare IDOMGroup
    *           objects and *not* for subclasses. Any attempt to set optional content
    *           details on any object that is not an IDOMGroup (getNodeType() is
    *           eDOMGroupNode) will result in an exception.
    *           Note that an IDOMGroup may have optional content details or marked
    *           content details, but not both.
    *  @param   details The Optional Content details
    */
    virtual void setOptionalContentDetails(const JawsMako::IOptionalContentDetailsPtr &details) = 0;
    
    /**
    *  @brief   Get the JawsMako Marked Content details for this group, or NULL if the group
    *           is not marked.
    *  @return  @b JawsMako::IMarkedContentDetailsPtr A smart pointer to the Marked Content details
    */
    virtual JawsMako::IMarkedContentDetailsPtr getMarkedContentDetails() const = 0;
    
    /**
    * @brief    Set the JawsMako Marked Content details for this group, or NULL to remove.
    *           Note that this is only allowed for bare IDOMGroup objects and *not* for
    *           subclasses. Any attempt to set marked content details on any object that
    *           is not an IDOMGroup (getNodeType() is eDOMGroupNode) will result in an exception.
    *           Note that an IDOMGroup may have optional content details or marked
    *           content details, but not both.
    * @param    details The Marked Content details
    */
    virtual void setMarkedContentDetails(const JawsMako::IMarkedContentDetailsPtr &details) = 0;

};
#define edlobj2IDOMGroup(src) edl_cast((IDOMGroup *)NULL, src) 

/**
* @class    IDOMTransparencyGroup
* @ingroup  IDom
* @brief    IDOMTransparencyGroup interface. Analogous to PDF Transparency groups.
*/
class IDOMTransparencyGroup : public IDOMGroup
{
#define IDOMTransparencyGroupClassID 0x67dd9c06, 0x1e0f42b5, 0xba609ed6, 0x9931c071
public:
    /**
    * @brief    Retrieves class id of IDOMTransparencyGroup
    * @return   @b CClassID class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMTransparencyGroupClassID);
        return id;
    };
	
    /**
    * @brief    Simplified creation function for IDOMGroup.
    *           Throws an IEDLError exception on failure.
    * @param    pFactory The EDL class factory
    * @param    transform The desired render transform
    * @param    clip The desired clip
    * @param    opacity The desired opacity
    * @param    opacityMask The desired opacity mask
    * @param    blendMode The desired blend mode
    * @param    colorSpace The desired group color space
    * @param    isolated Group isolated parameter
    * @param    knockout Group knockout parameter
	* @return   @b IDOMTransparencyGroupPtr A smart pointer to the new transprency group
    */
    static EDL_API IDOMTransparencyGroupPtr create(IEDLClassFactory *pFactory,
                                                   const FMatrix &transform = FMatrix(),
                                                   const IDOMPathGeometryPtr &clip = IDOMPathGeometryPtr(),
                                                   float opacity = 1.0f,
                                                   const IDOMBrushPtr &opacityMask = IDOMBrushPtr(),
                                                   eBlendMode blendMode = eBlendModeNormal,
                                                   const IDOMColorSpacePtr &colorSpace = IDOMColorSpacePtr(),
                                                   bool isolated = false,
                                                   bool knockout = false);

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data(): opacity(1.0), isolated(false), knockout(false), blendMode(eBlendModeNormal)
        {}
        float               opacity;
        eBlendMode          blendMode;
        IDOMColorSpacePtr   colorSpace;
        bool                isolated;
        bool                knockout;
        FMatrix             renderTransform;
        IDOMBrushPtr        ptrOpacityMask;
        IDOMPathGeometryPtr ptrClip;
    };

    /**
    * @brief  Get the group alpha/opacity.
    * @return @b float The group opacity.
    */
    virtual float getOpacity() const = 0;

    /**
    * @brief  Set the group opacity.
    * @param  opacity The desired opacity.
    * @return @b bool True on success.
    */
    virtual bool setOpacity(float opacity) = 0;

    /**
    * @brief  Get the blend mode to be used for compositing this group with the backdrop.
    * @param  blendMode Reference to receive the blend mode
    * @return @b bool True on success
    */
    virtual bool getBlendMode(eBlendMode &blendMode) = 0;

    /**
    * @brief  Set the blend mode to be used for compositing this group with the backdrop.
    * @param  blendMode The blend mode for the group.
    * @return @b bool True on success
    */
    virtual bool setBlendMode(eBlendMode blendMode) = 0;

    /**
    * @brief  Get the group colorspace.
    * @param  colorSpace Reference to receive a smart pointer to the groups
    *         color space
    * @return @b bool True on success
    */
    virtual bool getColorSpace(IDOMColorSpacePtr &colorSpace) = 0;

    /**
    * @brief Set the group colorspace.
    * @details This is an optional entry in most circumstances, and
    *          may not be used in all situations.
    *          This is only used for the following situations:
    *          - If the group is isolated (optional)
    *          - If the group is used in a SoftMask Brush, with the
    *          soft mask using the luminosity of the group.
    *
    *        Not all color spaces can be used. See section 7.5.5 
    *        of the PDF 1.7 specification for further information.
    *
    * @param  colorSpace The desired color space
    * @return @b bool True on success
    */
    virtual bool setColorSpace(const IDOMColorSpacePtr &colorSpace) = 0;

    /**
    * @brief  Is the group an isolated group?
    *         See section 7.5.5 of the PDF 1.7 spec for details.
    * @return @b bool True if the group is isolated.
    */
    virtual bool getIsIsolated() const = 0;

    /**
    * @brief  Set whether the group is isolated.
    *         See section 7.5.5 of the PDF 1.7 spec for details.
    * @param  isolated New value of isolated
    * @return @b bool True if the group is isolated.
    */
    virtual bool setIsIsolated(bool isolated) = 0;

    /**
    * @brief  Is the group a knockout group?
    *         See section 7.5.5 of the PDF 1.7 spec for details.
    * @return @b bool True if the group is isolated.
    */
    virtual bool getIsKnockout() const = 0;

    /**
    * @brief  Set whether the group is a knockout group.
    *         See section 7.5.5 of the PDF 1.7 spec for details.
    * @param  knockout New value of knockout
    * @return @b bool True if the group is isolated.
    */
    virtual bool setIsKnockout(bool knockout) = 0;

    /**
    * @brief    Retrieves smart pointer to opacity mask
    * @param    ptrOpacityMask Smart pointer to opacity mask
    * @return   @b bool True on success
    */
    virtual bool getOpacityMask(IDOMBrushPtr &ptrOpacityMask) const = 0;

    /**
    * @brief    Sets opacity mask
    * @param    ptrOpacityMask Smart pointer to brush
    * @return   @b bool True on success
    */
    virtual bool setOpacityMask(const IDOMBrushPtr &ptrOpacityMask) = 0;
};
#define edlobj2IDOMTransparencyGroup(src) edl_cast((IDOMTransparencyGroup *)NULL, src) 

_END_EDL_DOM_NAMESPACE


#endif /* EDLIDOMGROUP_H */


