/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomnode.h
* @ingroup  IDom
* @brief    IDOMNode Interface
*/

#ifndef __IDOMNODE_H__
#define __IDOMNODE_H__

#include <edl/iedlobject.h>
#include <edl/iedlenum.h>
#include <edl/edltypes.h>
#include <edl/idomid.h>
#include <edl/edlproperty.h>
#include <edl/edlgeom.h>
#include <edl/edlvector.h>


_BEGIN_EDL_DOM_NAMESPACE

/**
* @enum     eDOMNodeType
* @ingroup  IDom
* @brief    DOM node type enumeration
*/
typedef enum
{
    eDOMContentRootNode,               	//!< Content Root Node
    eDOMDocumentSequenceNode,          	//!< Document Sequence Node
    eDOMDocumentNode,                  	//!< Document Node
    eDOMFixedDocumentNode,             	//!< Fixed Document Node
    eDOMPageNode,                      	//!< Page Node
    eDOMFixedPageNode,                 	//!< FixedPage Node
    eDOMGroupNode,                     	//!< Group Node
    eDOMCharPathGroupNode,             	//!< Char Path Group Node
    eDOMTransparencyGroupNode,         	//!< Transparency GroupNode
    eDOMCanvasNode,                    	//!< Canvas Node
    eDOMGlyphsNode,                    	//!< Glyphs Node
    eDOMGlyphNode,                     	//!< Glyph Node
    eDOMPathNode,                      	//!< Path Node
    eDOMRefNode,                       	//!< Ref Node
    eDOMJobTkContentNode,              	//!< JobTicket Content Node
    eDOMJobTkNodeNode,                 	//!< JobTicket Node Node
    eDOMJobTkValueNode,                	//!< JobTicket Value Node
    eDOMJobTkGenericNodeNode,          	//!< JobTicket Generic Node Node
    eDOMJobTkGenericCharacterDataNode, 	//!< JobTicket Generic Character Data Node
    eDOMVisualRootNode,                	//!< Visual Root Node
    eDOMOutlineNode,                   	//!< Outline Node
    eDOMOutlineEntryNode,              	//!< Outline Entry Node
    eDOMAnnotationAppearanceNode,      	//!< Annotation Appearance Node
    eDOMFormNode,                      	//!< Form Node
    eDOMFormInstanceNode,              	//!< Form Instance Node
    eDOMFragmentNode,                  	//!< Fragment Node
    eDOMTileNode,                      	//!< Tile Node
    eDOMNodeTypeCnt,                   	//!< Node Type Container

} eDOMNodeType;

class IDOMNode;
DECL_SMART_PTR(IDOMNode);

/**
* @class    IDOMNodeFlags
* @ingroup  IDom
* @brief    A collection of bit flags used to signal various conditions of the node.
* For example, the eNodeRenderFlag flag identifies nodes that require rendering.
*
* The IDOMNodeFlags class keeps primary, secondary and tertiary flags. The first eight bits (0..7) 
* are reserved for EDL private use - OEM user cannot change these bits.
* The next eight bits (8..15) are reserved for EDL public use - an OEM user can change these bits and 
* the bit's meaning is published. The next sixteen bits (16..32) are reserved for OEM usage.
*/
class IDOMNodeFlags
{
public:

    /**
    * @brief  DOM node flag enumeration
    * First eight bits (0..7) are reserved for EDL private use - OEM user cannot change these bits.
    * next eight bits (8..15) are reserved for EDL public use - OEM user can change these bits 
    * the bits meaning is "published" for OEM, for example bit 8 is the eNodeRenderFlag
    * next sixteen bits (16..32) are reserved for OEM usage
    */
    enum DOMNodeFlags
    {
        eNodeRenderFlag      = 8,
        eNodeUnpackFlag      = 9,
        eNodeDirtyFlag       = 10, /*!< eNodeDirtyFlag marks a node as dirty (i.e with changed content) */
        eNodeInterestingFlag = 11, /*!< eNodeInterestingFlag marks a node as interesting for rendering purposes.
                                    *   See <b>IJawsRenderer</b> for details. */
        eNodeSelectedFlag    = 15,
        eNodeUserFlag        = 16
    };

    /**
    * @brief    Tests whether the bit at a specified position is set to 1.
    * @param    position The bit position to test.
    * @return	@b bool Returns true if the flag bit is set to 1, false if it is set to zero.
    */
    virtual bool get(uint32 position) const = 0;

    /**
    * @brief    Sets the bit at the specified position to the specified value.
    * @param    position The bit position to set.
    * @param    value The bit value to set.
    */
    virtual void set(uint32 position, bool value = true) = 0;

};


/**
* @class    IDOMNode
* @ingroup  IDom
* @brief    Abstract class providing the interface to basic DOM node functionality.
* IDOMNode is the base class for many of the other DOM node types, and defines 
* many of the basic functions of DOM nodes.
*/
class IDOMNode : public IEDLObject
{
public:
    /**
    * @brief    virtual destructor
    */
    virtual ~IDOMNode()
    {}

    /**
    * @brief    Retrieves the node ID.
    * @return   DOMid The numeric value that uniquely identifies the node.
    */
    virtual DOMid getDOMid() const = 0;

    /**
    * @brief    Sets the node ID.
    * @param    id The new DOM ID for the node.
    */
    virtual void setDOMid(DOMid id) = 0;

    /**
    * @brief    Retrieves the DOM node type.
    * @return   @b eDOMNodeType The DOM node type.
    */
    virtual eDOMNodeType getNodeType() const = 0;

    /**
    * @brief    Retrieves the value of a property.
    * The EDL DOM node can store non-content or relationship
    * information through the use of the "properties" feature of the node. 
    * The data is represented as key-value pairs; the key being a string and 
    * the value being an abstract container called a PValue. PValues can represent
    * integers, strings, DOM nodes, and so on.
    * @param    propertyName The name of the property.
    * @param    propertyValue Smart pointer to receive the value of the property.
    * @return	@b bool True on success, false if the call fails.
    */
    virtual bool getProperty(const EDLSysString &propertyName, PValue &propertyValue) const = 0;

    /**
    * @brief    Sets the value of a property.
    * The EDL DOM node can store non-content or relationship
    * information through the use of the "properties" feature of the node. 
    * The data is represented as key-value pairs; the key being a string and 
    * the value being an abstract container called a PValue. PValues can represent
    * integers, strings, DOM nodes, and so on.
    * @param    propertyName The name of the property.
    * @param    propertyValue The new value for the property.
    * @return	@b bool Returns true on success, false if the call fails.
    */
    virtual bool setProperty(const EDLSysString &propertyName, const PValue &propertyValue) = 0;

    /**
    * @brief    Removes property.
    * @details  The EDL DOM node can store non-content or relationship
    *           information through the use of the "properties" feature of the node. 
    *           The data is represented as key-value pairs; the key being a string and 
    *           the value being an abstract container called a PValue. PValues can represent
    *           integers, strings, DOM nodes, and so on.
    * @param    propertyName The name of the property.
    */
    virtual void removeProperty(const EDLSysString &propertyName) = 0;

    /**
    * @brief    Retrieves a navigable list of the property names stored on this node.
    * @return   @b IEDLSysStringCollectionEnumPtr A smart pointer to the property names list.
    */
    virtual IEDLSysStringCollectionEnumPtr getPropertyCollectionEnum() = 0;

    /**
    * @brief    Function that indicates whether this node is a parent to other nodes.
    * @return	@b bool Returns true if the node has child nodes, false if it does not.
    */
    virtual bool hasChildNodes() const = 0;

    /**
    * @brief    Gets the parent node of this node.
    * @return   @b IDOMNodePtr A smart pointer to the parent node.
    */
    virtual IDOMNodePtr getParentNode () const = 0;

    /**
    * @brief    Gets the first child node of this node.
    * @return   @b IDOMNodePtr A smart pointer to the first child node.
    */
    virtual IDOMNodePtr getFirstChild () const = 0;

    /**
    * @brief    Gets the last child node of this node.
    * @return   @b IDOMNodePtr A smart pointer to the last child node.
    */
    virtual IDOMNodePtr getLastChild () const = 0;

    /**
    * @brief    Gets the child node which follows the node passed in.
    * @param    child The "current" child node; the node whose next sibling is required.
    * @return   @b IDOMNodePtr A smart pointer to the next child node. If the child node 
    *           passed in was the last one under the node, returns a smart pointer to the node's first child.
    */
    virtual IDOMNodePtr getNextChild (IDOMNodePtr &child) const = 0;

    /**
    * @brief    Gets the child node which precedes the node passed in.
    * @param    child The "current" child node; the node whose previous sibling is required.
    * @return   @b IDOMNodePtr A smart pointer to the previous child node. If the child node 
    *           passed in was the last one under the node, returns a smart pointer to the node's last child.
    */
    virtual IDOMNodePtr getPreviousChild (IDOMNodePtr &child) const = 0;

    /**
    * @brief    Retrieves the node's previous sibling node.
    * @return   @b IDOMNodePtr A smart pointer to the node's previous sibling.
    */
    virtual IDOMNodePtr getPreviousSibling () const = 0;

    /**
    * @brief    Retrieves node's next sibling node.
    * @return   @b IDOMNodePtr A smart pointer to the node's next sibling.
    */
    virtual IDOMNodePtr getNextSibling () const = 0;

    /**
    * @brief    Appends a node to the end of the node's child list.
    * @param    child The child node to append.
    * @return	@b bool True on success, false if the call fails.
    */
    virtual bool appendChild (const IDOMNodePtr &child) = 0;

    /**
    * @brief    Insert a child node after ptrPreviousSibling
    * @param    ptrPreviousSibling Smart pointer to the previous sibling node. 
    * If ptrPreviousSibling is NULL then child will be inserted as the first node
    * @param    child The child node to insert.
    * @param    bCheckComplete If false insert node even if parent is complete.
    * @return	@b bool True on success, false if the call fails.
    */
    virtual bool insertChild (const IDOMNodePtr &ptrPreviousSibling, const IDOMNodePtr &child, bool bCheckComplete = true) = 0;

    /**
    * @brief    Extracts (that is, finds and removes) a child node from the node children.
    * After extraction the child node is no longer a part of the DOM.  If no node is 
    * specified, the first available node will be extracted from the node's children.
    * @param    child A pointer to the child node to extract. If set to NULL, the first 
    * available child node will be extracted.
    * @return   @b IDOMNodePtr A smart pointer to the extracted child node.
    */
    virtual IDOMNodePtr extractChild(const IDOMNodePtr &child) = 0;

    /**
    * @brief    Replaces the child node with another.
    * @param    oldChild Pointer to the child node to be replaced.
    * @param    newChild Pointer to the replacing node.
    * @return	@b bool True on success, false if the call fails.
    */
    virtual bool replaceChild(const IDOMNodePtr &oldChild, const IDOMNodePtr &newChild) = 0;

    /**
    * @brief    Signals the completeness of the node.  
    * A complete node is one that has no more children to be added to it.
    *
    * @return	@b bool True if the node is complete, false if the node is incomplete.
    */
    virtual bool isComplete() const = 0;

    /**
    * @brief    Sets the node's completeness status to "true".
    */
    virtual void setComplete() = 0;

    /**
    * @brief    Retrieves the node's flags property.
    * @return   @b IDOMNodeFlags A pointer to the node's flags property.
    */
    virtual IDOMNodeFlags * getFlags() = 0;

    /**
    * @brief    Sets the parent node.
    * @param    ptrParent Smart pointer to the new parent node.
    * @return	@b bool True on success, false if the call fails.
    */
    virtual bool setParentNode (const IDOMNodePtr &ptrParent) = 0;

    /**
    * @brief    Sets the previous sibling node.
    * @param    ptrPreviousSibling Smart pointer to the new previous sibling node.
    * @return	@b bool True on success, false if the call fails.
    */
    virtual bool setPreviousSibling(const IDOMNodePtr &ptrPreviousSibling) = 0;

    /**
    * @brief    Sets the next sibling node.
    * @param    ptrNextSibling Smart pointer to the new next sibling node.
    * @return	@b bool True on success, false if the call fails.
    */
    virtual bool setNextSibling (const IDOMNodePtr &ptrNextSibling) = 0;

    /**
    * @brief    Function tests whether a candidate node is a descendant of the node.
    * @param    ptrCandidate Smart pointer to the candidate node.
    * @return	@b bool True if ptrCandidate is a descendant of this node.
    */
    virtual bool isAncestor (const IDOMNodePtr &ptrCandidate) = 0;

    // Bounds
    /**
    * @brief Find the conservative bounding box of the marking content of the node. 
    *
    *        For composite objects such as canvases
    *        and pages, this function will recurse through its current children.
    * @param bounds A reference to an FRect object to receive the bounding box.
    * @param applyTransform Controls whether or not the receiver's
    *        RenderTransform is applied to the bounds (if it has one)
    *        - Pass true (default) to return results in the coordinate
    *        space of the object enclosing the receiver
    *        - Pass false to return results in the coordinate space active
    *        inside the object
    * @param applyClip Controls whether or not the receiver's
    *        Clip is applied to the bounds (if it has a clip).
    *  
    *        This parameter only applies to the current node clip path, it does not apply to 
    *        child nodes. It is generally used to test if the current nodes clip path is effective.
    * @return @b bool True on success, false if the call fails.
    */
    virtual bool getBounds(FRect &bounds, bool applyTransform = true, bool applyClip = true)
    {
        // Get the bounds of the children
        IDOMNodePtr node = getFirstChild();
        while (node)
        {
            FRect childBounds;

            if (!node->getBounds(childBounds, applyTransform, applyClip))
                return false;

            bounds.unionRect(childBounds);
            node = node->getNextSibling();
        }

        return true;
    }
    
    // Exception-throwing version of getBounds().
    /**
    * @brief Find the conservative bounding box of the marking content of the node. 
    *        An exception of type IEDLError is thrown on failure.
    *  
    *        For composite objects such as canvases
    *        and pages, this function will recurse through its current children.
    * @param applyTransform Controls whether or not the receiver's
    *        RenderTransform is applied to the bounds (if it has one)
    *        - Pass true (default) to return results in the coordinate
    *        space of the object enclosing the receiver
    *        - Pass false to return results in the coordinate space active
    *        inside the object
    * @param applyClip Controls whether or not the receiver's
    *        Clip is applied to the bounds (if it has a clip).
    *  
    *        This parameter only applies to the current node clip path, it does not apply to 
    *        child nodes. It is generally used to test if the current nodes clip path is effective.
    * @return @b FRect The bounding box
    */
	virtual FRect getBounds(bool applyTransform = true, bool applyClip = true) = 0;

    /**
    * @brief    Copy the properties collection, the flags and the DOM ID from the given source node to this one.
    * @param    pSourceNode Smart pointer to the source node.
    * @return	@b bool True on success, false if the call fails.
    */
    virtual bool copyNodeData(IDOMNode *pSourceNode) = 0;
    
    /**
    * @brief    Simpified node cloning.
    *           An exception of type IEDLError will be thrown on failure.
    * @param    pFactory The EDL class factory.
    * @return   @b IDOMNodePtr The cloned node.
    */
    virtual IDOMNodePtr cloneNode(IEDLClassFactory *pFactory) const = 0;
    
    /**
    * @brief    Clone the tree of nodes beginning at this node.
    *           An exception of type IEDLError will be thrown on failure.
    * @param    pFactory The EDL class factory.
    * @return	@b IDOMNodePtr The cloned node.
    */
    virtual IDOMNodePtr cloneTree(IEDLClassFactory *pFactory) const = 0;
    
    /**
    * @brief    Clone the tree of nodes beginning at this node,
    *           and append the result to the destination tree.
    *           An exception of type IEDLError will be thrown on failure.
    * @param    pFactory The EDL class factory.
    * @param    dest The destination node
    */
    virtual void cloneTreeAndAppend(IEDLClassFactory *pFactory, const IDOMNodePtr &dest) const = 0;

    /**
    * @brief    Mark the entire tree from this point as complete.
    *           An exception of type IEDLError will be thrown on failure.
    *           You should not ordinarily need to call this function.
    */
    virtual void completeTree() = 0;
    
    /**
    * @brief    Mark the entire tree from this point as complete.
    *           An exception of type IEDLError will be thrown on failure.
    *           You should not ordinarily need to call this function.
    */
    virtual void removeCompleteFlagFromTree() = 0;
    
    /**
    * @brief    Find all children of this node with the given type,
    *           appending to the given vector.
    *           Does not descend into brushes.
    * @param    type the type of node to find
    * @param    nodes Vector to receive the found nodes.
    * @param    searchForms If true, search inside forms referenced
    *           from form instance nodes.
    */
    virtual void findChildrenOfType(eDOMNodeType type, CEDLVector<IDOMNodePtr> &nodes, bool searchForms = false) = 0;

    typedef bool (*WalkTreeFunc)(void *priv, const IDOMNodePtr &node);

    /**
    * @brief    Walk through the DOM calling a given function on each node.
    *           The function is allowed to:
    *            - edit or inspect the node in any way
    *            - replace the node (via parent->replaceNode())
    *            - remove the node (via parent->extractChild())
    *            - edit or modify the node's children in any way
    *           However the result is undefined if the nodes siblings are removed or reordered.
    *           Keep in mind that if a node within a brush or form is altered, all uses of that
    *           brush or form will see the change.
    *
    * @param    func The function to be called for each node. Return false from this function
    *           if you wish to stop the walking. It is also safe to throw exceptions from this
    *           function.
    * @param    priv A bare pointer that is passed to func for each call.
    * @param    descendIntoBrushes If true, the walker will descend into composite brushes,
    *           only once per invocation.
    * @param    descendIntoForms If true, the walker will descend into forms,
    *           only once per invocation.
    */
    virtual void walkTree(WalkTreeFunc func, void *priv, bool descendIntoBrushes = false, bool descendIntoForms = false) = 0;
    
    /** 
    * @brief    Attempt to find the effective transformation matrix external to the specified node
    *           relative to either a containing page or ultimate parent.
    * @param    node The specified node
	* @return   @b FMatrix The effective transformation matrix
    */
    static EDL_API FMatrix effectiveTransformationOfNode(const IDOMNodePtr &node);

};

#define edlobj2IDOMNode(src) edl_cast((IDOMNode *)NULL, src)


/**
* @brief    Clones a full node tree and (optionally) appends it to the children of a parent node.
* The clone is made starting with ptrNode and recurses down from there, using pFactory.  If 
* a value is supplied for ptrParent, the resulting cloned tree will be appended to that
* node's child list.
* @param    ptrNode Smart pointer to the source node.
* @param    ptrParent Smart pointer to the parent node where the cloned tree is to be appended(can be NULL).
* @param    pFactory Pointer to the class factory interface to use when cloning the tree.
* @return	<b>IDOMNodePtr</b> A smart pointer to the cloned node tree.
*/
EDL_API IDOMNodePtr cloneNodeWithChildren(IDOMNodePtr &ptrNode, IDOMNodePtr &ptrParent, IEDLClassFactory *pFactory);

/**
* @brief    Clones a full node tree and (optionally) appends it to the children of a parent node.
* The clone is made starting with ptrNode and recurses down from there, using pFactory.  If 
* a value is supplied for ptrParent, the resulting cloned tree will be appended to that
* node's child list. This variant of cloning function also includes cloning of sharing objects such
* as forms. Also this function allows to discard properties on the clonned nodes.
* @param    ptrNode Smart pointer to the source node.
* @param    ptrParent Smart pointer to the parent node where the cloned tree is to be appended(can be NULL).
* @param    pFactory Pointer to the class factory interface to use when cloning the tree.
* @param    discardProperties Array of pointers to property names to discard. Last pointer must be NULL.
* @return	<b>IDOMNodePtr</b> A smart pointer to the cloned node tree.
*/
EDL_API IDOMNodePtr deepCloneNodeWithChildren(IDOMNodePtr &ptrNode, IDOMNodePtr &ptrParent, IEDLClassFactory *pFactory, const char ** discardProperties);

_END_EDL_DOM_NAMESPACE

#endif /* __IDOMNODE_H__ */

