/*
* Copyright (C) 2007-2017 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file    idomoutline.h
* @ingroup IDom
*/

#ifndef __EDLIDOMOUTLINE_H__
#define __EDLIDOMOUTLINE_H__

#include <edl/edltypes.h>
#include <edl/iedltree.h>
#include <edl/idomnode.h>
#include <edl/idomtarget.h>
#include <edl/idomcolor.h>

_BEGIN_EDL_DOM_NAMESPACE

class IDOMOutlineEntry;
DECL_SMART_PTR(IDOMOutlineEntry);

#define IDOMOutlineTreeNodeClassID 0xc50ff402, 0xeb5146bd, 0xb71cb56d, 0xb2917eb8
typedef IEDLTreeNode<IDOMOutlineEntryPtr> IDOMOutlineTreeNode;
DECL_SMART_PTR(IDOMOutlineTreeNode);
#define edlobj2IDOMOutlineTreeNode(src) edl_cast((IDOMOutlineTreeNode *)NULL, src)

/**
* @class    IDOMOutlineEntry
* @ see     IDOMOutline
* @ingroup  IDom
* @brief    Represents an index to a specific location in the document or a specific location external to the document.
*
* IDOMOutlineEntry objects are used for the individual components of the IDOMOutline. You can use the document 
* outline information to support interactive functionality.
*
* Every outline entry associates a text description of the bookmark with certain location within the document
* (represented as an IDOMInternalTarget or an IDOMPageTarget) or with an external location (represented as
* an IDOMExternalTarget). IDOMOutlineEntry inherits from IDOMNode and the outline entries are organized into a
* tree-like structure. There is no limit set on number of outline entries, so there can be as many as the 
* available memory allows.
* 
* Using this information it is possible to create a navigation pane that uses the Unicode value of the
* description attribute of the outline entry node. The corresponding location is specified by the
* Target attribute.
*/
class IDOMOutlineEntry : public IEDLObject
{
#define IDOMOutlineEntryClassID 0x8779c348, 0x5b8647ad, 0x998a9ac5, 0xf917346d
public:

    /**
    * @enum     eTextStyle
    * @ingroup  IDom
    * @brief    Specifies an outline text style
    */
    typedef enum
    {
        eTextStyleNone       = 0x00,    //!< Default text style
        eTextStyleItalic     = 0x01,    //!< Italic text style
        eTextStyleBold       = 0x02,    //!< Bold text style 
        eTextStyleBoldItalic = eTextStyleItalic | eTextStyleBold     //!< Bold and italic text style
    } eTextStyle;

    DECLARE_CLASS_ID(IDOMOutlineEntryClassID);
    
    /**
    * @brief    Simplified creator to create an outline tree entry with a new
    *           outline tree node.
    * @param    pFactory The EDL factory to use
    * @param    description The description of the outline entry
    * @param    expanded If the outline entry's children should be shown by default
    * @param    target The destination for the outline entry, or NULL for no target.
    * @param    textColor The color for the outline entry, or NULL for default.
    *           If provided, it must use the DeviceRGB color space.
    * @param    textStyle The desired text style
    * @param    language The language code to use (see #setLanguage below)
    */
    static EDL_API IDOMOutlineTreeNodePtr createNode(IEDLClassFactory *pFactory,
                                                     const EDLString &description,
                                                     bool expanded = true,
                                                     const IDOMTargetPtr &target = IDOMTargetPtr(),
                                                     const IDOMColorPtr &textColor = IDOMColorPtr(),
                                                     eTextStyle style = eTextStyleNone,
                                                     EDLString language = EDLString(L"und"));

    
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data()
        {}
        EDLString language;
        EDLString description;
        IDOMTargetPtr ptrTarget;
        IDOMColorPtr ptrTextColor;
        eTextStyle textStyle;
        bool expanded;
        IEDLObjectPtr ptrStructureElement;
    };

    /**
    * @brief    Retrieves the default language of the outline entry node.
    *
    * English is defined as en_GB and American English as en_US. There is no 
    * default setting. If the language is not known it is set to "und"
    * (undetermined). For further information see http://www.w3.org/International/articles/language-tags/.
    * The language is specified according to RFC 3066.
    *
    * @param    strLanguage Default language
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getLanguage (EDLString &strLanguage) const = 0;

    /**
    * @brief    Retrieves the default language of the outline entry node.
    *
    * English is defined as en_GB and American English as en_US. There is no 
    * default setting. If the language is not known it is set to "und"
    * (undetermined). For further information see http://www.w3.org/International/articles/language-tags/.
    * The language is specified according to RFC 3066.
    *
    * @param    strLanguage Default language
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setLanguage (const EDLString &strLanguage) = 0;

    /**
    * @brief    Retrieves the description of the outline entry node.
    * @param    strDescription Reference parameter to receive the outline entry node descipriton.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getDescription(EDLString &strDescription) const = 0;

    /**
    * @brief    Sets the description of the outline entry node.
    * @param    strDescription The new descripton for the outline entry node.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setDescription (const EDLString &strDescription) = 0;

    /**
    * @brief    Retrieves the target of the outline entry node.
    *           If outline entry doesn't have target then ptrTarget will be set to NULL
    * @param    ptrTarget Smart pointer to receive the target of the outline entry node.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getTarget(IDOMTargetPtr &ptrTarget) const = 0;

    /**
    * @brief    Sets the target of the outline entry node.
    *           NULL is a valid value of ptrTarget parameter
    * @param    ptrTarget Smart pointer to the new target of the outline entry node.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setTarget(const IDOMTargetPtr &ptrTarget) = 0;

    /**
    * @brief    Retrieves the color to be used for the outline entry's text.
    * @param    ptrColor Smart pointer to the outline entry text's color.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getTextColor(IDOMColorPtr &ptrColor) const = 0;

    /**
    * @brief    Sets the color to be used for the outline entry's text.
    * @param    ptrColor Smart pointer to the new outline entry text color.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setTextColor(const IDOMColorPtr &ptrColor) = 0;

    /**
    * @brief    Retrieves the style to be used for the outline entry's text.
    * @see        eTextStyle
    * @return   eTextStyle. Returns the style value for the outline entry's text.
    */
    virtual eTextStyle getTextStyle() const = 0;

    /**
    * @brief    Sets the outline entry's text style.
    * @see        eTextStyle
    * @param    style The new value for the outline entry's text style.
    */
    virtual void setTextStyle(eTextStyle style) = 0;

    /**
    * @brief    Retrieves "expanded" flag value.
    * @return   bool. Returns true if the outline entry is expanded, false otherwise.
    */
    virtual bool getExpanded() const = 0;

    /**
    * @brief    Sets the outline entry's "expanded" flag.
    * @param    expanded New value for the outline entry's "expanded" flag.
    */
    virtual void setExpanded(bool expanded) = 0;

    /**
    * @brief    Retrieves the structure element.
    * @param    ptrSE Smart pointer to receive the structure element
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getStructureElement(IEDLObjectPtr &ptrSE) const = 0;

    /**
    * @brief    Sets the structure element.
    * @param    ptrSE Smart pointer to the new structure element.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setStructureElement(const IEDLObjectPtr &ptrSE) = 0;

};

#define edlobj2IDOMOutlineEntry(src) edl_cast((IDOMOutlineEntry *)NULL, src)

#define IDOMOutlineTreeClassID 0xe506e704, 0x6e2a44ac, 0x90492ec9, 0xf0030b95
typedef IEDLTree<IDOMOutlineEntryPtr> IDOMOutlineTree;
DECL_SMART_PTR(IDOMOutlineTree);
#define edlobj2IDOMOutlineTree(src) edl_cast((IDOMOutlineTree *)NULL, src)

class IDOMOutline;
DECL_SMART_PTR(IDOMOutline);

/**
* @class    IDOMOutline
* @see      IDOMOutlineEntry
* @ingroup  IDom
* @brief    Represents the outline of the document, which is the collection of bookmarks for the document.
* 
* IDOMOutline is similar to the table of contents in a book. Each entry into the outline is represented by an 
* IDOMOutlineEntry object.  IDOMOutlineEntry  inherits from the IDOMNode and these outline entries are 
* organized into a tree-like structure. IDOMOutline holds the reference to the root node of that tree.
*/
class IDOMOutline : public IEDLObject
{
#define IDOMOutlineClassID 0x236225f5, 0xf0ab4443, 0xa0f0b200, 0x138611e3
public:

    DECLARE_CLASS_ID(IDOMOutlineClassID);
    
    /**
    * @brief    Simplified creator to create an outline
    * @param    pFactory The EDL factory to use
    * @param    language The language code to use (see #setLanguage below)
    */
    static EDL_API IDOMOutlinePtr create(IEDLClassFactory *pFactory, const EDLString &language = EDLString(L"und"));
    
    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data()
        {}
        EDLString language;
    };

    /**
    * @brief    Retrieves the default language of the outline node.
    *
    * English is defined as en_GB and American English as en_US. There is no 
    * default setting. If the language is not known it is set to "und"
    * (undetermined). For further information see http://www.w3.org/International/articles/language-tags/.
    * The language is specified according to RFC 3066.
    *
    * @param    strLanguage Reference parameter to receive the default language.
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool getLanguage (EDLString &strLanguage) const = 0;

    /**
    * @brief    the default language of the outline node.
    *
    * English is defined as en_GB and American English as en_US. There is no 
    * default setting. If the language is not known it is set to "und"
    * (undetermined). For further information see http://www.w3.org/International/articles/language-tags/.
    * The language is specified according to RFC 3066.
    *
    * @param    strLanguage The new default language
    * @return   bool. Returns true on success, false if the call fails.
    */
    virtual bool setLanguage (const EDLString &strLanguage) = 0;

    /**
    * @brief    Retrieves the outline tree.
    * @return   IDOMOutlineTreePtr. Returns a smart pointer to the outline tree.
    */
    virtual IDOMOutlineTreePtr getOutlineTree() = 0;

};

#define edlobj2IDOMOutline(src) edl_cast((IDOMOutline *)NULL, src)


_END_EDL_DOM_NAMESPACE

#endif // __EDLIDOMOUTLINE_H__
