/*
* Copyright (C) 2008-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file    idomsecurity.h
* @ingroup IDom
*/

#ifndef __EDLIDOMSECURITY_H__
#define __EDLIDOMSECURITY_H__

#include <edl/edltypes.h>
#include <edl/edlstring.h>
#include <edl/iedlobject.h>

_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMSecurityInfo
* @ingroup  IDom
* @brief    Base DOM security class
*/
class IDOMSecurityInfo : public IEDLObject
{
};
DECL_SMART_PTR(IDOMSecurityInfo);
#define edlobj2IDOMSecurityInfo(src) edl_cast((IDOMSecurityInfo *)NULL, src)

class IDOMPDFSecurityInfo : public IDOMSecurityInfo
{
public:
    /**
    * @brief    Retrieves the name of the security handler
    */
    virtual EDLSysString getSecurityHandlerName() const = 0;
    /**
    * @brief    Retrieves encryption algorithm code (0, 1, 2, 3 or 4)
    */
    virtual int32 getAlgorithmCode() const = 0;
    /**
    * @brief    Retrieves the length of the encryption key (rnagind from 40 to 128)
    */
    virtual int32 getKeyLength() const = 0;
};
DECL_SMART_PTR(IDOMPDFSecurityInfo);
#define edlobj2IDOMPDFSecurityInfo(src) edl_cast((IDOMPDFSecurityInfo *)NULL, src)


/**
* @class    IDOMStandardPDFSecurityInfo
* @ingroup  IDom
* @brief    Represents security information from PDF Standard encryption handler
*/
class IDOMStandardPDFSecurityInfo : public IDOMPDFSecurityInfo
{
#define IDOMStandardPDFSecurityInfoClassID 0x04e1352e, 0xf6554791, 0x9d4c410a, 0xe798d64d
public:

    DECLARE_CLASS_ID(IDOMStandardPDFSecurityInfoClassID);

    /**
    * @brief    Retrieves the revision of the handler
    */
    virtual int32 getHandlerRevision() const = 0;
    /**
    * @brief    Retrieves the user access password
    */
    virtual EDLSysString getUserPassword() const = 0;
    /**
    * @brief    Returns true is document level metadata stream is to be encrypted meaningful only when algorithm code = 4
    */
    virtual bool encryptMetadata() const = 0;
    /**
    * @brief    Returns true if the owner password was used to open the PDF
    */
    virtual bool isOwnerAccess() const = 0;
    
    /**
    * @brief    Is printing allowed?
    */
    bool isPrintingAllowed() const
    {
        return ((getPermissionFlags() & ePrintAllowed) != 0);
    }
    
    /**
    * @brief    Is high quality printing allowed?
    */
    bool isHighQualityPrintingAllowed() const
    {
        // For handler version 2, high quality printing is allowed if
        // any printing is allowed
        if (getHandlerRevision() == 2)
        {
            return isPrintingAllowed();
        }
        else
        {
            return ((getPermissionFlags() & eHighQualityPrintAllowed) != 0);
        }
    }
    
    /**
    * @brief     Is modification of the contents by operations other than those specified
    *            by \link isEditingAnnotationsAllowed() \endlink, \link isFillingFormAllowed() \endlink and \link isDocAssemblyAllowed() \endlink
    *            allowed?
    */
    bool isEditingAllowed() const
    {
        return ((getPermissionFlags() & eEditingAllowed) != 0);
    }
    
    /**
    * @brief     Is copying of text and graphics from the document for operations other than
    *            specified by \link isExtractionAllowed() \endlink allowed?
    */
    bool isCopyingAllowed() const
    {
        return ((getPermissionFlags() & eCopyingAllowed) != 0);
    }
    
    /**
    * @brief     Is editing of annotations and forms alowed? 
    */
    bool isAnnotationEditingAllowed() const
    {
        return ((getPermissionFlags() & eAnnotationEditingAllowed) != 0);
    }
    
    /**
    * @brief     Is form-filling allowed?
    */
    bool isFillingFormsAllowed() const
    {
        // If annotation editing is allowed, then so is form filling
        if (isAnnotationEditingAllowed())
        {
            return true;
        }
        // Otherwise if revision 3 or greater, check bit 9
        if (getHandlerRevision() >= 3)
        {
            return ((getPermissionFlags() & eFormFillingAllowed) != 0);
        }
        else if (getHandlerRevision() == 2)
        {
            // If editing is allowed, then so is filling forms
            return isEditingAllowed();
        }
        else
        {
            // Not currently possible
            return false;
        }
    }
    
    /**
    * @brief    Is extracting for content accessibility allowed?
    */
    bool isContentAccessibilityExtractionAllowed() const
    {
        // If the handler revision is 2, then extraction is allowed if copying is allowed.
        if (getHandlerRevision() == 2)
        {
            return isCopyingAllowed();
        }
        else if (isCopyingAllowed())
        {
            // If copying is allowed, then it is inferred that extraction allowed
            return true;
        }
        else
        {
            return ((getPermissionFlags() & eContentAccessibilityExtractionAllowed) != 0);
        }
    }
    
    /**
    * @brief    Is document assembly allowed?
    */
    bool isDocumentAssemblyAllowed() const
    {
        // For handler revision 2, assembly is allowed if other editing is allowed.
        if (getHandlerRevision() == 2)
        {
            return isEditingAllowed();
        }
        else if (isEditingAllowed())
        {
            // If other editing is true, it implies that document assembly
            // is allowed.
            return true;
        }
        else
        {
            return ((getPermissionFlags() & eDocumentAssemblyAllowed) != 0);
        }
    }
    
    /**
    * @brief    Retrieve the raw permissions flags according to the PDF 1.7 spec, table 3.2.0.
    *           These should be interpreted with regard to the handler revision, as some flags
    *           are not available for all security handler revisions.
    */
    virtual uint32 getPermissionFlags() const = 0;
    
    /*! Bit values for each permission flag as available from getPermissionFlags */
    typedef enum
    {
        ePrintAllowed = 0x04UL,                           /*!< Allow printing (bit 3)*/
        eEditingAllowed = 0x08UL,                         /*!< Allow editing  (bit 4)*/
        eCopyingAllowed = 0x10UL,                         /*!< Allow copying (bit 5) */
        eAnnotationEditingAllowed = 0x20UL,               /*!< Allow annotation editing (bit 6)*/
        eFormFillingAllowed = 0x100UL,                    /*!< Allow form filling (bit 9)*/
        eContentAccessibilityExtractionAllowed = 0x200UL, /*!< Allow content extraction in support of accessibility (bit 10)*/
        eDocumentAssemblyAllowed = 0x400UL,               /*!< Allow document assembly (bit 11)*/
        eHighQualityPrintAllowed = 0x800UL,               /*!< Allow low-resolution printing (bit 12)*/
        eEverythingAllowed = 0xfffffffcUL                 /*!< All operations permitted */
    } ePermissionsFlags;

};
DECL_SMART_PTR(IDOMStandardPDFSecurityInfo);
#define edlobj2IDOMStandardPDFSecurityInfo(src) edl_cast((IDOMStandardPDFSecurityInfo *)NULL, src)

/**
  * @brief Callback typedef for the password getter.
  *
  * The callback will be called repeatedly until a valid password is supplied or
  * the callback returns false.
  *
  * @param pPriv A private pointer that is passed with each invocation
  * @param pBuff The buffer allocated by the library where the password is to be written as 
  *        a NULL terminated string. The password is expected in the Hex form.
  *        The buffer size is 65 bytes - sufficient to store a 32 byte long password
  *        in hex form as a null terminated string.
  * @return true on success, false if no valid password could be provided.
  */
typedef bool (*GetPasswordFunc) (void *pPriv, unsigned char *pBuff);


_END_EDL_DOM_NAMESPACE

#endif // __EDLIDOMSECURITY_H__
