/*
* Copyright (C) 2009-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     idomshape.h
* @ingroup  IDom
* @brief    Classes for handling Shapes
*/

#include <edl/edlnamespaces.h>
#include <edl/smartptr.h>

#ifndef EDLIDOMSHAPE_H
#define EDLIDOMSHAPE_H


// Forward declarations
_BEGIN_EDL_DOM_NAMESPACE
class IDOMShape;
DECL_SMART_PTR(IDOMShape);
_END_EDL_DOM_NAMESPACE


#include <edl/edltypes.h>
#include <edl/edlgeom.h>
#include <edl/iedlobject.h>
#include <edl/isession.h>
#include <edl/idombrush.h>
#include <edl/idompathgeometry.h>


_BEGIN_EDL_DOM_NAMESPACE

/**
* @class    IDOMShape
* @ingroup  IDom
* @brief    Interface to an IDOMShape
*
* A shape describes a scan-converted path. That is, a path that has been scanned to find
* which pixels would be affected by the path.
*
* This is useful for a number of things. Once a shape has been made from a path it may
* be compared to other shapes to test for things like intersection and occulsion using
* higher accuracy than can be obtained by simply comparing bounding boxes.
*
* A shape may be compared only with another shape scan-converted at the same resolution.
*
* As of current this interface does not allow access to the low level details of the
* scan converted result. Rather it is intended to allow for simple comparisons.
*/

class IDOMShape : public IEDLObject
{
public:
#define IDOMShapeClassID 0x57F23317, 0xA8BF49B1, 0x8EE836DD, 0x0233DC32
    /**
    * @brief    Retrieves class id of IDOMShape
    * @return   @b CClassID Class id of the element
    */
    static const CClassID &classID()
    {
        static CClassID id(IDOMShapeClassID);
        return id;
    }

    /**
    * @class    Data
    * @ingroup  IDom
    * @brief    Initialization data
    */
    class Data : public CClassParams
    {
    public:
        Data() : resolution(96.0f), privateData(NULL)
        {}
        float resolution;
        void  *privateData; /* EDL Private Shape Data */
    };
    
    /**
    * @brief Create a rectangular shape.
    */
    static EDL_API IDOMShapePtr createRect(IEDLClassFactory *pFactory, float resolution, const FRect &rect);

    /**
    * @brief Return in \c image an 1-bit-per-pixel DeviceGray image
    * representation of this shape.
    *
    * @param [out] image A reference to an IDOMImage; the caller should
    * ensure this is NULL, as this method does the work of populating
    * it with real content.
    *
    * @param [in] session The current session.
    *
    * @param [in] bUseTempFileForImage If true, the method creates a
    * temporary file (the lifecycle of which EDL manages itself) to
    * store the underlying image data in.  If false, it uses an
    * in-memory stream to store the image data.  Client code can
    * determine the size of the image that will be created using this
    * class's \c getBounds() method, and choose which value to send
    * for this flag.  Note that as the image returned is 1 bit per
    * pixel, the number of bytes required to store an image is the
    * product of the bounds divided by the size of a byte.
    *
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getAsImage(IDOMImagePtr &image,
                            ISessionPtr &session,
                            bool bUseTempFileForImage) const = 0;


    /**
    * @brief    Get the bounds (in pixels) of the shape
    * @param    [out] bounds A reference to receive the bounds of the shape.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool getBounds(FRect &bounds) const = 0;

    /**
    * @brief    Get the resolution of the shape
    * @return   @b float Returns the resolution of the shape.
    */
    virtual float getResolution() const = 0;

    /**
    * @brief    Detect an empty shape
    * @return   @b bool True if the shape is empty.
    */
    virtual bool getIsEmpty() const = 0;

    /**
    * @brief    Detect if a shape is a rectangle
    * @return   @b bool True if the shape is rectangular.
    */
    virtual bool getIsRect() const = 0;

    /**
    * @brief    Check this shape for equality to another shape.
    * @param    [in] ptrShape The shape to compare.
    * @return   @b bool True if the shapes are equivalent.
    */
    virtual bool isEqualTo(const IDOMShapePtr &ptrShape) const = 0;

    /**
    * @brief    Unite this shape with another.
    * @param    [in] ptrShape The shape with which to unite.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool unite(const IDOMShapePtr &ptrShape) = 0;

    /**
    * @brief    Reduce this shape to its intersection with another shape.
    * @param    [in] ptrShape The shape with which to intersect.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool intersect(const IDOMShapePtr &ptrShape) = 0;

    /**
    * @brief    Reduce this shape by subtracting another shape.
    * @param    [in] ptrShape The shape to subtract.
    * @return   @b bool True on success, false if the call fails.
    */
    virtual bool difference(const IDOMShapePtr &ptrShape) = 0;

    /**
    * @brief    Detect if this shape intersects with another.
    * @param    [in] ptrShape The shape to test with.
    * @return   @b bool True if the shapes intersect, false otherwise.
    */
    virtual bool intersects(const IDOMShapePtr &ptrShape) const = 0;

    /**
    * @brief    Detect if another shape fits completely inside this shape.
    * @param    [in] ptrShape The shape to test.
    * @return   @b bool True if the given shape fits completely in this shape.
    */
    virtual bool completelyContainsShape(const IDOMShapePtr &ptrShape) const = 0;
};

#define edlobj2IDOMShape(src) edl_cast((IDOMShape *)NULL, src)

_END_EDL_DOM_NAMESPACE

#endif /* EDLIDOMSHAPE_H */
