/*
* Copyright (C) 2006-2019 Global Graphics Software Ltd. All rights reserved.
*
*/

/**
* @file iedlfactory.h
* @ingroup IFactory
* @brief EDL Factory Interface allows one part of the EDL infrastructure to register
* class creation methods identified by either GUIDs and/or names (strings)
* and then another part of the EDL infrastructure to request the creation of instances of one or more of these classes
* by quoting the same GUID or name.
*/

#ifndef __EDLIFACTORY_H__
#define __EDLIFACTORY_H__

#include <edl/edltypes.h>
#include <edl/edlstring.h>
#include <edl/iedlobject.h>
#include <edl/ircobject.h>
#include <edl/objclassid.h>

_BEGIN_EDL_NAMESPACE

/**
* @class    IEDLClassFactory
* @brief    EDL Class Factory
*/
class EDL_API IEDLClassFactory
{
public:
    virtual ~IEDLClassFactory()
    {}
    typedef IEDLObjectPtr (*creatorFunc)(CClassParams *, IEDLClassFactory *);

    /**
    * @brief    Register a GUID under a string name
    * @param    strName Name for registartion
    * @param    id Class GUID
    * @param    overwrite If overwrite is true then overwites old name registration else returns false
    * @return   @b bool True on success
    */
    virtual bool registerNamedClass(const EDLSysString &strName, const CClassID &id, bool overwrite = true) = 0;

    /**
    * @brief    Retrieve GUID registred under the string name
    * @param    strName Name to find
    * @param    id Placeholder for result
    * @return   @b bool True on success
    */
    virtual bool findNamedClass(const EDLSysString &strName, CClassID &id) = 0;

    /**
    * @brief    Register a GUID with creator function
    * @param    id Class GUID
    * @param    f Function that creates the class
    * @return   @b bool True on success
    */
    virtual bool registerClass(const CClassID &id, creatorFunc f) = 0;

    /**
    * @brief    Creates the registered class by CClassID
    * @param    id Class GUID
    * @param    pParams Parameters for class initialization
    * @return   @b IEDLObjectPtr A smart pointer to EDLObject interface to newly created class
    */
    virtual IEDLObjectPtr createInstance(const CClassID &id, CClassParams *pParams = NULL) = 0;

    /**
    * @brief    Creates the EDL singleton (for example FontLibrary or ColorManager)
    * @param    id Class GUID
    * @return   @b IEDLObjectPtr A smart pointer to the IEDLObject
    */
    virtual IEDLObjectPtr getSingleton(const CClassID &id) = 0;
};

template <class T>
inline CSmartPtr<T> createInstance(IEDLClassFactory * pFactory,CClassParams *pParams = NULL)
{
    return edl_cast((T *)NULL, pFactory->createInstance(T::classID(),pParams));
}

template <class T>
inline CSmartPtr<T> createInstance(IEDLClassFactory * pFactory, const CClassID & id,CClassParams *pParams = NULL)
{
    return edl_cast((T *)NULL, pFactory->createInstance(id,pParams));
}

template <class T>
inline CSmartPtr<T> createInstance(IEDLClassFactory * pFactory, const EDLSysString &strName, CClassParams *pParams = NULL)
{
    CClassID id;
    if (pFactory->findNamedClass(strName, id))
        return edl_cast((T *)NULL, pFactory->createInstance(id, pParams));
    else
        return CSmartPtr<T>(NULL);
}

template <class T>
inline CSmartPtr<T> getSingleton(IEDLClassFactory * pFactory)
{
    return edl_cast((T *)NULL, pFactory->getSingleton(T::classID()));
}

template <class T>
inline CSmartPtr<T> getSingleton(IEDLClassFactory * pFactory, const CClassID & id)
{
    return edl_cast((T *)NULL, pFactory->getSingleton(id));
}

_END_EDL_NAMESPACE

#endif /* __EDLIFACTORY_H__ */
