/*
* Copyright (C) 2007-2019 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file     iedlobject.h
* @ingroup  IFactory
* @brief    EDL Object Interface
*/

#ifndef __EDLOBJECT_H__
#define __EDLOBJECT_H__

#include <edl/edltypes.h>
#include <edl/smartptr.h>
#include <edl/ircobject.h>
#include <edl/objclassid.h>

_BEGIN_EDL_NAMESPACE

/**
* @class    CClassParams
* @brief    When an EDL object is created via a class factory
* the created object can be passed a collection of initialization parameters.
* This collection is passed into the EDL class factory object creation method
* as a pointer to a sub-class of a CClassParams
*/

class CClassParams 
{
public:
    virtual ~CClassParams()
    {}
};

class IEDLClassFactory;
class IEDLObject;
DECL_SMART_PTR(IEDLObject);

/**
* @class    IEDLObject
* @brief    IEDLObject is an abstract base class that is used
* by all classes that are intended to be created via an EDL class factory
*
* IEDLObjects are always internally reference-counted
* so that any user of an IEDLObject does not need to consider
* memory-management/object-lifetime/object-deletion issues
*/

class IEDLObject : public IRCObject
{
public:

    /**
    * @brief    Returns class ID of IEDLObject
    * @return   CClassID Returns reference to class ID
    */
    virtual const CClassID &getClassID() const = 0;

    /**
    * @brief    The init() method is called to perform  any post-construction initialization
    * of an IEDLObject that has been created by the EDL class factory, before it is actually
    * returned by the factory.
    * @param    pData It is upto the caller/user of the class factory to construct any
    *           sub-class of CClassParams that must be supplied to this init() method
    * @return   bool Returns true on success
    */
    virtual bool init(CClassParams *pData)
    {
        UNREFERENCED_PARAMETER(pData);
        return false;
    }

    /**
    * @brief    Create a copy of EDLObject
    * @param    ptrObject Smart pointer to the source object
    * @param    pFactory Pointer to the EDL class factory
    * @return   bool Returns true on success
    */
    virtual bool clone(IEDLObjectPtr &ptrObject, IEDLClassFactory *pFactory)
    {
        UNREFERENCED_PARAMETER(ptrObject);
        UNREFERENCED_PARAMETER(pFactory);
        return false;
    }
};

template<class Type>
CSmartPtr<Type> edl_cast(Type *dst, IEDLObjectPtr src)
{
    UNREFERENCED_PARAMETER(dst);
    if (src)
        return CSmartPtr<Type>((Type *)dynamic_cast<Type *>((IEDLObject *)src), true);
    else 
        return CSmartPtr<Type>();
};


/**
* @brief	Convenience template to clone a node, returning a reference to the clone
*			using the original type. Throws an IEDLError on failure.
* @param	src Smart pointer to the source node
* @param    pFactory Pointer to the EDL class factory
* @return   <b>result</b> Smart pointer to the cloned node object
*/
template <class Type>
CSmartPtr<Type> clone(const CSmartPtr<Type> &src, IEDLClassFactory *pFactory)
{
    extern void throwCloneError();
    IEDLObjectPtr cloned;
    if (!src || !pFactory || !src->clone(cloned, pFactory) || !cloned)
        throwCloneError();
    CSmartPtr<Type> result = edl_cast((Type*) NULL, cloned);
    if (!result)
        throwCloneError();
    return result;
}

_END_EDL_NAMESPACE

#endif /* __EDLOBJECT_H__ */
