/*
* Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
*/

/**
* @file isession.h
* @ingroup IController
*/

#ifndef __ISESSION_H__
#define __ISESSION_H__

#include <edl/iedlobject.h>
#include <edl/iedlfactory.h>
#include <edl/edlfwd.h>
#include <edl/iedltempstore.h>

_BEGIN_EDL_NAMESPACE

/**
 * @class   ISession
 * @ingroup Controller
 * @brief   EDL session class
 */
class ISession : public IEDLObject
{
#define ISessionClassID 0xbe1d3ca8, 0x6c6e46ee, 0xa9ff2d08, 0x0c883055
public:

    DECLARE_CLASS_ID(ISessionClassID);

    /**
    * @brief    initializes Session by setting EDL class Factory
    * @param    pFactory Ptr to EDL Class Factory
    * @return   bool Returns true on success
    */
    virtual bool setFactory(IEDLClassFactory *pFactory) = 0;

    /**
    * @brief    EDL Class Factory getter method
    * @return   IEDLClassFactory* Returns pointer to EDL class factory
    */
    virtual IEDLClassFactory * getFactory() = 0;

    /**
    * @brief Obtain the session's message handler.
    *
    * Each session has a message handler, used to deal with messages sent
    * between various components being used in the context of a particular
    * session. This returns a pointer to a session's message handler.
    *
    * @return    IMessageHandlerPtr Smart pointer to a message handler object.
    */
    virtual IMessageHandlerPtr getMessageHandler() = 0;

    /**
    * @brief Obtain the session's litemessage handler.
    *
    * Each session also contains a litemessage handler, used to deal with litemessages.
    * The Litemessage framework will eventually be taken over by the proper messaging framework.
    *
    * @return    ILiteMessageHandlerPtr Returns Smart pointer to a litemessage handler object.
    */
    virtual ILiteMessageHandlerPtr getLiteMessageHandler() = 0;

    /**
    * @brief    Set the temporary directory to be used by EDL and filters for this session.
    * @param    sTempDirectory The path to the temp directory. This directory must exist and
    *           the path must not be zero length. Note that for safety, this should be an
    *           absolute path.
    * @return   bool Returns true on success.
    */
    virtual bool setTemporaryDirectory(const EDLSysString &sTempDirectory) = 0;

    /**
    * @brief    Get the temporary directory to be used by EDL and filters for this session.
    * @param    sTempDirectory A reference to receive path to the temp directory. 
    * @return   bool Returns true on success.
    */
    virtual bool getTemporaryDirectory(EDLSysString &sTempDirectory) = 0;
    
    /**
    * @brief    Get the temporary store for this session.
    *           The temporary directory must be set before calling this member.
    * @param    tmpStore a reference to receive the temp store.
    * @return   bool Returns true on success.
    */
    virtual bool getTempStore(IEDLTempStorePtr &tempStore) = 0;
    
    /**
    * @brief    Convenience method to get the temporary store.
    *           Throws an IEDLError on failure.
    * @return   IEDLTempStorePtr The temp store.
    */
    virtual IEDLTempStorePtr getTempStore() = 0;

    /**
    * @brief    Set the startup directory. This is meaningfull for executable environments. Note the client must set this during initalisation for it to be valid.
    * @param    sStartupDirectory The path of the directory that the executable started.
    * @return   bool Returns true on success.
    */
    virtual bool setStartupDirectory(const EDLSysString &sStartupDirectory) = 0;

    /**
    * @brief    Get the startup directory. 
    * @param    sStartupDirectory The path of the directory that the execuatble started.
    * @return   bool Returns true on success.
    */
    virtual bool getStartupDirectory(EDLSysString &sStartupDirectory) = 0;
};

DECL_SMART_PTR(ISession);
#define edlobj2ISession(src) edl_cast((ISession *)NULL, src) 


/**
 * @class   IRunnable
 * @ingroup Controller
 * @brief   Interface to filter's runnable classes
 */
class IRunnable : public virtual IEDLObject
{
#define IRunnableClassID 0x5ec5e587, 0xdd2b45c9, 0x83464637, 0xe5a89832

public:
    DECLARE_CLASS_ID(IRunnableClassID);

    typedef enum 
    {
        rcError = -1,
        rcFinished = 0,
        rcContinue
    } RunCode;

public:
    /**
    * @brief    Run runnable object
    * @return   RunCode Returns run code
    */
    virtual RunCode run() = 0;
};

DECL_SMART_PTR(IRunnable);
#define edlobj2IRunnable(src) edl_cast((IRunnable *)NULL, src)

/**
* @brief    Sets a directory that is used as a root
*           for temporary directories of the EDL sessions.
*           For every new session a unique subfolder under this root
*           directory will be created and used as this session's 
*           temp folder.
* @param    sTempRootDirectory temp root directory
* @return   bool Returns true on success
*/
EDL_API bool setSessionTempRootDirectory(const EDLSysString &sTempRootDirectory);

_END_EDL_NAMESPACE

#endif //__ISESSION_H__
