/*
 * Copyright (C) 2007 Global Graphics Software Ltd. All rights reserved.
 */

/**
 * @file smartptr.h
 * @ingroup IController
 * @brief EDL smart pointers which, in conjunction with the IRCOject class
 * provide reference-counted and automatically garbage-collected IEDLObjects
 * which are typically returned by one of the EDL class factories
 */

#ifndef __cplusplus
#error This file is intended for use with C++ only
#endif

#ifndef __CSMARTPTR_H
#define __CSMARTPTR_H

#include <stddef.h>
#include <edl/edlnamespaces.h>

_BEGIN_EDL_NAMESPACE

template <class T>
class CSmartPtr
{
public:
    explicit CSmartPtr(T *src = NULL): m_ptr(src)
    {
    }
    explicit CSmartPtr(T *src , bool bAddref): m_ptr(src)
    {
        if (bAddref && m_ptr)
            m_ptr->addRef();
    }
    template <class T1>
    CSmartPtr<T1> getPtr(T1 *fake) const
    {
        return CSmartPtr<T1>((T1*)m_ptr, true);
    }

    template <class T1>
    CSmartPtr(const CSmartPtr<T1> &src): m_ptr(src.operator T1 *())
    {
        if (m_ptr)
            m_ptr->addRef();
    }
    CSmartPtr(const CSmartPtr<T> &src): m_ptr(src.operator T *())
    {
        if (m_ptr)
            m_ptr->addRef();
    }
    ~CSmartPtr()
    {
        if (m_ptr)
            m_ptr->decRef();
    }

    template <class T1>
    CSmartPtr<T> &operator=(const CSmartPtr<T1> &src)
    {
        if (m_ptr)
            m_ptr->decRef();
        if ((m_ptr = src.operator T1 *()))
            m_ptr->addRef();
        return *this;
    }

    CSmartPtr<T> &operator=(const CSmartPtr<T> &src)
    {
        T * ptr = m_ptr;
        if ((m_ptr = src.m_ptr))
            m_ptr->addRef();
        if (ptr)
            ptr->decRef();
        return *this;
    }
    CSmartPtr &operator=(T *src)
    {
        if (m_ptr)
            m_ptr->decRef();
        m_ptr = src;
        return *this;
    }

    operator T*() const
    {
        return m_ptr;
    }
    T* operator->() const
    {
        return m_ptr;
    }

protected:
    T* m_ptr;
};

template <class T>
class CConstSmartPtr
{
public:
    explicit CConstSmartPtr(const T *src = NULL): m_ptr((T*)src)
    {
        if (m_ptr)
            m_ptr->addRef();
    }
    explicit CConstSmartPtr(const T *src , bool bAddref): m_ptr((T *)src)
    {
        if (bAddref && m_ptr)
            m_ptr->addRef();
    }
    template <class T1>
    CConstSmartPtr<T1> getConstPtr(T1 *fake)
    {
        return CConstSmartPtr<T1>((T1*)m_ptr, true);
    }
    template <class T1>
    CSmartPtr<T1> getPtr(T1 *fake)
    {
        return CSmartPtr<T1>((T1*)m_ptr, true);
    }
    CSmartPtr<T> unConst()
    {
        return CSmartPtr<T>((T*)m_ptr, true);
    }

    template <class T1>
    CConstSmartPtr(const CConstSmartPtr<T1> &src): m_ptr((T1 *)(src.operator const T1*()))
    {
        if (m_ptr)
            m_ptr->addRef();
    }
    template <class T1>
    CConstSmartPtr(const CSmartPtr<T1> &src): m_ptr(src.operator T1 *())
    {
        if (m_ptr)
            m_ptr->addRef();
    }
    CConstSmartPtr(const CConstSmartPtr<T> &src): m_ptr((T *)(src.operator const T*()))
    {
        if (m_ptr)
            m_ptr->addRef();
    }
    ~CConstSmartPtr()
    {
        if (m_ptr)
            m_ptr->decRef();
    }

    CConstSmartPtr &operator=(const CSmartPtr<T> &src)
    {
        T * ptr = m_ptr;
        if ((m_ptr = src.operator T *()))
            m_ptr->addRef();
        if (ptr)
            ptr->decRef();
        return *this;
    }

    CConstSmartPtr &operator=(const CConstSmartPtr<T> &src)
    {
        T * ptr = m_ptr;
        if ((m_ptr = src.m_ptr))
            m_ptr->addRef();
        if (ptr)
            ptr->decRef();
        return *this;
    }
    CConstSmartPtr &operator=(const T *src)
    {
        if (m_ptr)
            m_ptr->decRef();
        m_ptr = (T *)src;
        return *this;
    }
    operator const T*() const
    {
        return m_ptr;
    }
    const T* operator->() const
    {
        return m_ptr;
    }

protected:
    T* m_ptr;
};

#define DECL_SMART_PTR(cls) typedef CSmartPtr<cls> cls##Ptr; typedef CConstSmartPtr<cls> cls##ConstPtr;

_END_EDL_NAMESPACE

#endif // !__CSMARTPTR_H
