/*
* Copyright (C) 2008-2019 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef _JAWSMAKO_H_
#define _JAWSMAKO_H_

/**
* @defgroup JawsMako API
* @brief    The JawsMako API.
*
* JawsMako encompasses an API that is intended to provide a
* simpler and more flexible interface than that provided by
* its predecessor, EDL. It allows for simplified access to 
* supported input types, providing easy APIs for accessing 
* document assemblies, documents and pages, and for performing
* common tasks. It also provides methods of managing the
* amount of memory used by JawsMako.
*
* Errors are handled through exceptions of type JawsMako::IError
* which is currently a synonym for EDL::IEDLError.
*
* This API will be extended in future JawsMako releases.
* @{
*/

/**
* @file jawsmako.h
* @brief The JawsMako library API
*/

#include <edl/iabort.h>
#include <jawsmako/types.h>
#include <jawsmako/interactive.h>
#include <jawsmako/optionalcontent.h>

namespace JawsMako
{
    using namespace EDL;

	/*! Document format enumeration */
    typedef enum
    {
        eFFPDF,     /*!< PDF */
        eFFXPS,     /*!< XPS */
        eFFPS,      /*!< PostScript */
        eFFSVG,     /*!< SVG */
        eFFPCL5,    /*!< PCL5 */
        eFFPCLXL,   /*!< PCL/XL */
        eFFUnknown  /*!< None of the above */
    } eFileFormat;

    /**
    * @class    IJawsMako
    * @brief    An instance of the IJawsMako library.
    *           Only one instance of this object is currently allowed.
    *           This class may also be used as both an EDL factory
    *           and an EDL session, and passed to any EDL API that
    *           requires these objects.
    */
    class IJawsMako : virtual public IEDLClassFactory, virtual public ISession
    {
      public:
        virtual ~IJawsMako() {}
        
        /**
        * @brief    Create an IJawsMako instance.
        *           Only one may be created at any one time.
        *
        * @param    libDir the absolute path to the directory containing the
        *           support libraries (ie the corerip and edlpspdfout libraries
        *           and executables). May be set to an empty string if these
        *           components are not used (such as on iOS or Android).
        * @param    tempDir An absolute  path to a directory JawsMako can use to store
        *           temporary files. If a non-empty string is passed, the directory
        *           must exist. If an empty string is passed, JawsMako will choose an
        *           appropriate temporary location based on the host operating system.
        * @param    cacheDir An absolute path to a directory where JawsMako may
        *           persistently store cached information. If an empty string is
        *           passed, JawsMako will choose an appropriate location based on the
        *           host operating system. If a directory is passed it must exist.
        * @param    tempStoreParams The desired temporary store parameters. See
         *           #CTemporaryStoreParams for details.
        *
        * @return   IJawsMakoPtr the new Instance.
        */
        static JAWSMAKO_API IJawsMakoPtr create(const U8String &libDir = U8String(""),
                                                const U8String &tempDir = U8String(""),
                                                const U8String &cacheDir = U8String(""),
                                                const CTemporaryStoreParameters &tempStoreParams = CTemporaryStoreParameters());

        /**
         * @brief   Enable PDF Input for JawsMako. Not available on iOS or Android
         */
        static JAWSMAKO_API void enablePDFInput(const IJawsMakoPtr &jawsMako);

        /**
         * @brief   Enable PDF Output for JawsMako.
         *          Requires the following additional libraries to be linked on iOS
         *           - libedlpspdfout (iOS only)
         */
        static JAWSMAKO_API void enablePDFOutput(const IJawsMakoPtr &jawsMako);

        /**
         * @brief   Enable PostScript Output for JawsMako.
         *          Requires the following additional libraries to be linked on iOS
         *           - libedlpspdfout (iOS only)
         */
        static JAWSMAKO_API void enablePSOutput(const IJawsMakoPtr &jawsMako);

        /**
         * @brief   Enable all JawsMako features
         */
        static JAWSMAKO_API void enableAllFeatures(const IJawsMakoPtr &jawsMako)
        {
            enablePDFInput(jawsMako);
            enablePDFOutput(jawsMako);
            enablePSOutput(jawsMako);
        }

        /**
        * @brief    Inform JawsMako that system memory is being exhausted.
        *           This will cause JawsMako to free up cached information and discard
        *           any unmodified pages.
        */
        virtual void freeMemory() = 0;
        
        /**
        * @brief    Find a font from the system fonts, and the font index if applicable.
        *           
        *           Only TrueType, TrueType collections or OpenType/CFF fonts are supported.
        *           Matching is currently exact; there is no attempt to find a font that is
        *           similarly named. Fonts are matched based on their Full or PostScript
        *           font names.
        *
        *           The list of fonts on the system is cached and refreshed once per
        *           JawsMako instance creation.
        *
        *           An IError of code JM_ERR_FONT_NOT_FOUND will be thrown if the font
        *           could not be found.
        *
        * @param    fontName The font name of the desired font.
        * @param    fontIndex If the requested font is part of a TrueType collection, this
        *           will be set to the index of the font within the collection, or 0 otherwise.
        * @return   IDOMFontPtr The found font.
        */
        virtual IDOMFontPtr findFont(const U8String &fontName, uint32 &fontIndex) = 0;
    };

    /**
    * @class    IInput
    * @brief    Abstract input source that can open files from disk or a stream
    *           and create an IDocumentAssembly for the contents.
    */
    class IInput : public IRCObject
    {
      public:
        virtual ~IInput() {}

        /**
         * @brief	Create an input for reading source documents in the
         *          given format. The following formats are currently
		 *			supported:
		 *          - PDF
         *          - XPS
		 *          - PCL5
		 *          - PCL/XL
         *
         * @return  IInputPtr the input
         */
        static JAWSMAKO_API IInputPtr create(const IJawsMakoPtr &jawsMako, eFileFormat format, const IAbortPtr &abort = IAbortPtr ());

        /**
        * @brief    Open a file on disk, returning the IDocumentAssembly
        *           representing the contents.
        * @return   IDocumentAssemblyPtr the document assembly.
        */
        virtual IDocumentAssemblyPtr open(const U8String &pathToFile) = 0;

        /**
        * @brief    Open a file on disk, returning the IDocumentAssembly
        *           representing the contents. Takes a wide character string.
        * @return   IDocumentAssemblyPtr the document assembly.
        */
        virtual IDocumentAssemblyPtr open(const String &pathToFile) = 0;

        /**
        * @brief    Open a stream, returning the IDocumentAssembly
        *           representing the contents.
        * @return   IDocumentAssemblyPtr the document assembly.
        */
        virtual IDocumentAssemblyPtr open(const IInputStreamPtr &inputStream) = 0;
        
    };
    


    /**
    * @class    IOutput
    * @brief    Abstract output sink that can output DOM to a file or stream
    *           in a given output format.
    */
    class IOutput : public IRCObject
    {
      public:
        virtual ~IOutput() {}

        /**
        * @brief   Create an output for writing source in the given format.
        *          
        *          The following formats are currently supported:
        *          - PDF
        *          - XPS
        *          - PostScript
        *          - PCL/XL
        *          - PCL5e
        *          - PCL5c
        *
        * @return  IOutputPtr the new output
        */
        static JAWSMAKO_API IOutputPtr create(const IJawsMakoPtr &jawsMako, eFileFormat format, const IAbortPtr &abort = IAbortPtr ());
        
        /*
         * Configuration
         */
        
        /**
        * @brief   Configure the output according to a general preset.
        *          Please see the supplied documentation for details of these presets.
        *          The default is "Preserve" which will attempt to produce output as
        *          close to the input as possible for the output format. A string
        *          value can be used for any parameter and will be converted as
        *          necessary.
        */
        virtual void setPreset(const U8String &preset) = 0;
        
        /**
        * @brief   Apply a key value pair output parameter with a string value.
        *          The parameter name is case insensitive.
        *          Please refer to the supplied documentation for the details of the
        *          available parameters and their ranges.
        */
        virtual void setParameter(const U8String &param, const U8String &value) = 0;

        /**
        * @brief   Control whether or not assemblies with certain security permission
        *          flags are allowed to be written by this output.
        *
        *		   If allowed permissions is set to \link IDOMStandardPDFSecurityInfo::eEverythingAllowed \endlink
        *          then no checking for assembly permissions will be performed.
        *
        *          Otherwise, the parameter is the or'd combination of
        *          IDOMStandardPDFSecurity::ePermissionsFlags that is checked against
        *          the operations allowed by the assembly's security (from
        *          \link IAssembly::getSecurityInfo()) \endlink. The assembly allows an operation
        *          corresponding to a flag set in the parameter, then output will be
        *          allowed to continue. Otherwise, an IError exception with the error
        *          code JM_ERR_ASSEMBLY_WRITE_FORBIDDEN will be thrown.
        *
        *          If the source assembly has no permissions information, or shows that
        *          the owner password was supplied, the output will proceed regardless.
        *
        *          The default depends on the individual output. For print-centric outputs
        *          (PostScript, PCL5, PCLXL) the default is 
        *          \link IDOMStandardPDFSecurityInfo::eHighQualityPrintAllowed \endlink. That is,
        *          permission-protected jobs that are permitted to print to high
        *          resolution will be allowed to convert. For all other outputs, the
        *          default is 0, which means that if any permissions information is
        *          present and the owner password was not supplied, then no output will be
        *          allowed in order to preserve the intentions of the input assembly.
        *
        *          For example, if the requirement is to generate output to be fed to
        *          a printer, then calling:
        *          <code>setAllowedPermissions(IDOMStandardPDFSecurityInfo::eHighQualityPrintAllowed)</code>
        *          would allow permission-protected jobs that allow high quality output
        *          to proceed.
        *
        *          Note: only permissions specified in the standard security handler
        *          Revisions 2 and 3 are currently checked.
        *
        *          Equivalent to calling setParameter() with the param name "AllowedPermissions".
        */
        virtual void setAllowedPermissionsFlags(uint32 allowedPermissions) = 0;
        
        
        /**
        * @brief   A callback type for receiving progress information from 0.0 (starting) through 1.0 (complete)
        */
        typedef void (*ProgressCallbackFunc)(void *priv, float progress);

        /**
        * @brief    Write the given document assembly to a file on disk.
        * @param    pathToFile The path to the output file on disk
        * @param    progressCallback An optional function pointer to be called after
        *           each page has been written providing progress information.
        * @param    progressPriv A private pointer passed to the progress callback for
        *           each call.
        */
        virtual void writeAssembly(const IDocumentAssemblyPtr &assembly,
                                   const U8String &pathToFile,
                                   ProgressCallbackFunc progressFunc = NULL, void *progressPriv = NULL) = 0;

        /**
        * @brief    Write the given document assembly to a file on disk, specified
        *           by a wide character string.
        * @param    pathToFile The path to the output file on disk
        * @param    progressCallback An optional function pointer to be called after
        *           each page has been written providing progress information.
        * @param    progressPriv A private pointer passed to the progress callback for
        *           each call.
        */
        virtual void writeAssembly(const IDocumentAssemblyPtr &assembly,
                                   const String &pathToFile,
                                   ProgressCallbackFunc progressFunc = NULL, void *progressPriv = NULL) = 0;

        /**
        * @brief    Write the given document assembly to a stream.
        * @param    stream The stream to write to.
        * @param    progressCallback An optional function pointer to be called after
        *           each page has been written providing progress information.
        * @param    progressPriv A private pointer passed to the progress callback for
        *           each call.
        */
        virtual void writeAssembly(const IDocumentAssemblyPtr &assembly,
                                   const IOutputStreamPtr &stream,
                                   ProgressCallbackFunc progressFunc = NULL, void *progressPriv = NULL) = 0;

        /**
        * @brief    Create an output writer for the given assembly, targeting a file on disk.
        *           This is designed to allow streaming output, or to deal with situations where an
        *           operation would require too much memory to hold an entire edited assembly in memory
        *           at once.
        * @param    assembly The document assembly for output. The assembly need not be populated with documents.
        * @param    pathToFile The path to the output file on disk.
        */
        virtual IOutputWriterPtr openWriter(const IDocumentAssemblyPtr &assembly, const U8String &pathToFile) = 0;
        
        /**
        * @brief    Create an output writer for the given assembly, targeting a file on disk.
        *           As above, but with the file specified in a wide character string.
        * @param    assembly The document assembly for output. The assembly need not be populated with documents.
        * @param    pathToFile The path to the output file on disk.
        * @return   IOutputWriterPtr The new output writer.
        */
        virtual IOutputWriterPtr openWriter(const IDocumentAssemblyPtr &assembly, const String &pathToFile) = 0;

        /**
        * @brief    Create an output writer for the given assembly, targeting a stream.
        * @param    assembly The document assembly for output. The assembly need not be populated with documents.
        * @param    stream The stream for the output.
        * @return   IOutputWriterPtr The new output writer.
        */
        virtual IOutputWriterPtr openWriter(const IDocumentAssemblyPtr &assembly, const IOutputStreamPtr &stream) = 0;
    };
    


    /**
    * @class    IOutputWriter
    *
    * @brief    A writer for writing individual pages and documents to an output in piecemeal fashion.
    */
    class IOutputWriter : public IRCObject
    {
      public:
        virtual ~IOutputWriter() {}
        
        /**
        * @brief    Begin a document. This document need not be completely populated with pages.
        *
        *           Once this has been invoked, pages may be written using writePage().
        */
        virtual void beginDocument(const IDocumentPtr &document) = 0;
        
        /**
        * @brief    Finish a document.
        */
        virtual void endDocument() = 0;
        
        /**
        * @brief    Write a page to the output
        *
        *           beginDocument() must have been called to begin the document before any pages
        *           may be written. The page may be discarded after this is called.
        * @param    page  The page to write.
        */
        virtual void writePage(const IPagePtr &page) = 0;
        
        /**
        * @brief    Finish output and flush the results.
        */
        virtual void finish() = 0;
    };
    
    /**
    * @class    IDocumentAssembly
    * @brief    A self contained collection of IDocuments.
    */
    class IDocumentAssembly : public IRCObject
    {
      public:
        virtual ~IDocumentAssembly() {}

        /**
        * @brief    Create a new, empty document assembly.
        * @return   IDocumentAssemblyPtr the new assembly.
        */
        static JAWSMAKO_API IDocumentAssemblyPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());

        /**
        * @brief    Return the number of documents in the assembly, if known.
        * @return   uint32 The number of pages in the assembly.
        */
        virtual uint32 getNumDocuments() = 0;

        /**
        * @brief    Get the IDocument from the assembly at the given index.
        * @return   IDocumentPtr the requested document, 0 being the first
        *            document.
        */
        virtual IDocumentPtr getDocument(uint32 index = 0) = 0;
        
        /**
        * @brief    Insert a document into the assembly at the given index.
        * @param    document smart pointer to the document to be inserted.
        * @param    index The position in the assembly to insert the document,
        *           0 being the first position.
        */
        virtual void insertDocument(const IDocumentPtr &document, uint32 index = 0) = 0;
        
        /**
        * @brief    Remove the document at the given index.
        * @param    index The index of the page to be removed (0 being the
        *           first document).
        */
        virtual void removeDocument(uint32 index) = 0;

        /**
        * @brief    Remove the given document from the document. If the
        *           document is not present, an exception will be raised.
        */
        virtual void removeDocument(const IDocumentPtr &document) = 0;
        
        /**
        * @brief    Append a document to the assembly.
        * @param    page smart pointer to the page to be appended.
        */
        virtual void appendDocument(const IDocumentPtr &document) = 0;

        /**
        * @brief    Get the overall job ticket for the entire assembly, if present.
        * @return   IDOMJobTkPtr the job ticket, or NULL if not present
        */
        virtual IDOMJobTkPtr getJobTicket() const = 0;

        /**
        * @brief    Set the document job ticket.
        */
        virtual void setJobTicket(const IDOMJobTkPtr &jobTicket) = 0;

        /**
        * @brief    Find the page contatining the target with the given DOMid in the
        *           assembly, providing the index of the document and page within the
        *           assembly.
        *           Throws an IError if the target could not be found.
        */
        virtual void findTarget(DOMid targetId, uint32 &docNum, uint32 &pageNum) = 0;

        /**
        * @brief    Find the annotation with the given annotation reference within the
        *           document assembly.
        *           Throws an IError if the target could not be found.
        */
        virtual IAnnotationPtr findAnnotation(const IAnnotationReferencePtr &reference) = 0;
        
        /**
        * @brief    Find the annotation with the given annotation reference within the
        *           document assembly, providing the index of the document and page
        *           that contains the annotation.
        *           Throws an IError if the target could not be found.
        */
        virtual IAnnotationPtr findAnnotation(const IAnnotationReferencePtr &reference, uint32 &docNum, uint32 &pageNum) = 0;

        /**
        * @brief    Clone an IDocumentAssembly.
        *           Will also clone all the documents and pages in the assembly.
        * @return   IDocumentAssemblyPtr the clone.
        */
        virtual IDocumentAssemblyPtr clone() = 0;
        
        /**
        * @brief    Get the job metadata, if present (ie the document properties).
        * @return   IDOMMetadataPtr. Returns NULL if no such information
        *           was provided by the input.
        */
        virtual IDOMMetadataPtr getJobMetadata() const = 0;

        /**
        * @brief    Set the job metadata.
        */
        virtual void setJobMetadata(const IDOMMetadataPtr &metadata) = 0;
        
        /**
        * @brief    Get the PDF XMP packet, if present.
        * @return   IInputStreamPtr. Returns NULL if the XMP packet is not present.
        */
        virtual IInputStreamPtr getXmpPacket() const = 0;
        
        /**
        * @brief    Set the PDF XMP packet, if present.
        *           Note that the XMP packet in any PDF output will be modified to
        *           match the job metadata for compliance with specification.
        *           If NULL, the XMP packet will be reset.
        */
        virtual void setXmpPacket(const IInputStreamPtr &xmpPacket) = 0;
        
        /**
        * @brief    Get the security information that applied to the source
        *           file/stream. Curently only relevant for PDF.
        * @return   IDOMPDFSecurityInfoPtr. Returns NULL if no security information
        *           was provided by the input.
        */
        virtual IDOMSecurityInfoPtr getSecurityInfo() const = 0;
        
        /**
        * @brief    Get the thumbnail image for the assembly.
        * @return   IDOMImagePtr. Returns NULL if no thumbnail image is present.
        */
        virtual IDOMImagePtr getThumbnail() const = 0;
        
        /**
        * @brief    Sets a new thumbnail image for the document assembly.
        *           The image must be in either JPEG or PNG format. Setting thumbnail to NULL deletes the document assembly thumbnail.
        * @param    thumbnail Smart pointer to new thumbnail image.
        */
        virtual void setThumbnail(const IDOMImagePtr &thumbnail) = 0;
    };


    /**
    * @class    IDocument
    * @brief    A document from an IDocumentAssembly, allowing for high
    *           level document and page mangement, and providing on-demand
    *           lazy loading of page markup.
    */
    class IDocument : public IRCObject
    {
      public:
        virtual ~IDocument() {}

        /**
        * @brief    Create an empty document.
        * @return   IDocumentPtr the new page.
        */
        static JAWSMAKO_API IDocumentPtr create(const IJawsMakoPtr &jawsMako);

        /**
        * @brief    Return the number of pages in the document, if known.
        * @return   uint32 The number of pages in the document.
        */
        virtual uint32 getNumPages() = 0;

        /**
        * @brief    Get the IPage from the document at the given index.
        * @return   IPagePtr the requested page, 0 being the first page.
        */
        virtual IPagePtr getPage(uint32 index = 0) = 0;

        /**
        * @brief    Insert a page into the document at the given index.
        *           Note that the page will not be cloned, and so any changes
        *           to the added page will affect all users of the page.
        *           The interactive forms will be updated with any widget
        *           annotations present on the page. Note that if the names of
        *           any widgets being added clash with existing widgets or fields
        *           they will be forcibly renamed. Provide a sourceDocument
        *           if possible for improved form merging.
        * @param    page smart pointer to the page to be inserted.
        * @param    index The position in the document to insert the page,
        *           0 being the first position.
        * @param    sourceDocument (optional) the source document that the page
        *           came from. If provided, the target document's outline will
        *           be update with any outline entries present on the inserted
        *           page.
        */
        virtual void insertPage(const IPagePtr &page, uint32 index = 0, const IDocumentPtr &sourceDocument = IDocumentPtr(NULL)) = 0;
        
        /**
        * @brief    Append a page to the document
        * @param    page smart pointer to the page to be inserted.
        * @param    sourceDocument (optional) the source document that the page
        *           came from. If provided, the target document's outline will
        *           be update with any outline entries present on the inserted
        *           page.
        *           The interactive forms will be updated with any widget
        *           annotations present on the page. Note that if the names of
        *           any widgets being added clash with existing widgets or fields
        *           they will be forcibly renamed. Provide a sourceDocument
        *           if possible for improved form merging.
        */
        virtual void appendPage(const IPagePtr &page, const IDocumentPtr &sourceDocument = IDocumentPtr(NULL)) = 0;
        
        /**
        * @brief    Remove the page at the given index.
        * @param    index The index of the page to be removed (0 being the
        *           first page)
        */
        virtual void removePage(uint32 index) = 0;
        
        /**
        * @brief    Remove the given page from the document. If the page is
        *           not present, ane exception will result.
        */
        virtual void removePage(const IPagePtr &page) = 0;

        /**
        * @brief    Find the page contatining the target with the given DOMid in the
        *           document, providing the index of the page within the document.
        *           Throws an IError if the target could not be found.
        */
        virtual void findTarget(DOMid targetId, uint32 &pageNum) = 0;

        /**
        * @brief    Find the annotation with the given annotation reference within the
        *           document.
        *           Throws an IError if the target could not be found.
        */
        virtual IAnnotationPtr findAnnotation(const IAnnotationReferencePtr &reference) = 0;
        
        /**
        * @brief    Find the annotation with the given annotation reference within the
        *           document, providing the index of the page that contains the annotation.
        *           Throws an IError if the target could not be found.
        */
        virtual IAnnotationPtr findAnnotation(const IAnnotationReferencePtr &reference, uint32 &pageNum) = 0;
                
        /**
        * @brief    Get the document job ticket, if present.
        * @return   IDOMJobTkPtr the job ticket, or NULL if not present
        */
        virtual IDOMJobTkPtr getJobTicket() const = 0;
        
        /**
        * @brief    Set the document job ticket.
        */
        virtual void setJobTicket(const IDOMJobTkPtr &jobTicket) = 0;
        
        /**
        * @brief    Get the output intents, if present.
        */
        virtual COutputIntentVect getOutputIntents() const = 0;
        
        /**
        * @brief    Get the optional content if present.
        */
        virtual IOptionalContentPtr getOptionalContent() const = 0;
        
        /**
        * @brief    Set the optional content for the document, or NULL if if
        *           should be removed.
        */
        virtual void setOptionalContent(const IOptionalContentPtr &optionalContent) = 0;
        
        /**
        * @brief    Get the structure information if present.
        */
        virtual IStructurePtr getStructure() const = 0;
        
        /**
         * @brief    Set the structure content for the document, or NULL if if
         *           should be removed.
         */
        virtual void setStructure(const IStructurePtr &structure) = 0;
        
        /**
        * @brief    Get the document outline, if present.
        * @return   IDOMOutlinePtr the document outline, or NULL if not present.
        */
        virtual IDOMOutlinePtr getOutline() const = 0;

        /**
        * @brief    Set the document outline, if present.
        * @param    outline The desired document outline, or NULL to set no outline.
        */
        virtual void setOutline(const IDOMOutlinePtr &outline) = 0;

        /**
        * @brief    Get the document interactive form, if present.
        * @return   IFormPtr The document form, or NULL if not present.
        */
        virtual IFormPtr getForm() const = 0;
        
        /**
        * @brief    Set the document interactive form. May be NULL.
        */
        virtual void setForm(const IFormPtr &form) = 0;
        
        /**
        * @brief    Get any embedded streams or attachments attached to the document.
        *           This is currently a PDF-specific feature.
        */
        virtual CFileSpecAsEmbeddedDataVect getEmbeddedStreams() = 0;
        
        /**
        * @brief    Add an embedded file stream to the document.
        *           This is currently a PDF-specific feature.
        */
        virtual void addEmbeddedStream(const IFileSpecAsEmbeddedDataPtr &embeddedData) = 0;
        
        /**
        * @brief    Get any named destinations present in the document.
        */
        virtual CNamedDestinationVect getNamedDestinations() = 0;
        
        /**
        * @brief	Add a named destination to the document.
        *           Note that this will override any named destination with the same name
        *           that may be present.
		* @param	namedDestination The named destination to be added
        */
        virtual void addNamedDestination(const INamedDestinationPtr &namedDestination) = 0;
        
        /**
        * @brief    Replace the named destinations in the document with the given vector.
        *           Note that if there are multiple destinations with the same name,
        *           the results are undefined.
		* @param	namedDestinations The vector of named destinations
        */
        virtual void setNamedDestinations(const CNamedDestinationVect &namedDestinations) = 0;
        
        /**
        * @brief    Get the document threads, if present.
        * @return   IThreadsPtr The document threads, if present.
        */
        virtual IThreadsPtr getThreads() const = 0;
        
        /**
        * @brief    Clone an IDocument.
        *           Will also clone all the pages in the document.
        * @return   IDocumentPtr The clone.
        */
        virtual IDocumentPtr clone() = 0;
        
    };

    
    /**
    * @class    IPage
    * @brief    A page from an IDocument, allowing
    *           high level page management, and providing
    *           on-demand access to page contents.
    */
    class IPage : public IRCObject
    {
      public:
        virtual ~IPage() {}

        /**
        * @brief    Create an empty page
        * @return   IPagePtr the new page.
        */
        static JAWSMAKO_API IPagePtr create(const IJawsMakoPtr &jawsMako);
        
        /**
        * @brief    Get the page (media) width of the page.
        *           This does not require the page contents to be
        *           loaded.
        */
        virtual double getWidth() = 0;
        
        /**
        * @brief    Get the page (media) height of the page.
        */
        virtual double getHeight() = 0;
        
        /**
        * @brief    Get the CropBox for this page.
        *           The returned rect is relative to the top left of the page.
        */
        virtual FRect getCropBox() = 0;

        /**
        * @brief    Get the BleedBox for this page.
        *           The returned rect is relative to the top left of the page.
        */
        virtual FRect getBleedBox() = 0;
                
        /**
        * @brief    Get the TrimBox for this page.
        *           The returned rect is relative to the top left of the page.
        */
        virtual FRect getTrimBox() = 0;
                
        /**
        * @brief    Get the ContentBox (also know as the ArtBox) for this page.
        *           The returned rect is relative to the top left of the page.
        */
        virtual FRect getContentBox() = 0;
        
        /**
        * @brief    Get the view rotation for the page, in degrees clockwise.
        */
        virtual int32 getRotate() = 0;
        
        /**
        * @brief    Set the view rotation for the page, in degrees clockwise.
        *           Must be a multiple of 90 degrees.
        */
        virtual void setRotate(int32 rotate) = 0;
                
        /**
        * @brief    Return a smart pointer to the IFixedPage, loading it
        *           from the source document if necessary.
        * @return   IDOMFixedPagePtr the fixed page.
        */
        virtual IDOMFixedPagePtr getContent() = 0;
        
        /**
        * @brief    Mark the page as edited, returning the editable
        *           IFixedPage. Do this before attempting to edit the page
        *           contents. The markup DOM will be set to incomplete and
        *           may be edited in arbitrary ways.
        *           This also prevents the fixed page from being purged in
        *           low memory conditions.
        *           Note: This member may return a different instance than
        *           returned by previous invocations of getContent(),
        *           however once this routine has been called, subsequent
        *           calls to  getContent() will return the same instance as
        *           that returned here.
        */
        virtual IDOMFixedPagePtr edit() = 0;
        
        /**
        * @brief    Revert any edits to a page and mark the page as non-edited
        *           if possible.
        *
        *           This may not be used if the page was created as a new page,
        *           as there is nothing to revert to. In this case an IError
        *           exception will be thrown.
        */
        virtual void revert() = 0;

        /**
        * @brief    Has the page been loaded? That is, has the DOM been fetched?
        */
        virtual bool isLoaded() = 0;
        
        /**
        * @brief    Has the page been edited? That is, has the DOM been fetched?
        */
        virtual bool isEdited() = 0;
        
        /**
        * @brief    Let the page know that the content metadata has changed.
        *           For example if either of the MediaBox, CropBox, TrimBox 
        *           or ArtBox has changed. 
        */
        virtual void setMetadataChanged() const = 0;
        
        /**
        * @brief    Replace the content with the given fixed page.
        */
        virtual void setContent(const IDOMFixedPagePtr &content) = 0;

        /**
        * @brief    Release the reference to the page content, if possible.
        *           If a page is not edited, and the original source is available,
        *           the IPage's reference to the page content will be released.
        *           This is useful to release the memory associated with the page
        *           content if a page is not to be accessed again. The content
        *           may be safely requested again using getContent().
        *
        *           If the page content cannot be released, nothing will happen.
        */
        virtual void release() = 0;
        
        /**
        * @brief    Get the annotations for the page, as a vector.
        */
        virtual CAnnotationVect getAnnotations() = 0;
        
        /**
        * @brief    Add the given annotation.
        */
        virtual void addAnnotation(const IAnnotationPtr &annotation) = 0;
        
        /**
        * @brief    Find the annotation with the given annotation reference within the
        *           page.
        *           Throws an IError if the target could not be found.
        */
        virtual IAnnotationPtr findAnnotation(const IAnnotationReferencePtr &reference) = 0;
        
        /**
        * @brief    Remove the annotation at the given index.
        * @param    index The index of the annotation to be removed (0 being the
        *           first document).
        */
        virtual void removeAnnotation(uint32 index) = 0;
        
        /**
        * @brief    Remove the given annotation from the page. If the
        *           page is not present, an exception will be thrown.
        */
        virtual void removeAnnotation(const IAnnotationPtr &annotation) = 0;
        
        /**
        * @brief    Remove all the annotations from the page.
        */
        virtual void removeAnnotations() = 0;
        
        /**
        * @brief    Get the page's text runs.
        *           Text present in Tiling brushes will not be returned.
        */
        virtual void getTextRuns(CTextRunVect &runs) = 0;
        
        /**
        * @brief    Get the IDOMCatalog associated with this page.
        */
        virtual IDOMCatalogPtr getCatalog() const = 0;
        
        /**
        * @brief    Get the Id of the page
        */
        virtual DOMid getPageId() = 0;
        
        /**
        * @brief    Get the output intents, if present.
        */
        virtual COutputIntentVect getOutputIntents() = 0;

        /**
        * @brief    Get the page job ticket, if present.
        * @return   IDOMJobTkPtr the job ticket, or NULL if not present
        */
        virtual IDOMJobTkPtr getJobTicket() const = 0;
        
        /**
        * @brief    Set the page job ticket.
        */
        virtual void setJobTicket(const IDOMJobTkPtr &jobTicket) = 0;
        
        /**
        * @brief    Clone an IPage.
        *           For edited pages, this will clone the DOM tree.
        * @return   IPagePtr the fixed page.
        */
        virtual IPagePtr clone() = 0;
    };

    
    /**
    * @class    IJawsRenderer
    * @brief    A renderer that uses the Jaws RIP to create images from arbitrary  DOM.
    */
    class IJawsRenderer : public IRCObject
    {
      public:
        virtual ~IJawsRenderer() {}

        /**
        * @brief    Create a Jaws renderer instance.
        * @param    JawsMako The JawsMako instance.
        * @return   IJawsRendererPtr A jaws renderer instance.
        */
        static JAWSMAKO_API IJawsRendererPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /**
        * @brief    Render a node to an image.
        * @param    node The node to render.
        * @param    dpi The desired resolution.
        * @param    colorSpace The desired colour space. If NULL, DeviceRGB will be used.
        *           If provided, this must be an instance of one of the following colorspace types:
        *             - DeviceRGB,
        *             - sRGB,
        *             - DeviceGray,
        *             - sGray,
        *             - DeviceCMYK,
        *             - ICCBased,
        * @param    bounds The desired rendering area. If the rect is empty
        *           an appropriate size is chosen based on the input node.
        * @param    retainedSpotColorNames Used only if the color space is a three or four
        *           channel type (ie some form of RGB or CMYK color space).
        *           If provided, the renderer will provide channels in the output image for
        *           these spot colors and generate a DeviceN color space for the image as a
        *           whole. Note that the generated DeviceN color space is a best guess and
        *           is suitable for separated output only.
        * @param    optionalContent The optional content properties of the document the content
        *           is taken from. If provided, the renderer will apply optional content rules
        *           to the content during rendering. If not provided, all content will be
        *           rendered.
        * @param    optionalContentUsage The desired usage for optional content. Ignored if
        *           #optionalContent is NULL. eOCEUnknown must not be used.
        * @return   IDOMImagePtr the rendered image.
        */
        virtual IDOMImagePtr render(const IDOMNodePtr &node, uint32 dpi = 72, const IDOMColorSpacePtr &colorSpace = IDOMColorSpacePtr(NULL),
                                    const FRect &bounds = FRect(),
                                    bool  maskToInterestingNodes = false,
                                    const CSpotColorNames &retainedSpotColors = CSpotColorNames(),
                                    const IOptionalContentPtr &optionalContent = IOptionalContentPtr(),
                                    eOptionalContentEvent optionalContentUsage = eOCEView) = 0;
        
        /**
        * @brief    Render a node to a set of separated images, with optional anti-aliasing.
        *           For ease of use, the output is scaled to fit the destination size, described
        *           in pixels.
        * @param    node The node to render.
        * @param    colorSpace The desired colour space. If NULL, DeviceRGB will be used.
        *           If provided, this must be an instance of one of the following colorspace types:
        *             - DeviceRGB,
        *             - sRGB,
        *             - DeviceGray,
        *             - sGray,
        *             - DeviceCMYK,
        *             - ICCBased,
        * @param    bounds The desired rendering area. If the rect is empty
        *           an appropriate size is chosen based on the input node.
        * @param    destWidth The desired output width, in pixels. If zero, the input size is used.
        * @param    destHeight The desired output height, in pixels. If zero, the input size is used.
        * @param    retainedSpotColorNames Used only if the color space is a three or four
        *           channel type (ie some form of RGB or CMYK color space).
        *           If provided, the renderer will produce separate images for these colorants.
        * @param    optionalContent The optional content properties of the document the content
        *           is taken from. If provided, the renderer will apply optional content rules
        *           to the content during rendering. If not provided, all content will be
        *           rendered.
        * @param    optionalContentUsage The desired usage for optional content. Ignored if
        *           #optionalContent is NULL. eOCEUnknown must not be used.
        * @return   CEDLVector<IDOMImagePtr> the rendered separations. The main channels (as
        *           determined from the target space) will be first, followed by spot components
        *           (if requested) in the same order as retainedSpotColors.
        */
        virtual CEDLVector<IDOMImagePtr> renderSeparations(const IDOMNodePtr &node,
                                                           const IDOMColorSpacePtr &colorSpace = IDOMColorSpacePtr(NULL),
                                                           bool antiAliased = false,
                                                           const FRect &bounds = FRect(),
                                                           uint32 destWidth = 0,
                                                           uint32 destHeight = 0,
                                                           const CSpotColorNames &retainedSpotColors = CSpotColorNames(),
                                                           const IOptionalContentPtr &optionalContent = IOptionalContentPtr(),
                                                           eOptionalContentEvent optionalContentUsage = eOCEView) = 0;

        /**
        * @brief    Render a node to a supplied frame buffer, at 8 bits per component.
        *
        * @param    node The node to render.
        * @param    buffer The target frame buffer.
        * @param    destWidth The desired output width in pixels. The input content will
        *           be scaled to fit this size.
        * @param    destHeight The desired output height in pixels. The input content will
        *           be scaled to fit this size.
        * @param    stride The distance in bytes between the start of one scanline in the
        *           buffer to the next scanline in y.
        * @param    colorSpace The desired colour space. If NULL, DeviceRGB will be used.
        *           If provided, this must be an instance of one of the following colorspace types:
        *             - DeviceRGB,
        *             - sRGB,
        *             - DeviceGray,
        *             - sGray,
        *             - DeviceCMYK,
        *             - ICCBased,
        * @param    bounds The desired rendering area. If the rect is empty
        *           an appropriate size is chosen based on the input node.
        * @param    optionalContent The optional content properties of the document the content
        *           is taken from. If provided, the renderer will apply optional content rules
        *           to the content during rendering. If not provided, all content will be
        *           rendered.
        * @param    optionalContentUsage The desired usage for optional content. Ignored if
        *           #optionalContent is NULL. eOCEUnknown must not be used.
        * @return   IDOMImagePtr the rendered image.
        */
        virtual void renderToFrameBuffer(const IDOMNodePtr &node,
                                         void *buffer, uint32 stride,
                                         uint32 destWidth,
                                         uint32 destHeight,
                                         const IDOMColorSpacePtr &colorSpace = IDOMColorSpacePtr(NULL),
                                         const FRect &bounds = FRect(),
                                         const IOptionalContentPtr &optionalContent = IOptionalContentPtr(),
                                         eOptionalContentEvent optionalContentUsage = eOCEView) = 0;
        
        /**
        * @brief    As per renderToFrameBuffer, but allowing for pixel byte order and padding
        *           control.
        *
        *           Amongst other things this allows the use of XRGB, RGBX, XBGR or BGRX frame
        *           buffers. For example, to render to:
        *
        *           - XRGB: Set pixelStride to 4 and pass the address of the second frame buffer byte
        *           - RGBX: Set pixelStride to 4 and pass the buffer address as usual
        *           - XBGR: Set pixelStride to 4, pass the address of the second frame buffer byte,
        *                   and set reverseComponents to true
        *           - BGRX: Set pixelStride to 4, pass the buffer address as usual, and set
        *                   reverseComponents to true
        *
        *           Note that any pad bytes will not be altered during rendering.
        *
        * @param    node The node to render.
        * @param    buffer The target frame buffer.
        * @param    destWidth The desired output width in pixels. The input content will
        *           be scaled to fit this size.
        * @param    destHeight The desired output height in pixels. The input content will
        *           be scaled to fit this size.
        * @param    stride The distance in bytes between the start of one scanline in the
        *           buffer to the next scanline in y.
        * @param    pixelStride The distance in bytes between one pixel in x and the next.
        *           Must be at least as large as the number of components in the colour space.
        * @param    reverseComponents If true, the order of samples within a pixel will be
        *           reversed. For example, if you are rendering RGB to a BGR frame buffer,
        *           set this to true.
        * @param    colorSpace The desired colour space. If NULL, DeviceRGB will be used.
        *           If provided, this must be an instance of one of the following colorspace types:
        *             - DeviceRGB,
        *             - sRGB,
        *             - DeviceGray,
        *             - sGray,
        *             - DeviceCMYK,
        *             - ICCBased,
        * @param    bounds The desired rendering area. If the rect is empty
        *           an appropriate size is chosen based on the input node.
        * @param    optionalContent The optional content properties of the document the content
        *           is taken from. If provided, the renderer will apply optional content rules
        *           to the content during rendering. If not provided, all content will be
        *           rendered.
        * @param    optionalContentUsage The desired usage for optional content. Ignored if
        *           #optionalContent is NULL. eOCEUnknown must not be used.
        * @return   IDOMImagePtr the rendered image.
        */
        virtual void renderToFrameBufferPadAndReverse(const IDOMNodePtr &node,
                                                      void *buffer, uint32 stride,
                                                      uint32 pixelStride, bool reverseComponents,
                                                      uint32 destWidth,
                                                      uint32 destHeight,
                                                      const IDOMColorSpacePtr &colorSpace = IDOMColorSpacePtr(NULL),
                                                      const FRect &bounds = FRect(),
                                                      const IOptionalContentPtr &optionalContent = IOptionalContentPtr(),
                                                      eOptionalContentEvent optionalContentUsage = eOCEView) = 0;

        /**
        * @brief    Render a node to an antialiased, DeviceRGB image suitable for display,
        *           using supersampling.
        * @param    node The node to render.
        * @param    dpi The desired resolution.
        * @param    quality The desired anti-aliasing quality, from 0 (no antialiasing) to 15 (maximum).
        * @return   IDOMImagePtr the rendered image.
        */
        virtual IDOMImagePtr renderAntiAliased(const IDOMNodePtr &node, uint32 dpi = 72, uint8 quality = 3, const FRect &bounds = FRect()) = 0;
        
        /**
        * @brief    EXPERIMENTAL: Render a node to an antialiased, 8-bit RGBA (DeviceRGB) frame buffer,
        *           suitable for display.
        *           Does not use full supersampling as renderAntiAliased(), but uses a faster method.
        * @param    node The node to render.
        * @param    frameBuffer The destination frame buffer
        * @param    stride The offset in bytes from one frame buffer scanline to the next, which may be negative
        * @param    width The frame buffer width, in pixels
        * @param    height The frame buffer height, in pixels
        * @param    dpi The desired resolution.
        * @param    quality The desired anti-aliasing quality, from 1 (low quality) to 15 (maximum).
        * @param    bounds The desired rendering area. If the rect is empty
        *           an appropriate size is chosen based on the input node.
        */
        virtual void renderAntiAliasedToFrameBuffer(const IDOMNodePtr &node, void *frameBuffer, int32 stride, uint32 width, uint32 height,
                                                    uint32 dpi = 72, uint8 quality = 3, const FRect &bounds = FRect()) = 0;
        /**
         * @class   IHalftone
         * @brief   An abstract base class for communicating halftone information to the Jaws
         *          renderer, for use with renderMonochrome() and renderMonochromeToFrameBuffer()
         */
        class IHalftone
        {
        public:
            virtual ~IHalftone() {}
        };
        
        /**
         * @class   CSpotHalftone
         * @brief   Description of a simple spot halftone, at 45 degrees, using Jaws's
         *          default spot function.
         */
        class CSpotHalftone : public IHalftone
        {
          public:
            // The frequency must be greater than 0.
            CSpotHalftone(float freq, bool useFullResForFlattening = false)
            {
                frequency = freq;
                useFullResolutionForFlattening = useFullResForFlattening;
            }
            float frequency; //!< The frequency of the spot function, in lines per inch.
            bool useFullResolutionForFlattening; //!< Normally, the resolution at which flattening for transparent
                                                 //   images occurs is capped at twice the frequency. For some
                                                 //   content, this may not be appropriate. Set this to true to
                                                 //   use the full device resolution.
        };
        
        /**
         * @class   CThresholdArrayHalftone
         * @brief   Description of a Type 3 8-bit threshold array halftone for use
         *          with renderMonochrome. Please refer to section 7.4.5 of the
         *          PostScript language reference manual, 3rd edition.
         */
        class CThresholdArrayHalftone : public IHalftone
        {
          public:
            CThresholdArrayHalftone()
            {
                width = 0;
                height = 0;
            }
            
            uint32 width;  //!< The width of the halftone cell in pixels
            uint32 height; //!< the height of the halftone cell in pixels
            CEDLVector<uint8> thresholdArray; //<! The threshold array. Must be width x height bytes in size, and no larger than 65535 bytes.
        };
        
        /**
         * @class   CThresholdHalftone
         * @brief   A halftone representing a simple threshold.
         */
        class CThresholdHalftone : public IHalftone
        {
          public:
            CThresholdHalftone(uint8 thresh = 127)
            {
                threshold = thresh;
            }
            
            uint8 threshold; // Range is 8 bits, 1 to 254 inclusive.
        };
        
        /**
         * @brief   Render a node to a 1-bit monochrome image.
         * @param   node The node to render.
         * @param   dpi The desired resolution.
         * @param   halftone The Halftone to use. If NULL, a 60lpi spot function will be used.
         * @param   colorSpace The desired colour space. If NULL, DeviceGray will be used.
         *          If provided, only DeviceGray or sGray may be used.
         * @param   bounds The desired rendering area. If the rect is empty
         *          an appropriate size is chosen based on the input node.
         * @return  IDOMImagePtr the rendered image.
         */
        virtual IDOMImagePtr renderMonochrome(const IDOMNodePtr &node,
                                              uint32 dpi = 72, const IHalftone *halftone = NULL,
                                              const IDOMColorSpacePtr &colorSpace = IDOMColorSpacePtr(NULL),
                                              const FRect &bounds = FRect()) = 0;
        
        /**
         * @brief   Render a node to a 1-bit monochrome frame buffer
         *
         * @param   [in] node The node to render.
         * @param   [in] buffer The target frame buffer.
         * @param   [in] stride The distance in bytes between the start of one scanline in the
         *                buffer to the next scanline in y.
         * @param   [in] halftone The Halftone to use. Must not be NULL.
         *                Spot function (CSpotHalftone) and threshold (CThresholdHalftone) supported.\n
         *                (Adjust the frequency of spot function so that it works in 48 dpi.   
         *                For example, to achieve 60 lpi at 600dpi, set a frequency of 60 * 48 / 600 = 4.8 lpi).
         * @param   [in] colorSpace The desired colour space. If NULL, DeviceGray will be used.
         *               If provided, only DeviceGray or sGray may be used.
         * @param   [in] bounds The desired rendering area. If the rect is empty
         *               an appropriate size is chosen based on the input node.
         * @param   [in] optionalContentUsage The desired usage for optional content. Ignored if
         *               optionalContent is NULL. eOCEUnknown must not be used.
         */
        virtual void renderMonochromeToFrameBuffer(const IDOMNodePtr &node,
                                                   void *buffer, uint32 stride,
                                                   uint32 destWidth, uint32 destHeight,
                                                   const IHalftone *halftone,
                                                   const IDOMColorSpacePtr colorSpace = IDOMColorSpacePtr(NULL),
                                                   const FRect &bounds = FRect(),
                                                   const IOptionalContentPtr &optionalContent = IOptionalContentPtr(),
                                                   eOptionalContentEvent optionalContentUsage = eOCEView) = 0;

    };


    /**
     * @class   IProgressTick
     * @brief   An abstract class allowing a callback to monitor the progress of a task. 
     *          The callback can either pass an integer (from 0 to a maximum value)
     *          or a float (0.0 to 1.0) which represents the progress of the task.
     *          The task needs to call the tick() method to indicate incremental progress.
     *          If the task has completed (even if it's premature) then tickMax() should
     *          be called instead of tick().
     */
    class IProgressTick : public IRCObject
    {
    public:

        /**
         * @brief   A callback type for receiving progress information from 0.0 (starting) through 1.0 (complete)
         */
        typedef void (*FloatProgressCallbackFunc)(void *priv, float progress);

        /**
         * @brief   A callback type for receiving progress information as an integer
         */
        typedef void (*IntProgressCallbackFunc)(void *priv, uint32 currentCount, uint32 maxCount);

        virtual ~IProgressTick() {}

        /**
         * @brief   create the IProgressTick object
         *
         * @param   [in] tickMaxVal         The maximum value for the tick counter.
         * @param   [in] callbackInterval   The interval of the tick counts between each callback invocation.
         *                                  This can be usefull if we expect the callback invocation to be expensive
         *                                  and we want to minimize the impact of the callback on performance.
         */
        static IProgressTickPtr create (uint32 tickMaxVal, uint32 callbackInterval = 1);

        /**
         * @brief   Set the integer type callback function
         *
         * @param   [in] cbFunc             The integer type callback function to be invoked
         * @param   [in] cbPriv             The parameter to be passed back to the callback whenever it is invoked.
         */
        virtual void setCallback (IntProgressCallbackFunc cbFunc, void *cbPriv) = 0;

        /**
         * @brief   Set the integer type callback function
         *
         * @param   [in] cbFunc             The integer type callback function to be invoked
         * @param   [in] cbPriv             The parameter to be passed back to the callback whenever it is invoked.
         */
        virtual void setCallback (FloatProgressCallbackFunc cbFunc, void *cbPriv) = 0;

        /**
         * @brief   Increment the tick counter and optionally invoke the callback function.
         */
        virtual void tick () = 0;

        /**
         * @brief   Increment the tick counter to the maximum value and invoke the callback function.
         */
        virtual void tickMax () = 0;
    };

#   define PROGRESSTICK(progressTick)      { if (progressTick) (progressTick)->tick(); }
#   define PROGRESSTICKMAX(progressTick)      { if (progressTick) (progressTick)->tickMax(); }


}

/** @} */
#endif
