/*
 * Copyright (C) 2017 Global Graphics Software Ltd. All rights reserved.
 */

#ifndef JAWSMAKO_OUTPUTINTENT_H
#define JAWSMAKO_OUTPUTINTENT_H

/**
 *  @file    outputintent.h
 *  @ingroup JawsMako
 *  @brief   Declarations of interfaces for querying output intents.
 *
 *           Please see the PDF specification for details.
 */

#include <jawsmako/types.h>
#include <edl/ifilespec.h>

namespace JawsMako
{
    using namespace EDL;
    
    /**
     * @ingroup JawsMako
     * @class   IOutputIntent
     * @brief   Interface class representing a PDF output intent.
     */
    class IOutputIntent : public IRCObject
    {
      public:
        virtual ~IOutputIntent() {}
        
        
        /**
         * @brief Create an IOutputIntent
         */
        static JAWSMAKO_API IOutputIntentPtr create(const IJawsMakoPtr &jawsMako,
                                                    const U8String &subtype,
                                                    const U8String &outputCondition,
                                                    const U8String &outputConditionIdentifier,
                                                    const U8String &registryName,
                                                    const U8String &info,
                                                    const IDOMICCProfilePtr &profile = IDOMICCProfilePtr(),
                                                    // PDF 2.0 output intent additions
                                                    const RawString &checkSum = RawString(),
                                                    const CU8StringVect &colorantTable = CU8StringVect(),
                                                    uint32 profileVersion = 0,
                                                    const RawString &profileCS = RawString(),
                                                    const U8String  &profileName = U8String(),
                                                    const CFileSpecVect &fileSpecs = CFileSpecVect());
        
        /**
         * @brief Clone the output intent.
         */
        virtual IOutputIntentPtr clone() = 0;
        
        /**
         * @brief Retrieve the subtype.
         */
        virtual U8String getSubtype() const = 0;

        /**
         * @brief Retrieve the output condition, if present.
         *        An empty string is returned if there is no output condition provided.
         */
        virtual U8String getOutputCondition() const = 0;
        
        /**
         * @brief Retrieve the output condition identifier.
         */
        virtual U8String getOutputConditionIdentifier() const = 0;
        
        /**
         * @brief Retrieve the registry name, if present.
         *        An empty string is returned if there is no registry name provided.
         */
        virtual U8String getRegistryName() const = 0;
        
        /**
         * @brief Retrieve the output intent information string, if present.
         *        An empty string is returned if there is no info string provided.
         */
        virtual U8String getInfo() const = 0;
        
        /**
         * @brief Retrieve the output ICC profile if provided
         *        An empty string is returned if there is no output profile provided.
         */
        virtual IDOMICCProfilePtr getProfile() const = 0;
        
        /**
         * PDF 2.0 Features
         */
        
        /**
         * @brief Retrieve the 16-byte MD5 checksum of the profile, if provided.
         *        An empty string is returned if there is no checksum.
         */
        virtual RawString getCheckSum() const = 0;
        
        /**
         * @brief Retrieve the colorant table for the output intent, if provided.
         *        An empty string is returned if there is no colorant table.
         */
        virtual CU8StringVect getColorantTable() const = 0;
        
        /**
         * @brief Retrieve the ICC profile version, if provided, as a 32 bit value.
         *        0 will be returned if the version is not provided.
         */
        virtual uint32 getProfileVersion() const = 0;
        
        /**
         * @brief Retrieve the four-byte color space signature of the IFF profile,
         *        if provided. Otherwise, an empty string will be returned.
         */
        virtual RawString getProfileColorSpaceSignature() const = 0;
        
        /**
         * @brief Retrieve the profile name, if provided.
         *        Otherwise, an empty string will be returned.
         */
        virtual U8String getProfileName() const = 0;
        
        /**
         * @brief Retrieve any file specifications referring to external
         *        profiles (IFileSpecAsUrl) or embedded profiles
         *        (IFileSpecAsEmbeddedData).
         *
         *        For convenience, the first embedded profile in this list
         *        will be returned by getProfile() unless a top-level profile
         *        has also been provided.
         */
        virtual CFileSpecVect getProfileFileSpecifications() const = 0;
    };

}
#endif /* JAWSMAKO_OUTPUTINTENT_H */
