/*
* Copyright (C) 2008-2019 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef _JAWSMAKO_PDFINPUT_H_
#define _JAWSMAKO_PDFINPUT_H_

/**
* @file     pdfinput.h
* @ingroup  JawsMako
* @brief    PDF Input-Specific Features.
*
* This header describes the IPDFInput interface.
* These are a subclass of IInput. Instances of this type are returned by
* IInput::create() when eFFPDF is specified. This allows access to PDF
* specific input features and modes of operation.
*/

#include <jawsmako/jawsmako.h>

namespace JawsMako
{
    using namespace EDL;

    class IPDFInput;
    DECL_SMART_PTR(IPDFInput);

    /**
    * @ingroup  JawsMako
    * @class    IPDFInput
    * @brief    An instance of the JawsMako PDF input class.
    */
    class IPDFInput : public IInput
    {
      public:
        virtual ~IPDFInput() {}

        /**
         * @brief   Create an input for reading source documents in PDF format.
         *
         * @return  IPDFInputPtr the PDF input
         */
        static JAWSMAKO_API IPDFInputPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());

        /**
        * @brief    Set the password that should be used when attempting to open PDF files
        *           using this input. The default is no password.
        * @param    password The password to use. Assumed to be UTF8.
        */
        virtual void setPassword(const U8String &password) = 0;
        
        /**
        * @brief    Set whether or not to fail when a font cannot be found and PDF input
        *           needs to use a fallback font or create and emulated font.
        *
        *           If false, font substitution and/or font emulation for missing fonts
        *           is allowed, which is sensible behaviour for general use cases. If
        *           a font is found to be invalid, a font may be sought on the users
        *           system or substituted.
        *
        *           If true, PDF input will fail and an exception thrown if a font
        *           requires the use of a fallback font or an emulated font, or if
        *           a damaged font is found and needs to be substituted.
        *
        *           The default is false.
        */
        virtual void setFailOnFontFallback(bool failOnFontFallback) = 0;
        
        /**
        * @brief    Information about a font in a PDF file, obtained by
        *           scanning the PDF font structures.
        */
        class CPdfFontInfo
        {
          public:
            CPdfFontInfo()
            {
                embedded = false;
            }
            
            bool      embedded;     //< True if the PDF provides an embedded font stream for this font
            RawString fontType;     //< The PDF Font Type as a string
            RawString fontName;     //< The name of the font, according to the font's FontDescriptor entry, if present.
            RawString baseFontName; //< The "base" (PostScript) name of the font according to the Font dictionary.
        };
        
        typedef CEDLVector<CPdfFontInfo> CPdfFontInfoVect;
        
        /**
         * @brief   A callback type for receiving progress information in terms of page numbers
         *          within a document.
         */
        typedef void (*ProgressCallbackFunc)(void *priv, uint32 currentPage, uint32 numPages);

        /**
         * @brief  Quickly scan the PDF file at the given UTF-8 path for fonts.
         *         Does not attempt to validate the fonts, nor use them in any way.
         *         Instead, the PDF page tree is scanned and the PDF font objects interrogated.
         *         Annotations are scanned also.
         *
         *         You can pass in an optional callback function to receive progress information.
         *         From this function, execution may be aborted by throwing an abort exception
         *         using throwEDLError(EDL_ERR_ABORTED) which will propagate back to the caller.
         *
         *         Note: Even if a font is embedded, a given PDF consumer may decide not to use
         *         it and instead use a substitute, for example if the font is invalid or broken.
         */
        virtual CPdfFontInfoVect scanPdfForFonts(const U8String &pathToFile, ProgressCallbackFunc progressFunc = NULL, void *progressPriv = NULL) = 0;
        
        /**
         * @brief  Quickly scan the PDF file at the given wide-character Unicode path for fonts.
         *         Does not attempt to validate the fonts, nor use them in any way.
         *         Instead, the PDF page tree is scanned and the PDF font objects interrogated.
         *         Annotations are scanned also.
         *
         *         You can pass in an optional callback function to receive progress information.
         *         From this function, execution may be aborted by throwing an abort exception
         *         using throwEDLError(EDL_ERR_ABORTED) which will propagate back to the caller.
         *
         *         Note: Even if a font is embedded, a given PDF consumer may decide not to use
         *         it and instead use a substitute, for example if the font is invalid or broken.
         */
        virtual CPdfFontInfoVect scanPdfForFonts(const String &pathToFile, ProgressCallbackFunc progressFunc = NULL, void *progressPriv = NULL) = 0;
        
        /**
         * @brief  Quickly scan the PDF file in the given stream for fonts.
         *         Does not attempt to validate the fonts, nor use them in any way.
         *         Instead, the PDF page tree is scanned and the PDF font objects interrogated.
         *         Annotations are scanned also.
         *
         *         You can pass in an optional callback function to receive progress information.
         *         From this function, execution may be aborted by throwing an abort exception
         *         using throwEDLError(EDL_ERR_ABORTED) which will propagate back to the caller.
         *
         *         Note: Even if a font is embedded, a given PDF consumer may decide not to use
         *         it and instead use a substitute, for example if the font is invalid or broken.
         */
        virtual CPdfFontInfoVect scanPdfForFonts(const IInputStreamPtr &pdfStream, ProgressCallbackFunc progressFunc = NULL, void *progressPriv = NULL) = 0;

        
        /**
         * @brief  Basic information about an ink used in a PDF file, obtained
         *         by scanning the PDF page tree.
         *
         *         The returned information does not provide details about the ink itself;
         *         instead, this information may used to scan the appropriate page in the
         *         PDF where the ink is encountered using #IRendererTransform::findInks()
         */
        class CPdfScannedInk
        {
          public:
            RawString   inkName;  //< The name of the ink
            CUInt32Vect pageNums; //< The page numbers where this ink is encountered
                                  // (where 0 indicates the first page)
        };
        
        typedef CEDLVector<CPdfScannedInk> CPdfScannedInkVect;

        /**
         * @brief Quickly scan the PDF file at the given UTF-8 path and find any mentions
         * of inks in color spaces.
         */
        virtual CPdfScannedInkVect scanPdfForInks(const U8String &pathToFile) = 0;
        
        /**
         * @brief Quickly scan the PDF file at the given wide character path and find any mentions
         * of inks in color spaces.
         */
        virtual CPdfScannedInkVect scanPdfForInks(const String &pathToFile) = 0;
        
        /**
         * @brief Quickly scan the given PDF stream and find any mentions of inks in color spaces.
         */
        virtual CPdfScannedInkVect scanPdfForInks(const IInputStreamPtr &pdfStream) = 0;

    };
    #define obj2IPDFInput(obj) IPDFInputPtr(dynamic_cast<IPDFInput *>((IRCObject *) obj), true)
}

#endif
