/*
 * Copyright (C) 2014-2019 Global Graphics Software Ltd. All rights reserved.
 */
 
#ifndef _JAWSMAKO_PSOUTPUT_H_
#define _JAWSMAKO_PSOUTPUT_H_

/**
* @file     psoutput.h
* @ingroup  JawsMako
* @brief    JawsMako PS Output.
*/

#include <jawsmako/jawsmako.h>
#include <jawsmako/transforms.h>

namespace JawsMako
{
    using namespace EDL;

    class IPSOutput;
    DECL_SMART_PTR(IPSOutput);
    class IPSInjector;

    /**
    * @ingroup  JawsMako
    * @class    IPSOutput
    * @brief    Interface for the PS IOutput class.
    *
    *           Note that there is a limit on the number of PDF and PS
    *           output operations that may be in progress simultaneously.
    *           These are:
    *           - iOS - 1 PDF or PS output operation at a time
    *           - Android - 1 PDF or PS output operation at a time
    *           - Windows/MacOS/X 32 bit - 8 PDF or PS output operations at a time, subject to available memory
    *           - Windows/MacOS/X 64 bit - 96 PDF or PS output operations at a time for most tool chains
    *                                    - VS2015 Static builds are currently limited to 48
    *           - Windows UWP - 1 PDF or PS output operation at a time
    */
    class IPSOutput : public IOutput
    {
      public:
        virtual ~IPSOutput() {}
        
        /**
        * @brief   Create a PS Output instance.
        */
        static IPSOutputPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
         * PS Output-specific configuration
         */
         
        /**
        * @brief  Set the target resolution for the output, in dots per inch.
        *         The default is 600.
        *         Equivalent to calling setParameter with the parameter name
        *         "TargetResolution"
        */
        virtual void setResolution(float resolution) = 0;
        
        /**
        * @brief  Set whether or not streaming output should be enabled.
        *
        *         Streaming output provides output as soon as it is ready, sending
        *         it to the output file or stream. Fonts are defined incrementally
        *         and a number of standard document structure comments are
        *         deferred to the end job trailer. This mode is suitable for use
        *         with streaming consumers, such as printers.
        *
        *         If streaming output is set to false, then the output is withheld
        *         until the entire assembly has been processed. This enables
        *         providing all document resources (such as fonts and color
        *         spaces up front), and may be more useful for applications that
        *         further process the PostScript.
        *         The default is true,
        *
        *         Equivalent to calling setParameter with the parameter name
        *         "StreamingOutput".
        */
        virtual void setStreamingOutput(bool enable = true) = 0;
        
        /**
        * @brief  Set the target colour space for the intended output device.
        *         The default is DeviceCMYK.
        *         Equivalent to calling setParameter with the param name "TargetColorSpace"
        *         with appropriate values (please refer to documentation).
        * @param  colorSpace The desired color space. Must not be LAB, Indexed, DeviceN
        *         or scRGB. Any ICC space must have one, three, or four components.
        */
        virtual void setTargetColorSpace(const IDOMColorSpacePtr &targetSpace) = 0;
        
        /**
        * @brief  Set the target color space for the intended output device using an ICC profile.
        *         Equivalent to calling setParameter() with the param name "TargetProfile"
        *         with the value as the path to the profile.
        * @param  profile The desired profile.
        */
        virtual void setTargetProfile(const IDOMICCProfilePtr &profile) = 0;
        
        /**
        * @brief  Sets whether or not all objects should be converted to the target color
        *         space. If false, only objects that require rendering, or use a color space
        *         that cannot will represented in PostScript will be forcibly converted to
        *         the target colour space. Exquivalent to calling setParameter() with the
        *         param name "ConvertAllObjectsToTargetColorSpace".
        *
        *         The default is true.
        */
        virtual void setConvertAllObjectsToTargetColorSpace(bool convert = true) = 0;
        
        /**
        * @brief  Sets whether to ignore DeviceGray objects when color conversion
        *         is to be performed. The default is true.
        *         Equivalent to calling setParameter() with the param name
        *         "IgnoreDeviceGrayDuringColorConversion".
        */
        virtual void setIgnoreDeviceGrayDuringColorConversion(bool ignore = true) = 0;
        
        /**
        * @brief  Set the IPSInjector instance to use to inject PostScript directly into
        *         the output PostScript stream at strategic points in the output process.
        *         Note that this routine does not take ownership of the passed pointer, 
        *         which must remain allocated until after the output is complete.
        */
        virtual void setInjector(IPSInjector *injector) = 0;

        /**
         * @brief  Enumeration for image compression formats.
         */
        typedef enum
        {
            eICFlate,
            eICDCT, // Jpeg
            eICLZW,
            eICCCITT,
        } eImageCompression;

        /**
         * @brief  Set the desired image compression for color images that need to be
         *         reencoded. The default is eICDCT.
         *
         *         CCITT is not allowed for colour images.
         *
         *         Note: this is advisory only and may not be honoured in all
         *         cases.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "ColorImageCompression" with appropriate values (please refer to
         *         documentation).
         * @param  compression The desired compression.
         */
        virtual void setPreferredColorImageCompression(eImageCompression compression) = 0;

        /**
         * @brief  Set the desired image compression for gray images that need to be
         *         reencoded. The default is eICDCT.
         *
         *         CCITT is not allowed for gray images.
         *
         *         Note: this is advisory only and may not be honoured in all
         *         cases.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "GrayImageCompression" with appropriate values (please refer to
         *         documentation).
         * @param  compression The desired compression.
         */
        virtual void setPreferredGrayImageCompression(eImageCompression compression) = 0;

        /**
         * @brief  Set the desired image compression for monochrome images that need to be
         *         reencoded. The default is eICCCITT. JPEG/DCT is not allowed for mono
         *         images.
         *
         *         Note: this is advisory only and may not be honoured in all
         *         cases.
         *
         *         Equivalent to calling setParameter() with the parameter name
         *         "MonoImageCompression" with appropriate values (please refer to
         *         documentation).
         * @param  compression The desired compression.
         */
        virtual void setPreferredMonoImageCompression(eImageCompression compression) = 0;

        /**
         * @brief  Set the JPEG quality to use when compressing images in DCT format.
         *         Equivalent to calling setParameter() with the parameter name
         *         "JPEGQuality" and the value being the required quality.
         *         The default is 3 - normal quality.
         * @param  quality The desired quality level, with 1 being lowest quality
         *         and 5 being highest quality.
         */
        virtual void setJPEGQuality(uint8 quality) = 0;

        /**
         * @brief  Set the desired maximum resolution, threshold and downsampling method
         *         for colour images.
         *
         *         The default behaviour is leave the image resolution unchanged.
         *         Equivalent to calling setParameter() with the param names
         *         "ColorImageDownsamplingResolution", "ColorImageDownsamplingThreshold"
         *         and "ColorImageDownsamplingMethod" with the respective values.
         *
         * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
         *         images unchanged, which is the default.
         * @param  threshold The threshold above which images will be reduced to the
         *         desired resolution. Pass 0 to use the resolution.
         * @param  method The method to use when downsampling. The default is bicubic
         *         for colour images.
         */
        virtual void setColorImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eBicubic) = 0;

        /**
         * @brief  Set the desired maximum resolution, threshold and downsampling method
         *         for gray images.
         *
         *         The default behaviour is leave the image resolution unchanged.
         *         Equivalent to calling setParameter() with the param names
         *         "GrayImageDownsamplingResolution", "GrayImageDownsamplingThreshold"
         *         and "GrayImageDownsamplingMethod" with the respective values.
         *
         * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
         *         images unchanged, which is the default.
         * @param  threshold The threshold above which images will be reduced to the
         *         desired resolution. Pass 0 to use the resolution.
         * @param  method The method to use when downsampling. The default is bicubic
         *         for gray images.
         */
        virtual void setGrayImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eBicubic) = 0;

        /**
         * @brief  Set the desired maximum resolution, threshold and downsampling method
         *         for monochrome images.
         *
         *         The default behaviour is leave the image resolution unchanged.
         *         Equivalent to calling setParameter() with the param names
         *         "MonoImageDownsamplingResolution", "MonoImageDownsamplingThreshold"
         *         and "MonoImageDownsamplingMethod" with the respective values.
         *
         * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
         *         images unchanged, which is the default.
         * @param  threshold The threshold above which images will be reduced to the
         *         desired resolution. Pass 0 to use the resolution.
         * @param  method The method to use when downsampling. The default is subsample
         *         for monochrome images; using any other method will result in grayscale
         *         output.
         */
        virtual void setMonoImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eSubsample) = 0;
    };
    #define obj2IPSOutput(obj) IPSOutputPtr(dynamic_cast<IPSOutput *>((IRCObject *) obj), true)
    
    /**
    * @ingroup  JawsMako
    * @class    IPSInjector
    * @brief    Interface allowing users of IPSOutput to inject raw PostScript directly into
    *           the output stream at strategic points in the output process. Use IPSOutput::setInjector()
    *           to install subclasses of this type.
    */
    class IPSInjector
    {
      public:
        virtual ~IPSInjector() {}
        
        /**
        * @brief Invoked by the PostScript writer after the output PostScript stream is opened,
        *        but before any data is written to the output stream.
        * @param psStream The PostScript stream. Do not retain this stream after your handler
        *        exits, and only write during the execution of your handler.
        */
        virtual void beforeFirstByte(const IOutputStreamPtr &psStream) {}
        
        /**
        * @brief Invoked by the PostScript writer just before the PostScript header is written
        *        to the stream. Currently, this is effectively the same as beforeFirstByte()
        *        above as Mako will not currently write anything at the top of the PostScript
        *        stream before the PostScript header.
        * @param psStream The PostScript stream. Do not retain this stream after your handler
        *        exits, and only write during the execution of your handler.
        */
        virtual void beforePsHeader(const IOutputStreamPtr &psStream) {}
        
        /**
        * @brief Invoked by the PostScript writer just after %%BeginSetup is written to the stream.
        * @param psStream The PostScript stream. Do not retain this stream after your handler
        *        exits, and only write during the execution of your handler.
        */
        virtual void afterBeginSetup(const IOutputStreamPtr &psStream) {}
        
        /**
        * @brief Invoked by the PostScript writer just before %%EndSetup is written to the stream.
        * @param psStream The PostScript stream. Do not retain this stream after your handler
        *        exits, and only write during the execution of your handler.
        */
        virtual void beforeEndSetup(const IOutputStreamPtr &psStream) {}

        /**
        * @brief Invoked by the PostScript writer just after %%BeginPageSetup is written to the stream.
        * @param pageNumber The page number being prepared, with 0 indicating the first page.
        * @param psStream The PostScript stream. Do not retain this stream after your handler
        *        exits, and only write during the execution of your handler.
        */
        virtual void afterBeginPageSetup(uint32 pageNumber, const IOutputStreamPtr &psStream) {}
        
        /**
        * @brief Invoked by the PostScript writer just before %%EndPageSetup is written to the stream.
        * @param pageNumber The page number being prepared, with 0 indicating the first page.
        * @param psStream The PostScript stream. Do not retain this stream after your handler
        *        exits, and only write during the execution of your handler.
        */
        virtual void beforeEndPageSetup(uint32 pageNumber, const IOutputStreamPtr &psStream) {}
        
        /**
        * @brief Invoked by the PostScript writer just before "showpage" is written to the stream.
        * @param pageNumber The page number being output, with 0 indicating the first page.
        * @param psStream The PostScript stream. Do not retain this stream after your handler
        *        exits, and only write during the execution of your handler.
        */
        virtual void beforeShowpage(uint32 pageNumber, const IOutputStreamPtr &psStream) {}
        
        /**
        * @brief Invoked by the PostScript writer after the last byte of normal PostScript data
        *        is written to the stream.
        * @param psStream The PostScript stream. Do not retain this stream after your handler
        *        exits, and only write during the execution of your handler.
        */
        virtual void afterLastByte(const IOutputStreamPtr &psStream) {}
        
    };

}

#endif

