/*
 * Copyright (C) 2018 Global Graphics Software Ltd. All rights reserved.
 */

#ifndef JAWSMAKO_STRUCTURE_H
#define JAWSMAKO_STRUCTURE_H

/**
 *  @file    structure.h
 *  @ingroup JawsMako
 *  @brief   Declarations of interfaces for querying and manipulating logical structure,
 *           tagging, and marked content.
 *           Logical Structure is a PDF feature, built on marked content, allowing for
 *           tagging of content in a PDF file to facilitate reflowing, understanding the 
 *           logical document structure, and improved accessibility.
 */

#include <jawsmako/types.h>

namespace JawsMako
{
    using namespace EDL;
    
    class IStructureElementChild;
    DECL_SMART_PTR(IStructureElementChild);
    typedef CEDLVector<IStructureElementChildPtr> CStructureElementChildVect;
    class IStructureElementReferenceChild;
    DECL_SMART_PTR(IStructureElementReferenceChild);
    class IStructureMarkedContentReferenceChild;
    DECL_SMART_PTR(IStructureMarkedContentReferenceChild);
    class IStructureObjectReferenceChild;
    DECL_SMART_PTR(IStructureObjectReferenceChild);
    class IMarkedContentArtifactDetails;
    DECL_SMART_PTR(IMarkedContentArtifactDetails);

    /**
     * @ingroup JawsMako
     * @class   IMarkedContentDetails
     * @brief   Details of Marked Content applied to an IDOMGroup
     */
    class IMarkedContentDetails : public IRCObject
    {
      public:
        virtual ~IMarkedContentDetails() {}
        
        /**
         * @brief Create general-purpose marked content details.
         */
        static JAWSMAKO_API IMarkedContentDetailsPtr create(const IJawsMakoPtr &jawsMako,
                                                            const U8String &tag, const IRCObjectPtr &properties,
                                                            bool isPoint);
        
        /**
         * @brief Obtain the marked content's tag.
         */
        virtual U8String getTag() const = 0;
        
        /**
         * @brief Obtain the properties, if present.
         *        This is currently a private structure.
         */
        virtual IRCObjectPtr getProperties() const = 0;
        
        /**
         * @brief Do the marked content details represent a single point?
         */
        virtual bool getIsPoint() const = 0;
    };
    
    class IMarkedContentStructureDetails;
    DECL_SMART_PTR(IMarkedContentStructureDetails);
    
    /**
     * @ingroup JawsMako
     * @class   IMarkedContentStructureDetails
     * @brief   A subclass of IMarkedContentDetails that is created when the marked
     *          content is associated with the document's structure.
     */
    class IMarkedContentStructureDetails : public IMarkedContentDetails
    {
      public:
        virtual ~IMarkedContentStructureDetails() {}
        
        /**
         * @brief Create structure marked content details.
         */
        static IMarkedContentStructureDetailsPtr create(const IJawsMakoPtr &jawsMako,
                                                        const U8String &tag, const IRCObjectPtr &properties,
                                                        bool isPoint, const IStructureElementReferencePtr &elementReference);

        /**
         * @brief Obtain a reference for the structure element that this content refers to.
         *        Will never return NULL.
         */
        virtual IStructureElementReferencePtr getStructureElementReference() const = 0;
    };
    #define obj2IMarkedContentStructureDetails(obj)IMarkedContentStructureDetailsPtr(dynamic_cast<IMarkedContentStructureDetails *>((IRCObject *) obj), true)

    /**
     * @ingroup JawsMako
     * @class   IMarkedContentArtifactDetails
     * @brief   A subclass of IMarkedContentDetails that is created when the content is
     *          a logical structure Artifact.
     */
    class IMarkedContentArtifactDetails : public IMarkedContentDetails
    {
      public:
        virtual ~IMarkedContentArtifactDetails() {}
        
        /**
         * @brief Basic marked content artifact details creation.
         */
        static JAWSMAKO_API IMarkedContentArtifactDetailsPtr create(const IJawsMakoPtr &jawsMako,
                                                                    const U8String &tag, const IRCObjectPtr &properties,
                                                                    bool isPoint);
        /**
         * @brief Create marked content details for a logical structure Artifact
         *        In this context, an artifact is any object that are not relevant
         *        for the understanding of the content.
         */
        static JAWSMAKO_API IMarkedContentArtifactDetailsPtr create(const IJawsMakoPtr &jawsMako,
                                                                    const U8String &type,
                                                                    const U8String &subType);
        
        /**
         * @brief Get the type of the artifact, if present. Returns an empty
         *        string if no Type information is provided.
         */
        virtual U8String getType() const = 0;
        
        /**
         * @brief Get the subtype of the artifact, if present. Returns an empty
         *        string if no Type information is provided.
         */
        virtual U8String getSubtype() const = 0;
    };
    #define obj2IMarkedContentArtifactDetails(obj) IMarkedContentArtifactDetailsPtr(dynamic_cast<IMarkedContentArtifactDetails *>((IRCObject *) obj), true)

    /**
     * @ingroup JawsMako
     * @class   IStructureElementReference
     * @brief   A token-like class encapsulating a reference to a structure element.
     */
    class IStructureElementReference : public IRCObject
    {
      public:
        virtual ~IStructureElementReference() {}
        
        /**
         * @brief  Determine if another structure element reference refers to the same element
         */
        virtual bool equals(const IStructureElementReferencePtr &other) const = 0;
    };
    
    
    
    /**
     * @ingroup JawsMako
     * @class   IStructure
     * @brief   Top level tracking structure describing the logical structure of the document
     */
    class IStructure : public IRCObject
    {
      public:
        virtual ~IStructure() {}
        
        /**
         * @brief Create a new Structure.
         */
        static JAWSMAKO_API IStructurePtr create(const IJawsMakoPtr &jawsMako);
        
        /**
         * @brief Perform a deep clone of the structure.
         */
        virtual IStructurePtr clone() const = 0;
        
        /**
         * @brief Tag an IDOMNode from the given page with the given element.
         *
         *        If the node is an IDOMGroup (which isn't already optional content or
         *        tagged with structure information) or IDOMForm, the node will have
         *        it's marked content information set accordingly and the returned result
         *        will be that node.
         *        However, if the node is some other graphical node, it will be moved
         *        inside an IDOMGroup with the optional content information set. If the
         *        node is in a tree, the node will be replaced with the group. The
         *        group will be returned.
         *
         *        The given element must be present in the structure.
         */
        virtual IDOMNodePtr tagNode(const IStructureElementPtr &element, const IPagePtr &page, const IDOMNodePtr &nodeToTag) = 0;
        
        /**
         * @brief Add an element to the end of the children at the top level.
         *        An exception will be thrown if the element already exists
         *        in the structure tree.
         */
        virtual void appendElement(const IStructureElementPtr &element) = 0;
        
        /**
         * @brief Insert an element into the children at the top level at the
         *        given index, where an index of 0 is the head of the list.
         *        An exception will be thrown if the element already exists
         *        in the structure tree.
         */
        virtual void insertElement(const IStructureElementPtr &element, uint32 index) = 0;
        
        /**
         * @brief Remove the element at the given index from the list of children.
         *        Note: This will also cause all child elements to be removed.
         */
        virtual void removeElement(uint32 index) = 0;
        
        /**
         * @brief Get the child structure elements
         */
        virtual const CStructureElementVect &getChildElements() const = 0;
        
        /**
         * @brief Find the structure element matching the given reference,
         *        or NULL if it does not exist.
         */
        virtual IStructureElementPtr findStructureElementWithReference(const IStructureElementReferencePtr &reference) const = 0;

    };
    

    /**
     * @ingroup JawsMako
     * @class   IStructureElement
     * @brief   A structure element in the structure tree.
     */
    class IStructureElement : public IRCObject
    {
      public:
        virtual ~IStructureElement() {}
        
        /**
         * @brief Create a structure element.
         */
        static JAWSMAKO_API IStructureElementPtr create(const IJawsMakoPtr &jawsMako, const U8String &structureType);
        
        /**
         * @brief Obtain a clone of the element.
         */
        virtual IStructureElementPtr clone() const = 0;
        
        /**
         * @brief Get the structure type
         */
        virtual U8String getStructureType() const = 0;
        
        /**
         * @brief Set the structure type. Must be non-zero length.
         */
        virtual void setStructureType(const U8String &structureType) = 0;
        
        /*
         * @brief Get the "Title" of the structure element, or an empty string if none is present.
         */
        virtual U8String getTitle() const = 0;
        
        /**
         * @brief Set the "Title" of the structure element, or an empty string to remove
         */
        virtual void setTitle(const U8String &title) = 0;
        
        /**
         * @brief Get the language specifier for this element, or an empty string if none is specified.
         */
        virtual U8String getLanguage() const = 0;
        
        /**
         * @brief Set the language specifier for this element, or an empty string to remove
         */
        virtual void setLanguage(const U8String &language) = 0;
        
        /**
         * @brief Get the alternate description of the structure element, in human readable form.
         *        Returns an empty string if none is provided.
         */
        virtual U8String getAlternate() const = 0;
        
        /**
         * @brief Set the alternate description of the structure element, or an empty string to remove
         */
        virtual void setAlternate(const U8String &alternateDescription) = 0;
        
        /**
         * @brief Get the actual text of the structure element.
         *        Returns an empty string if none is provided.
         */
        virtual U8String getActualText() const = 0;
        
        /**
         * @brief Set the alternate description of the structure element, or an empty string to remove
         */
        virtual void setActualText(const U8String &actualText) = 0;
        
        /**
         * @brief Add an element to the end of the children.
         *        An exception will be thrown if the element already exists
         *        in the structure tree.
         *        The structure containing the target element must be provided.
         */
        virtual void appendElement(const IStructurePtr &structure, const IStructureElementPtr &element) = 0;
        
        /**
         * @brief Insert an element at the given index, where 0 is the head.
         *        An exception will be thrown if an element is added that already exists
         *        in the structure tree.
         */
        virtual void insertElement(const IStructurePtr &structure,
                                   const IStructureElementPtr &element,
                                   uint32 index) = 0;

        /**
         * @brief Add a marked content reference to the end of the children.
         */
        virtual void appendMarkedContentReferenceChild(const IStructureMarkedContentReferenceChildPtr &child) = 0;
        
        /**
         * @brief Insert a marked content reference into the children at the given
         *        index, where an index of 0 is the head.
         */
        virtual void insertMarkedContentReferenceChild(const IStructureMarkedContentReferenceChildPtr &child,
                                                       uint32 index) = 0;
        
        /**
         * @brief Add an object reference targetting the given form to the end of the chidren.
         *        In some situations the form may have its properties edited. Pass in the page
         *        (required) that the form will appear on.
         */
        virtual void appendObjectReferenceChild(const IPagePtr &page, const IDOMFormPtr &targetForm) = 0;

        /**
         * @brief Insert an object reference targetting the given form into the children at the
         *        given index, where 0 is the head.
         *        In some situations the form may have its properties edited. Pass in the page
         *        (required) that the form will appear on.
         */
        virtual void insertObjectReferenceChild(const IPagePtr &page,
                                                const IDOMFormPtr &targetForm,
                                                uint32 index) = 0;

        /**
         * @brief Remove the child at the given index from the list of children.
         *        Note: This will also cause all child elements of that element to
         *        be removed.
         */
        virtual void removeChild(const IStructurePtr &structure, uint32 index) = 0;

        /**
         * @brief Create a IStructureMarkedContentReferenceChild and an IMarkedContentStructureDetails
         *        pair for a node present on a given page. Cannot be used with form nodes.
         *        Neither the node nor the target element are edited, but in some situations the
         *        parents of the node may have their properties edited.
         *
         *        See also IStructure::tagNode() for a simplified method for applying structure to an
         *        individual node.
         */
        virtual void createMarkedContentReferencePair(const IStructurePtr &structure,
                                                      const IPagePtr &page, const IDOMNodePtr &node,
                                                      IStructureMarkedContentReferenceChildPtr &resultReference,
                                                      IMarkedContentStructureDetailsPtr &resultDetails) = 0;
        
        /**
         * @brief Get the child structure information
         */
        virtual const CStructureElementChildVect &getChildren() const = 0;

        /**
         * @brief Obtain a reference for the structure element.
         */
        virtual IStructureElementReferencePtr getStructureElementReference() const = 0;

        /**
         * @brief Find the structure element matching the given reference,
         *        in the children. Returns NULL if it does not exist.
         */
        virtual IStructureElementPtr findStructureElementWithReference(const IStructureElementReferencePtr &reference) const = 0;

    };
    
    /**
     * @ingroup JawsMako
     * @class   IStructureElementChild
     * @brief   A child of a structure element. Either points to actual marked content,
     *          or another structure element.
     */
    class IStructureElementChild : public IRCObject
    {
      public:
        virtual ~IStructureElementChild() {}
        
        typedef enum
        {
            eSCTStructureElement,
            eSCTMarkedContentReference,
            eSCTObjectReference
        } eStructureChildType;
        
        /**
         * @brief Get the type of this child.
         */
        virtual eStructureChildType getType() const = 0;
        
        /**
         * @brief Clone the element child.
         */
        virtual IStructureElementChildPtr clone() const = 0;
        
    };

    /**
     * @ingroup JawsMako
     * @class   IStructureElementReferenceChild
     * @brief   A child of a structure element that points to another structure element.
     */
    class IStructureElementReferenceChild : public IStructureElementChild
    {
      public:
        virtual ~IStructureElementReferenceChild() {}
        
        /**
         * @brief Obtain a reference to this structure element.
         */
        virtual IStructureElementPtr getElement() const = 0;
        
    };
    #define obj2IStructureElementReferenceChild(obj) IStructureElementReferenceChildPtr(dynamic_cast<IStructureElementReferenceChild *>((IRCObject *) obj), true)
    

    /**
     * @ingroup JawsMako
     * @class   IStructureMarkedContentReferenceChild
     * @brief   A child of a structure element that points to a piece of marked content.
     *          Note; to create these, please see IStructureElement::createMarkedContentReferencePair()
     */
    class IStructureMarkedContentReferenceChild : public IStructureElementChild
    {
      public:
        virtual ~IStructureMarkedContentReferenceChild() {}
        
        /**
         * @brief Check to see if the given IMarkedContentStructureDetails object
         *        matches this reference. That is, does this child refer to a group
         *        with these details. Pass in the parent structure element; it is
         *        needed to check that the values match.
         */
        virtual bool matches(const IStructureElementPtr &parentElement, const IMarkedContentStructureDetailsPtr &details) const = 0;
    };
    #define obj2IStructureMarkedContentReference(obj) IStructureMarkedContentReferenceChildPtr(dynamic_cast<IStructureMarkedContentReferenceChild *>((IRCObject *) obj), true)
    
    /**
     * @ingroup JawsMako
     * @class   IStructureObjectReferenceChild
     * @brief   A child of a structure element that points to a piece of marked content.
     *          These cannot be created directly. Instead use
     *          IStructureElement::appendObjectReferenceChild() or 
     *          IStructureElement::insertObjectReferenceChild()
     */
    class IStructureObjectReferenceChild : public IStructureElementChild
    {
      public:
        virtual ~IStructureObjectReferenceChild() {}
        
        /**
         * @brief Check to see if the given IDOMForm is pointed to by this object
         *        reference.
         */
        virtual bool matches(const IDOMFormPtr &form) const = 0;
    };
    #define obj2IStructureObjectReferenceChild(obj) IStructureObjectReferenceChildPtr(dynamic_cast<IStructureObjectReferenceChild *>((IRCObject *) obj), true)
    
    
}

#endif /* JAWSMAKO_STRUCTURE_H */
