/*
* Copyright (C) 2014-2016 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef SVGGENERATOR_H
#define SVGGENERATOR_H

/**
* @file svggenerator.h
* @ingroup  JawsMako
* @brief A svg generator for JawsMako
*/

#include <jawsmako/jawsmako.h>
#include <jawsmako/interactive.h>

namespace JawsMako
{
    using namespace EDL;
    
    class ISVGGenerator;
    DECL_SMART_PTR(ISVGGenerator);

    /**
    * @ingroup JawsMako
    * @class   ISVGGenerator
    * @brief   A SVG generator for JawsMako, allowing simple generation of
    *          SGG fragments for individual DOM nodes or entire pages.
    *
    *          The SVG is provided in a stream, with resources used by the
    *          SVG presented as streams tracked by instances of this object,
    *          which can be requested via getResource(). Resources are reused
    *          where possible for multiple SVG fragments.
    */
    class ISVGGenerator : public IRCObject
    {
      public:
        virtual ~ISVGGenerator() {}
        
        /**
        * @brief   Create a SVG generator instance.
        * @param   JawsMako The JawsMako instance.
        * @param   resourcePrefix A path fragment to prepend to resource names when
        *          generating SVG. Useful for cases where the resources are to be
        *          written to disk somewhere else than alongside the SVG fragment.
        * @return  ISVGGeneratorPtr The new instance.
        */
        static JAWSMAKO_API ISVGGeneratorPtr create(const IJawsMakoPtr &jawsMako, const U8String &resourcePrefix = U8String(), const IAbortPtr &abort = IAbortPtr ());
        
        /**
        * @brief   Generate SVG for the given DOM Node, returning the
        *          result in a stream.
        * @param   node The DOM node to be represented as SVG.
        * @return  IRAInputStreamPtr The SVG stream.
        */
        virtual IRAInputStreamPtr generateSVG(const IDOMNodePtr &node) = 0;
        
        /**
        * @brief   Alternate form of generateSVG() when an existing stream
        *          should be used.
        * @param   node The DOM node to be represented as SVG.
        * @param   IOutputStreamPtr The destination stream for the SVG.
        */
        virtual void generateSVG(const IDOMNodePtr &node, const IOutputStreamPtr &outputStream) = 0;
        
        /**
        *  @brief  Get the stream for a named resource.
        *          An exception will be thrown if the resource cannot be found.
        *  @param  name The name of the resource.
        *  @return IInputStreamPtr The resource stream.
        */
        virtual IInputStreamPtr getResource(const U8String &name) = 0;
        
        class CResourceEntry
        {
          public:
            U8String       name;
            IInputStreamPtr resource;
        };
        typedef CEDLVector<CResourceEntry> CSVGResourceVect;

        /**
        *  @brief  Get all the resources in a vector.
        *  @param  resources A reference to a vector to receive the entries.
        */
        virtual void getResources(CSVGResourceVect &resources) = 0;
        
        /*
         * Configuration.
         */
        
        /**
        * @brief   Set the target resolution for display.
        *          The default is 96dpi.
        */
        virtual void setResolution(float resolution) = 0;
        
        /**
        * @brief   Enable or disable image downsampling.
        *
        *          If true, images whose resolution is 50% higher than the
        *          target resolution (see #setResolution()) will be downsampled
        *          to the target resolution.
        *
        *          If false, images will not be downsampled.
        *
        *          The default is true.
        */
        virtual void setEnableImageDownsampling(bool downsample) = 0;
        
        typedef U8String (*UrlCallbackForTarget)(void *priv, DOMid target);
        typedef U8String (*UrlCallbackForPage)(void *priv, uint32 pageNumber);
        
        /**
        * @brief   Set a callback to provide the Url (absolute or relative) of
        *          the object where a target may be found. Return either the
        *          URL of the containing object, or an empty string if no such
        *          target exists.
        *
        *          The priv argument will be provided to the callback at each
        *          invocation.
        *
        *          See svggenerator.cpp for an example of its use.
        */
        virtual void setTargetResolutionCallback(void *priv, UrlCallbackForTarget callback) = 0;

        /**
        * @brief   Set a callback to provide the Url (absolute or relative) of
        *          the given page number. Return either the URL of the page, or
        *          an empty string if the page will not be reachable.
        *
        *          The priv argument will be provided to the callback at each
        *          invocation.
        *
        *          See svggenerator.cpp for an example of its use.
        */
        virtual void setPageResolutionCallback(void *priv, UrlCallbackForPage callback) = 0;
};
    
}

#endif

        
