/*
* Copyright (C) 2018-2019 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef JAWSMAKO_TEXT_H
#define JAWSMAKO_TEXT_H

/**
* @file     text.h
* @ingroup  JawsMako
* @brief    JawsMako Text Conveniences
*
* This header contains definitions of interfaces for extracting text
* information from a page.
*/

#include <jawsmako/types.h>
#include <jawsmako/interactive.h>
#include <edl/idomglyphs.h>

namespace JawsMako
{
    using namespace EDL;

    class IUnicodeHelper;
    DECL_SMART_PTR(IUnicodeHelper);
    
    enum eLanguageType
    {
        eLTArabic
    };

    /**
    * @ingroup  JawsMako
    * @class    IUnicodeHelper
    * @brief    An interface into language specific unicode helpers.
    */
    class IUnicodeHelper : public IRCObject
    {
      private:
        /**
         * @brief A bsearch compare function for type uint16
         */
        static int compare(const void *a, const void *b);
        
      public:
        virtual ~IUnicodeHelper() {}

        static JAWSMAKO_API IUnicodeHelperPtr create(eLanguageType language);
        
        /**
         * @brief Returns true if ch is a unicode space character.
         */
        static bool isSpaceCharacter(wchar_t ch);

        /**
         * @brief  Returns true is wchar is within the languages combining character ranges
         */
        virtual bool isCombiningCharacter(const wchar_t wchar) const = 0;
        
        /**
         * @brief Performs a reverse mapping on a contextual character and returns the original.
         */
        virtual wchar_t getNonContextualCharacter(const wchar_t ch) const = 0;
        
        /**
         * @brief  Returns true if the language has contextual forms
         */
	    virtual bool hasContextualForms() const = 0;
        
        /**
         * @brief  Creates a string from a CTextRunVect by applying diacritic fixes and contextual
         *         substitution, if required. The supplied runs collection must be contain unicode strings of length one.
         *         Basically the glyphs need to be in their single character form.
         */
        virtual String constructStringFromRuns(CTextRunVect &runs) const = 0;
                
        /**
         * @brief  Checks and attempts to fix diacritic characters that are incorrectly ordered
         */
        virtual void applyDiacriticFixes(CTextRunVect &runs) const = 0;
        
        /**
         * @brief  Applys the languages contextual substitution rules, in place.
         */
        virtual void applyContextualSubstitution(String &input) const = 0;
    };
    
    /**
    * @ingroup  JawsMako
    * @class    ITextRun
    * @brief    A run of text, containing unicode information, the position,
    *           transformation and bounds of the text.
    */
    class ITextRun : public IRCObject
    {
      public:
        virtual ~ITextRun() {}
        
        /**
        * @brief  Get the Unicode string for the run as available.
        */
        virtual const String &getUnicode() const = 0;
        
        /**
        * @brief  Get the Unicode string for the run as available, in UTF-8.
        */
        virtual U8String getUTF8() const = 0;

        /**
        * @brief  Get the transformation active where the glyph run is positioned,
        *         relative to the page. The transformation includes any transformation
        *         present in the IDOMGlyphs node where the text is represented.
        */
        virtual FMatrix getTransform() const = 0;
        
        /**
        * @brief  Get the origin of the first character, in glyph-local coordinates.
        */
        virtual FPoint getLocalOrigin() const = 0;
        
        /**
        * @brief Get the origin of the first character, in page coordinates.
        *        Convenience.
        */
        FPoint getOriginOnPage() const
        {
            FPoint origin = getLocalOrigin();
            getTransform().transform(origin, origin);
            return origin;
        }
        
        /**
        * @brief  Get the bounds of the run, in glyph-local coordinates.
        * @param  tight  Whether the bounding box should be 'tight' around the actual
        *                glyph markup, or expanded to include the full em height.
        */
        virtual FRect getLocalBounds(bool tight = true) const = 0;
        
        /**
        * @brief  Get the bounds of the run, in page coordinates.
        *         Convenience.
        * @param  tight  Whether the bounding box should be 'tight' around the actual
        *                glyph markup, or expanded to include the full em height.
        */
        FRect getBoundsOnPage(bool tight = true) const
        {
            FRect bounds = getLocalBounds(tight);
            getTransform().transformRect(bounds);
            return bounds;
        }

        /**
        * @brief  Get the corner points of the run (in the clockwise order), in page coordinates.
        */
        virtual void getCornersOnPage(FPoint &p1, FPoint &p2, FPoint &p3, FPoint &p4, bool tight) const = 0;

        /**
        * @brief  Split the run into the smallest possible units, returning the
        *         split runs. If the run cannot be split, the result will be a
        *         vector containing this run.
        */
        virtual CTextRunVect split() = 0;
        
        /**
        * @brief  Determine the width of a space in the font used for this run.
        *         If the font does not have a space character then an estimate
        *         will be returned.
        *         The returned value will be in ems.
        */
        virtual double getSpaceWidth() const = 0;
        
        /**
        * @brief  Determine the width of a space in the font used for this run.
        *         If the font does not have a space character then an estimate
        *         will be returned.
        *         The returned value will be in page units.
        */
        virtual double getSpaceWidthOnPage() const = 0;

        /**
        * @brief  Determine the height of the font used for this run.
        *         The returned value will be in page units.
        */
        virtual double getFontHeightOnPage() const = 0;

        /**
        * @brief  Get the IDOMGlyphs node that represents this run.
        */
        virtual IDOMGlyphsPtr getGlyphs() const = 0;
    };

    class IPageLayout;
    DECL_SMART_PTR(IPageLayout);

	/*! IPageLayout analysis options */
    typedef enum {
        ePAAll = -1, /*!< Complete all page analysis steps (build, join and reorder text runs) in one operation */
        ePABuildPage, /*!< Locate all glyphs nodes on the page */
        ePAJoinHRuns, /*!< Find all compatible horizonal runs of text */
        ePAJoinVRuns, /*!< Find all compatible vertical runs of text */
        ePADefineReadingOrder /*!<  Sort the runs of text into a suitable reading order */
    } ePageAnalysis;
	
	/*! IPageLayoutNode types */
    typedef enum {
        ePLTRoot, /*!< Whole page */
        ePLTColumn, /*!< Column */
        ePLTTextRun /*!< Run of text */
    } ePageLayoutType;

    class IPageLayoutData;
    DECL_SMART_PTR(IPageLayoutData);

    class IPageLayoutNode;
    DECL_SMART_PTR(IPageLayoutNode);

	/*! A collection of IPageLayoutNodes */
    typedef CEDLVector<IPageLayoutNodePtr> IPageLayoutNodeCollection;

    /**
    * @ingroup  JawsMako
    * @class    IPageLayoutData
    * @brief    Provides a representation of the analyzed page layout by organizing and allowing access to collections of IPageLayoutNodes
    */
    class IPageLayoutData : public IRCObject
    {
    public:
        virtual ~IPageLayoutData() {}

		/**
		* @brief    Creation function for IPageLayoutData that provides a representation 
		*           of the analyzed page layout, by organizing and allowing access to collections of IPageLayoutNodes
		* @param    data The collection of IPageLayotNodes
		* @return   IPageLayoutDataPtr The new representation
		*/
		static JAWSMAKO_API IPageLayoutDataPtr create(IPageLayoutNodeCollection data);

        /**
        * @brief  Get a collection of IPageLayoutNodes, representing the content of the specified column number.
        *         A columnNumber of zero will return the root content
        */
        virtual IPageLayoutNodeCollection getColumn(uint32 columnNumber) = 0;

        /**
        * @brief  Get the number of columns (this does not include the root item)
        */
        virtual uint32 getNumberOfColumns() const = 0;
    };

    /**
    * @ingroup  JawsMako
    * @class    IPageLayoutNode
    * @brief    Simple data type representing a part of an analyzed page.
    */
    class IPageLayoutNode : public IRCObject
    {
    public:
        virtual ~IPageLayoutNode() {}
		
		/**
		* @brief    Creation function for an IPageLayoutNode, a simple data type representing a part of an analyzed page
		* @param    rect The bounds of the text area
		* @param    type The layout node type
		* @param    content The Unicode string content
        * @param    contentGlyphs The glyphs node from which the content was obtained
		* @param    columnNumber The zero-indexed column number
		* @return   IPageLayoutNodePtr The new node
		*/
        static JAWSMAKO_API IPageLayoutNodePtr create(FRect rect, ePageLayoutType type, String content, const IDOMGlyphsPtr &contentGlyphs, uint32 columnNumber);

        /**
        * @brief  Get the unicode string content of the node, if it's of type ePLTTextRun
        *         otherwise returns an empty string.
        */
        virtual String getContent() const = 0;
        
        /**
        * @brief  Get the IDOMGlyphsNode that represents the content of the node, if it's of type ePLTTextRun
        *         otherwise returns a NULL.
        */
        virtual IDOMGlyphsPtr getContentGlyphs() const = 0;

        /**
        * @brief  Gets the page bounds of the node.
        */
        virtual FRect getPageBounds() const = 0;

        /**
        * @brief  Get the type of node.
        */
        virtual ePageLayoutType getType() const = 0;

        /**
        * @brief  Get the column number that the node belongs to. Zero is the root element.
        */
        virtual uint32 getColumnNumber() const = 0;
    };    

    /**
    * @ingroup  JawsMako
    * @class    IPageLayout
    * @brief    Analyze the layout of a FixedPage, grouping together text deemed to be
    *           in horizontal and/or vertical blocks. Useful for text search and selection.
    */
    class IPageLayout : public IRCObject
    {
    public:
        virtual ~IPageLayout() {}

		/**
		* @brief    Creation function for an IPageLayout, a fixed page layout analyser
		*           Throws an IEDLError on failure.
		* @param    pFactory The factory to use.
		* @param    page The fixed page to be analysed
		* @return   IPageLayoutPtr The new page layout analyser
		*/
        static JAWSMAKO_API IPageLayoutPtr create(IEDLClassFactory *factory, const IDOMFixedPagePtr &page);
		
		/**
		* @brief    Creation function for an IPageLayout, a fixed page layout analyser
		*           Throws an IEDLError on failure
		* @param    pFactory The factory to use
		* @param    runs The arrary of text runs to be analysed
		* @return   IPageLayoutPtr The new page layout analyser
		*/
        static JAWSMAKO_API IPageLayoutPtr create(IEDLClassFactory *factory, const CTextRunVect &runs);

        /**
        * @brief  Get the FixedPage being processed.
        */
        virtual IDOMFixedPagePtr getFixedPage() const = 0;

        /**
        * @brief  Process the page find the blocks of text.
        *         Can optionally perform each analysis phase independently (which
        *         can be useful when debugging).
        */
        virtual void analyze(ePageAnalysis analysisToPerform = ePAAll) = 0;

        /**
        * @brief  Get a textual description of the page content, useful for debugging purposes.
        */
        virtual String getLayoutInfo() const = 0;

        /**
        * @brief  Get a processed representation of the page content.
        */
        virtual IPageLayoutDataPtr getLayoutData() const = 0;

        /**
        * @brief  Get a flat collection of page content nodes.
        */
        virtual IPageLayoutNodeCollection getLayoutNodeCollection() const = 0;

        /**
        * @brief  Return all page text.
        */
        virtual String getPageText() const = 0;
    };

    class ITextSearch;
    DECL_SMART_PTR(ITextSearch);

    /**
    * @ingroup JawsMako
    * @class   ITextSearch
    * @brief   Perform text searching using the page information obtained from an IPageLayout.
    */
    class ITextSearch : public IRCObject
    {
    public:
        virtual ~ITextSearch() {};

        /**
		* @brief    Creation function for ITextSearch that performs text searching using page information obtained from an IPageLayout. 
		*           Throws an IEDLError on failure
		* @param    pFactory The factory to use
		* @param    pageLayout The page layout to use
		* @return   ITextSearchPtr The new text searcher
		*/
		static JAWSMAKO_API ITextSearchPtr create(IEDLClassFactory *factory, const IPageLayoutPtr &pageLayout);

        /**
        * @brief  Return a collection of quadpoint data covering all found text.
        *         Each entry in the CFPointVectVect represents a unique search 'hit'.
        */
        virtual CFPointVectVect search(const String &targetText, CEDLStringVect &found, bool caseSensitive, bool ignoreSpaces) const = 0;
    };

    class ITextSelect;
    DECL_SMART_PTR(ITextSelect);

    /**
    * @ingroup JawsMako
    * @class   ITextSelect
    * @brief   Perform text selection using the page information obtained from an IPageLayout.
    */
    class ITextSelect : public IRCObject
    {
    public:
        virtual ~ITextSelect() {};
		
        /**
		* @brief    Creation function for an ITextSelect that performs text selection using page information obtained from an IPageLayout. 
		*           Throws an IEDLError on failure
		* @param    pFactory The factory to use
		* @param    pageLayout The page layout to use
		* @return   ITextSelectPtr The new text selector
		*/
        static JAWSMAKO_API ITextSelectPtr create(IEDLClassFactory *factory, const IPageLayoutPtr &pageLayout);

        /**
        * @brief Return the unicode string located within the bounds of 'pageArea', using the specified 'language' unicode helper.
        *        Result will be empty if no string is found.
        */
        virtual String getTextAtRect(const FRect &pageArea, eLanguageType language = eLTArabic) const = 0;

        /**
        * @brief  Return quadpoint data covering all text within the specified
        *         page area, optionally returning the actual text as well.
        */
        virtual CFPointVect selectArea(const FRect &pageArea, String *selectedText) const = 0;

        /**
        * @brief  Return quadpoint data covering all text within the specified
        *         page start/end points, optionally returning the actual text as well.
        */
        virtual CFPointVect selectLines(const FPoint &startPoint, const FPoint &endPoint, String *selectedText) const = 0;
    };
}

#endif
