/*
* Copyright (C) 2013-2016 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef JAWSMAKO_TRANSFORMS_H
#define JAWSMAKO_TRANSFORMS_H

/**
*  @file    transforms.h
*  @ingroup JawsMako
*  @brief   Transforms for JawsMako, allowing direct modification of individual brushes,
*           nodes, or entire pages.
*/

#include <jawsmako/jawsmako.h>
#include <edl/idompath.h>
#include <edl/idomglyphs.h>
#include <edl/idombrush.h>
#include <edl/iabort.h>

namespace JawsMako
{
    using namespace EDL;
    
    class ITransform;
    DECL_SMART_PTR(ITransform);
    class ITransformChain;
    DECL_SMART_PTR(ITransformChain);
    
    /**
    *  @ingroup JawsMako
    *  @brief When a CTransformState has been pushed for a brush,
    *         this indicates the usage of that brush.
    *         If we descend into a brush, this allows the transform
    *         to know what kind of brush it is in.
    */
    typedef enum
    {
        eBUNone = 0,
        eBUGeneral,
        eBUFill,
        eBUStroke,
        eBUOpacityMask,
        eBUAlternateImage,
    } eBrushUsage;
    
    /**
    *  @ingroup JawsMako
    *  @brief Class for tracking the graphics state leading to the point
    *         where a transform is applied.
    *         
    *         Consider for example the IImageDownsamplerTransform described
    *         later in this header. In order to determine how to downsample
    *         an image, the transform needs to know how large the image will
    *         eventually be. The CTransformState provides this information
    *         by providing the combined transform that applies to the image
    *         based on the RenderTransforms of all the nodes entered leading
    *         to the point where the image is actually encountered.
    *
    *         Other transforms need access to other information, such as the
    *         approximate clip area, the current group color space, the
    *         renderingIntent (if present), the current antialiasing mode
    *         (edge mode) and/or how a brush is used.
    */
    class JAWSMAKO_API CTransformState
    {
      public:
        CTransformState() : hasClipBounds(false), hasRenderingIntent(false), renderingIntent(eRelativeColorimetric),
                            blackPointCompensation(eBPCDefault), brushUsage(eBUNone), edgeMode(eEMDefault),
                            inUncoloredTilingBrush(false), transformPriv(NULL)
        {}
        
        /**
        * @brief  Initialise to the state that would be active inside the given node.
        *         An atttempt will be made to determine the complete state by
        *         contatenating the state of all the parents of the given node.
        *         The best results will occur when the node's parents lead
        *         to an IDOMFixedPage.
        */
        CTransformState(const IDOMNodePtr &node);
        
        /**
        * @brief  Return a new state consisting of this state concatenated
        *         with the state implied by the given node.
        * @param  node The node whose state we wish to apply.
        * @param  updateTransform If true, update the transform entry, and also the
        *         related black point compensation entry.
        * @param  updateClip If true, update the clipBounds entry
        * @param  updateRenderingIntent If true, update the rendering intent.
        * @return The new state.
        */
        CTransformState stateInsideNode(const IDOMNodePtr &node, bool updateTransform = true, bool updateClip = true, bool updateRenderingIntent = true, bool updateEdgeMode = true) const;
        
        /**
        * @brief  Return a new state consisting of this state concatenated
        *         with the state implied by the given brush.
        * @param  brush The brush whose state we wish to apply.
        * @param  brushUsage The way the brush would be used; ie for filling,
        *         stroking, or for use as an opacity mask.
        * @return The new state.
        */
        CTransformState stateInsideBrush(const IDOMBrushPtr &brush, eBrushUsage brushUsage) const;
        
     public:
        FMatrix                 transform;              ///< The current transformation from default coordinates
        bool                    hasClipBounds;          ///< True if clipBounds is valid
        FRect                   clipBounds;             ///< The bounds of the current clipping area
        IDOMColorSpacePtr       groupColorSpace;        ///< The current group colour space
        bool                    hasRenderingIntent;     ///< True if an explicit rendering intent has been applied
        eRenderingIntent        renderingIntent;        ///< Ignored if hasRenderingIntent is false
        eBlackPointCompensation blackPointCompensation; ///< The current black point compensation
        eEdgeMode               edgeMode;               ///< The current edge mode
        eBrushUsage             brushUsage;             ///< Set when we descend into a brush
        bool                    inUncoloredTilingBrush; ///< Set to true when we descend into an uncolored tiling brush
                                                        ///  (An IDOMTilingPatternBrush with PaintType == 2)
        void                   *transformPriv;          ///< Private contextual information for use by the transform
                                                        ///  implementation
    };
    
    /**
    *  @ingroup JawsMako
    *  @brief ITransforms provide a method of applying common operations
    *         on DOM objects such as brushes, nodes, colours, colourspaces
    *         or entire trees. Not all transforms will operate on all kinds
    *         of objects, as noted in their descriptions.
    *
    *         ITransform instances attempt to return the same object for
    *         cases where identical transforms have been applied. For
    *         example, if an image has been previously transformed, the
    *         same object will be returned as the previous call if the
    *         results would be identical.
    */
    class ITransform : public IRCObject
    {
      public:
        virtual ~ITransform() {}
      
        /**
        *  @brief  Apply the transform to the given brush, if applicable.
        *          These transforms are thread safe.
        *  @param  brush The brush to be processed.
        *  @param  usage What the brush is to be used for.
        *  @param  state The active CTransformState when this brush is encountered.
        *          Not all transforms require this information.
        *  @return IDOMBrushPtr The result. If modifications have been applied the
        *          result will be a new brush. If NULL, this indicates that the brush
        *          should be dropped entirely.
        */
        virtual IDOMBrushPtr transform(const IDOMBrushPtr &brush, eBrushUsage usage = eBUGeneral, const CTransformState &state = CTransformState()) = 0;
        
        /**
        *  @brief  Apply the transform to the given image, if applicable.
        *          These transforms are thread safe.
        *  @param  image The image to be processed.
        *  @param  state The active CTransformState when this image is encountered.
        *          Not all transforms require this information.
        *  @return IDOMImagehPtr The result. If modifications have been applied the
        *          result will be a new image. If NULL, this indicates that the image
        *          should be dropped entirely.
        */
        virtual IDOMImagePtr transform(const IDOMImagePtr &image, const CTransformState &state = CTransformState()) = 0;
        
        /**
        *  @brief  Apply the transform to the given color, if applicable.
        *          These transforms are thread safe.
        *  @param  color The color to be processed.
        *  @param  state The active CTransformState when this color is encountered.
        *          Not all transforms require this information.
        *  @return IDOMColorPtr The result. If modifications have been applied the
        *          result will be a new color.
        */
        virtual IDOMColorPtr transform(const IDOMColorPtr &color, const CTransformState &state = CTransformState()) = 0;
        
        /**
        *  @brief  Apply the transform to the given colorspace, if applicable.
        *          These transforms are thread safe.
        *  @param  colorSpace The colorSpace to be processed.
        *  @param  state The active CTransformState when this color space is encountered.
        *          Not all transforms require this information.
        *  @return IDOMColorSpacePtr The result. If modifications have been applied the
        *          result will be a new color space.
        */
        virtual IDOMColorSpacePtr transform(const IDOMColorSpacePtr &colorSpace, const CTransformState &state = CTransformState()) = 0;
        
        /**
        *  @brief  Apply the transform to the given node, if applicable.
        *          These transforms are thread safe, providing no other transforms are being
        *          applied to the same nodes at the same time.
        *  @param  node The node to be processed.
        *  @param  changed Will be set to true on return if any changes have been made.
        *  @param  transformChildren If true, the transform will also apply to any children
        *          of the node. For some transforms, this must be set; see the description
        *          of the individual transforms for details.
        *  @param  state The active CTransformState when this node is encountered.
        *          Not all transforms require this information.
        *  @return IDOMNodePtr The result. It may be a new node, a modified version,
        *          or NULL indicating that the node is to be dropped entirely.
        */
        virtual IDOMNodePtr transform(const IDOMNodePtr &node, bool &changed, bool transformChildren = true, const CTransformState &state = CTransformState()) = 0;

        /**
        *  @brief  Apply the transform to the given page, if applicable.
        *          These transforms are thread safe, providing no other transforms are being
        *          applied to the same nodes at the same time.
        *          The transform will also apply to the annotations appearances.
        *  @param  page The page to be processed.
        *  @param  transformContent If true, apply the transform to the page content
        *  @param  transformAnnotations If true, apply the transform to the annotations
        */
        virtual void transformPage(const IPagePtr &page, bool transformContent = true, bool transformAnnotations = true) = 0;
        
        /**
        *  @brief  Flush the caches used by the transform.
        *          Most transforms cache recently transformed results to improve
        *          the performance of repeated transformations of equivalent
        *          results. However, it is possible that some cached results may
        *          point to entities that no longer exist, such as content inside
        *          an XPS file that no longer exists. If you are deleting or
        *          replacing files where transforms have been used, it is advisable
        *          to invoke this routine to clear the caches.
        */
        virtual void flushCaches() = 0;

        /**
        *  @brief  Returns a pointer to an IProgressTick object which allows
        *          for monitoring the progress of the transform.
        */
        virtual IProgressTickPtr getProgressTick () = 0;
 
        /**
        *  @brief  Set the IProgressTick object for this transform to allow
        *          for monitoring the progress of the transform.
        */
        virtual void setProgressTick (const IProgressTickPtr &progressTick) = 0;
 
    };
    
    /**
    *  @ingroup JawsMako
    *  @brief A vector of transform instances.
    */
    typedef CEDLVector<ITransformPtr> CTransformVect;
    
    /**
    *  @ingroup JawsMako
    *  @brief ITransformChain represents a change of ITransforms, and provides
    *         a method of applying a range of transforms to an entire DOM tree.
    *         Instances of this type attempt to ensure that shared resources are
    *         modified once only.
    */
    class ITransformChain : public IRCObject
    {
      public:
        virtual ~ITransformChain() {}
        
        /**
        *  @brief  Create a new transform chain
        *  @param  jawsMako The JawsMako instance.
        *  @param  transforms The initial vector of ITransforms for the chain.
        *  @return ITransformChainPtr The new instance.
        */
        static JAWSMAKO_API ITransformChainPtr create(const IJawsMakoPtr &jawsMako, const CTransformVect &transforms = CTransformVect());
        
        /**
         *  @brief  Remove the ITransform at the specified index from the ITransformChain
         *  @param  index The index of the ITransform to remove
         */
        virtual void removeTransform(uint32 index) = 0;
        
        /**
        *  @brief  Push an ITransform onto the end of the ITransformChain.
        *          Should not be called if another thread is currently using the transform.
        *  @param  transform The transform to push.
        */
        virtual void pushTransform(const ITransformPtr &transform) = 0;
        
        /**
        *  @brief  Push an ITransform onto the front of the ITransformChain.
        *          Should not be called if another thread is currently using the transform.
        *  @param  transform The transform to push.
        */
        virtual void pushTransformFront(const ITransformPtr &transform) = 0;
        
        /**
        *  @brief  Get the transforms in a vector.
        *  @return CTransformVect The transforms.
        */
        virtual CTransformVect getTransforms() const = 0;
        
        /**
        *  @brief  Apply the transform chain to the given node and its children,
        *          returning the resulting node. This is thread safe.
        *  @param  node The node to transform.
        *  @return IDOMNodePtr the modified node.
        */
        virtual IDOMNodePtr transform(const IDOMNodePtr &node) = 0;

        /**
        *  @brief  Apply the transform chain to the given node and its children,
        *          returning the resulting node, and providing an indication if
        *          any changes were actually made.
        *  @param  node The node to transform.
        *  @param  changed On exit changes will be set to true if any modifications
        *          were made, false otherwise.
        *  @return IDOMNodePtr the modified node.
        */
        virtual IDOMNodePtr transform(const IDOMNodePtr &node, bool &changed) = 0;
        
        /**
        *  @brief  Apply the transform chain to the given page, if applicable.
        *          The transform will also apply to the annotations appearances.
        *  @param  page The page to be processed.
        *  @param  transformContent If true, apply the transform to the page content
        *  @param  transformAnnotations If true, apply the transform to the annotations
        */
        virtual void transformPage(const IPagePtr &page, bool transformContent = true, bool transformAnnotations = true) = 0;

        /**
        *  @brief  Flush the caches used by the transforms in the chain.
        *          Most transforms cache recently transformed results to improve
        *          the performance of repeated transformations of equivalent
        *          results. However, it is possible that some cached results may
        *          point to entities that no longer exist, such as content inside
        *          an XPS file that no longer exists. If you are deleting or
        *          replacing files where transforms have been used, it is advisable
        *          to invoke this routine to clear the caches.
        */
        virtual void flushCaches() = 0;
        
        /**
        * @brief Clones this ITransformChain into a new object that can be manipulated independently of the original
        */
        virtual ITransformChainPtr clone() = 0;
    };
    
    /*
     * Individual ITransforms
     */

    class IImageEncoderTransform;
    DECL_SMART_PTR(IImageEncoderTransform);
     
    /**
    *  @ingroup JawsMako
    *  @brief  A simple transform for image encoding. Most useful for
    *          encoding abstract images such as IDOMRecombineImage,
    *          IDOMRawImage and IDOMFilteredImage as PNG, Tiff or Jpeg.
    *          Images may be colour converted if they are not compatible
    *          with the desired image type.
    */
    class IImageEncoderTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  jawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IImageEncoderTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
        *  Configuration.
        */
        
        /**
        * @brief Sets whether or not all images should be reencoded.
        *        The default is false; that is, only image types other than
        *        JPEG, TIFF, HDPHOTO and PNG will be re-encoded.
        */
        virtual void setReencodeAllImages(bool value) = 0;
        
        /**
        * @brief Set whether or not TIFF should ever be allowed.
        *        The default value is true. This is needed as some output
        *        formats do not support TIFF. If a TIFF is seen it will
        *        be re-encoded as something else.
        */
        virtual void setEnableTIFFEncoding(bool value) = 0;
        
        /**
        * @brief Available target image formats.
        */
        typedef enum
        {
            eEFAuto, /// Attempt to choose an appropriate format for each image
            eEFJPEG,
            eEFPNG,
            eEFTIFF,
        } eEncodeFormat;
        
        /**
        * @brief Set the preferred format for all images. This is only
        *        a preference and may not be honoured in all cases.
        *        The default is eEFAuto.
        * @param format The preferred format.
        */
        virtual void setPreferredFormat(eEncodeFormat format) = 0;

        /**
        * @brief Set the preferred format for monochrome images. This is only
        *        a preference and may not be honoured in all cases.
        *        The default is eEFAuto.
        * @param format The preferred format.
        */
        virtual void setPreferredMonoFormat(eEncodeFormat format) = 0;
        
        /**
        * @brief Get the preferred format for monochrome images.
        */
        virtual eEncodeFormat getPreferredMonoFormat() const = 0;

        /**
        * @brief Set the preferred format for gray images. This is only
        *        a preference and may not be honoured in all cases.
        *        The default is eEFAuto.
        * @param format The preferred format.
        */
        virtual void setPreferredGrayFormat(eEncodeFormat format) = 0;

        /**
        * @brief Get the preferred format for gray images.
        */
        virtual eEncodeFormat getPreferredGrayFormat() const = 0;

        /**
        * @brief Set the preferred format for colour images.
        *        The default is eEFAuto.
        * @param format The preferred format.
        */
        virtual void setPreferredColorFormat(eEncodeFormat format) = 0;

        /**
        * @brief Get the preferred format for color images.
        */
        virtual eEncodeFormat getPreferredColorFormat() const = 0;

        /**
        * @brief Set the JPEG encoding quality when JPEG encoding is used.
        * @param quality The desired quality level, with 1 being lowest quality
        *        and 5 being highest quality.
        */
        virtual void setJPEGQuality(uint8 quality) = 0; 
        
        /**
        * @brief Get the currently set JPEG encoding quality, where 1 being lowest
        *        quality and 5 being highest quality.
        */
        virtual uint8 getJPEGQuality() const = 0;
        
        /**
        * @brief Set the preferred TIFF compression when TIFF encoding is used.
        * @param scheme The preferred TIFF compression scheme. May not always be
        *        used as not all compression types can be used for all types.
        */
        virtual void setTIFFCompression(IDOMTIFFImage::eTIFFCompression scheme) = 0;

        /**
        * @brief Set the preferred TIFF compression when TIFF encoding is used
        *        for monochome images. Any compresion scheme may be used.
        * @param scheme The preferred TIFF compression scheme. May not always be
        *        used as not all compression types can be used for all types.
        */
        virtual void setMonoTIFFCompression(IDOMTIFFImage::eTIFFCompression scheme) = 0;

        /**
        * @brief Set the preferred TIFF compression when TIFF encoding is used
        *        for gray images. Only PackBits, LZW or None may be used.
        * @param scheme The preferred TIFF compression scheme. May not always be
        *        used as not all compression types can be used for all types.
        */
        virtual void setGrayTIFFCompression(IDOMTIFFImage::eTIFFCompression scheme) = 0;

        /**
        * @brief Set the preferred TIFF compression when TIFF encoding is used
        *        for color images. Only PackBits, LZW or None may be used.
        * @param scheme The preferred TIFF compression scheme. May not always be
        *        used as not all compression types can be used for all types.
        */
        virtual void setColorTIFFCompression(IDOMTIFFImage::eTIFFCompression scheme) = 0;
        
        /**
        * @brief Set whether or not IDOMMaskedBrushes where the sub-brush is a
        *        solid colour should be encoded as an image with an alpha channel,
        *        rather than encoding the image separately. The default is true.
        */
        virtual void setEncodeSolidColorMaskedBrushesAsSingleImage(bool encode) = 0;
    };


    class IImageDownsamplerTransform;
    DECL_SMART_PTR(IImageDownsamplerTransform);
     
    /**
    *  @ingroup JawsMako
    *  @brief  A transform for downsampling images above a given effective resolution
    *          to a desired target effective resolution.
    */
    class IImageDownsamplerTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  JawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IImageDownsamplerTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
        *  Configuration.
        */
        
        /**
        * @brief Set the desired downsampling method for color images.
        *        The default is bicubic.
        */
        virtual void setColorDownsamplingMethod(IDOMImageDownsamplerFilter::eDownsamplingMethod method) = 0;
        
        /**
        * @brief Get the downsampling method to be used for color images.
        */
        virtual IDOMImageDownsamplerFilter::eDownsamplingMethod getColorDownsamplingMethod() const = 0;
        
        /**
        * @brief Set the desired downsampling method for gray images
        *        The default is bicubic.
        */
        virtual void setGrayDownsamplingMethod(IDOMImageDownsamplerFilter::eDownsamplingMethod method) = 0;
        
        /**
        * @brief Get the downsampling method to be used for gray images.
        */
        virtual IDOMImageDownsamplerFilter::eDownsamplingMethod getGrayDownsamplingMethod() const = 0;

        /**
        * @brief Set the desired downsampling method for monochrome images.
        *        The default is subsample.
        *        NB: using any other method other than subsampling for monochrome
        *        images will result in grayscale output. 
        */
        virtual void setMonoDownsamplingMethod(IDOMImageDownsamplerFilter::eDownsamplingMethod method) = 0;

        /**
        * @brief Get the downsampling method to be used for mono images.
        */
        virtual IDOMImageDownsamplerFilter::eDownsamplingMethod getMonoDownsamplingMethod() const = 0;

        /**
        * @brief Set a blanket target resolution (dpi) for downsampling all images.
        *        This will also set the threshold resolution to the same value.
        */
        virtual void setTargetResolution(float resolution) = 0;
        
        /**
        * @brief Set a blanket threshold resolution (dpi) for downsampling all images.
        *        Only images with an effective resolution of at least this
        *        value will be downsampled.
        */
        virtual void setThresholdResolution(float resolution) = 0;

        /**
        * @brief Set a target resolution (dpi) for downsampling color images.
        *        This will also set the threshold resolution to the same value.
        *        Setting this to 0.0 will result in no downsampling for this image type.
        *        The default is 0.0.
        */
        virtual void setColorTargetResolution(float resolution) = 0;
        
        /**
        * @brief Get the target resolution (dpi) for downsampling color images.
        */
        virtual float getColorTargetResolution() = 0;
        
        /**
        * @brief Set a threshold resolution (dpi) for downsampling color images.
        *        Only images with an effective resolution of at least this
        *        value will be downsampled.
        */
        virtual void setColorThresholdResolution(float resolution) = 0;

        /**
        * @brief Get the theshold resolution (dpi) for downsampling color images.
        */
        virtual float getColorThresholdResolution() = 0;

        /**
        * @brief Set a target resolution (dpi) for downsampling gray images.
        *        This will also set the threshold resolution to the same value.
        *        The default is 300dpi.
        *        Setting this to 0.0 will result in no downsampling for this image type.
        *        The default is 0.0.
        */
        virtual void setGrayTargetResolution(float resolution) = 0;
        
        /**
        * @brief Get the target resolution (dpi) for downsampling gray images.
        */
        virtual float getGrayTargetResolution() = 0;

        /**
        * @brief Set a threshold resolution (dpi) for downsampling gray images.
        *        Only images with an effective resolution of at least this
        *        value will be downsampled.
        */
        virtual void setGrayThresholdResolution(float resolution) = 0;

        /**
        * @brief Get the theshold resolution (dpi) for downsampling gray images.
        */
        virtual float getGrayThresholdResolution() = 0;

        /**
        * @brief Set a target resolution (dpi) for downsampling monochrome images.
        *        This will also set the threshold resolution to the same value.
        *        The default is 1200dpi.
        *        Setting this to 0.0 will result in no downsampling for this image type.
        *        The default is 0.0.
        */
        virtual void setMonoTargetResolution(float resolution) = 0;
        
        /**
        * @brief Get the target resolution (dpi) for downsampling mono images.
        */
        virtual float getMonoTargetResolution() = 0;

        /**
        * @brief Set a threshold resolution (dpi) for downsampling monochrome images.
        *        Only images with an effective resolution of at least this
        *        value will be downsampled.
        */
        virtual void setMonoThresholdResolution(float resolution) = 0;

        /**
        * @brief Get the theshold resolution (dpi) for downsampling mono images.
        */
        virtual float getMonoThresholdResolution() = 0;
        
        /**
        * @brief Set whether or not to downsample masked images
        *
        *        This applies to cases where an image is masked by a separate
        *        masked image, such as types of PDF masked or soft-masked images.
        *        These are represented in the DOM using IDOMMaskedBrush, where the
        *        sub-brush is an image.
        *
        *        The default is true.
        */
        virtual void setDownsampleMaskedImages(bool downsampleMaskedImages) = 0;
        
        /**
        * @brief Set whether or not the mask resolution setting should be applied
        *        to the image portion of a masked image.
        *
        *        This applies to cases where an image is masked by a separate
        *        masked image, such as types of PDF masked or soft-masked images.
        *        These are represented in the DOM using IDOMMaskedBrush, where the
        *        sub-brush is an image.
        *
        *        If false, then the mask and the image are evaluated separately and
        *        a downsampling resolution and threshold are chosen. For the mask,
        *        this is normally either grayscale or monochrome. The image data
        *        can be anything. In this mode it is possible for the downsampled
        *        image and mask to be downsampled to different resolutions.
        *
        *        If true, then whatever target resolution and threshold is applied
        *        to the mask will also be applied to the image samples. If these
        *        images have the same effective resolution before downsampling,
        *        then they will also share the same effective resolution after
        *        downsampling.
        *
        *        The default is false.
        */
        virtual void setUseMaskResolutionForMaskedImages(bool useMaskResolutionSettingForMaskedImages) = 0;

        
    };
    
    
    class IColorConverterTransform;
    DECL_SMART_PTR(IColorConverterTransform);
     
    /**
    *  @ingroup JawsMako
    *  @brief  A transform for color conversion, converting all appropriate
    *          DOM contents to a desired target color space.
    *
    *          Conversion for individual color space types can be enabled or
    *          disabled. This is useful for example for consumers who only
    *          support a subset of the JawsMako supported color space set.
    *
    *          When handling objects using indexed colour spaces, instances of
    *          this transform may opt to use a modified indexed color space rather
    *          than actually convert the underlying color. This is usually much
    *          faster. An IComplexColorSimplifierTransform can be applied after
    *          this transform if required.
    *
    *          Objects using DeviceN color spaces are converted only if the
    *          alternate space would require conversion. Again, an 
    *          IComplexColorSimplifierTransform can be used if required.
    */
    class IColorConverterTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  JawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IColorConverterTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
        *  Configuration.
        */
        
        /**
        * @brief  Set the desired target space for conversion.
        *         The default is DeviceRGB. Indexed or DeviceN color spaces
        *         cannot be used as a target space.
        * @param  targetSpace The target color space to be used.
        */
        virtual void setTargetSpace(const IDOMColorSpacePtr &targetSpace) = 0;
        
        /**
        * @brief  Get the target space for conversion.
        */
        virtual IDOMColorSpacePtr getTargetSpace() = 0;
        
        /**
        * @brief  Set the desired target space using an ICC profile.
        * @param  profile The ICC profile to use.
        */
        virtual void setTargetProfile(const IDOMICCProfilePtr &profile) = 0;
        
        /**
        * @brief  Set the desired target space to the default SWOP color profile.
        */
        virtual void setSWOPTargetSpace() = 0;
        
        /**
        * @brief  Sets an override rendering intent.
        *         By default this is not set. If set, this intent is used
        *         instead of any DOM resident information.
        */
        virtual void setOverrideRenderingIntent(eRenderingIntent intent) = 0;
        
        /**
        * @brief  Set whether or not to use explicit rendering intents in
        *         the DOM when performing conversion. The PDF input for example
        *         can set explicit rendering intents for individual nodes in
        *         the DOM, and these will be ignored if this property is set
        *         to false. The default is true; that is to honour the explicit
        *         rendering intent information in the DOM.
        *         This is ignored if an overriding rendering intent has been set
        *         by setOverrideRenderingIntent();
        */
        virtual void useDOMRenderingIntents(bool use) = 0;
        
        /**
        * @brief  Sets whether or not only device-independent colors should be
        *         converted. Useful for cases where device independent colour
        *         spaces are not allowed, or will not produce reliable results.
        *         The default is false.
        */
        virtual void setConvertOnlyDeviceIndependentColors(bool convert) = 0;
        
        /**
        * @brief  If setConvertOnlyDeviceIndependentColors() has been set to true,
        *         this controls the conversion behaviour for affected spaces. If
        *         set to treu, the converter will convert to the most "similar"
        *         Device space; for example 3 channel colours will be converted to
        *         DeviceRGB and so forth. If false, the target space will be used.
        *         The default is false.
        */
        virtual void setConvertIndependentToSimilarDeviceSpace(bool convert) = 0;
        
        /**
        * @brief  Sets whether or not images can be ignored if their color space is
        *         deemed to be similar to the target space (see IDOMColorSpace::similar()
        *         for details about the similarity test). This is useful to avoiding
        *         needing to be re-encode images in some situations.
        *         The default is false.
        */
        virtual void setIgnoreImagesWithSimilarColorSpaces(bool ignore) = 0;
        
        /**
        * @brief  Sets whether or not color conversion of image samples should be
        *         forced, even for cases where the source image color space is
        *         considered similar to the target space. The default is false;
        *         that is, if an image's color space is similar to the target
        *         then the individual color samples will not be color converted.
        *         In both cases the target color space will be applied to the
        *         resulting image.
        */
        virtual void setForceImageSampleConversion(bool force) = 0;
        
        /**
         * @brief Set whether or not objects using a certain color space
         *        type should be converted. The default for all is true,
         *        although use of setConvertOnlyDeviceIndependentColors()
         *        will affect this default.
         */
        virtual void setShouldConvertForSpaceType(IDOMColorSpace::eColorSpaceType type, bool convert = true) = 0;
        
        /**
         * @brief Get whether or not objects using a certain color space
         *        type would be converted.
         */
        virtual bool wouldConvertForSpaceType(IDOMColorSpace::eColorSpaceType type) = 0;
        
        /**
         * @brief Set whether objects with ICC color spaces with the given number
         *        of components should be converted.
         *        The default is true for all.
         *        numComponents is from 1 to EDL_CMM_MAX_COMPONENTS
         *        Calling setShouldConvertForSpaceType(IDOMColorSpace::eICCBased, true)
         *        however will cause all possibilities to be true. Likewise calling
         *        setShouldConvertForSpaceType(IDOMColorSpace::eICCBased, false) will
         *        cause all possibilities to be false.
         */
        virtual void setConvertICCColorsWithNumComponents(uint8 numComponents, bool convert = true) = 0;
        
        /**
         * @brief Set the maximum allowed version of ICC Profiles allowed. Anything
         *        with a higher version will be converted to the target space.
         *        The default is 0.0, which disables this feature.
         */
        virtual void setConvertICCSpacesWithProfileVersionGreaterThan(uint8 majorVersion, uint8 minorVersion) = 0;
        
        /**
         * @brief Set whether the contents of Luminosity Soft Masks should be converted or not.
         *
         *        The default is true.
         *
         *        Luminosity soft masks use the luminosity of the contents of a soft mask to
         *        generate the opacity that is applied to the objects being masked. If the
         *        colors inside such a soft mask are converted to a different colour space,
         *        then this will affect the calculation of the opacity, and potentially
         *        undesirable results in some contexts.
         */
        virtual void setConvertColorsInsideLuminositySoftMasks(bool convert = true) = 0;
    };


    class IComplexColorSimplifierTransform;
    DECL_SMART_PTR(IComplexColorSimplifierTransform);
     
    /**
    *  @ingroup JawsMako
    *  @brief  A simple transform that looks for DeviceN or Indexed
    *          color spaces, and where found, simplifies the hosting
    *          objects to use the underlying color space (for Indexed
    *          cases) or the alternate color space (for DeviceN cases).
    *          Useful in particular for consumers that do not support
    *          such color spaces.
    */
    class IComplexColorSimplifierTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  JawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IComplexColorSimplifierTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
        *  There is no configuration for this transform.
        */
    };


    class IImageMergerTransform;
    DECL_SMART_PTR(IImageMergerTransform);
     
    /**
    *  @ingroup JawsMako
    *  @brief  A simple transform that looks for nearby images and attempts
    *          to glom them together in a single image. Some producers can
    *          break images up into images consisting of a single scanline;
    *          this transform attempts to put them back together again.
    *          This transform can handle images with a mask channel, but
    *          does not attempt to merge images with an alpha channel.
    */
    class IImageMergerTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  JawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IImageMergerTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
        *  Configuration
        */
        
        /**
        * @brief  Sets whether or a mask channel is allowed in the merged result.
        *
        *         The merger deals with images that do not exactly abut alone a single
        *         edge by producing a mask channel that describes the area of the
        *         resulting image that contains merged image data. If set to false this
        *         transform will not produce masked images, and as a result will not
        *         be able to merge some images. The default is true.
        */
        virtual void setAllowMaskedResults(bool allowMasked) = 0;
        
    };


    class IOptionalContentFixerTransform;
    DECL_SMART_PTR(IOptionalContentFixerTransform);
     
    /**
    *  @ingroup JawsMako
    *  @brief  A simple transform that strips the DOM of any PDF optional content
    *          that is not visible for the given document use.
    *          This transform also selects from any alternate images, if present.
    */
    class IOptionalContentFixerTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  jawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IOptionalContentFixerTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
        *  Configuration
        */
        
        typedef enum
        {
            eOCUView,
            eOCUPrint,
            eOCUExport
        } eOptionalContentUsage;
        
        /**
        * @brief  Sets the optional content data to use when making decisions.
        *         Without this, the status of the optional content at the time
        *         of creation will be used.
        */
        virtual void setOptionalContent(const IOptionalContentPtr &optionalContent) = 0;
        
        /**
        * @brief  Sets the usage of the optional content items that
        *         should be retained. The default is eOCUView.
        */
        virtual void setOptionalContentUsage(eOptionalContentUsage usage) = 0;
    };


    class ICFFCIDSplitterTransform;
    DECL_SMART_PTR(ICFFCIDSplitterTransform);
     
    /**
    *  @ingroup JawsMako
    *  @brief  A simple transform that looks for CID CFF Fonts containing multiple
    *          SubFonts. Some viewers do not support these fonts, or do so poorly.
    *          If found, this transform will split out the sub fonts into individual
    *          font streams, and adjust the Glyphs nodes where they are used
    *          accordingly.
    *
    *          Operates on nodes. Note that when operating on a glyphs node, the
    *          operation may result in multiple nodes, in which case they will be
    *          returned encapsulated in an IDOMGroup or IDOMTransparencyGroup
    *          depending on its attributes.
    *
    *          Useful whenever a CID CFF with multiple subfonts would not be allowed.
    */
    class ICFFCIDSplitterTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  jawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API ICFFCIDSplitterTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
        *  There is no configuration for this transform.
        */
    };


    class IStrokerTransform;
    DECL_SMART_PTR(IStrokerTransform);
     
    /**
    *  @ingroup JawsMako
    *  @brief  A transform for converting some or all stroked paths into plain filled
    *          paths.
    *
    *          Different rendering engines have different stroking abilities. XPS for
    *          example supports several stroking capabilities that, say, PDF or SVG
    *          support, such as triangular line caps, using a different start and end
    *          line caps, and the ability to mark sections of a path as non-stroking.
    *
    *          Further, XPS and most other renderers differ in how miters are treated,
    *          with XPS "clipping" miters to the miter limit where PDF and SVG renderers
    *          would simply bevel instead.
    *
    *          This filter provides a method for converting strokes to plain fills
    *          in all cases or only in certain circumstances.
    *
    *          For best results the transform entry in the state given to the transform()
    *          routines should be valid.
    */
    class IStrokerTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  JawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IStrokerTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
        *  Configuration
        */
        
        /**
        * @brief  Sets the target resolution (dpi) of the eventual consumer. In some
        *         situations the stroker will need to "flatten" sections of the path
        *         and this parameter will help the stroker decide how much error is
        *         allowable in such situations. The default is 300dpi.
        */
        virtual void setTargetResolution(float resolution) = 0;
        
        /**
        * @brief  Sets up the stroker transform for output to an XPS style consumer.
        *         
        *         Equivalent to (on a freshly instantiated transform):
        *            setConvertAllStrokes(false);
        *            setConvertForMiterTreatment(IDOMPathNode::eBevelLongMiters);
        */
        virtual void setupForXPSStyleOutput() = 0;
        
        /**
        * @brief  Sets up the stroker transform for output to a PDF style consumner.
        *         
        *         Equivalent to (on a freshly instantiated transform):
        *            setConvertAllStrokes(false);
        *            setConvertForMiterTreatment(IDOMPathNode::eClipLongMiters);
        *            setConvertForLineCap(IDOMPath::eTriangleCap);
        *            setConvertForDashCap(IDOMPath::eTriangleCap);
        *            setConvertForMismatchedCaps(true);
        *            setConvertForMismatchedDashCap(true);
        *            setConvertForNonStrokingSegments(true);
        *            setConvertForNonTilingVisualOrImageStrokes(true);
        */
        virtual void setupForPDFStyleOutput() = 0;
        
        /**
        * @brief  Sets whether or not the stroker should convert all stroked paths
        *         into fills. The default is true.
        */
        virtual void setConvertAllStrokes(bool convert) = 0;
        
        /* @brief Conditions. If any are true a stroker will be converted to a fill */ 
        
        /**
        * @brief  Sets whether or not to convert paths with dashed strokes.
        *         The default is false but this is ignored if setConvertAllStrokes()
        *         has been set to true.
        */
        virtual void setConvertForDashedStrokes(bool convert) = 0;
        
        /**
        * @brief  Sets whether or not to convert paths with non-tiling visual or image
        *         brushes. The default is false but this is ignored if
        *         setConvertAllStrokes() has been set to true. This results in simpler
        *         processing for many consumers.
        */
        virtual void setConvertForNonTilingVisualOrImageStrokes(bool convert) = 0;
        
        /**
        * @brief  Sets whether or not the stroker should convert paths with the given
        *         miter treatment mode. Note that if this is set the stroker will not
        *         convert paths where the miters would not be seen, such as if none
        *         of the corners in the path are acute enough to generate miters longer
        *         than the threshold of the path. The default is false in all cases,
        *         but this is ignored if setConvertAllStrokes() has been set to true.
        */
        virtual void setConvertForMiterTreatment(IDOMPathNode::eStrokeMiterLimitTreatment treatment, bool convert = true) = 0;
        
        /**
        * @brief  Sets the approximate allowable amount of error, in pixels at the target
        *         resolution allowed for miter joins. This does not affect the result of
        *         converting a stroke to a filled path, but does affect whether or not to
        *         convert if setConvertForMiterTreatment() has been set to true for the
        *         applicable miter treatment method. It is designed to allow thin paths
        *         where miters would not be easily discernable to be left as is.
        *         The default is 0.0.
        */
        virtual void setMiterErrorThreshold(float maxError) = 0;
        
        /**
        * @brief  Sets whether or not strokes containing the given line cap type
        *         should be converted. In all cases the default is false, but this is
        *         ignored if setConvertAllStrokes() has been set to true.
        */
        virtual void setConvertForLineCap(IDOMPathNode::eStrokeLineCap cap, bool convert = true) = 0;
        
        /**
        * @brief  Sets whether or not strokes containing the given dash cap type
        *         should be converted. In all cases the default is false, but this is
        *         ignored if setConvertAllStrokes() has been set to true.
        */
        virtual void setConvertForDashCap(IDOMPathNode::eStrokeLineCap cap, bool convert = true) = 0;
        
        /**
        * @brief  Sets whether or not strokes where the start and end line cap do not
        *         match should be converted. The default is false, but ignored if
        *         setConvertAllStrokes() has been set to true.
        */
        virtual void setConvertForMismatchedCaps(bool convert) = 0;
        
        /**
        * @brief  Sets whether or not strokes where the line is dashed and the dash
        *         dash cap does not match both of the line capse. The default is false
        *         but ignored if setConvertAllStreoks() has been set to true.
        */
        virtual void setConvertForMismatchedDashCap(bool convert) = 0;
        
        /**
        * @brief  Sets whether or not strokes containing the given line join type
        *         should be converted. In all cases the default is false, but this is
        *         ignored if setConvertAllStrokes() has been set to true.
        */
        virtual void setConvertForJoin(IDOMPathNode::eStrokeLineJoin join, bool convert = true) = 0;
        
        /**
        * @brief  Sets whether or not strokes with any segments marked non-stroking
        *         should be converted. The default is false, but this is ignored if
        *         setConvertAllStrokes() has been set to true.
        */
        virtual void setConvertForNonStrokingSegments(bool convert) = 0;
        
    };


    class IFormUnpackerTransform;
    DECL_SMART_PTR(IFormUnpackerTransform);
     
    /**
    *  @ingroup JawsMako
    *  @brief  A transform for unpacking an IDOMFormInstance directly into the DOM tree.
    *          That is, in the DOM tree the IDOMFormInstance is replaced with the unpacked
    *          contents of the referenced IDOMForm.
    *
    *          The results will be unpacked into the simplest grouping node possible;
    *          either an IDOMGroup, IDOMCanvas or IDOMTransparencyGroup as required.
    *
    *          Useful for consumers that cannot directly handle PDF and PS Form objects.
    *
    *          Note that for this transform, transformPage() does not operate on annotations
    *          as the annotation appearances are themselves, forms.
    */
    class IFormUnpackerTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  JawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IFormUnpackerTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
        *  Configuration
        */
        
        /**
        * @brief  Sets whether or not only IDOMFormInstances with the eNodeUnpackFlag flag
        *         set will be unpacked. The default is false.
        */
        virtual void setUnpackFlaggedFormInstancesOnly(bool unpack) = 0;
        
    };


    class IRendererTransform;
    DECL_SMART_PTR(IRendererTransform);

    /**
    *  @ingroup JawsMako
    *  @brief  A transform for selective rendering of sections of a DOM tree,
    *          replacing the rendered items with an image representation.
    *          Currently only operates on IDOMFixedPages; this restriction should
    *          be eased in future versions.
    *
    *          Useful for rendering sections of the DOM that could not be consumed
    *          by a consumer any other way than as an image. For example, this
    *          transform is used to flatten objects with certain PDF transparency
    *          attributes before generating XAML or XPS output.
    */
    class IRendererTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  JawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IRendererTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /**
        * @brief  Probe the node, checking to see if the renderer transform would
        *         have any effect on this node if transformed.
        */
        virtual bool probe(const IDOMNodePtr &node) = 0;

        /*
        *  Configuration
        */
        
        /**
        * @brief  Sets the target resolution for rendering, in dpi.
        *         The default is 300dpi.
        */
        virtual void setResolution(uint32 resolution) = 0;
        
        /**
        * @brief  Get the target resolution for rendering.
        */
        virtual uint32 getResolution() const = 0;
        
        /**
        * @brief  Sets the target final colour space for rendered content.
        *         scRGB, Indexed, DeviceN or LAB spaces are not allowed.
        *         Further limitations exist if monochrome mode is used;
        *         see the description of setMonochromeMode() for details.
        *         The default is DeviceRGB.
        */
        virtual void setTargetSpace(const IDOMColorSpacePtr &space) = 0;
        
        /**
        * @brief  Get the target colour space for rendering.
        */
        virtual IDOMColorSpacePtr getTargetSpace() const = 0;

        /**
        * @brief  Convenience; Set the target space to CMYK using the default SWOP profile.
        *         This will fail if monochrome mode is used; see the description
        *         of setMonochromeMode() for details.
        */
        virtual void setSWOPTargetSpace() = 0;
        
        /**
        * @brief  Sets the target final colour space for rendered content, using a profile.
        */
        virtual void setTargetProfile(const IDOMICCProfilePtr &profile) = 0;
        
        /**
        * @brief  Sets whether or not to use monochrome output.
        *
        *         Use of this mode is supported only in specific circumstances.
        *         please contact Global Graphics support for details.
        *         
        *         The default is false; that is, continuous tone output will be
        *         produced, in color or grayscale as determined by the selected
        *         target colour space or profile.
        *
        *         If set to true, the output will be one-bit black and white
        *         halftoned images for each sections. This should only be used
        *         for specialised output requirements where the characteristics
        *         of the output device are well known and understood.
        *
        *         If enabling:
        *           - If the current target colour space is not DeviceGray or
        *             sGray, sGray will be set as the target colour space.
        *           - setGenerateMasks() will be set to false. Any attempt to
        *             change to masked output with setGenerateMasks() will result
        *             in an exception being thrown.
        *           - setPreserveSpots() is set to an empty list. Any attemp to
        *             re-enable spot colour preservation will result in an
        *             exception being thrown.
        */
        virtual void setMonochromeMode(bool enable = true) = 0;

        /**
        * @brief  Set the simple spot haltone to be used when monochrome mode is
        *         enabled.
        *
        *         The spot halftone is always at 45 degrees. The default is 60.0f.
        *         The halftone is not used if monochrome mode is not enabled.
        *         The halftone frequency must be greater than 0.
        *         Normally, the resolution at which compositing for transparent
        *         images occurs is capped at twice the halftone frequency. For
        *         some content, this may not be appropriate. Set
        *         useFullResolutionForFlattening for such cases.
        */
        virtual void setSpotHalftone(float frequency, bool useFullResolutionForFlattening = false) = 0;
        
        typedef CEDLVector<uint8> CThresholdArray;
        
        /**
        * @brief Set a threshold array halftone to be used when monochrome mode is
        *        enabled.
        *
        *        Only 8 bit threshold arrays are supported, and the threshold array
        *        must not be larger than 65535 entries. Please refer to section 7.4.5
        *        of the PostScript language reference manual, 3rd edition.
        *
        * @param width The width of the halftone cell, in pixels
        * @param height The height of the halftone cell, in pixels
        * @param thresholdArray The threshold array. Must be width * height in length,
        *        and no longer than 65535 entries.
        */
        virtual void setThresholdHalftone(uint32 width, uint32 height, const CThresholdArray &thresholds) = 0;

        /**
        * @brief  Provide a list of spots to retain in the output, if present.
        *         In the default case, no spots will be retained.
        *         This function will fail if monochrome mode is used; see the
        *         description of setMonochromeMode() for details.
        */
        virtual void setPreserveSpots(const CSpotColorNames &preservedSpots) = 0;
        
        /**
        * @brief  Alternatively preserve all spot colors in the output.
        *         The default is false.
        *         This function will fail if monochrome mode is used; see the
        *         description of setMonochromeMode() for details.
        */
        virtual void setPreserveAllSpots(bool preserve) = 0;
        
        /**
        * @brief  Set whether or not to generated masked images as the rendererd result.
        *         If true, the individual images generated as part of rendering will
        *         have a bilevel alpha channel providing a mask to just the silhouette of
        *         the dom nodes targettted for rendering. This generally results in better
        *         looking results at the intersection between rendered and non-rendered
        *         sections. The default is true.
        *
        *         If monochrome mode is used this function may only be called with false.
        *         See the description of setMonochromeMode() for details.
        */
        virtual void setGenerateMasks(bool generate) = 0;
        
        /**
        * @brief  Set whether the transform should use multiple threads when rendering
        *         sections of the tree. When enabled, a global thread pool is used.
        *         The default is true.
        */
        virtual void setUseMultipleThreads(bool useMultipleThreads) = 0;
        
        /**
        * @brief  Set whether or not to render each renderable object into its own image.
        *         The default is false. Normally the renderer will attempt to produce a
        *         single image for overlapping or nearby objects in an attempt to reduce
        *         the number of images generated and improve performance. Setting this
        *         to true will instead produce a single image in the output for every
        *         object that would be rendered.
        *         If set to true, this overrides any setting made with
        *         setRenderOncePerRenderableArea().
        */
        virtual void setRenderObjectsIndividually(bool renderIndividually) = 0;
        
        /**
        * @brief  Set whether or not to render just one image per renderable region.
        *         That is, if true, the renderer will not produce any overlapping
        *         rendered images in the result. The default is false.
        *         This setting is ignored if setRenderObjectsIndividually() has been
        *         set to true.
        */
        virtual void setRenderOncePerRenderableArea(bool renderOnce) = 0;
        
        /**
        * @brief  Set the maximum amount of allowed overdraw in the rendered results.
        *         Ordinarily the renderer attempts to retain non-renderable content
        *         in its original form, which can require rendering to multiple images.
        *         For cases where the dom tree consists of highly layered combinations
        *         of renderable and non-renderable content, the amount of image data
        *         generated can be very high. This provides a method of putting a cap
        *         on the amount of image data that can be generated. The parameter
        *         is a threshold in units of multiples of the entire candidate area.
        *         That is if a value of 5 is specified, and in ordinary circumstances
        *         rendering would generate more than 5 times the candidate area, then
        *         the renderer will render the results in a single image.
        */
        virtual void setMaximumImageAreaMultiple(float limit) = 0;
        
        /*
        * Object selection for rendering. The rules are cumulative; that is
        * if an object is to be rendered according to any of these if shall be
        * rendered.
        */
        
        /**
        * @brief  Set whether nodes with the IDOMNodeFlags::eNodeRenderFlag set should be
        *         rendered. This is useful if arbitrary objects need to be marked for
        *         rendering within a tree, or if none of the rules below are a good fit.
        *         The default is false.
        */
        virtual void renderNodesWithRenderFlagSet(bool render) = 0;
        
        typedef bool (*ShouldRenderNodeFunc)(void *priv, const IDOMNodePtr &node);

        /**
        * @brief  Provides a callback function, called for each node that is not chosen
        *         for rendering according to any of the other rules. This function returns
        *         true if the node should be rendered. Perhaps simpler in some situations
        *         than applying the marker flag externally and using
        *         renderNodesWithRenderFlagSet(true). priv is passed to the callback
        *         function for every call. The function can also interrogate the children
        *         siblings and parent of the node.
        */
        virtual void setShouldRenderCallback(ShouldRenderNodeFunc func, void *priv) = 0;
        
        /**
        * @brief  Set whether transparent objects should be renderered.
        *         Useful for blanket flattening of all transparency in a DOM tree.
        *         The default is false.
        */
        virtual void renderTransparentNodes(bool render) = 0;
        
        /**
        * @brief  Set whether to render transparent nodes if the page blending group
        *         color space differs from the rendering target color space.
        *
        *         This affects how blending is performed, and non-rendered transparent
        *         content may result in colour differences compared with rendered content
        *         if there is a mismatch.
        *
        *         To avoid this issue, use true here. If true, and the renderer transform
        *         detects that the page blending group color space and the target color
        *         space, then all transparent content will be rendered.
        *         #IDOMColorSpace::similar() is used for this purpose.
        *
        *         If false, the current renderTransparentNodes() setting will be honored.
        *
        *         The default is false.
        */
        virtual void setRenderTransparentNodesOnPageGroupMismatch(bool render) = 0;
        
        /**
        * @brief  Set whether nodes using a blend mode other than eBlendModeNormal
        *         should be rendered. Useful for preparing PDF content for consumers
        *         that do not support these blend modes, such as XPS. The default is
        *         false.
        */
        virtual void renderNonNormalBlendedNodes(bool render) = 0;
        
        /**
        * @brief  Set whether transparency groups that cannot be represented as a canvas
        *         should be rendered. The default is false.
        */
        virtual void renderNonCanvasTransparencyGroups(bool render) = 0;

        /**
        * @brief  Set whether images with boolean mask channels should be rendered.
        *         The default is false.
        */
        virtual void renderMaskedImages(bool render) = 0;
        
        /**
        * @brief  Set whether nodes with opacity masks should be rendered.
        *         The default is false.
        */
        virtual void renderOpacityMaskedNodes(bool render) = 0;
        
        /**
        * @brief  Set whether the brush of the given type should be rendered.
        *         In all cases, the default is false.
        */
        virtual void renderBrush(IDOMBrush::eBrushType brushType, bool render = true) = 0;
        
        /**
        * @brief  Set whether the brush of the given type should be ignored when
        *         scanning for items to render. If true, brushes of that type will
        *         not trigger rendering.
        *         In all cases the default is false.
        */
        virtual void ignoreBrushWhileScanning(IDOMBrush::eBrushType brushType, bool ignore = true) = 0;
        
        /**
        * @brief  Set whether to render Type 3 shading pattern brushes that cannot currently be
        *         converted to a simpler form by IDOMShadingPatternType3Brush::getEquivalentSimpleBrush().
        *         The default is false.
        */
        virtual void renderComplexType3ShadingPatterns(bool render) = 0;
        
        /**
        * @brief  Set whether to render tiling pattern brushes that cannot currently be
        *         converted to a visual brush form by IDOMTilingPatternBrush::getEquivalentVisualBrush()
        *         such as self-intersecting patterns.
        *         The default is false.
        */
        virtual void renderComplexTilingPatterns(bool render) = 0;
        
        /**
        * @brief  Set whether visual brushes with a ViewPort brushes above a certain size should
        *         be rendered. Some consumers cannot reliably handle large visual brushes, and this
        *         provides a method of dealing with this. The default is 0.0, which disables this
        *         feature.
        */
        virtual void renderVisualBrushesWithViewPortLargerThan(double val) = 0;
        
        /**
        * @brief  Set whether radial gradients with a cone shape should be rendererd.
        *         The default is false.
        */
        virtual void renderConeGradients(bool render) = 0;
        
        /**
        * @brief  Set whether glyphs with style simulations should be rendered.
        *         The default is false.
        */
        virtual void renderGlyphsWithStyleSimulation(bool render) = 0;
        
        /**
        * @brief  Set whether glyphs using a restricted font should be rendered. That is, a font
        *         which has bit 1 of the OS/2 fsType field set, or are marked as bitmap only.
        *         The default is false.
        */
        virtual void renderGlyphsWithRestrictedFont(bool render) = 0;

        /**
        * @brief  Set whether glyphs using a font that is not allowed to be subset (according
        *         to its OS/2 fsType field) should be rendered.
        *         The default is false.
        */
        virtual void renderGlyphsWithNonSubsettableFont(bool render) = 0;

        /**
        * @brief  Set whether glyphs using a font that is not allowed in an edited document (according
        *         to its OS/2 fsType field) should be rendered.
        *         The default is false.
        */
        virtual void renderGlyphsWithNonEditableFont(bool render) = 0;

        /**
        * @brief  Set whether glyphs using a Type 3 font should be rendered.
        *         The default is false.
        */
        virtual void renderGlyphsWithType3Font(bool render) = 0;
        
		/**
		* @brief  Set whether a TrueType font's .notdef glyph should be rendered.
		*         Note that this does not control whether or not a containing glyphs node is actually rendered.
		*         However, if a glyphs node is required to be rendered, and a TrueType notdef glyph is required,
		*         this controls whether or not the notdef glyph will be present in the rendered result.
		*         The default is false.
		*/
		virtual void enableTrueTypeNotDef(bool enable) = 0;

        class CInkInfo
        {
          public:
            U8String     inkName;
            IDOMColorPtr inkColor;
        };
        typedef CEDLVector<CInkInfo> CInkInfoVect;
        
        /**
        * @brief   Find all the inks used in the given DOM node tree.
        *          Utility. Also makes an attempt to determine the colour
        *          value of the colourant.
        */
        static JAWSMAKO_API CInkInfoVect findInks(const IJawsMakoPtr &jawsMako, const IDOMNodePtr &nodeTree, const IAbortPtr &abort = IAbortPtr ());
    };
    
    class IRedactorTransform;
    DECL_SMART_PTR(IRedactorTransform);
    
    /**
    *  @ingroup JawsMako
    *  @brief  A transform for applying redaction redactions.
    *
    *          This transform uses rendering to render the area covered by each
    *          redaction annotation including that annotation into an image, and
    *          removing any objects underneath that intersect with the annotation
    *          from the DOM tree entirely.
    *          This transform is only effective for page content, not for
    *          annotations and other off-page graphical content.
    *
    *          As it requires knowledge of the annotations, this transform may
    *          only be applied to IPage objects; anything else will result in an
    *          exception.
    *
    *          As such only transformPage() can be used and any other transform()
    *          routine will throw an exception. If the transformContent argument to
    *          transformPage() is not true the transform will do nothing. Further,
    *          the transformAnnotations parameter will be ignored.
    *
    *          The redaction annotations are removed after they are applied.
    */
    class IRedactorTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  JawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IRedactorTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /**
        * @brief  Set the colour space to use when rendering redacted sections.
        *         The default is DeviceRGB. The colour space must be supported by the
        *         IRendererTransform.
        */
        virtual void setTargetSpace(const IDOMColorSpacePtr &targetSpace) = 0;
        
        /**
        * @brief  Set the target resolution to use when rendering redacted sections.
        *         The default is 300dpi. The resolution must be supported by the
        *         IRendererTransform.
        */
        virtual void setResolution(uint32 resolution) = 0;
    };


    class IType3UnpackerTransform;
    DECL_SMART_PTR(IType3UnpackerTransform);
     
    /**
    *  @ingroup JawsMako
    *  @brief  A transform for unpacking glyphs using a Type 3 font into regular DOM.
    *
    *          Glyphs are recoloured if a plain solid colour brush was used in the glyphs
    *          fill. Otherwise, opacity masking is used to apply the glyphs brush to the
    *          glyphs.
    *
    *          Useful for consumers that cannot directly handle Type 3 fonts.
    */
    class IType3UnpackerTransform : public ITransform
    {
      public:
        /**
        * @brief  Create the transform.
        * @param  JawsMako The JawsMako instance.
        * @return The new instance.
        */
        static JAWSMAKO_API IType3UnpackerTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());

        /*
        *  There is no configuration for this transform.
        */
        
    };
    
    class IOverprintSimulationTransform;
    DECL_SMART_PTR(IOverprintSimulationTransform);

    /**
    *  @ingroup JawsMako
    *  @brief   A transform that modifies DOM such that any overprint present in the
    *           DOM will be visible when written or rendered in an environment that
    *           does not support overprint.
    *
    *           This will usually involve rendering overprinted content and replacing
    *           with an image. The result is a simulation/approximation and may not
    *           exactly match the results achieved on a real world printing device
    *           where all spot colorants are available as real inks.
    *
    *           Note that in order to perform this operation, all colours will be
    *           converted to DeviceCMYK colour space. Use an IColorConverterTransform
    *           after this transform if a different output colour space is required.
    *
    *           It can only operate on entire IDOMFixedPage or IPage nodes. Any attempt
    *           to apply this transform to an individual node or subtree will result in
    *           no changes being made.
    *           Does not apply to content in annotations.
    */
    class IOverprintSimulationTransform: public ITransform
    {
      public:
        static JAWSMAKO_API IOverprintSimulationTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /**
         * @brief  Set the target resolution to use when any rendering must be performed.
         *         The default is 300dpi. The resolution must be supported by the
         *         IRendererTransform.
         */
        virtual void setResolution(uint32 resolution) = 0;
        
        /**
         * @brief  Set whether or not overprint simulation should be performed for 100% black
         *         DeviceGray text.
         *
         *         The default is true.
         */
        virtual void setSimulateBlackDeviceGrayTextOverprint(bool simulate) = 0;
    };

    class IPageCropperTransform;
    DECL_SMART_PTR(IPageCropperTransform);
    
    /**
    *  @brief  Very simple transform for cropping pages to one of the standard boxes.
    */
    class IPageCropperTransform : public ITransform
    {
      public:
        
        typedef enum
        {
            eBCrop,
            eBContent,
            eBTrim,
            eBBleed
        } eBox;
        
        
        /**
        * @brief  Create the transform.
        * @param  JawsMako The JawsMako instance.
        * @return The new instance.
        */
        static IPageCropperTransformPtr create(const IJawsMakoPtr &jawsMako, const IAbortPtr &abort = IAbortPtr ());
        
        /*
        *  Configuration.
        */
        
        /**
        * @brief Sets the box to crop to. The default is the crop box.
        */
        virtual void setCropBox(eBox cropBox) = 0;
        
        /**
        * @brief Sets whether or not the area being cropped should
        *        be clipped also. The default is true.
        */
        virtual void setShouldClip(bool clip) = 0;
    };

}

#endif
