/*
* Copyright (C) 2012-2019 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef JAWSMAKO_TYPES
#define JAWSMAKO_TYPES

/**
 * @file types.h
 * @ingroup JawsMako
 * @brief Common types and required headers for the JawsMako interface.
 *
 * @note Currently, these are simply imported from the EDL namespace.
 * This may change in future releases.
 */

#include <edl/edlfwd.h>
#include <edl/edlstring.h>
#include <edl/smartptr.h>
#include <edl/ircobject.h>
#include <edl/iedlfactory.h>
#include <edl/isession.h>
#include <edl/edlerrors.h>
#include <edl/edltypes.h>
#include <edl/edlvector.h>
#include <edl/idomid.h>
#include <edl/ifilespec.h>

#define JAWSMAKO_API EDL_API

namespace JawsMako
{
    using namespace EDL;
    
    // Forward declaration of common JawsMako classes
    class IJawsMako;
    DECL_SMART_PTR(IJawsMako);
    class IDocument;
    DECL_SMART_PTR(IDocument);
    class IInput;
    DECL_SMART_PTR(IInput);
    class IOutput;
    DECL_SMART_PTR(IOutput);
    class IOutputWriter;
    DECL_SMART_PTR(IOutputWriter);
    class IDocumentAssembly;
    DECL_SMART_PTR(IDocumentAssembly);
    class IPage;
    DECL_SMART_PTR(IPage);
    class IJawsRenderer;
    DECL_SMART_PTR(IJawsRenderer);
    class ITextRun;
    DECL_SMART_PTR(ITextRun);
    class IOptionalContent;
    DECL_SMART_PTR(IOptionalContent);
    class IOptionalContentGroup;
    DECL_SMART_PTR(IOptionalContentGroup);
    class IOptionalContentGroupReference;
    DECL_SMART_PTR(IOptionalContentGroupReference);
    class IOptionalContentDetails;
    DECL_SMART_PTR(IOptionalContentDetails);
    class IOutputIntent;
    DECL_SMART_PTR(IOutputIntent);
    class IMarkedContentDetails;
    DECL_SMART_PTR(IMarkedContentDetails);
    class IMarkedContentStructureDetails;
    DECL_SMART_PTR(IMarkedContentStructureDetails);
    class IStructure;
    DECL_SMART_PTR(IStructure);
    class IStructureElement;
    DECL_SMART_PTR(IStructureElement);
    class IStructureElementReference;
    DECL_SMART_PTR(IStructureElementReference);
    class INamedDestination;
    DECL_SMART_PTR(INamedDestination);
    class IPJLParser;
    DECL_SMART_PTR(IPJLParser);
    class IProgressTick;
    DECL_SMART_PTR(IProgressTick);
  
	/**
     *  @ingroup JawsMako
     *  @brief An error type used for exceptions. Synonymous with IEDLError
     */
    typedef IEDLError IError;

	/**
     *  @ingroup JawsMako
     *  @brief A wide character string (UTF-16 on Windows, UTF-32 on all other platforms)
     */
    typedef EDLString String;
    
	/**
     *  @ingroup JawsMako
     *  @brief A UTF-8 String
     */
    typedef EDLSysString U8String;

	/**
     *  @ingroup JawsMako
     *  @brief A raw, 8 bit string. Encoding depends on context.
     */
    typedef EDLSysString RawString;

	/**
     *  @ingroup JawsMako
     *  @brief An explicit UTF-16 string, regardless of platform
     */
    typedef EDLU16String U16String;
    
	/**
     *  @ingroup JawsMako
     *  @brief An explicit UTF-32 string, regardless of platform
     */	
	typedef EDLU32String U32String;
	
    // Vector types
    
    typedef CEDLStringVect CStringVect;
    typedef CEDLSysStringVect CU8StringVect;
    typedef CEDLSysStringVect CRawStringVect;
    typedef CEDLVector<U8String> CSpotColorNames;
    typedef CEDLVector<EDL::DOMid> CDOMidVector;
    typedef CEDLVector<ITextRunPtr> CTextRunVect;
    typedef CEDLVector<IOptionalContentGroupReferencePtr> COptionalContentGroupReferenceVect;
    typedef CEDLVector<IOutputIntentPtr> COutputIntentVect;
    typedef CEDLVector<IStructureElementPtr> CStructureElementVect;
    typedef CEDLVector<IStructureElementReferencePtr> CStructureElementReferenceVect;
    typedef CEDLVector<INamedDestinationPtr> CNamedDestinationVect;
    typedef CEDLVector<int32> CInt32Vect;
    typedef CEDLVector<uint32> CUInt32Vect;
    typedef CEDLVector<float> CFloatVect;
    typedef CEDLVector<double> CDoubleVect;
    
  
	/**
	 * @brief Optional content Event types
	 */        
	typedef enum
    {
        eOCEUnknown, ///< Unknown
        eOCEView,	 ///< View 
        eOCEPrint,	 ///< Print
        eOCEExport	 ///< Export
    } eOptionalContentEvent;
    
    /**
     *  @brief Convert UTF-8 string to String (wide char)
     *  @param string String to be converted
     *  @return @b String Result of conversion
     */	
	String    U8StringToString(const U8String &string);
    
	/**
     *  @brief Convert UTF-8 string to UTF-16 string
     *  @param string String to be converted
     *  @return @b U16String Result of conversion
     */
    U16String U8StringToU16String(const U8String &string);
    
	/**
     *  @brief Convert UTF-8 string to UTF-32 string
     *  @param string String to be converted
     *  @return @b U32String Result of conversion
     */
    U32String U8StringToU32String(const U8String &string);
    
    /**
     *  @brief Convert String (wide char) to UTF-8 string
     *  @param string String to be converted
     *  @return @b U8String Result of conversion
     */
    U8String  StringToU8String(const String &string);
    
	/**
     *  @brief Convert String (wide char) to UTF-16 string
     *  @param string String to be converted
     *  @return @b U16String Result of conversion
     */
    U16String StringToU16String(const String &string);
	
    /**
     *  @brief Convert String (wide char) to UTF-32 string
     *  @param string String to be converted
     *  @return @b U32String Result of conversion
     */
    U32String StringToU32String(const String &string);

	/**
     *  @brief Convert UTF-16 string to String (wide char)
     *  @param string String to be converted
     *  @return @b String Result of conversion
     */
    String U16StringToString(const U16String &string);

   	/**
     *  @brief Convert UTF-16 string to UTF-8 string
     *  @param string String to be converted
     *  @return @b U8String Result of conversion
     */
	U8String  U16StringToU8String(const U16String &string);
    
	/**
     *  @brief Convert UTF-16 string to UTF-32 string
     *  @param string String to be converted
     *  @return @b U32String Result of conversion
     */
    U32String U16StringToU32String(const U16String &string);
    
    /**
     *  @brief Convert UTF-16 string to String (wide char)
     *  @param string String to be converted
     *  @return @b String Result of conversion
     */
    String    U32StringToString(const U32String &string);

    /**
     *  @brief Convert UTF-32 string to UTF-8 string
     *  @param string String to be converted
     *  @return @b U8String Result of conversion
     */
    U8String  U32StringToU8String(const U32String &string);

    /**
     *  @brief Convert UTF-32 string to UTF-16 string
     *  @param string String to be converted
     *  @return @b U16String Result of conversion
     */
    U16String U32StringToU16String(const U32String &string);

    /**
     *  @brief Normalize to Unicode for comparison purposes
     *  @param string String to be normalized
     *  @return @b String Result of normalization
     */
    String NormalizeStringForCompare(const String &string);

    /**
	 * @brief Normalization forms (for use with \link NormalizeString \endlink )
	 */  
    enum eNormalizationForm
    {
        eNFNFKD, ///< Compatibility Decomposition
        eNFNFKC, ///< Compatibility Decomposition, followed by Canonical Composition
        eNFNFD,  ///< Canonical Decomposition
        eNFNFC   ///< Canonical Decomposition, followed by Canonical Composition
    };

    /**
     *  @brief Normalize to Unicode using the supplied normalization form
     *  @param string            String to be normalized
     *  @param normalizationForm Required normalization form
     *  @return @b String Normalized result
     */
    String NormalizeString(const String &string, eNormalizationForm normalizationForm);

    /**
     *  @brief There are chars that Unicode was not sure whether to classify them as ligatures.
     *         They were not ligatures in 1.0, became ligatures in 1.1 but dropped out in 2.0.
     *  @param string String to be processed
     *  @return @b String Result after processing
     */
    String ReplaceNonListedLigatures(String string);

	/**
	 *  @brief Remove combination characters
	 *  @param string Description for string
	 */    
	void RemoveCombinationCharacters(String &string);

    /**
     *  @brief Combination characters are in the range 0300-036F in Unicode table.
     *         https://en.wikipedia.org/wiki/Combining_character#Unicode_ranges
     *  @param c Character to test
     *  @return @b bool True if combination character, otherwise false
     */
    bool IsCombinationChar(wchar_t c);
    
    /**
     * @class CTemporaryStoreParameters
	 * @ingroup JawsMako
     * @brief Allows the temporary storage parameters to be optionally overridden.
     */
    class CTemporaryStoreParameters
    {
      public:
        /**
        * @brief    Initialise the temporary store parameter block
        *
        * @param    memoryLimit Set the limit, in bytes, on data stored in
        *           memory by the temporary store (IEDLTempStore). Pass zero to use
        *           the default for the current platform. The maximum is 8GB, though
        *           of course on 32 bit systems, it will need to be less than 4GB.
        *           The value given will be rounded down to a multiple of the block
        *           size.
        * @param    diskLimit Set the limit, in bytes, on data stored on
        *           disk by the temporary store (IEDLTempStore). Pass zero to use
        *           the default for the current platform. The maximum is currently
        *           256GB, however for Android this is currently limited to 2GB.
        *           The value given will be rounded down to a multiple of the
        *           block size.
        * @param    blockSize Set the block size to use. The default is
        *           currently 4096 bytes. Each file in the temporary store will
        *           occupy a multiple of the block size.
        *           Note that currently, the total number of blocks must be less than
        *           2147483647. That is, (diskLimit + memoryLimit) / blockSize must
        *           be less than 2147483647.
        */
        CTemporaryStoreParameters(uint64 memoryLimit = 0,
                                  uint64 diskLimit = 0,
                                  uint32 blockSize = 0)
        {
            m_memoryLimit = memoryLimit;
            m_diskLimit = diskLimit;
            m_blockSize = blockSize;
        }
        
      public:
        uint64 m_memoryLimit;
        uint64 m_diskLimit;
        uint32 m_blockSize;
    };

    /**
     * @class IHashable
     * @ingroup JawsMako
     * @brief Simple interface to provide a consisting hashing method for Mako objects.
     */
    class IHashable
    {
      public:
        virtual ~IHashable() {}
        
        /**
         * @brief Obtain a 64 bit hash of the receiving object
         * @return @b uint64 The hash
         */
        virtual uint64 hash() const
        {
            uint64 hashVal = 0;
            updateHash(hashVal);
            return hashVal;
        }
        
        /**
         * @brief Update the given hash to include the receiver
         * @param hash The hash to update
         */
        virtual void updateHash(uint64 &hash) const = 0;
    };

}

// EDL headers required by JawsMako interfaces
#include <edl/idomfont.h>
#include <edl/idomform.h>
#include <edl/idomjobtk.h>
#include <edl/idommetadata.h>
#include <edl/idomoutline.h>
#include <edl/idompage.h>
#include <edl/idomcatalog.h>
#include <edl/idomsecurity.h>
#include <edl/idomedgemode.h>
#include <edl/idomresources.h>

#endif
