/*
* Copyright (C) 2013-2015 Global Graphics Software Ltd. All rights reserved.
*/
 
#ifndef _XAMLGENERATOR_H_
#define _XAMLGENERATOR_H_

/**
* @file    xamlgenerator.h
* @ingroup JawsMako
* @brief   A xaml generator for JawsMako
*/

#include <jawsmako/jawsmako.h>
#include <jawsmako/interactive.h>
#include <jawsmako/transforms.h>

namespace JawsMako
{
    using namespace EDL;
    
    class IXAMLGenerator;
    DECL_SMART_PTR(IXAMLGenerator);

    /**
    * @ingroup JawsMako
    * @class   IXAMLGenerator
    * @brief   A XAML generator for JawsMako, allowing simple generation of
    *          XAML fragments for individual DOM nodes or entire pages.
    *
    *          The XAML is provided in a stream, with resources used by the
    *          XAML presented as streams tracked by instances of this object,
    *          which can be requested via getResource(). Resources are reused
    *          where possible for multiple XAML fragments.
    */
    class IXAMLGenerator : public IRCObject
    {
      public:
        virtual ~IXAMLGenerator() {}
        
        /**
        * @brief   Create a XAML generator instance.
        * @param   JawsMako The JawsMako instance.
        * @param   resourcePrefix A path fragment to prepend to resource names when
        *          generating XAML. Useful for cases where the resources are to be
        *          written to disk somewhere else than alongside the XAML fragment.
        * @return  IXAMLGeneratorPtr The new instance.
        */
        static JAWSMAKO_API IXAMLGeneratorPtr create(const IJawsMakoPtr &jawsMako, const U8String &resourcePrefix = U8String(), const IAbortPtr &abort = IAbortPtr ());
        
        /**
        * @brief   Generate XAML for the given DOM Node, returning the
        *          result in a stream.
        * @param   node The DOM node to be represented as XAML.
        * @param   catalog The catalog associated with the node. Optional,
        *          but required if the names of named elements are to be present
        *          in the generated XAML.
        * @return  IRAInputStreamPtr The XAML stream.
        */
        virtual IRAInputStreamPtr generateXAML(const IDOMNodePtr &node, const IDOMCatalogPtr &catalog = IDOMCatalogPtr()) = 0;
        
        /**
        * @brief   Alternate form of generateXAML() when an existing stream
        *          should be used.
        * @param   node The DOM node to be represented as XAML.
        * @param   IOutputStreamPtr The destination stream for the XAML.
        * @param   catalog The catalog associated with the node. Optional,
        *          but required if the names of named elements are to be present
        *          in the generated XAML.
        */
        virtual void generateXAML(const IDOMNodePtr &node, const IOutputStreamPtr &outputStream, const IDOMCatalogPtr &catalog = IDOMCatalogPtr()) = 0;
        
        /**
        * @brief   Class for receiving XAML generated for annotation appearances in a bulk fashion.
        */
        class CAnnotationXAML
        {
          public:
            IAnnotationPtr           annotation; //!< The annotation for which the XAML was generated
            IAnnotationAppearancePtr appearance; //!< The appearance for which the XAML was generated
            IRAInputStreamPtr        stream;     //!< The generated XAML stream
        };
        typedef CEDLVector<CAnnotationXAML> CAnnotationXAMLVect;

        /**
        * @brief   Generate XAML for the given IPage and all the annotation appearances
        *          present on the page. The XAML for the page will be in the returned stream,
        *          and appearanceXAMLs will be populated with the XAML for all annotation
        *          appearances. This will usually be faster than using generateXAML() on the
        *          page contents and then separately invoking
        *          generateXAMLForPageAnnotationAppearances().
        * @param   page The DOM node to be produced as XAML
        * @param   appearanceXAMLs A vector to receive the XAML for the annotation appearances.
        * @return  IRAInputStreamPtr The XAML stream.
        */
        virtual IRAInputStreamPtr generateXAMLForPageAndAnnotationAppearances(const IPagePtr &page, CAnnotationXAMLVect &appearanceXAMLs) = 0;
        
        /**
        * @brief   Generate XAML for a given annotation appearance.
        *          Provide the annotation and page from which the appearance comes.
        *          The page is required as some annotations may need to be composited
        *          against the page backdrop.
        *          This is slower than invoking generateXAMLForPageAnnotationAppearances()
        *          which is in turn slower than using generateXAMLForPageAndAnnotations(). 
        * @param   appearance The appearance to be presented as XAML.
        * @param   annotation The annotation that contains the appearance. Required
        *          in order to position the annotation against the page background
        *          correctly.
        * @param   page The page the appearance appears on.
        */
        virtual IRAInputStreamPtr generateXAMLForAppearance(const IAnnotationAppearancePtr &appearance, const IAnnotationPtr &annotation, const IPagePtr &page) = 0;
        
        /**
        * @brief   Alternate form of generateXAMLForAppearance() where an existing 
        *          stream should be used.
        *          Provide the annotation and page from which the appearance comes.
        *          The page is required as some annotations may need to be composited
        *          against the page backdrop.
        *          This is slower than invoking generateXAMLForPageAnnotationAppearances()
        *          which is in turn slower than using generateXAMLForPageAndAnnotations(). 
        * @param   appearance The appearance to be presented as XAML.
        * @param   annotation The annotation that contains the appearance. Required
        *          in order to position the annotation against the page background
        *          correctly.
        * @param   page The page the appearance appears on.
        */
        virtual void generateXAMLForAppearance(const IAnnotationAppearancePtr &appearance, const IAnnotationPtr &annotation, const IPagePtr &page, const IOutputStreamPtr &outputStream) = 0;
        
        
        /**
        * @brief   Generate XAML for all the annotation appearances on the given page.
        *          This is generally more efficient than using generateXAMLForAppearance()
        *          repeatedly for every page, but not as fast as using 
        *          generateXAMLForPageAndAnnotations() to generate XAML for the page and
        *          the annotations in one bulk operation.
        * @param   page The page whose annotations should be represented as XAML.
        * @param   appearanceXAMLs A vector to receive the XAML for the annotation appearances.
        */
        virtual void generateXAMLForPageAnnotationAppearances(const IPagePtr &page, CAnnotationXAMLVect &appearanceXAMLs) = 0;
        
        /**
        *  @brief  Get the stream for a named resource.
        *          An exception will be thrown if the resource cannot be found.
        *  @param  name The name of the resource.
        *  @return IInputStreamPtr The resource stream.
        */
        virtual IInputStreamPtr getResource(const U8String &name) = 0;
        
        class CResourceEntry
        {
          public:
            U8String       name;
            IInputStreamPtr resource;
        };

        /**
        *  @brief  Get all the resources in a vector.
        *  @param  resources A reference to a vector to receive the entries.
        */
        virtual void getResources(CEDLVector<CResourceEntry> &resources) = 0;
        
        /*
         * Configuration.
         */
         
        /**
        * @brief   Set whether fonts should be subset in the output.
        *
        *          The default is false. Equivalent to calling setParameter with
        *          "SubsetFonts" as the parameter name.
        *          
        *          Note that all fonts may not be subsetted; for some subsetting may be
        *          forbidden, and for others there may be little gain in subsetting.
        */
        virtual void setSubsetFonts(bool subset) = 0;
        
        /**
        * @brief   Set whether or not an attempt will be made to merge disparate
        *          subsets of a font into a single font.
        * 
        *          The default is false. Equivalent to calling setParameter with
        *          "MergeFonts" as the parameter name.
        *
        *          Some formats such as PostScript and XPS tend to include many
        *          font subsets and for output it is often avantageous to attempt
        *          to merge these fonts into a single font where possible.
        *
        *          Note that it is possible to enable both font subsetting and
        *          merging at the same time. In this case merging happens first,
        *          followed by subsetting of the merged results.
        */
        virtual void setMergeFonts(bool merge) = 0;

        /**
        * @brief   Set if the XAML writer should attempt to merge adjacent images.
        *          The default is true.
        */
        virtual void setMergeImages(bool merge = true) = 0;
        
        /**
        * @brief  Set the desired maximum resolution, threshold and downsampling method
        *         for colour images for XAML output.
        *
        *         The default behaviour is leave the image resolution unchanged.
        * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
        *         images unchanged, which is the default.
        * @param  threshold The threshold above which images will be reduced to the
        *         desired resolution. Pass 0 to use the resolution.
        * @param  method The method to use when downsampling. The default is bicubic
        *         for colour images.
        */
        virtual void setColorImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eBicubic) = 0;
        
        /**
        * @brief  Set the desired maximum resolution, threshold and downsampling method
        *         for gray images.
        *
        *         The default behaviour is leave the image resolution unchanged.
        * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
        *         images unchanged, which is the default.
        * @param  threshold The threshold above which images will be reduced to the
        *         desired resolution. Pass 0 to use the resolution.
        * @param  method The method to use when downsampling. The default is bicubic
        *         for gray images.
        */
        virtual void setGrayImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eBicubic) = 0;
        
        /**
        * @brief  Set the desired maximum resolution, threshold and downsampling method
        *         for monochrome images.
        *
        *         The default behaviour is leave the image resolution unchanged.
        * @param  resolution The desired output resolution, in dpi. Pass 0 to leave
        *         images unchanged, which is the default.
        * @param  threshold The threshold above which images will be reduced to the
        *         desired resolution. Pass 0 to use the resolution.
        * @param  method The method to use when downsampling. The default is subsample
        *         for monochrome images; using any other method will result in grayscale
        *         output.
        */
        virtual void setMonoImageMaxResolution(float resolution, float threshold = 0.0f, IDOMImageDownsamplerFilter::eDownsamplingMethod method = IDOMImageDownsamplerFilter::eSubsample) = 0;
        
        /**
        * @brief  Set the target color space for the output.
        *         The default behaviour is to, where possible, leave the color space
        *         of objects unchanged.
        * @param  colorSpace The desired color space. Must be a simple or ICC space.
        */
        virtual void setTargetColorSpace(const IDOMColorSpacePtr &targetSpace) = 0;
        
        /**
        * @brief  Set the target color space for the output using an ICC profile.
        *         The default behaviour is to, where possible, leave the color space
        *         of objects unchanged.
        * @param  profile The desired profile.
        */
        virtual void setTargetProfile(const IDOMICCProfilePtr &profile) = 0;
        
        /**
        * @brief  Apply the given color converter transform to the contents before
        *         writing XAML. This supersedes the target color space parameters
        *         described above. This allows for more advanced configuration of
        *         the color spaces of the output.
        */
        virtual void applyColorConverterTransform(const IColorConverterTransformPtr &transform) = 0;
        
        /**
        * @brief  Set the resolution to use if page content requires rendering in
        *         order to be output as XAML.
        *         The default is 150dpi. This is affected also by the maximum image
        *         resolution parameters.
        * @param  resolution The desired resolution in dpi.
        */
        virtual void setRenderResolution(uint32 resolution) = 0;
        
        /**
        * @brief  Apply the given renderer transform to the contents before
        *         writing XAML. This supersedes the target color space parameters
        *         described above. This allows for more advanced configuration of
        *         rendering.
        */
        virtual void applyRendererTransform(const IRendererTransformPtr &transform) = 0;
        
        /**
        * @brief  Set the desired image format for color images that need to be
        *         reencoded for XAML output. The default is eEFAuto.
        *         Note: this is advisory only and may not be honoured in all
        *         cases if the image cannot be represented in XAML in the desired
        *         form.
        * @param  format The desired format.
        */
        virtual void setPreferredColorImageFormat(IImageEncoderTransform::eEncodeFormat format) = 0;
        
        /**
        * @brief  Set the desired image format for gray images that need to be
        *         reencoded for XAML output. The default is eEFAuto.
        *         Note: this is advisory only and may not be honoured in all
        *         cases if the image cannot be represented in XPS in the desired
        *         form.
        * @param  format The desired format.
        */
        virtual void setPreferredGrayImageFormat(IImageEncoderTransform::eEncodeFormat format) = 0;
        
        /**
        * @brief  Set the desired image format for monochrome images that need to be
        *         reencoded for XAML output. The default is eEFAuto.
        *         Note: this is advisory only and may not be honoured in all
        *         cases if the image cannot be represented in XPS in the desired
        *         form.
        * @param  format The desired format.
        */
        virtual void setPreferredMonoImageFormat(IImageEncoderTransform::eEncodeFormat format) = 0;
        
        /**
        * @brief  Set the JPEG quality to use when encoding images in JPEG format.
        *         Equivalent to calling setParameter() with the parameter name
        *         "JPEGQuality" and the value being the required quality.
        * @param  quality The desired quality level, with 1 being lowest quality
        *         and 5 being highest quality.
        */
        virtual void setJPEGQuality(uint8 quality) = 0;
        
        /**
        * @brief  Apply the given image encoder transform to the contents before
        *         writing to XAML. This supersedes the image encoding parameters
        *         described above. This allows for more advanced configuration of
        *         image encoding.
        */
        virtual void applyEncoderTransform(const IImageEncoderTransformPtr &transform) = 0;

    };
    
}

#endif

        
